------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Gnat.Command_Line;
USE Gnat.Command_Line;
WITH  Gnat.Os_Lib;
USE Gnat.Os_Lib;

WITH Text_Io;
USE Text_Io;

WITH Version;
USE Version;

WITH Ocarina.Aadl;
USE Ocarina.Aadl;
WITH Types;
USE Types;
WITH Ocarina;
USE Ocarina;
WITH Ocarina.Aadl.Printer;
USE Ocarina.Aadl.Printer;
WITH Ocarina.Aadl;
WITH Ocarina.Aadl.Parser;
WITH Ocarina.Analyzer;
WITH Ocarina.Aadl.Parser;
USE Ocarina.Aadl.Parser;




WITH Ada.Io_Exceptions;
USE  Ada.Io_Exceptions;
WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Ocarina.Parser;
USE  Ocarina.Parser;


PROCEDURE Aadl2aadl IS


   File_Name : Unbounded_String := Empty_String;


   PROCEDURE Usage IS
   BEGIN
      Copyright("aadl2aadl");
      Put_Line (
         "aadl2aadl is a program which displays an aadl file to the screen.");

      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : aadl2aadl [switch] aadl-file-to-display");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      New_Line;
   END Usage;



   Root    : Node_Id := No_Node;
   Success : Boolean := False;

BEGIN

   -- Get arguments
   --
   LOOP
      CASE Getopt ("u v ") IS
         WHEN Ascii.Nul =>
            EXIT;

         WHEN 'u' =>
            Usage;
            Os_Exit(0);

         WHEN OTHERS =>
            Usage;
            Os_Exit(0);
      END CASE;
   END LOOP;

   LOOP
      DECLARE
         S : CONSTANT String := Get_Argument (Do_Expansion => True);

      BEGIN
         EXIT WHEN S'Length = 0;
         File_Name := File_Name & S;
      END;
   END LOOP;


   Copyright("aadl2aadl");



   Ocarina.Initialize;
   Ocarina.Aadl.Initialize;
   Ocarina.Reset;


   Root:=Ocarina.Aadl.Parser.Process ("./AADL_Properties.aadl", No_Node);
   Root:=Ocarina.Aadl.Parser.Process ("./AADL_Project.aadl", Root);
   Root:=Ocarina.Aadl.Parser.Process ("./Cheddar_Properties.aadl", Root);
   Root:=Ocarina.Aadl.Parser.Process (
      "./User_Defined_Cheddar_Properties.aadl", Root);


   -- Read and Parse file
   --
   Root := Ocarina.Parser.Parse  (To_String(File_Name), Root);


   -- Run AADL AST analysis
   --
   IF Root /= No_Node THEN
      Success := Ocarina.Analyzer.Check_Tree_Structure (Root, True);

      IF NOT Success THEN
         Root := No_Node;
      END IF;
   ELSE
      Os_Exit(0);
   END IF;



   --  Check that the AADL specification is ok
   --
   IF Root = No_Node THEN
      Put_Line("aadl2aadl.adb : Syntax error into the AADL specification");
   ELSE
      Ocarina.Aadl.Printer.Print_Node (Standout, Root);
   END IF;




EXCEPTION

   WHEN Gnat.Command_Line.Invalid_Switch    =>
      BEGIN
         Put_Line ("aadl2aadl.adb : invalid Switch ; " & Full_Switch);
         Usage;
      END;
   WHEN Gnat.Command_Line.Invalid_Parameter =>
      BEGIN
         Put_Line ("aadl2aadl.adb : missing parameter for switch ; " &
            Full_Switch);
         Usage;
      END;

   WHEN Ada.Io_Exceptions.Name_Error =>
      Put_Line("aadl2aadl.adb : Can not open project files, Name_Error");
   WHEN Ada.Io_Exceptions.Status_Error =>
      Put_Line("aadl2aadl.adb : Can not open project files, Status_Error");
   WHEN Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("aadl2aadl.adb : Can not open project files, Mode_Error");
   WHEN Ada.Io_Exceptions.Use_Error     =>
      Put_Line("aadl2aadl.adb : Can not open project files, Use_Error");
   WHEN Ada.Io_Exceptions.Device_Error  =>
      Put_Line("aadl2aadl.adb : Can not open project files, Device_Error");
   WHEN Ada.Io_Exceptions.End_Error    =>
      Put_Line("aadl2aadl.adb : Can not open project files, End_Error");
   WHEN Ada.Io_Exceptions.Data_Error   =>
      Put_Line("aadl2aadl.adb : Can not open project files, Data_Error");
   WHEN Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("aadl2aadl.adb : Can not open project files, Layout_Error");

   WHEN OTHERS =>
      Put_Line("This is an internal aadl2aadl bug ... sorry");
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line(
         "Please, send a bug report to cheddar@listes.univ-brest.fr");
      Put_Line(
         "Do not forget to join the AADL files with your bug report");

END Aadl2aadl;


