------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;

WITH Gnat.Command_Line;
USE Gnat.Command_Line;
WITH  Gnat.Os_Lib;
USE Gnat.Os_Lib;

WITH Ada.Io_Exceptions;
USE  Ada.Io_Exceptions;
WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Systems;
USE Systems;
WITH Task_Set;
USE Task_Set;
WITH Processor_Set;
USE Processor_Set;
WITH Buffer_Set;
USE Buffer_Set;
WITH Resource_Set;
USE Resource_Set;
WITH Address_Space_Set;
USE Address_Space_Set;

WITH Xml_Parsers;
USE Xml_Parsers;
WITH Aadl_Parsers;
USE Aadl_Parsers;

WITH Text_Io;
USE Text_Io;

WITH Version;
USE Version;

WITH Project;
USE Project;



PROCEDURE Aadl2xml IS


   File_Name : constant Unbounded_String_Ptr :=
   NEW Unbounded_String;


   PROCEDURE Usage IS
   BEGIN
      Copyright("aadl2xml");
      Put_Line (
         "aadl2xml is a program which translate an AADL specification into an XML Cheddar's project file.");

      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : aadl2xml [switch] aadl-file-to-translate");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line("            -v verbose mode ");
      Put_Line("            -o file-name, write XML file into file-name ");
      New_Line;
   END Usage;


   -- Is the command should run in verbose mode ?
   --
   Verbose : Boolean := False;


   -- The XML to create
   --
   Output_File      : Boolean          := False;
   Output_File_Name : Unbounded_String;


   -- The resulting system
   --
   Sys : System;

BEGIN

   -- Get arguments
   --
   LOOP
      CASE Getopt ("u v o:") IS
         WHEN Ascii.Nul =>
            EXIT;

         WHEN 'o' =>
            Output_File:=True;
            Output_File_Name :=To_Unbounded_String(Parameter);
         WHEN 'v' =>
            Verbose:=True;
         WHEN 'u' =>
            Usage;
            Os_Exit(0);

         WHEN OTHERS =>
            Usage;
            Os_Exit(0);
      END CASE;
   END LOOP;

   LOOP
      DECLARE
         S : CONSTANT String := Get_Argument (Do_Expansion => True);

      BEGIN
         EXIT WHEN S'Length = 0;
         File_Name.All := File_Name.All & S;
      END;
   END LOOP;
   Add(Aadl_Project_File_Name, File_Name);



   Copyright("aadl2xml");

   IF(File_Name.All = "")
         THEN
      Usage;
      Os_Exit(0);
   END IF;


   -- Transform  the file
   --
   IF Verbose THEN
      Put("Transforming the file ");
      Put(To_String(File_Name.All) & " ...." );
   END IF;
   New_Line;
   New_Line;
   New_Line;
   New_Line;

   Read_From_Aadl_File(Sys, Project_File_Dir_List, Aadl_Project_File_Name);


   IF Output_File
         THEN
      Write_To_Xml_File(Sys, Output_File_Name);
   ELSE
      Put_Xml(Sys);
   END IF;


EXCEPTION
   WHEN Task_Set.Invalid_Parameter =>
      Put_Line ("aadl2xml.adb : invalid task argument ; " &
         Exception_Message);
   WHEN Address_Space_Set.Invalid_Parameter =>
      Put_Line ("aadl2xml.adb : invalid addresss space argument ; " &
         Exception_Message);
   WHEN Processor_Set.Invalid_Parameter =>
      Put_Line ("aadl2xml.adb : invalid processor argument ; " &
         Exception_Message);
   WHEN Buffer_Set.Invalid_Parameter =>
      Put_Line ("aadl2xml.adb : invalid buffer argument ; " &
         Exception_Message);
   WHEN Resource_Set.Invalid_Parameter =>
      Put_Line ("aadl2xml.adb : invalid resource argument ; " &
         Exception_Message);


   WHEN Gnat.Command_Line.Invalid_Switch    =>
      BEGIN
         Put_Line ("aadl2xml.adb : invalid Switch ; " & Full_Switch);
         Usage;
      END;
   WHEN Gnat.Command_Line.Invalid_Parameter =>
      BEGIN
         Put_Line ("aadl2xml.adb : missing parameter for switch ; " &
            Full_Switch);
         Usage;
      END;

   WHEN Xml_Read_Error =>
      Put_Line(
         "aadl2xml.adb : Can not read project file ; Xml Fatal Error ; "
         & Exception_Message);
   WHEN Aadl_Read_Error =>
      Put_Line(
         "aadl2xml.adb : Can not read project files ; AADL parsing Error ; "
         &  Exception_Message);

   WHEN Ada.Io_Exceptions.Name_Error =>
      Put_Line("aadl2xml.adb : Can not open project files, Name_Error");
   WHEN Ada.Io_Exceptions.Status_Error =>
      Put_Line("aadl2xml.adb : Can not open project files, Status_Error");
   WHEN Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("aadl2xml.adb : Can not open project files, Mode_Error");
   WHEN Ada.Io_Exceptions.Use_Error     =>
      Put_Line("aadl2xml.adb : Can not open project files, Use_Error");
   WHEN Ada.Io_Exceptions.Device_Error  =>
      Put_Line("aadl2xml.adb : Can not open project files, Device_Error");
   WHEN Ada.Io_Exceptions.End_Error    =>
      Put_Line("aadl2xml.adb : Can not open project files, End_Error");
   WHEN Ada.Io_Exceptions.Data_Error   =>
      Put_Line("aadl2xml.adb : Can not open project files, Data_Error");
   WHEN Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("aadl2xml.adb : Can not open project files, Layout_Error");

   WHEN OTHERS =>
      Put_Line("This is an internal aadl2xml bug ... sorry");
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line(
         "Please, send a bug report to cheddar@listes.univ-brest.fr");
      Put_Line(
         "Do not forget to join the AADL/XML Cheddar project files with your bug report");

END Aadl2xml;



