------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE  Gtk;
WITH Gtk.Main;


WITH Ada.Io_Exceptions;
USE  Ada.Io_Exceptions;

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Ada.Strings.Unbounded;
USE  Ada.Strings.Unbounded;


WITH Version;
USE  Version;
WITH Framework_Config;
USE  Framework_Config;

WITH Project;
USE  Project;
WITH Systems;
USE  Systems;
WITH Task_Set;
USE Task_Set;
WITH Processor_Set;
USE Processor_Set;
WITH Buffer_Set;
USE Buffer_Set;
WITH Resource_Set;
USE Resource_Set;
WITH Address_Space_Set;
USE Address_Space_Set;

WITH Xml_Parsers;
USE Xml_Parsers;
WITH Aadl_Parsers;
USE Aadl_Parsers;

WITH Root_Window_Pkg;
USE  Root_Window_Pkg;
WITH Graph_Window_Pkg;
USE  Graph_Window_Pkg;
WITH My_Canvas;
USE  My_Canvas;


WITH Gtk.Widget;
USE  Gtk.Widget;
WITH Gdk.Font;
USE  Gdk.Font;
WITH Gtk.Style;
USE  Gtk.Style;

WITH Gnat.Command_Line;
USE  Gnat.Command_Line;

WITH Gnat.Os_Lib;
USE  Gnat.Os_Lib;

WITH Text_Io;
USE  Text_Io;

WITH Framework_Config;
USE Framework_Config;

WITH Translate;
USE Translate;
WITH Translate.English_Labels;
WITH Translate.Francais_Labels;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;

WITH Debug;
USE Debug;


PROCEDURE Cheddar IS



   PROCEDURE Usage IS
   BEGIN
      Copyright("Cheddar");
      Put_Line (
         "Cheddar is a free real time scheduling simulator.");
      Put_Line(
         "This program provides services to automatically check temporal constraints of real time tasks.");
      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : cheddar [switches] foo");
      Put_Line("      where foo can be : ");
      Put_Line("            - an unique XML file ");
      Put_Line("            - one or several AADL files");
      New_Line;
      Put_Line("   Switches can be :");

      Put_Line("            -u get this help");

      Put_Line(
         "            -l select a language : ""fr"" for franais; ""en"" for english ");
      Put("               Default language is ");
      IF(Current_Language = English)
            THEN
         Put("English.");
      ELSE
         Put("Franais.");
      END IF;
      New_Line;

      Put(
         "            -a the file names given to cheddar contain AADL descriptions instead of an XML file");
      Put_Line(
         " Only one XML file can be provided to Cheddar but several AADL files can be sent to Cheddar.");

      Put_Line(
         "            -I directory-name : gives an extra directory name where to look for XML or AADL files. By default, Cheddar only looks for project files into the Cheddar's current directory.");
      Put_Line(
         "            -d activate Cheddar's debug mode ");
      Put_Line(
         "            -c put the current Cheddar's configuration at the screen");

   END Usage;




   My_Style          : Gtk_Style;
   Global_Font       : Gdk_Font;
   Aadl_File         : Boolean;
   A_Dir_Name        : Unbounded_String_Ptr;
   Project_File_Name : Unbounded_String_Ptr;
   Project_File_List : Unbounded_String_List;
e : positive :=1;

BEGIN

e:=e-1;

   Copyright("cheddar");


   -- New project
   --
   Aadl_File:=False;
   Initialize(Project_File_List);
   Do_New;



   -- Get arguments
   --
   LOOP
      CASE Getopt ("c d u l: a I:") IS
         WHEN Ascii.Nul =>
            EXIT;
         WHEN 'c' =>
            Framework_Config.Put;
            
         WHEN 'a' =>
            Aadl_File:=True;

         WHEN 'd' =>
            Cheddar_Debug:=True;

         WHEN 'I' =>
            A_Dir_Name:= NEW Unbounded_String;
            A_Dir_Name.All:=To_Unbounded_String(Parameter);
            Put_Debug("New project file directory : " & To_String(
                  A_Dir_Name.All));
            Add(Project_File_Dir_List, A_Dir_Name);

         WHEN 'u' =>
            Usage;
            Os_Exit(0);

         WHEN 'l' =>
            IF (Parameter /= "fr") AND
                  (Parameter /= "en")
                  THEN
               Put_Line("cheddar.adb : invalid language parameter");
               New_Line;
               Usage;
               Os_Exit(0);
            ELSE
               IF Parameter = "fr"
                     THEN
                  Current_Language:=Francais;
               ELSE
                  Current_Language:=English;
               END IF;
            END IF;

         WHEN OTHERS =>
            Usage;
            Os_Exit(0);
      END CASE;
   END LOOP;


   LOOP
      Project_File_Name:=NEW Unbounded_String;
      Project_File_Name.All:=To_Unbounded_String(Get_Argument (
            Do_Expansion =>
            True));

      EXIT WHEN Project_File_Name.All = Empty_String;

      Put_Debug("Project file to load : " & To_String(
            Project_File_Name.All));

      Add(Project_File_List, Project_File_Name);
   END LOOP;



   IF (Get_Number_Of_Elements(Project_File_List) > 1) AND (NOT Aadl_File)
         THEN
      Put_Line(
         "cheddar.adb : can not load more than one XML file at the same time ");
      Os_Exit(0);
   END IF;


   Translate.Initialize;
   IF Current_Language=English THEN
      Translate.English_Labels.Initialize;
   ELSE
      Translate.Francais_Labels.Initialize;
   END IF;


   IF(Get_Number_Of_Elements(Project_File_List) > 0)
         THEN
      Is_New:=False;
      Is_Saved:=True;
      IF (Aadl_File)
            THEN
         Duplicate(Project_File_List, Aadl_Project_File_Name);
         Systems.Read_From_Aadl_File(Sys, Project_File_Dir_List,
            Aadl_Project_File_Name);

         --
         -- The piece of code below allows us to call aadl2xml externally instead of
         -- directly calling the package aadl_parser.ads
         --
         --         declare
         --            Ok                        : Boolean;
         --            Spawn_Args                : String_List (1 .. 2);
         --            Temp_File_Name            : aliased String       := "-o cheddar_temp_file.xml";
         --            Aadl_File_Name_To_Perform : aliased String       := To_String
         --              (Aadl_Project_File_Name);
         --         begin
         --            Spawn_Args(1) := Temp_File_Name'Unchecked_Access;
         --            Spawn_Args(2):=Aadl_File_Name_To_Perform'Unchecked_Access;
         --            Spawn("aadl2xml", Spawn_Args, Ok);
         --            if not Ok then
         --               Put_Line("marche pas");
         --            end if;
         --         end;

      ELSE
         Project_File_Name:=Get_Head(Project_File_List);
         Xml_Project_File_Name:=Project_File_Name.All;
         Systems.Read_From_Xml_File(Sys, Project_File_Dir_List,
            Xml_Project_File_Name);
      END IF;
   END IF;



   -- Initialize the graphical editor
   --
   Gtk.Main.Set_Locale;
   Gtk.Main.Init;

   Load (Global_Font,
      "-*-*-*-*-*-*-*-*-*-*-*-*-iso8859-1");
   IF Global_Font = Null_Font
         THEN
      Put_Line("cheddar.adb : warning, can not load ISO 8859-1 font ");
   END IF;

   Gtk_New(My_Style);
   Set_Font(My_Style, Global_Font);
   Set_Default_Style (My_Style);

   Gtk_New (Root_Window);
   Show_All(Root_Window);


   -- Graphical editor event loop
   --
   Gtk.Main.Main;



EXCEPTION

   WHEN Task_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid task argument ; " &
         Exception_Message);
   WHEN Address_Space_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid addresss space argument ; " &
         Exception_Message);
   WHEN Processor_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid processor argument ; " &
         Exception_Message);
   WHEN Buffer_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid buffer argument ; " &
         Exception_Message);
   WHEN Resource_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid resource argument ; " &
         Exception_Message);

   WHEN Gnat.Command_Line.Invalid_Switch    =>
      BEGIN
         Put_Line ("cheddar.adb : invalid Switch ; " & Full_Switch);
         Usage;
      END;
   WHEN Gnat.Command_Line.Invalid_Parameter =>
      BEGIN
         Put_Line ("cheddar.adb : missing parameter for switch ; " &
            Full_Switch);
         Usage;
      END;

   WHEN Xml_Read_Error =>
      Put_Line(
         "cheddar.adb : Can not read project file ; Xml Fatal Error ; "
         & Exception_Message);
   WHEN Aadl_Read_Error =>
      Put_Line(
         "cheddar.adb : Can not read project files ; AADL parsing Error ; "
         &  Exception_Message);

   WHEN Ada.Io_Exceptions.Name_Error =>
      Put_Line("cheddar.adb : Can not open project files, Name_Error");
   WHEN Ada.Io_Exceptions.Status_Error =>
      Put_Line(
         "cheddar.adb : Can not open project files, Status_Error");
   WHEN Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("cheddar.adb : Can not open project files, Mode_Error");
   WHEN Ada.Io_Exceptions.Use_Error     =>
      Put_Line("cheddar.adb : Can not open project files, Use_Error");
   WHEN Ada.Io_Exceptions.Device_Error  =>
      Put_Line(
         "cheddar.adb : Can not open project files, Device_Error");
   WHEN Ada.Io_Exceptions.End_Error    =>
      Put_Line("cheddar.adb : Can not open project files, End_Error");
   WHEN Ada.Io_Exceptions.Data_Error   =>
      Put_Line("cheddar.adb : Can not open project files, Data_Error");
   WHEN Ada.Io_Exceptions.Layout_Error  =>
      Put_Line(
         "cheddar.adb : Can not open project files, Layout_Error");


   WHEN OTHERS =>
      Put_Line("This is an internal Cheddar bug ... sorry");
      Put_Line(
         "WARNING           : your project was saved in the file cheddar_bug.xml");
      Put_Line("Exception name    : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);

      IF Graph_Window/=NULL
            THEN
         Get_Coordinates(Graph_Window.Buffer, Sys);
      END IF;
      Systems.Write_To_Xml_File(Sys, To_Unbounded_String(
            "cheddar_bug.xml"));

      Put_Line(
         "Please, send a bug report to cheddar@listes.univ-brest.fr");
      Put_Line(
         "Do not forget to join your AADL/XML/SC Cheddar project files with your bug report");

END Cheddar;




