------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Gnat.Command_Line;
USE Gnat.Command_Line;
WITH  Gnat.Os_Lib;
USE Gnat.Os_Lib;

WITH Ada.Io_Exceptions;
USE  Ada.Io_Exceptions;
WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Systems;
USE Systems;

WITH Parser;
USE Parser;
WITH  Scheduler_Io;
USE  Scheduler_Io;
WITH Text_Io;
USE Text_Io;

WITH Version;
USE Version;

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;

WITH Expressions;
USE Expressions;
USE Expressions.Variables_Type_Package;


WITH Io_Tools;
USE Io_Tools;


PROCEDURE Sc IS


   Sc_File,
   File_Name : Unbounded_String;




   PROCEDURE Usage IS
   BEGIN
      Copyright("sc");
      Put_Line (
         "Sc is a compiler that checks if a user-defined code file is correct or not.");

      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : sc [switch] sc-file-to-check");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line("            -v verbose mode ");
      New_Line;
   END Usage;


   -- Is the sc command should run in verbose mode ?
   --
   Verbose : Boolean := False;


   -- The parser have to read a system description to
   -- set user defined parameters ...
   -- This is a "empty" dummy application description.
   --
   Dummy : System;




BEGIN

   -- Get arguments
   --
   LOOP
      CASE Getopt ("u v") IS
         WHEN Ascii.Nul =>
            EXIT;

         WHEN 'v' =>
            Verbose:=True;
         WHEN 'u' =>
            Usage;
            Os_Exit(0);

         WHEN OTHERS =>
            Usage;
            Os_Exit(0);
      END CASE;
   END LOOP;

   LOOP
      DECLARE
         S : CONSTANT String := Get_Argument (Do_Expansion => True);

      BEGIN
         EXIT WHEN S'Length = 0;
         File_Name := File_Name & S;
      END;
   END LOOP;


   Copyright("sc");

   IF(File_Name = Empty_String)
         THEN
      Usage;
      Os_Exit(0);
   END IF;


   -- Parse the file
   --
   IF Verbose THEN
      Put("Parsing the file ");
      Put(To_String(File_Name) & " ...." );
   END IF;
   New_Line;
   New_Line;
   New_Line;
   New_Line;


   Sc_File := Read_Sequential_File(File_Name);

   -- Start parser
   --
   Open_Input(Sc_File);
   Parser.Init_Parser;



   Parser.Yyparse;


   -- Display syntax tree and compiling information
   --
   IF Verbose THEN
      Put(Root_Statement_Pointer);
      Put(Variables_Table);
   END IF;




   New_Line;
   New_Line;
   New_Line;
   Put_Line("Syntax of the file " & To_String(File_Name) & " is ok ");
   New_Line;


EXCEPTION
   WHEN Expressions.Syntax_Error =>
      BEGIN
         New_Line;
         New_Line;
         Put_Line(To_String(Lb_Syntax_Error(Current_Language) & Lb_Colon &
               To_Unbounded_String(Exception_Message)));

         New_Line;
      END;
   WHEN Gnat.Command_Line.Invalid_Switch    =>
      BEGIN
         Put_Line ("sc.adb : invalid Switch " & Full_Switch);
         Usage;
      END;
   WHEN Gnat.Command_Line.Invalid_Parameter =>
      Put_Line ("sc.adb : missing parameter for switch " &
         Full_Switch);

   WHEN Ada.Io_Exceptions.Name_Error =>
      Put_Line("sc.adb : Can not open project files, Name_Error");
   WHEN Ada.Io_Exceptions.Status_Error =>
      Put_Line("sc.adb : Can not open project files, Status_Error");
   WHEN Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("sc.adb : Can not open project files, Mode_Error");
   WHEN Ada.Io_Exceptions.Use_Error     =>
      Put_Line("sc.adb : Can not open project files, Use_Error");
   WHEN Ada.Io_Exceptions.Device_Error  =>
      Put_Line("sc.adb : Can not open project files, Device_Error");
   WHEN Ada.Io_Exceptions.End_Error    =>
      Put_Line("sc.adb : Can not open project files, End_Error");
   WHEN Ada.Io_Exceptions.Data_Error   =>
      Put_Line("sc.adb : Can not open project files, Data_Error");
   WHEN Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("sc.adb : Can not open project files, Layout_Error");


   WHEN OTHERS =>
      Put_Line("This is an internal sc bug ... sorry");
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line(
         "Please, send a bug report to cheddar@listes.univ-brest.fr");
      Put_Line(
         "Do not forget to join the XML/SC Cheddar files with your bug report");



END Sc;



