------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Finalization;

WITH Systems;
USE Systems;


WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;



PACKAGE Aadl_Parsers IS


   -- Exception raised by clients of Xml_Parses when something goes
   -- wrong during XML parsing
   --
   Aadl_Read_Error : EXCEPTION;


   -- aadl parser
   --
   TYPE Aadl_Project_Parser IS NEW Ada.Finalization.Controlled WITH PRIVATE;


   -- Methods for all Cheddar XML parsers
   --
   PROCEDURE Initialize_Project_Parser (
         Handler : IN OUT Aadl_Project_Parser);

   FUNCTION Get_Parsed_System (
         Handler : IN     Aadl_Project_Parser)
     RETURN System;

   PROCEDURE Initialize (
         Handler : IN OUT Aadl_Project_Parser);

   PROCEDURE Parse (
         Handler           : IN OUT Aadl_Project_Parser;
         Dir_List          : IN     Unbounded_String_List;
         Project_File_List : IN     Unbounded_String_List);



PRIVATE


   TYPE Aadl_Project_Parser IS NEW Ada.Finalization.Controlled WITH
      RECORD
         -- At the end of the parser execution, the project
         -- is store in the objet below
         --
         Parsed_System : System;
      END RECORD;






END Aadl_Parsers;



