------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ocarina.Aadl.Parser;
WITH Ocarina.Aadl.Printer;
WITH Ocarina.Analyzer;
WITH Ocarina.Aadl;
USE Ocarina.Aadl;
WITH Types;
USE Types;
WITH Ocarina.Debug;
USE Ocarina.Debug;
WITH Ocarina.Entities;
USE Ocarina.Entities;
WITH Ocarina.Entities.Components;
USE  Ocarina.Entities.Components;
WITH Ocarina.Entities.Properties;
USE Ocarina.Entities.Properties;

WITH Ocarina.Entities.Components.Connections;
USE Ocarina.Entities.Components.Connections;
WITH Ocarina.Nutils;
USE Ocarina.Nutils;
WITH Ocarina.Nodes;
USE Ocarina.Nodes;
WITH Ocarina.Aadl_Values;
USE Ocarina.Aadl_Values;
WITH Ocarina.Aadl.Tokens;
USE Ocarina.Aadl.Tokens;
WITH Ocarina.Aadl.Printer.Properties;
USE Ocarina.Aadl.Printer.Properties;
WITH Ocarina.Aadl.Printer.Components.Connections;
USE Ocarina.Aadl.Printer.Components.Connections;
WITH Ocarina.Aadl.Printer.Components;
USE Ocarina.Aadl.Printer.Components;
WITH Ocarina.Aadl.Printer.Components.Modes;
USE Ocarina.Aadl.Printer.Components.Modes;
WITH Ocarina.Aadl.Printer.Identifiers;
USE Ocarina.Aadl.Printer.Identifiers;
WITH Output;
USE Output;
WITH Ocarina.Analyzer;
USE Ocarina.Analyzer;
WITH Ocarina.Aadl.Tokens;
USE Ocarina.Aadl.Tokens;




WITH Text_Io;
USE Text_Io;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;
WITH Config;
USE Config;
WITH Task_Offsets;
USE Task_Offsets;
USE Task_Offsets.Offsets_Table_Package;
WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Ada.Strings.Unbounded.Text_Io;
USE Ada.Strings.Unbounded.Text_Io;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Ada.Strings;
USE Ada.Strings;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;

WITH Tables;
WITH Ada.Exceptions;
USE Ada.Exceptions;


WITH Resources;
USE Resources;
USE Resources.Task_Lists;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Event_Analyzer_Set;
USE Event_Analyzer_Set;
USE Event_Analyzer_Set.Generic_Event_Analyzer_Set;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Buffers;
USE Buffers;
USE Buffers.Buffer_Roles;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;


WITH Task_Dependencies;
USE Task_Dependencies;

WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;

WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;

WITH Queueing_System;
USE Queueing_System;

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Scheduler;
USE Scheduler;

WITH Io_Tools;
USE Io_Tools;

WITH Ada.Strings.Maps;
USE Ada.Strings.Maps;


WITH Translate;
USE Translate;




PACKAGE BODY  Aadl_Parsers IS




   -- Set to true means we already had initialize Ocarina
   --
   First_Aadl_Parsing : Boolean := True;




   TYPE Pass_Number_Type IS NEW Integer RANGE 1 .. 3;


   Critical_Section_List : Unbounded_String_List;




   -- Look for an AADL file from a set of directory and load it
   -- into ocarina
   --
   FUNCTION Load_Aadl_File_From_Directory (
         Root      : IN     Node_Id;
         File_Name : IN     Unbounded_String;
         Dir_List  : IN     Unbounded_String_List)
     RETURN Node_Id;
   FUNCTION Load_Aadl_File_From_Directory (
         Root      : IN     Node_Id;
         File_Name : IN     String;
         Dir_List  : IN     Unbounded_String_List)
     RETURN Node_Id;







   -- Information retrieved from the aadl parser
   --

   Dummy_Identifier : Unbounded_String := To_Unbounded_String ("__missing_cheddar_aadl_cpu_identifier__");

   Name                 : Unbounded_String;
   Deadline             : Integer;
   Start_Time           : Integer;
   Blocking_Time        : Integer;
   Criticality          : Integer;
   State                : Integer;
   Period               : Integer;
   Capacity             : Integer;
   Cpu_Name             : Unbounded_String;
   Address_Space_Name   : Unbounded_String;
   Jitter               : Integer;
   Policy               : Policies;
   Task_Type            : Tasks_Type;
   Seed                 : Integer;
   Predictable          : Boolean;
   Quantum              : Integer;
   Priority             : Integer;
   A_Scheduler          : Scheduler_Type;
   A_Network            : Unbounded_String;
   Protocol             : Resources_Type;
   Is_Preemptive        : Preemptive_Type;
   Text_Memory_Size     : Integer;
   Heap_Memory_Size     : Integer;
   Data_Memory_Size     : Integer;
   Stack_Memory_Size    : Integer;
   Parametric_File_Name : Unbounded_String;

   Roles             : Buffer_Roles_Table;
   A_Role            : Buffer_Role;
   Offset            : Offsets_Table;
   A_Task_List_Entry : Affected_Task_List;
   A_Task_List       : Task_List;
   Param             : User_Defined_Parameters_Table;
   A_Param           : Parameter_Ptr;

   Task1 : Task_Ptr;

   Ok : Boolean;


   -- Data to store processor and bus bindings
   --
   TYPE Binding_Record_Type IS
      RECORD
         Cpu_Name           : Unbounded_String;
         Address_Space_Name : Unbounded_String;
      END RECORD;

   PROCEDURE Put (
         A_Binding : Binding_Record_Type) IS
   BEGIN
      Put(A_Binding.Cpu_Name);
      Put(A_Binding.Address_Space_Name);
   END Put;


   PACKAGE Processor_Binding_Package IS NEW Tables(Binding_Record_Type,
      Config.Max_Address_Space, Put);
   USE Processor_Binding_Package;

   SUBTYPE Processor_Binding_Table IS Processor_Binding_Package.Table;

   Cpu_Bindings : Processor_Binding_Table;




   PROCEDURE Initialize_Parsed_Data IS

   BEGIN

      State:=0;
      Task_Type:=Aperiodic_Type;
      Policy:=Sched_Fifo;
      Is_Preemptive:=Preemptive;
      Seed := 0;
      Predictable:=True;
      Priority:=1;
      Name:=To_Unbounded_String("");
      Deadline:=0;
      Period:=0;
      Start_Time:=0;
      Criticality:=0;
      Blocking_Time:=0;
      Capacity:=0;
      Jitter:=0;
      Quantum:=0;
      A_Scheduler:=Rate_Monotonic_Protocol;
      Text_Memory_Size:=0;
      Heap_Memory_Size:=0;
      Data_Memory_Size:=0;
      Stack_Memory_Size:=0;
      Protocol:=No_Protocol;

      A_Network:= To_Unbounded_String("");
      Parametric_File_Name:=To_Unbounded_String("");
      Cpu_Name:=To_Unbounded_String("");
      Address_Space_Name:=To_Unbounded_String("");
      Parametric_File_Name:=To_Unbounded_String("");

      Initialize(A_Task_List_Entry);
      Initialize(A_Task_List);
      Initialize(Offset);
      Initialize(Param);
      Initialize(Roles);
      Initialize(A_Role);
      Initialize(Cpu_Bindings);
   END Initialize_Parsed_Data;


   PROCEDURE Initialize_Project_Parser (
         Handler : IN OUT Aadl_Project_Parser) IS
   BEGIN
      Initialize(Handler.Parsed_System);
      Initialize_Parsed_Data;
   END Initialize_Project_Parser;




   FUNCTION Get_Parsed_System (
         Handler : IN     Aadl_Project_Parser)
     RETURN System IS
   BEGIN
      RETURN Handler.Parsed_System;
   END Get_Parsed_System;


   PROCEDURE Initialize (
         Handler : IN OUT Aadl_Project_Parser) IS
   BEGIN
      Initialize(Handler.Parsed_System);
      Initialize_Parsed_Data;
   END Initialize;



   ------------------------------------------------------------------------
   -- Parse AADL properties
   ------------------------------------------------------------------------





   PROCEDURE Read_Property_Associations (
         Handler                 : IN OUT Aadl_Project_Parser;
         Component_Type          : IN     Node_Id;
         Component_Instance_Name : IN     Unbounded_String;
         Path                    : IN     Unbounded_String;
         Pass                    : IN     Pass_Number_Type) IS


      -- Display a property (debug only)
      --
      PROCEDURE Display_Property (
            A_Property_Node : IN     Node_Id) IS
      BEGIN
         IF Aadl_Debug THEN
            IF Pass = 1 THEN
               Put("First");
            ELSE
               IF Pass = 2 THEN
                  Put("Second");
               ELSE
                  Put("Third");
               END IF;
            END IF;
            Put(" pass ; ");
            Print_Property_Association(A_Property_Node);
         END IF;
      END Display_Property;


      -- aadl enumeration properties
      --

      PROCEDURE Read_Concurrency_Control_Protocol_Property (
            A_Property_Value : Node_Id) IS
         Protocol_String : Unbounded_String;
      BEGIN
         Protocol_String:=To_Lower(To_Unbounded_String(Image(Display_Name(
                     A_Property_Value))));

         IF (Protocol_String /= "no_protocol")
               AND (Protocol_String /= "priority_inheritance_protocol")
               AND (Protocol_String /= "priority_ceiling_protocol")
               AND (Protocol_String /=
               "immediate_priority_ceiling_protocol")
               THEN
            Raise_Exception(Aadl_Read_Error'Identity,
               "Property Concurrency_Control_Protocol should store No_Protocol, Priority_Ceiling_Protocol, Immediate_Priority_Ceiling_Protocol or Priority_Inheritance_Protocol");
         END IF;
         IF Protocol_String = "no_protocol"
               THEN
            Protocol:=No_Protocol;
         END IF;
         IF Protocol_String = "priority_inheritance_protocol"
               THEN
            Protocol:=Priority_Inheritance_Protocol;
         END IF;
         IF Protocol_String = "priority_ceiling_protocol"
               THEN
            Protocol:=Priority_Ceiling_Protocol;
         END IF;
         IF Protocol_String = "immediate_priority_ceiling_protocol"
               THEN
            Protocol:=Immediate_Priority_Ceiling_Protocol;
         END IF;
      END Read_Concurrency_Control_Protocol_Property;

      PROCEDURE Read_Scheduling_Protocol_Property (
            A_Property_Value : Node_Id) IS
         Scheduler_Name : Unbounded_String;
      BEGIN
         Scheduler_Name:=To_Unbounded_String(Image(Display_Name(
                  A_Property_Value)));
         To_Scheduler(To_Upper(Scheduler_Name), A_Scheduler, Ok);
         IF NOT Ok THEN
            Raise_Exception(Aadl_Read_Error'Identity,
               "Property Scheduling_Protocol should store a valid scheduler name");
         END IF;
      END Read_Scheduling_Protocol_Property;

      PROCEDURE Read_Dispatch_Protocol_Property (
            A_Property_Value : Node_Id) IS
         Activation : Unbounded_String;
      BEGIN
         Activation:=To_Lower(To_Unbounded_String(Image(Display_Name(
                     A_Property_Value))));

         IF (Activation /= "periodic")
               AND (Activation /= "poisson_process")
               AND (Activation /= "parametric")
               AND (Activation /= "sporadic")
               AND (Activation /= "background")
               THEN
            Raise_Exception(Aadl_Read_Error'Identity,
               "Property Dispatch_Protocol should store Periodic, Sporadic, Background, Poisson_Process or Parametric");
         END IF;

         IF (Activation = "periodic")
               THEN
            Task_Type := Periodic_Type;
         END IF;
         IF (Activation = "parametric")
               THEN
            Task_Type := Parametric_Type;
         END IF;
         IF (Activation = "poisson_process")
               THEN
            Task_Type := Poisson_Type;
         END IF;
         IF (Activation = "sporadic")
               THEN
            Task_Type := Sporadic_Type;
         END IF;
         IF (Activation = "background")
               THEN
            Task_Type := Aperiodic_Type;
         END IF;

      END Read_Dispatch_Protocol_Property;

      PROCEDURE Read_Posix_Scheduling_Policy_Property (
            A_Property_Value : Node_Id) IS
         Policy_String : Unbounded_String;
      BEGIN
         Policy_String:=To_Unbounded_String(Image(Display_Name(
                  A_Property_Value)));
         To_Policy(To_Upper(Policy_String), Policy, Ok);
         IF NOT Ok THEN
            Raise_Exception(Aadl_Read_Error'Identity,
               "Property POSIX_Scheduling_Policy should store SCHED_FIFO, SCHED_OTHERS of SCHED_RR");
         END IF;
      END Read_Posix_Scheduling_Policy_Property;



      --  aadlinteger/aadlfloat properties
      --

      PROCEDURE Read_Compute_Execution_Time_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Capacity:=Integer'Value(Ocarina.Aadl_Values.Image(Value(
                  Number_Value(Upper_Bound(A_Property_Value)))));
      END Read_Compute_Execution_Time_Property;

      PROCEDURE Read_Period_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Period:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(
                     A_Property_Value))));
      END Read_Period_Property;

      PROCEDURE Read_Deadline_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Deadline:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(
                     A_Property_Value))));
      END Read_Deadline_Property;

      PROCEDURE Read_Data_Concurrency_State_Property (
            A_Property_Value : Node_Id) IS

      BEGIN
         IF Sign(A_Property_Value)
               THEN
            State:=-1;
         ELSE
            State:=1;
         END IF;
         State:=State*Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Data_Concurrency_State_Property;

      PROCEDURE Read_Source_Code_Size_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Text_Memory_Size:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Source_Code_Size_Property;

      PROCEDURE Read_Source_Data_Size_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Data_Memory_Size:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Source_Data_Size_Property;

      PROCEDURE Read_Source_Heap_Size_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Heap_Memory_Size:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Source_Heap_Size_Property;

      PROCEDURE Read_Source_Stack_Size_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Stack_Memory_Size:=Integer'Value(Ocarina.Aadl_Values.Image (
               Value(
                  Number_Value(A_Property_Value))));
      END Read_Source_Stack_Size_Property;

      PROCEDURE Read_Criticality_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Criticality:=Integer'Value(Ocarina.Aadl_Values.Image (
               Value(
                  Number_Value(A_Property_Value))));
      END Read_Criticality_Property;

      PROCEDURE Read_Scheduler_Quantum_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Quantum:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Scheduler_Quantum_Property;

      PROCEDURE Read_Fixed_Priority_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Priority:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Fixed_Priority_Property;

      PROCEDURE Read_Dispatch_Jitter_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Jitter:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(
                     A_Property_Value))));
      END Read_Dispatch_Jitter_Property;

      PROCEDURE Read_Bound_On_Data_Blocking_Time_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Blocking_Time:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Bound_On_Data_Blocking_Time_Property;

      PROCEDURE Read_Dispatch_Absolute_Time_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Start_Time:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(A_Property_Value))));
      END Read_Dispatch_Absolute_Time_Property;

      PROCEDURE Read_Dispatch_Seed_Value_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Seed:=Integer'Value(Ocarina.Aadl_Values.Image (Value(
                  Number_Value(
                     A_Property_Value))));
      END Read_Dispatch_Seed_Value_Property;


      -- aadl list properties
      --

      PROCEDURE Read_Task_Dependencies_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         NULL;
      END Read_Task_Dependencies_Property;


      PROCEDURE Read_Critical_Section_Property (
            A_Property_Value : IN     Node_Id) IS


         A_Resource : Resource_Ptr;

         List_Node        : Node_Id;
         Cs_Start,
         Cs_End,
         Resource_Name,
         Thread_Name      : Unbounded_String;
         Integer_Ok       : Boolean;
         Integer_Cs_Start,
         Integer_Cs_End   : Integer;

         Resource_Str_Ptr : Unbounded_String_Ptr;

      BEGIN

         List_Node := First_Node (List_Id (A_Property_Value));

         -- Parse critical sections now
         --

         WHILE Present (List_Node) LOOP
            -- Resource name
            --
            IF Present (List_Node)
                  THEN
               Resource_Name:=To_Lower(To_Unbounded_String(
                     Ocarina.Aadl_Values.Image (
                        Value (List_Node))));
               Resource_Name:=To_Unbounded_String(Slice(
                     Resource_Name,2,Length(
                        Resource_Name)-1) );
            ELSE
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Cheddar_Properties::Critical_Section value list should start with a resource name");
            END IF;
            List_Node := Next_Node (List_Node);


            -- Thread name
            --
            IF Present (List_Node)
                  THEN
               Thread_Name:=To_Lower(To_Unbounded_String(
                     Ocarina.Aadl_Values.Image (
                        Value (List_Node))));
               Thread_Name:=To_Unbounded_String(Slice(
                     Thread_Name,2,Length(
                        Thread_Name)-1) );
            ELSE
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Cheddar_Properties::Critical_Section, Thread name of critical section is missing");
            END IF;
            List_Node := Next_Node (List_Node);


            -- Start of the critical section
            --
            IF Present (List_Node)
                  THEN
               Cs_Start:=To_Unbounded_String(Ocarina.Aadl_Values.Image (
                     Value (List_Node)));
               Cs_Start:=To_Unbounded_String(Slice(
                     Cs_Start,2,Length(
                        Cs_Start )-1) );
            ELSE
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Cheddar_Properties::Critical_Section, Start of critical section is missing");
            END IF;
            List_Node := Next_Node (List_Node);

            -- End of the critical section
            --
            IF Present (List_Node)
                  THEN
               Cs_End:=To_Unbounded_String(Ocarina.Aadl_Values.Image (
                     Value (List_Node)));
               Cs_End:=To_Unbounded_String(Slice(
                     Cs_End,2,Length(
                        Cs_End)-1) );
            ELSE
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Cheddar_Properties::Critical_Section, End of critical section is missing");
            END IF;
            List_Node := Next_Node (List_Node);


            -- Check the critical section
            --

            -- Full path resource name
            --
            Resource_Name:= Path &
               Component_Instance_Name & "." &
               Resource_Name;

            A_Resource:=Search_Resource(Handler.Parsed_System.Resources,
               Resource_Name);


            -- Full path thread name
            --
            Thread_Name:= Path &
               Component_Instance_Name & "." &
               Thread_Name;



            To_Integer(Cs_Start, Integer_Cs_Start, Integer_Ok);
            IF NOT Ok
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  To_String(Lb_Task_Begin(Current_Language) &
                     Lb_Must_Be_Numeric(Current_Language)));
            END IF;



            To_Integer(Cs_End, Integer_Cs_End, Integer_Ok);
            IF NOT Integer_Ok
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  To_String(Lb_Task_End(Current_Language) &
                     Lb_Must_Be_Numeric(Current_Language)));
            END IF;



            Check_Task_Critical_Section (
               Handler.Parsed_System.Tasks,
               Resource_Name,
               A_Resource.Cpu_Name,
               Thread_Name,
               Integer_Cs_Start,
               Integer_Cs_End);


            -- Add the new critical section
            --
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Item:=Thread_Name;
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Data.Task_Begin:=Integer_Cs_Start;
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Data.Task_End:=Integer_Cs_End;
            A_Resource.Task_Tab.Nb_Entries:=
               A_Resource.Task_Tab.Nb_Entries+1;


            Resource_Str_Ptr := NEW Unbounded_String;
            Resource_Str_Ptr.All:=A_Resource.Name;
            Add(Critical_Section_List, Resource_Str_Ptr);


         END LOOP;

      END Read_Critical_Section_Property;



      -- aadlstring properties
      --

      PROCEDURE Read_Source_Text_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         Parametric_File_Name:=To_Unbounded_String(
            Ocarina.Aadl_Values.Image (
               Value (A_Property_Value)));
         Parametric_File_Name:=To_Unbounded_String(Slice(
               Parametric_File_Name,2,Length(
                  Parametric_File_Name)-1) );
      END Read_Source_Text_Property;


      PROCEDURE Read_Actual_Processor_Binding_Property (
            A_Property_Node : IN     Node_Id) IS

         Val        : CONSTANT Node_Id             := Property_Value (A_Property_Node);
         List_Node  :          Node_Id;
         L          :          List_Id;
         Applies_To : CONSTANT List_Id             := Applies_To_Prop (A_Property_Node);
         A_Binding  :          Binding_Record_Type;



      BEGIN
         IF Kind (Val) =   K_Reference_Term


               -- This processor binding is a reference processor binding type
               --
               THEN

            -- Process name
            --
            List_Node := First_Node (Applies_To);

            WHILE Present (List_Node) LOOP

               Address_Space_Name:=To_Lower(To_Unbounded_String(Image(
                        Display_Name(
                           List_Node))));
               List_Node := Next_Node (List_Node);
            END LOOP;



            -- Processor name
            --
            L:=List_Id(Val);

            IF NOT Is_Empty (L) THEN
               List_Node := First_Node (L);
               WHILE Present (List_Node) LOOP

                  CASE Kind (List_Node) IS
                     WHEN K_Identifier =>
                        Cpu_Name:=To_Lower(To_Unbounded_String(Image(
                                 Display_Name(
                                    List_Node))));
                     WHEN OTHERS =>
                        NULL;
                  END CASE;

                  List_Node := Next_Node (List_Node);
               END LOOP;
            END IF;


            -- This processor binding is not a reference processor binding type
            --
         ELSE
            Cpu_Name:=To_Lower(To_Unbounded_String(
                  Ocarina.Aadl_Values.Image (
                     Value (Val))));
            Cpu_Name:=To_Unbounded_String(Slice(
                  Cpu_Name,2,Length(
                     Cpu_Name)-1) );
         END IF;


         A_Binding.Cpu_Name:=Path & Cpu_Name;
         A_Binding.Address_Space_Name:=Path & Address_Space_Name;
         Add(Cpu_Bindings, A_Binding);


         IF Aadl_Debug
               THEN
            Put_Line("First pass ; Process/processor binding =  "
               & To_String(A_Binding.Address_Space_Name) & " " & To_String(
                  A_Binding.Cpu_Name));
         END IF;



      END Read_Actual_Processor_Binding_Property;




      -- aadlboolean properties
      --

      PROCEDURE Read_Dispatch_Seed_Is_Predictable_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         CASE Kind (A_Property_Value) IS
            WHEN K_True_Boolean_Term =>
               Predictable:=True;
            WHEN K_False_Boolean_Term =>
               Predictable:=False;
            WHEN OTHERS =>
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Property Dispatch_Seed_is_Predictable should stores a boolean value");
         END CASE;
      END Read_Dispatch_Seed_Is_Predictable_Property;

      PROCEDURE Read_Preemptive_Scheduler_Property (
            A_Property_Value : Node_Id) IS
      BEGIN
         CASE Kind (A_Property_Value) IS
            WHEN K_True_Boolean_Term =>
               Is_Preemptive:=Preemptive;
            WHEN K_False_Boolean_Term =>
               Is_Preemptive:=Not_Preemptive;
            WHEN OTHERS =>
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Property Preemptive_Scheduler should stores a boolean value");
         END CASE;
      END Read_Preemptive_Scheduler_Property;



      -- Main Read property subprogram
      --
      List_Node               : Node_Id;
      Property_Name           : Unbounded_String;
      User_Defined_Properties : Unbounded_String;
      Val                     : Node_Id;


   BEGIN

      IF NOT Is_Empty (Ocarina.Nodes.Properties (Component_Type)) THEN
         List_Node := First_Node (Ocarina.Nodes.Properties (
               Component_Type));


         WHILE Present (List_Node) LOOP

            -- The property name
            --
            Property_Name:=To_Lower(To_Unbounded_String(Image(
                     Display_Name(
                        Identifier(List_Node)))));


            IF Pass=1 THEN

               IF Property_Name=To_Unbounded_String(
                     "compute_execution_time")
                     THEN
                  Display_Property(List_Node);
                  Read_Compute_Execution_Time_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "concurrency_control_protocol")
                     THEN
                  Display_Property(List_Node);
                  Read_Concurrency_Control_Protocol_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("deadline")
                     THEN
                  Display_Property(List_Node);
                  Read_Deadline_Property(Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("period")
                     THEN
                  Display_Property(List_Node);
                  Read_Period_Property(Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("scheduling_protocol")
                     THEN
                  Display_Property(List_Node);
                  Read_Scheduling_Protocol_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::source_text")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Text_Property(Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("dispatch_protocol")
                     THEN
                  Display_Property(List_Node);
                  Read_Dispatch_Protocol_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("source_code_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Code_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("source_data_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Data_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("source_heap_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Heap_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::criticality")
                     THEN
                  Display_Property(List_Node);
                  Read_Criticality_Property(Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String("source_stack_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Stack_Size_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::source_global_text_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Code_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::source_global_data_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Data_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::source_global_heap_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Heap_Size_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::source_global_stack_size")
                     THEN
                  Display_Property(List_Node);
                  Read_Source_Stack_Size_Property(
                     Property_Value(
                        List_Node));
               END IF;

               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::scheduler_quantum")
                     THEN
                  Display_Property(List_Node);
                  Read_Scheduler_Quantum_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::posix_scheduling_policy")
                     THEN
                  Display_Property(List_Node);
                  Read_Posix_Scheduling_Policy_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::fixed_priority")
                     THEN
                  Display_Property(List_Node);
                  Read_Fixed_Priority_Property(Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::dispatch_jitter")
                     THEN
                  Display_Property(List_Node);
                  Read_Dispatch_Jitter_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::bound_on_data_blocking_time")
                     THEN
                  Display_Property(List_Node);
                  Read_Bound_On_Data_Blocking_Time_Property(
                     Property_Value(List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::dispatch_absolute_time")
                     THEN
                  Display_Property(List_Node);
                  Read_Dispatch_Absolute_Time_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::dispatch_seed_value")
                     THEN
                  Display_Property(List_Node);
                  Read_Dispatch_Seed_Value_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::dispatch_seed_is_predictable")
                     THEN
                  Display_Property(List_Node);
                  Read_Dispatch_Seed_Is_Predictable_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::preemptive_scheduler")
                     THEN
                  Display_Property(List_Node);
                  Read_Preemptive_Scheduler_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::data_concurrency_state")
                     THEN
                  Display_Property(List_Node);
                  Read_Data_Concurrency_State_Property(
                     Property_Value(
                        List_Node));
               END IF;

               -- Scan task user defined properties
               --
               IF Length(Property_Name) > 33
                     THEN
                  User_Defined_Properties:=To_Unbounded_String(Slice(
                        Property_Name, 1 , 33));
               ELSE
                  User_Defined_Properties:=Empty_String;
               END IF;
               IF User_Defined_Properties=
                     "user_defined_cheddar_properties::"
                     THEN

                  Val:=Property_Value(List_Node);
                  User_Defined_Properties:=To_Unbounded_String(Slice(
                        Property_Name, 34, Length(Property_Name)));

                  -- Find the property type
                  --
                  --                  case Kind (Val) is
                  --
                  --                     when K_Signed_Aadlreal =>
                  --                        A_Param:= new Parameter(Double_Parameter);
                  --                        A_Param.Name:=User_Defined_Properties;
                  --                        A_Param.Double_Value:=Ada.Numerics.Aux.Double'
                  --                           Value(
                  --                           Ocarina.Aadl_Values.Image (Value(
                  --                                 Number_Value(Val))));
                  --                        Add(Param, A_Param);

                  --                     when K_Signed_Aadlinteger =>
                  --                        A_Param:= new Parameter(Integer_Parameter);
                  --                        A_Param.Name:=User_Defined_Properties;
                  --                        A_Param.Integer_Value:=Integer'Value(
                  --                           Ocarina.Aadl_Values.Image (Value(
                  --                                 Number_Value(Val))));
                  --                        Add(Param, A_Param);

                  --                     when K_Signed_Aadlnumber =>
                  --                        A_Param:= new Parameter(Integer_Parameter);
                  --                        A_Param.Name:=User_Defined_Properties;
                  --                        A_Param.Integer_Value:=Integer'Value(
                  --                           Ocarina.Aadl_Values.Image (Value(
                  --                                 Number_Value(Val))));
                  --                        Add(Param, A_Param);

                  --                     when k_True =>
                  --                        A_Param:= new Parameter(Boolean_Parameter);
                  --                        A_Param.Name:=User_Defined_Properties;
                  --                        A_Param.Boolean_Value:=True;
                  --                        Add(Param, A_Param);

                  --                     when T_false_Boolean_Term =>
                  --                        A_Param:= new Parameter(Boolean_Parameter);
                  --                        A_Param.Name:=User_Defined_Properties;
                  --                        A_Param.Boolean_Value:=True;
                  --                        Add(Param, A_Param);

                  --                     when others =>
                  --                        Raise_Exception(Aadl_Read_Error'Identity,
                  --                           "Can not find task user defined parameter type");
                  --                  end case;
               END IF;



            END IF;


            IF Pass = 2 THEN
               IF Property_Name=To_Unbounded_String(
                     "actual_processor_binding")
                     THEN
                  Display_Property(List_Node);
                  Read_Actual_Processor_Binding_Property(List_Node);
               END IF;
            END IF;

            IF Pass = 3 THEN
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::task_dependencies")
                     THEN
                  Display_Property(List_Node);
                  Read_Task_Dependencies_Property(
                     Property_Value(
                        List_Node));
               END IF;
               IF Property_Name=To_Unbounded_String(
                     "cheddar_properties::critical_section")
                     THEN
                  Display_Property(List_Node);
                  Read_Critical_Section_Property(
                     Property_Value(
                        List_Node));
               END IF;
            END IF;

            List_Node := Next_Node (List_Node);
         END LOOP;
      END IF;

   END Read_Property_Associations;


   -------------------------------------------
   --   AADL syntax tree reading sub-programs
   -------------------------------------------



   PROCEDURE First_Pass (
         Handler : IN OUT Aadl_Project_Parser;
         Root    : IN     Node_Id) IS



      PROCEDURE Add_Data_Component (
            List_Node             : IN     Node_Id;
            Identifier_Path       : IN     Unbounded_String;
            Parent_Component_Name : IN     Unbounded_String) IS
      BEGIN
         Name:=To_Lower(To_Unbounded_String( Get_Name_Of_Entity (
                  List_Node)));
         Add_Resource(Handler.Parsed_System.Resources, Identifier_Path &
            Parent_Component_Name & "." & Name, State,
            Dummy_Identifier, Identifier_Path & Parent_Component_Name,
            Protocol, A_Task_List);
      END Add_Data_Component;

      PROCEDURE Add_Thread_Component (
            List_Node             : IN     Node_Id;
            Identifier_Path       : IN     Unbounded_String;
            Parent_Component_Name : IN     Unbounded_String) IS
      BEGIN
         Name:=To_Lower(To_Unbounded_String(Get_Name_Of_Entity (List_Node)));
         Add_Task(Handler.Parsed_System.Tasks, Task1, Identifier_Path &
            Parent_Component_Name & "." & Name,
            Dummy_Identifier,
            Identifier_Path & Parent_Component_Name,
            Task_Type, Start_Time, Capacity, Period, Deadline, Jitter,
            Blocking_Time, Priority, Criticality, Policy,
            Offset, Stack_Memory_Size, Text_Memory_Size, Param,
            Parametric_File_Name, Seed, Predictable);
      END Add_Thread_Component;

      PROCEDURE Add_Process_Component (
            List_Node       : IN     Node_Id;
            Identifier_Path : IN     Unbounded_String) IS
      BEGIN
         Name:=To_Lower(To_Unbounded_String(Get_Name_Of_Entity (List_Node)));
         Add_Address_Space(Handler.Parsed_System.Address_Spaces,
            Identifier_Path & Name,
            Dummy_Identifier,
            Text_Memory_Size, Stack_Memory_Size, Data_Memory_Size,
            Heap_Memory_Size);
      END Add_Process_Component;

      PROCEDURE Add_Processor_Component (
            List_Node       : IN     Node_Id;
            Identifier_Path : IN     Unbounded_String) IS
      BEGIN
         Name:=To_Lower(To_Unbounded_String(Get_Name_Of_Entity (List_Node)));
         Add_Processor(Handler.Parsed_System.Processors,
            Identifier_Path &
            Name, Is_Preemptive, Quantum, Parametric_File_Name,
            A_Scheduler, A_Network);
      END Add_Processor_Component;

      PROCEDURE Add_Event_Analyzers (
            Name : IN     Unbounded_String) IS

         -- A pair of index objects to keep track of the beginning and
         -- ending of the tokens isolated from the string:
         --
         First : Natural;
         Last  : Positive;

         -- A character set consisting of the "whitespace" characters
         -- that separate the tokens:
         --
         Comma   : CONSTANT Ada.Strings.Maps.Character_Set := Ada.Strings.Maps.To_Set
           (",");
         Subname,
         Token   :          Unbounded_String;
      BEGIN
         Subname:=Name;
         LOOP
            Find_Token (Subname,
               Set => Comma,
               Test => Ada.Strings.Outside,
               First => First,
               Last => Last);

            Token:=To_Unbounded_String(Slice(Subname, First, Last));
            Subname:=To_Unbounded_String( Slice(Subname,Last+1,Length(
                     Subname)) );

            Check_Event_Analyzer (
               Handler.Parsed_System.Event_Analyzers,
               Token);
            Add_Event_Analyzer (Handler.Parsed_System.Event_Analyzers,
               Token);
            EXIT WHEN Length(Subname)=0;
         END LOOP;

      EXCEPTION
         WHEN OTHERS =>
            Raise_Exception(Aadl_Read_Error'Identity, Exception_Message);
      END Add_Event_Analyzers;



      FUNCTION Find_Subcomponents (
            Parent_Component     : IN     Node_Id;
            Current_Process_Name :        Unbounded_String;
            Identifier_Path      : IN     Unbounded_String;
            Level                : IN     Integer          := 0)
        RETURN Integer IS




         List_Node        : Node_Id;
         Component_Number : Integer          := 0;
         Cat              : Byte;
         Cat2             : Byte;
         Path             : Unbounded_String := Identifier_Path;
         A_Process_Name   : Unbounded_String := Empty_String;

      BEGIN
         IF  (Parent_Component /= No_Node)
               AND  (Kind (Parent_Component) =
               K_Component_Implementation)
               THEN

            IF Subcomponents (Parent_Component) /= No_List THEN

               List_Node := First_Node (Subcomponents (Parent_Component));

               WHILE List_Node /= No_Node LOOP
                  Component_Number := Component_Number + 1;


                  -- We look for the component type
                  --
                  IF (Entity_Ref (List_Node) /= No_Node)
                        THEN

                     -- Get the type of the component
                     --
                     Cat:=Category (List_Node);

                     IF Aadl_Debug
                           THEN
                        Put_Line("First pass ; Component name =  " &
                           Get_Name_Of_Entity (
                              List_Node));
                        Put_Line("First pass ; Component type = " &
                           Get_Name_Of_Entity_Reference
                           (Entity_Ref(List_Node)));
                     END IF;



                     IF  Component_Category'Val (Cat) = Cc_Data
                           THEN

                        -- We do not take into account data declared into a other data
                        -- (nested data)
                        --
                        -- Get the type of the parent component
                        --
                        Cat2:=Category (Parent_Component);

                        IF  Component_Category'Val (Cat2) = Cc_Process
                              THEN

                           Put_Debug("Parsing an AADL data : " &
                              Get_Name_Of_Entity (List_Node));
                           Initialize_Parsed_Data;
                           Read_Property_Associations(Handler,
                              Get_Referenced_Entity(Entity_Ref (List_Node)),
                              To_Lower(To_Unbounded_String(
                                    Get_Name_Of_Entity (
                                       List_Node))),
                              Identifier_Path, 1);
                           Add_Data_Component(List_Node, Identifier_Path,
                              Current_Process_Name);
                        END IF;
                     END IF;


                     IF  Component_Category'Val (Cat) = Cc_Thread
                           THEN
                        Put_Debug("Parsing an AADL thread : " &
                           Get_Name_Of_Entity (List_Node));
                        Initialize_Parsed_Data;
                        Read_Property_Associations(Handler,
                           Get_Referenced_Entity(Entity_Ref (List_Node)),
                           To_Lower(To_Unbounded_String(
                                 Get_Name_Of_Entity(
                                    List_Node))),
                           Identifier_Path, 1);
                        Add_Thread_Component(List_Node, Identifier_Path,
                           Current_Process_Name);
                     END IF;


                     IF  Component_Category'Val (Cat) = Cc_Process
                           THEN
                        Put_Debug("Parsing an AADL process : " &
                           Get_Name_Of_Entity (List_Node));
                        Initialize_Parsed_Data;
                        A_Process_Name:=To_Lower(To_Unbounded_String(
                              Get_Name_Of_Entity (List_Node)));
                        Read_Property_Associations(Handler,
                           Get_Referenced_Entity(Entity_Ref (List_Node)),
                           To_Lower(To_Unbounded_String(
                                 Get_Name_Of_Entity(
                                    List_Node))),
                           Identifier_Path, 1);
                        Add_Process_Component(List_Node, Identifier_Path);
                     END IF;

                     IF  Component_Category'Val (Cat) = Cc_Processor
                           THEN
                        Put_Debug("Parsing an AADL processor : " &
                           Get_Name_Of_Entity (List_Node));
                        Initialize_Parsed_Data;
                        Read_Property_Associations(Handler,
                           Get_Referenced_Entity(Entity_Ref (List_Node)),
                           To_Lower(To_Unbounded_String(
                                 Get_Name_Of_Entity(
                                    List_Node))),
                           Identifier_Path, 1);
                        Add_Processor_Component(List_Node, Identifier_Path);
                     END IF;

                     IF  Component_Category'Val (Cat) = Cc_System
                           THEN
                        Put_Debug("Parsing an AADL system : " &
                           Get_Name_Of_Entity (List_Node));
                        Path := To_Lower(Identifier_Path  &
                           Get_Name_Of_Entity (
                              List_Node) & "");
                        Initialize_Parsed_Data;
                        Read_Property_Associations(Handler,
                           Get_Referenced_Entity(Entity_Ref (List_Node)),
                           To_Lower(To_Unbounded_String(
                                 Get_Name_Of_Entity(
                                    List_Node))),
                           Path, 1);

                        IF Parametric_File_Name/=Empty_String
                              THEN
                           Add_Event_Analyzers (Parametric_File_Name);
                        END IF;
                     END IF;

                     Component_Number := Component_Number +
                        Find_Subcomponents(Get_Referenced_Entity
                        (Entity_Ref (List_Node)), A_Process_Name, Path,
                        Level +
                        1);

                  ELSE

                     -- The component has no type ...
                     -- Currently, this is not supported
                     --
                     Raise_Exception(Aadl_Read_Error'Identity,
                        "Find_Processor_Component, AADL component must have a type");
                  END IF;
                  List_Node := Next_Node (List_Node);
               END LOOP;
            END IF;
         END IF;

         RETURN Component_Number;
      END Find_Subcomponents;



      List_Node               : Node_Id := No_Node;
      Global_Component_Number : Integer := 0;

   BEGIN

      -- Find and create all Cheddar's object sets
      --
      IF Ocarina.Nodes.Declarations (Root) /= No_List THEN

         List_Node := First_Node (Ocarina.Nodes.Declarations (Root));
         WHILE List_Node /= No_Node LOOP

            IF Kind (List_Node) = K_Component_Implementation
                  AND THEN Component_Category'Val (Category (List_Node)) =
                  Cc_System

                  THEN

               Put_Debug("Parsing a root AADL system component : " &
                  Get_Name_Of_Entity(List_Node));

               -- Read main system properties
               --
               Read_Property_Associations(Handler,
                  List_Node,
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                           List_Node))),
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity (
                           List_Node)) & "."),  1);

               IF Parametric_File_Name/=Empty_String
                     THEN
                  Add_Event_Analyzers (Parametric_File_Name);
               END IF;



               -- Find aadl sub components
               --
               Global_Component_Number := Find_Subcomponents(
                  List_Node, Empty_String, To_Lower(To_Unbounded_String(
                        Get_Name_Of_Entity (
                           List_Node)) & "."));
            END IF;

            List_Node := Next_Node (List_Node);
         END LOOP;
      END IF;

   END First_Pass;





   PROCEDURE Second_Pass (
         Handler : IN OUT Aadl_Project_Parser;
         Root    : IN     Node_Id) IS



      FUNCTION Find_Subcomponents (
            Parent_Component : IN     Node_Id;
            Identifier_Path  : IN     Unbounded_String;
            Level            : IN     Integer          := 0)
        RETURN Integer IS


         List_Node        : Node_Id;
         Component_Number : Integer          := 0;
         Cat              : Byte;
         Path             : Unbounded_String := Identifier_Path;

      BEGIN
         IF Subcomponents (Parent_Component) /= No_List THEN
            List_Node := First_Node (Subcomponents (Parent_Component));

            WHILE List_Node /= No_Node LOOP
               Component_Number := Component_Number + 1;


               -- We look for the component type
               --
               IF (Entity_Ref (List_Node) /= No_Node)
                     THEN

                  -- Get the type of the component
                  --
                  Cat:=Category (List_Node);

                  IF Aadl_Debug
                        THEN
                     Put_Line("Second pass ; Component name =  " &
                        Get_Name_Of_Entity (
                           List_Node));
                     Put_Line("Second pass ; Component type = " &
                        Get_Name_Of_Entity_Reference
                        (Entity_Ref(List_Node)) );
                  END IF;

                  IF  Component_Category'Val (Cat) = Cc_Process
                        THEN
                     Read_Property_Associations(Handler,
                        Get_Referenced_Entity(Entity_Ref(List_Node)),
                        To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                                 List_Node))),
                        Identifier_Path, 2);
                  END IF;

                  IF  Component_Category'Val (Cat) = Cc_System
                        THEN
                     Path := To_Lower(Identifier_Path  &
                        Get_Name_Of_Entity (
                           List_Node) & "");
                     Read_Property_Associations(Handler,
                        Get_Referenced_Entity(Entity(List_Node)),
                        To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                                 List_Node))),
                        Identifier_Path, 2);
                  END IF;


                  IF  (Component_Category'Val (Cat) = Cc_System)
                        OR  (Component_Category'Val (Cat) = Cc_Process)
                        THEN

                     Component_Number := Component_Number +
                        Find_Subcomponents(Get_Referenced_Entity
                        (Entity_Ref (List_Node)), Path, Level + 1);
                  END IF;

               ELSE

                  -- The component has no type ...
                  -- Currently, this is not supported
                  --
                  Raise_Exception(Aadl_Read_Error'Identity,
                     "Find_Processor_Component, AADL component must have a type");
               END IF;
               List_Node := Next_Node (List_Node);
            END LOOP;
         END IF;

         RETURN Component_Number;
      END Find_Subcomponents;


      A_Buffer             : Buffer_Ptr;
      My_Buffer_Iterator   : Buffers_Iterator;
      A_Resource           : Resource_Ptr;
      My_Resource_Iterator : Resources_Iterator;
      A_Task               : Task_Ptr;
      My_Task_Iterator     : Tasks_Iterator;

      An_Address_Space : Address_Space_Ptr;
      My_Iterator      : Address_Spaces_Iterator;

      List_Node               : Node_Id := No_Node;
      Global_Component_Number : Integer := 0;

   BEGIN

      Initialize_Parsed_Data;


      -- Find and perform analysis on all global systems
      --
      IF Ocarina.Nodes.Declarations (Root) /= No_List THEN
         List_Node := First_Node (Ocarina.Nodes.Declarations (Root));

         WHILE List_Node /= No_Node LOOP
            IF Kind (List_Node) = K_Component_Implementation
                  AND THEN Component_Category'Val (Category (List_Node)) =
                  Cc_System

                  THEN

               -- Read main system properties
               --
               Read_Property_Associations(Handler,
                  List_Node,
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                           List_Node))),
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity (
                           List_Node)) & "."),  2);

               -- Find aadl sub components
               --
               Global_Component_Number := Find_Subcomponents(
                  List_Node, To_Lower(To_Unbounded_String(
                        Get_Name_Of_Entity (
                           List_Node)) & "."));
            END IF;

            List_Node := Next_Node (List_Node);
         END LOOP;
      END IF;


      -- Now, update processor bindings of address spaces
      --



      IF NOT Is_Empty(Handler.Parsed_System.Address_Spaces) THEN
         Reset_Iterator(Handler.Parsed_System.Address_Spaces,
            My_Iterator);
         LOOP

            Current_Element(Handler.Parsed_System.Address_Spaces,
               An_Address_Space,
               My_Iterator);


            -- Find the process name in the table and assign
            -- processor
            --
            FOR  I IN 0..Cpu_Bindings.Nb_Entries-1 LOOP
               IF Cpu_Bindings.Entries(I).Address_Space_Name=
                     An_Address_Space.Name
                     THEN
                  An_Address_Space.Cpu_Name:=Cpu_Bindings.Entries(I).
                     Cpu_Name;
               END IF;
            END LOOP;

            EXIT WHEN Is_Last_Element(
               Handler.Parsed_System.Address_Spaces, My_Iterator);

            Next_Element(Handler.Parsed_System.Address_Spaces,
               My_Iterator);

         END LOOP;
      END IF;


      -- Check that no address space stays unbinded
      -- Bind threads, buffers and shared_resources
      --
      IF NOT Is_Empty(Handler.Parsed_System.Address_Spaces) THEN
         Reset_Iterator(Handler.Parsed_System.Address_Spaces,
            My_Iterator);
         LOOP

            Current_Element(Handler.Parsed_System.Address_Spaces,
               An_Address_Space,
               My_Iterator);


            -- Check and bind
            --
            IF (An_Address_Space.Cpu_Name = Dummy_Identifier)
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Process " & To_String(An_Address_Space.Name) &
                  " is not binded to a processor");
            END IF;



            IF NOT Is_Empty(Handler.Parsed_System.Tasks) THEN
               Reset_Iterator(Handler.Parsed_System.Tasks,
                  My_Task_Iterator);
               LOOP

                  Current_Element(Handler.Parsed_System.Tasks, A_Task,
                     My_Task_Iterator);

                  IF A_Task.Address_Space_Name = An_Address_Space.Name
                        THEN
                     A_Task.Cpu_Name:=An_Address_Space.Cpu_Name;
                  END IF;

                  EXIT WHEN Is_Last_Element(
                     Handler.Parsed_System.Tasks,
                     My_Task_Iterator);

                  Next_Element(Handler.Parsed_System.Tasks,
                     My_Task_Iterator);

               END LOOP;
            END IF;




            IF NOT Is_Empty(Handler.Parsed_System.Resources) THEN
               Reset_Iterator(Handler.Parsed_System.Resources,
                  My_Resource_Iterator);
               LOOP

                  Current_Element(Handler.Parsed_System.Resources,
                     A_Resource,
                     My_Resource_Iterator);

                  IF A_Resource.Address_Space_Name = An_Address_Space.Name
                        THEN
                     A_Resource.Cpu_Name:=An_Address_Space.Cpu_Name;
                  END IF;

                  EXIT WHEN Is_Last_Element(
                     Handler.Parsed_System.Resources,
                     My_Resource_Iterator);

                  Next_Element(Handler.Parsed_System.Resources,
                     My_Resource_Iterator);

               END LOOP;
            END IF;




            IF NOT Is_Empty(Handler.Parsed_System.Buffers) THEN
               Reset_Iterator(Handler.Parsed_System.Buffers,
                  My_Buffer_Iterator);
               LOOP

                  Current_Element(Handler.Parsed_System.Buffers,
                     A_Buffer,
                     My_Buffer_Iterator);

                  IF A_Buffer.Address_Space_Name = An_Address_Space.Name
                        THEN
                     A_Buffer.Cpu_Name:=An_Address_Space.Cpu_Name;
                  END IF;

                  EXIT WHEN Is_Last_Element(
                     Handler.Parsed_System.Buffers,
                     My_Buffer_Iterator);

                  Next_Element(Handler.Parsed_System.Buffers,
                     My_Buffer_Iterator);

               END LOOP;
            END IF;


            EXIT WHEN Is_Last_Element(
               Handler.Parsed_System.Address_Spaces, My_Iterator);

            Next_Element(Handler.Parsed_System.Address_Spaces,
               My_Iterator);

         END LOOP;
      END IF;



      -- Now, check that no task/resource/buffer stays unbinded
      --
      IF NOT Is_Empty(Handler.Parsed_System.Tasks) THEN
         Reset_Iterator(Handler.Parsed_System.Tasks,
            My_Task_Iterator);
         LOOP

            Current_Element(Handler.Parsed_System.Tasks, A_Task,
               My_Task_Iterator);

            IF (A_Task.Cpu_Name = Dummy_Identifier)
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Task " & To_String(A_Task.Name) &
                  " is not binded to a processor");
            END IF;

            EXIT WHEN Is_Last_Element(
               Handler.Parsed_System.Tasks,
               My_Task_Iterator);

            Next_Element(Handler.Parsed_System.Tasks,
               My_Task_Iterator);

         END LOOP;
      END IF;




      IF NOT Is_Empty(Handler.Parsed_System.Resources) THEN
         Reset_Iterator(Handler.Parsed_System.Resources,
            My_Resource_Iterator);
         LOOP

            Current_Element(Handler.Parsed_System.Resources,
               A_Resource,
               My_Resource_Iterator);

            IF (A_Resource.Cpu_Name = Dummy_Identifier)
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Data " & To_String(A_Resource.Name) &
                  " is not binded to a processor");
            END IF;

            EXIT WHEN Is_Last_Element(
               Handler.Parsed_System.Resources,
               My_Resource_Iterator);

            Next_Element(Handler.Parsed_System.Resources,
               My_Resource_Iterator);

         END LOOP;
      END IF;




      IF NOT Is_Empty(Handler.Parsed_System.Buffers) THEN
         Reset_Iterator(Handler.Parsed_System.Buffers,
            My_Buffer_Iterator);
         LOOP

            Current_Element(Handler.Parsed_System.Buffers,
               A_Buffer,
               My_Buffer_Iterator);

            IF (A_Buffer.Cpu_Name = Dummy_Identifier)
                  THEN
               Raise_Exception(Aadl_Read_Error'Identity,
                  "Buffer " & To_String(A_Buffer.Name) &
                  " is not binded to a processor");
            END IF;

            EXIT WHEN Is_Last_Element(
               Handler.Parsed_System.Buffers,
               My_Buffer_Iterator);

            Next_Element(Handler.Parsed_System.Buffers,
               My_Buffer_Iterator);

         END LOOP;
      END IF;


   END Second_Pass;





   -------------------------------------------
   ---   Read AADL connections
   -------------------------------------------


   PROCEDURE Read_Connections (
         Handler                  : IN OUT Aadl_Project_Parser;
         Component_Implementation : IN     Node_Id;
         Component_Instance_Name  : IN     Unbounded_String;
         Path                     : IN     Unbounded_String) IS


      PROCEDURE Buffer_Connections (
            A_Connection : Node_Id) IS

         Buffer_Name          : Unbounded_String;
         Consumer_Thread_Name : Unbounded_String;
         Producer_Thread_Name : Unbounded_String;

         A_Buffer          : Buffer_Ptr;
         A_Consumer_Thread : Task_Ptr;

         Already_Exist : Boolean;
         A_Role        : Buffer_Role;
         Roles         : Buffer_Roles_Table;

         -- A pair of index objects to keep track of the beginning and
         -- ending of the tokens isolated from the string:
         --
         First : Natural;
         Last  : Positive;

         -- A character set consisting of the "whitespace" characters
         -- that separate the tokens:
         --
         Dot     : CONSTANT Ada.Strings.Maps.Character_Set := Ada.Strings.Maps.To_Set
           (".");
         Subname :          Unbounded_String;

      BEGIN

         -- Get thread names
         --
         Producer_Thread_Name:=To_Lower(To_Unbounded_String(Image(
                  Display_Name(
                     Identifier(Source (A_Connection))))));
         Consumer_Thread_Name:=To_Lower(To_Unbounded_String(Image(
                  Display_Name(
                     Identifier(Destination (A_Connection))))));

         Subname:=Producer_Thread_Name;
         Find_Token (Subname,
            Set => Dot,
            Test => Ada.Strings.Outside,
            First => First,
            Last => Last);

         Producer_Thread_Name:=To_Unbounded_String( Slice(Subname,First,
               Last) );
         Producer_Thread_Name:= Path &
            Component_Instance_Name & "." &
            Producer_Thread_Name;


         Subname:=Consumer_Thread_Name;
         Find_Token (Subname,
            Set => Dot,
            Test => Ada.Strings.Outside,
            First => First,
            Last => Last);

         Consumer_Thread_Name:=To_Unbounded_String( Slice(Subname,First,
               Last) );
         Consumer_Thread_Name:= Path &
            Component_Instance_Name & "." &
            Consumer_Thread_Name;


         A_Consumer_Thread:=Search_Task(Handler.Parsed_System.Tasks,
            Consumer_Thread_Name);


         -- Is the buffer already exists ?
         -- if not, create it. If it already exist, add the new
         -- producer
         --
         Buffer_Name := Consumer_Thread_Name &
            To_Unbounded_String("_buffer");
         BEGIN
            Already_Exist := True;
            A_Buffer:=Search_Buffer(Handler.Parsed_System.Buffers,
               Buffer_Name);
         EXCEPTION
            WHEN OTHERS =>
               Already_Exist:=False;
         END;

         IF NOT Already_Exist
               THEN

            -- Create the buffer
            --
            Initialize(Roles);
            Initialize(A_Role);
            A_Role.The_Role:=Consumer;
            A_Role.Size:=1;
            A_Role.Time:=1;
            Add(Roles, Consumer_Thread_Name, A_Role);

            Add_Buffer (Handler.Parsed_System.Buffers,
               A_Buffer, Buffer_Name, 1, A_Consumer_Thread.Cpu_Name,
               A_Consumer_Thread.Address_Space_Name, Qs_Pp1,
               Roles);
         END IF;

         -- Add the new role (producer)
         --
         Initialize(A_Role);
         A_Role.The_Role:=Producer;
         A_Role.Size:=1;
         A_Role.Time:=1;
         Add(A_Buffer.Roles, Producer_Thread_Name, A_Role);

      END Buffer_Connections;



      PROCEDURE Shared_Resource_Connections (
            A_Connection : Node_Id) IS

         Resource_Name : Unbounded_String;
         Thread_Name   : Unbounded_String;

         Already_Exist    : Boolean;
         Resource_Str_Ptr : Unbounded_String_Ptr;
         List_Ite         : Unbounded_String_Iterator;

         A_Resource : Resource_Ptr;
         A_Thread   : Task_Ptr;

      BEGIN
         -- Check that no critical section are already memorized
         -- If we found no critical section, build a new one from
         -- the data access connection
         --
         Resource_Name:=To_Lower(To_Unbounded_String(Image(Display_Name(
                     Identifier(
                        Source (A_Connection))))));
         Resource_Name:= Path &
            Component_Instance_Name & "." &
            Resource_Name;

         -- Get resource and thread information
         --
         A_Resource:=Search_Resource(Handler.Parsed_System.Resources,
            Resource_Name);

         Thread_Name:=To_Lower(To_Unbounded_String(Image(Display_Name(
                     Identifier(Destination (A_Connection))))));

         DECLARE
            -- A pair of index objects to keep track of the beginning and
            -- ending of the tokens isolated from the string:
            --
            First : Natural;
            Last  : Positive;

            -- A character set consisting of the "whitespace" characters
            -- that separate the tokens:
            --
            Dot     : CONSTANT Ada.Strings.Maps.Character_Set := Ada.Strings.Maps.To_Set
              (".");
            Subname :          Unbounded_String;
         BEGIN
            Subname:=Thread_Name;

            Find_Token (Subname,
               Set => Dot,
               Test => Ada.Strings.Outside,
               First => First,
               Last => Last);

            Thread_Name:=To_Unbounded_String( Slice(Subname,First,
                  Last) );
            Thread_Name:= Path &
               Component_Instance_Name & "." &
               Thread_Name;
         END;


         A_Thread:=Search_Task(Handler.Parsed_System.Tasks,
            Thread_Name);

         -- Check is some critical section exists
         --
         Already_Exist:=False;
         IF NOT Is_Empty(Critical_Section_List) THEN

            Reset_Head_Iterator(Critical_Section_List, List_Ite);
            LOOP
               Current_Element (Critical_Section_List, Resource_Str_Ptr,
                  List_Ite);

               IF Resource_Str_Ptr.All = Resource_Name
                     THEN
                  Already_Exist:=True;
                  EXIT;
               END IF;

               IF  Is_Tail_Element (Critical_Section_List,
                     List_Ite) THEN
                  EXIT;
               END IF;
               Next_Element (Critical_Section_List, List_Ite);
            END LOOP;

         END IF;


         -- Add the critical section is the critical_section_list
         -- does not contain it
         --
         IF NOT Already_Exist
               THEN
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Item:=Thread_Name;
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Data.Task_Begin:=1;
            A_Resource.Task_Tab.Entries(A_Resource.Task_Tab.Nb_Entries).
               Data.Task_End:=A_Thread.Capacity;
            A_Resource.Task_Tab.Nb_Entries:=
               A_Resource.Task_Tab.Nb_Entries+1;
         END IF;




      END Shared_Resource_Connections;




      List_Node : Node_Id;
      Cat       : Connection_Type;

   BEGIN

      IF NOT Is_Empty (Ocarina.Nodes.Connections (
               Component_Implementation)) THEN

         List_Node := First_Node (Ocarina.Nodes.Connections (
               Component_Implementation));

         WHILE Present (List_Node) LOOP
            IF Aadl_Debug
                  THEN
               Print_Connection (List_Node);
            END IF;

            Cat :=Get_Category_Of_Connection (List_Node);

            BEGIN
               CASE Cat IS
                  WHEN Ct_Data | Ct_Data_Delayed =>
                     NULL;
                  WHEN Ct_Event =>
                     Buffer_Connections(List_Node);
                  WHEN Ct_Event_Data =>
                     Buffer_Connections(List_Node);
                  WHEN Ct_Port_Group =>
                     NULL;
                  WHEN Ct_Parameter =>
                     NULL;
                  WHEN Ct_Access_Bus =>
                     NULL;
                  WHEN Ct_Access_Data =>
                     Shared_Resource_Connections(List_Node);
               END CASE;

               -- Sometimes, the parsing of a connection may fail.
               -- Most of the time, it means that the AADL component were not
               -- translated towards Cheddar's component (for several reasons)
               -- Then, we do not stop the parser : the AADL translation
               -- should continue ...
               --
            EXCEPTION
               WHEN OTHERS =>
                  Put_Debug("Warning : fail to parse an AADL connection");
            END;


            List_Node := Next_Node (List_Node);
         END LOOP;

      END IF;

   END Read_Connections;





   PROCEDURE Third_Pass (
         Handler : IN OUT Aadl_Project_Parser;
         Root    : IN     Node_Id) IS



      FUNCTION Find_Subcomponents (
            Parent_Component : IN     Node_Id;
            Identifier_Path  : IN     Unbounded_String;
            Level            : IN     Integer          := 0)
        RETURN Integer IS


         List_Node        : Node_Id;
         Component_Number : Integer          := 0;
         Cat              : Byte;
         Path             : Unbounded_String := Identifier_Path;

      BEGIN
         IF Subcomponents (Parent_Component) /= No_List THEN
            List_Node := First_Node (Subcomponents (
                  Parent_Component));

            WHILE List_Node /= No_Node LOOP
               Component_Number := Component_Number + 1;


               -- We look for the component type
               --
               IF (Entity_Ref (List_Node) /= No_Node)
                     THEN

                  -- Get the type of the component
                  --
                  Cat:=Category (List_Node);

                  IF Aadl_Debug
                        THEN
                     Put_Line("Third pass ; Component name =  " &
                        To_String(
                           Identifier_Path) &
                        Get_Name_Of_Entity (
                           List_Node));
                     Put_Line("Third pass ; Component type = " &
                        Get_Name_Of_Entity_Reference
                        (Entity_Ref(List_Node)));
                  END IF;



                  IF  Component_Category'Val (Cat) = Cc_Process
                        THEN
                     Read_Property_Associations(Handler,
                        Get_Referenced_Entity(Entity_Ref (List_Node)),
                        To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                                 List_Node))),
                        Identifier_Path, 3);
                     Read_Connections(Handler,
                        Get_Referenced_Entity(Entity_Ref (List_Node)),
                        To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                                 List_Node))),
                        Identifier_Path);
                  END IF;


                  IF  Component_Category'Val (Cat) = Cc_System
                        THEN
                     Path := To_Lower(Identifier_Path  &
                        Get_Name_Of_Entity (
                           List_Node) & "");
                     Read_Property_Associations(Handler,
                        List_Node,
                        To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                                 List_Node))),
                        Identifier_Path, 3);
                  END IF;


                  IF  (Component_Category'Val (Cat) = Cc_System)
                        OR  (Component_Category'Val (Cat) = Cc_Process)
                        THEN

                     Component_Number := Component_Number +
                        Find_Subcomponents(Get_Referenced_Entity
                        (Entity_Ref (List_Node)), Path, Level + 1);
                  END IF;

               ELSE

                  -- The component has no type ...
                  -- Currently, this is not supported
                  --
                  Raise_Exception(Aadl_Read_Error'Identity,
                     "Find_Processor_Component, AADL component must have a type");
               END IF;
               List_Node := Next_Node (List_Node);
            END LOOP;
         END IF;

         RETURN Component_Number;
      END Find_Subcomponents;


      List_Node               : Node_Id := No_Node;
      Global_Component_Number : Integer := 0;

   BEGIN

      Initialize_Parsed_Data;


      -- Find and perform analysis on all global systems
      --
      IF Ocarina.Nodes.Declarations (Root) /= No_List THEN
         List_Node := First_Node (Ocarina.Nodes.Declarations (Root));

         WHILE List_Node /= No_Node LOOP
            IF Kind (List_Node) = K_Component_Implementation
                  AND THEN Component_Category'Val (Category (
                     List_Node)) =
                  Cc_System

                  THEN

               -- Read main system properties
               --
               Read_Property_Associations(Handler,
                  List_Node,
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity(
                           List_Node))),
                  To_Lower(To_Unbounded_String(Get_Name_Of_Entity (
                           List_Node)) & "."),  3);

               -- Find aadl sub components
               --
               Global_Component_Number := Find_Subcomponents(
                  List_Node, To_Lower(To_Unbounded_String(
                        Get_Name_Of_Entity (
                           List_Node)) & "."));
            END IF;

            List_Node := Next_Node (List_Node);
         END LOOP;
      END IF;


   END Third_Pass;






   PROCEDURE Parse (
         Handler           : IN OUT Aadl_Project_Parser;
         Dir_List          : IN     Unbounded_String_List;
         Project_File_List : IN     Unbounded_String_List) IS


      Root    : Node_Id := No_Node;
      Success : Boolean := False;

      A_Buff      : Buffer_Ptr;
      My_Iterator : Buffers_Iterator;



   BEGIN

      --  Initialization Ocarina
      --
      IF First_Aadl_Parsing THEN
         Ocarina.Initialize;
         First_Aadl_Parsing:=False;
      END IF;
      Ocarina.Aadl.Initialize;
      Ocarina.Reset;




      -- Parse standard files
      --
      Root:=Load_Aadl_File_From_Directory (Root, "AADL_Properties.aadl",
         Dir_List);
      Root:=Load_Aadl_File_From_Directory (Root, "AADL_Project.aadl",
         Dir_List);
      Root:=Load_Aadl_File_From_Directory (Root,
         "Cheddar_Properties.aadl", Dir_List);
      Root:=Load_Aadl_File_From_Directory (Root,
         "User_Defined_Cheddar_Properties.aadl", Dir_List);




      -- Read and Parse user files from Project_File_List
      --
      DECLARE
         A_File_Name : Unbounded_String_Ptr;
         List_Ite    : Unbounded_String_Iterator;

      BEGIN

         IF NOT Is_Empty(Project_File_List) THEN

            Reset_Head_Iterator(Project_File_List, List_Ite);
            LOOP
               Current_Element (Project_File_List, A_File_Name,
                  List_Ite);

               Root:=Load_Aadl_File_From_Directory (Root, A_File_Name.All,
                  Dir_List);

               IF  Is_Tail_Element (Project_File_List,
                     List_Ite) THEN
                  EXIT;
               END IF;
               Next_Element (Project_File_List, List_Ite);
            END LOOP;
         END IF;
      END;




      -- Run AADL AST analysis
      --
      IF Root /= No_Node THEN
         Success := Ocarina.Analyzer.Check_Tree_Structure (Root, True);

         IF NOT Success THEN
            Root := No_Node;
         END IF;
      END IF;



      --  Check that the AADL specification is ok
      --
      IF Root = No_Node THEN
         Raise_Exception(Aadl_Read_Error'Identity,
            "Syntax error into the AADL specification");
      ELSE

         -- Store subcomponents (systems, processor, thread and data components)
         --
         Initialize(Critical_Section_List);

         First_Pass(Handler, Root);
         IF Aadl_Debug THEN
            Put_Line("End of First pass");
         END IF;

         -- Perform processor and bus bindings
         --
         Second_Pass(Handler, Root);
         IF Aadl_Debug THEN
            Put_Line("End of Second pass");
         END IF;

         -- Set some component properties :  shared resources critical section and  task dependencies
         --
         Third_Pass(Handler, Root);
         IF Aadl_Debug THEN
            Put_Line("End of Third pass");
         END IF;


         -- Build buffer's dependencies
         --
         IF NOT Is_Empty(Handler.Parsed_System.Buffers)
               THEN
            Reset_Iterator(Handler.Parsed_System.Buffers,  My_Iterator);
            LOOP
               Current_Element(Handler.Parsed_System.Buffers, A_Buff,
                  My_Iterator);

               BEGIN
                  Add_All_Task_Dependencies(
                     Handler.Parsed_System.Dependencies,
                     Handler.Parsed_System.Tasks, A_Buff);

                  -- Sometimes, the parsing of a connection may fail.
                  -- Most of the time, it means that the AADL component were not
                  -- translated towards Cheddar's component (for several reasons)
                  -- Then, we do not stop the parser : the AADL translation
                  -- should continue ...
                  --
               EXCEPTION
                  WHEN OTHERS =>
                     Put_Debug(
                        "Warning : fail to translate an AADL connection to a task dependency");
               END;


               EXIT WHEN Is_Last_Element(Handler.Parsed_System.Buffers,
                  My_Iterator);
               Next_Element(Handler.Parsed_System.Buffers, My_Iterator);
            END LOOP;
         END IF;

         -- Free memory
         --
         -- Delete(Critical_Section_List);


      END IF;


   END Parse;



   FUNCTION Load_Aadl_File_From_Directory (
         Root      : IN     Node_Id;
         File_Name : IN     String;
         Dir_List  : IN     Unbounded_String_List)
     RETURN Node_Id IS
   BEGIN
      RETURN Load_Aadl_File_From_Directory(Root, To_Unbounded_String(
            File_Name),
         Dir_List);
   END Load_Aadl_File_From_Directory;



   FUNCTION Load_Aadl_File_From_Directory (
         Root      : IN     Node_Id;
         File_Name : IN     Unbounded_String;
         Dir_List  : IN     Unbounded_String_List)
     RETURN Node_Id IS

      A_Dir    : Unbounded_String_Ptr;
      List_Ite : Unbounded_String_Iterator;

   BEGIN

      IF NOT Is_Empty(Dir_List) THEN

         Reset_Head_Iterator(Dir_List, List_Ite);
         LOOP
            Current_Element (Dir_List, A_Dir,
               List_Ite);

            IF Can_Be_Read (A_Dir.All & Directory_Separator(A_Dir.All &
                     File_Name) & File_Name)
                  THEN
               Put_Debug("Load AADL file " & A_Dir.All &
                  Directory_Separator(A_Dir.All & File_Name) & File_Name);
               RETURN Ocarina.Aadl.Parser.Process (To_String(A_Dir.All &
                     Directory_Separator(A_Dir.All & File_Name) &
                     File_Name), Root);
            END IF;

            IF  Is_Tail_Element (Dir_List,
                  List_Ite) THEN
               EXIT;
            END IF;
            Next_Element (Dir_List, List_Ite);
         END LOOP;

      END IF;


      IF Can_Be_Read (File_Name)
            THEN
         Put_Debug("Load AADL file "  & File_Name);
         RETURN Ocarina.Aadl.Parser.Process (To_String(File_Name), Root);
      END IF;

      RETURN Root;
   END Load_Aadl_File_From_Directory;



END Aadl_Parsers;



