------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Objects;
USE Objects;

WITH Scheduler;
USE Scheduler;

WITH Unchecked_Deallocation;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;




PACKAGE Address_Spaces IS



   TYPE Address_Space IS NEW Generic_Object WITH
      RECORD
         Cpu_Name : Unbounded_String;

         Text_Memory_Size  : Natural := 0;
         Stack_Memory_Size : Natural := 0;
         Data_Memory_Size  : Natural := 0;
         Heap_Memory_Size  : Natural := 0;

         Scheduler : Scheduler_Ptr;
      END RECORD;

   TYPE Address_Space_Ptr IS ACCESS ALL Address_Space'Class;


   PROCEDURE Initialize (
         A_Addr : IN OUT Address_Space);


   PROCEDURE Put_Name (
         A_Addr : IN     Address_Space_Ptr);

   PROCEDURE Put (
         A_Addr : IN     Address_Space_Ptr);

   PROCEDURE Put (
         A_Addr : IN     Address_Space);

   FUNCTION Copy (
         A_Addr : IN     Address_Space_Ptr)
     RETURN Address_Space_Ptr;

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Address_Space'Class, Address_Space_Ptr);





END Address_Spaces;








