------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Framework_Config;
USE Framework_Config;


WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Ada.Strings.Unbounded.Text_Io;
USE Ada.Strings.Unbounded.Text_Io;

WITH Objects;
USE Objects;


WITH Unchecked_Deallocation;
WITH Convert_Strings;
WITH Convert_Unbounded_Strings;
WITH Indexed_Tables;

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;

WITH Queueing_System;
USE Queueing_System;

PACKAGE Buffers IS


   TYPE Role_Type IS
         (No_Role,
          Producer,
          Consumer);

   TYPE Buffer_Role IS
      RECORD
         The_Role : Role_Type;
         Size     : Natural;
         Time     : Natural;
      END RECORD;


   PROCEDURE Put (
         B : IN     Buffer_Role);
   PROCEDURE Initialize (
         B :    OUT Buffer_Role);


   PROCEDURE To_Role IS
   NEW Convert_Strings(Role_Type, No_Role);
   PROCEDURE To_Role IS
   NEW Convert_Unbounded_Strings(Role_Type, No_Role);



   PACKAGE Buffer_Roles IS NEW Indexed_Tables(Buffer_Role,
      Unbounded_String,
      Framework_Config.Max_Tasks, 0,
      Put, Initialize, Put);
   USE Buffer_Roles;

   SUBTYPE Buffer_Roles_Table IS Indexed_Table;
   SUBTYPE Buffer_Roles_Range IS Indexed_Table_Range;

   --------------------------------------------------------

   TYPE Buffer IS NEW Generic_Object WITH
      RECORD
         Cpu_Name : Unbounded_String := To_Unbounded_String ("");

         Address_Space_Name : Unbounded_String := To_Unbounded_String ("");

         Queueing_System_Type : Queueing_Systems_Type;

         Size  : Natural            := 0;
         Roles : Buffer_Roles_Table;
         X     : Natural            := 0;
         Y     : Natural            := 0;
      END RECORD;

   TYPE Buffer_Ptr IS ACCESS ALL Buffer'Class;


   PROCEDURE Initialize (
         A_Buff : IN OUT Buffer);

   PROCEDURE Put (
         My_Buff : IN     Buffer);


   PROCEDURE Put_Name (
         A_Buff : IN     Buffer_Ptr);

   PROCEDURE Put (
         A_Buff : IN     Buffer_Ptr);


   FUNCTION Copy (
         My_Buffer : IN     Buffer_Ptr)
     RETURN Buffer_Ptr;

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Buffer'Class, Buffer_Ptr);



   --------------------------------------------------------


   FUNCTION Is_Cons_Prod_Harmonic (
         My_Buff  : IN     Buffer_Ptr;
         My_Tasks : IN     Tasks_Set)
     RETURN Boolean;


   Flow_Constraint_Not_Respected : EXCEPTION;
   Task_Model_Error              : EXCEPTION;


   PROCEDURE Buffer_Flow_Control (
         My_Buff   : IN     Buffer_Ptr;
         My_Tasks  : IN     Tasks_Set;
         Flow_Cons : IN OUT Double;
         Flow_Prod : IN OUT Double);




END Buffers;





