------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Framework_Config;
USE Framework_Config;

WITH Translate;
USE Translate;
WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Scheduling_Sequences;
USE Scheduling_Sequences;
USE Scheduling_Sequences.Densities_Table_Package;
USE Scheduling_Sequences.Density_Table_Package;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Double_Util;
USE Double_Util;

WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;


WITH Call_Framework;
USE Call_Framework;
WITH Call_Scheduling_Framework;
USE Call_Scheduling_Framework;


WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;

WITH Text_Io;
USE Text_Io;

WITH Xml_Tag;
USE Xml_Tag;


PACKAGE BODY Call_Random_Framework IS





   PROCEDURE Compute_Response_Time_Density (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output) IS



      Proba : Densities_Table;


   BEGIN
      Result := To_Unbounded_String("");

      IF Output=Xml_Output THEN
         Set_Tag;
      ELSE
         Set_Empty;
      END IF;

      FOR I IN 0..Call_Scheduling_Framework.Sched.Nb_Entries-1 LOOP


         IF Call_Scheduling_Framework.Sched.Entries(I).Item=
               A_Processor THEN

            Initialize(Proba);
            Compute_Response_Time_Distribution (Sys.Tasks,
               Call_Scheduling_Framework.Sched.Entries(I).Data.Result,
               Call_Scheduling_Framework.Sched.Entries(I).Item.
               Name,
               Proba);
            FOR I IN 0..Proba.Nb_Entries-1 LOOP
               Result := Result & Lb_Minus & Lb_Task(
                  Current_Language) &
                  Unbounded_Sp & Start_Task
                  & To_String(Proba.Entries(I).Item.Name) & End_Task &
                  Lb_Colon &
                  Unbounded_Lf;
               FOR K IN 0..Proba.Entries(I).Data.Nb_Entries-1 LOOP
                  Result := Result & To_Unbounded_String("   ") &
                     To_Unbounded_String("P(") & Lb_Response_Time(
                     Current_Language) & Lb_Equal
                     & Start_Response_Time & To_Unbounded_String(Proba.
                     Entries(I).Data.Entries(K).Response_Time'Img) &
                     End_Response_Time & To_Unbounded_String(")=")  &
                     Start_Probability & Format(Proba.Entries(I).Data.
                     Entries(K).Probability) & End_Probability &
                     Unbounded_Lf;
               END LOOP;
            END LOOP;

         END IF;

      END LOOP;


      Result := Result & End_Response_Time_Density;


   EXCEPTION
      WHEN Task_Set.Task_Must_Be_Periodic=>
         Result := Result & Lb_Compute_Scheduling_Error_5(
            Current_Language) &
            Unbounded_Lf & End_Response_Time_Density;
      WHEN Task_Set.Task_Model_Error =>
         Result := Result & Lb_Compute_Scheduling_Error_6(
            Current_Language) &
            Unbounded_Lf & End_Response_Time_Density;

   END Compute_Response_Time_Density;





END Call_Random_Framework;





