------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Text_Io;
USE Text_Io;


PACKAGE BODY Dependencies IS




   PROCEDURE Initialize (
         My_Dep : IN OUT Dependency_Ptr) IS
   BEGIN
      My_Dep:=NULL;
   END Initialize;


   FUNCTION Copy (
         My_Dep : IN     Dependency_Ptr)
     RETURN Dependency_Ptr IS

      Ptr : Dependency_Ptr;
   BEGIN
      Ptr:=NEW Dependency(My_Dep.Depend_Type);
      Ptr.All:=My_Dep.All;

      RETURN Ptr;
   END Copy;


   PROCEDURE Put (
         My_Dep : IN     Dependency_Ptr) IS
   BEGIN
      IF (My_Dep/=NULL) THEN
         Put(My_Dep.Depend_Type'Img & '/');
         CASE My_Dep.Depend_Type IS

            WHEN Precedence_Dependency =>
               Put(To_String(My_Dep.Precedency_From_Task.Name) & "=>" &
                  To_String(
                     My_Dep.Precedency_To_Task.
                     Name));

            WHEN Buffer_Dependency =>
               IF (My_Dep.Buffer_Orientation=From_Object_To_Task) THEN
                  Put(To_String(My_Dep.Buffer_Dependency_Object.Name) &
                     "=>" &
                     To_String(
                        My_Dep.Buffer_Dependent_Task.
                        Name));
               ELSE
                  Put(To_String(My_Dep.Buffer_Dependent_Task.Name) & "=>" &
                     To_String(
                        My_Dep.Buffer_Dependency_Object.
                        Name));
               END IF;

            WHEN Communication_Dependency =>
               IF (My_Dep.Communication_Orientation=From_Object_To_Task) THEN
                  Put(To_String(My_Dep.Communication_Dependency_Object.
                        Name) &
                     "=>" & To_String(
                        My_Dep.Communication_Dependent_Task.
                        Name));
               ELSE
                  Put(To_String(My_Dep.Communication_Dependent_Task.Name) &
                     "=>" &
                     To_String(
                        My_Dep.Communication_Dependency_Object.
                        Name));
               END IF;
         END CASE;
      END IF;
   END Put;






END Dependencies;


