------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Text_Io;
USE Text_Io;

PACKAGE BODY Messages IS



   PROCEDURE Initialize (
         A_Message : IN OUT Generic_Message) IS
   BEGIN
      A_Message.Object_Type:=Message_Object_Type;
   END Initialize;



   PROCEDURE Initialize (
         A_Message : IN OUT Aperiodic_Message) IS
   BEGIN
      Initialize(Generic_Message(A_Message));
      A_Message.Message_Type:=Aperiodic_Type;
   END Initialize;


   PROCEDURE Initialize (
         A_Message : IN OUT Periodic_Message) IS
   BEGIN
      Initialize(Generic_Message(A_Message));
      A_Message.Message_Type:=Periodic_Type;
   END Initialize;


   --------------------------------------------------

   PROCEDURE Put (
         My_Message : IN     Generic_Message) IS
   BEGIN
      Put ("Name : " & To_String(My_Message.Name));
      Put (" ; Type : " & My_Message.Message_Type'Img);
      Put (" ; Size : " & My_Message.Size'Img & " ; " );
      Put ("Deadline : " & My_Message.Deadline'Img & " ; " );
      Put(" ; x : " & My_Message.X'Img);
      Put(" ; y : " & My_Message.Y'Img);
      Put(" ; Response : " & My_Message.Response_Time'Img);
      Put(" ; Communication : " & My_Message.Communication_Time'Img);
   END Put;


   PROCEDURE Put_Name (
         A_Message : IN     Message_Ptr) IS
   BEGIN
      Put(To_String(A_Message.Name));
   END Put_Name;

   PROCEDURE Put (
         A_Message : IN     Message_Ptr) IS
   BEGIN
      Put(A_Message.All);
   END Put;


   PROCEDURE Put (
         My_Message : IN     Periodic_Message) IS
   BEGIN
      Put(Generic_Message(My_Message));
      Put ("Period : " & My_Message.Period'Img & " ; " );
      Put ("Jitter : " & My_Message.Jitter'Img & " ; " );
   END Put;




   FUNCTION Copy (
         My_Message : IN     Message_Ptr)
     RETURN Message_Ptr IS

      New_Periodic_Message  : Periodic_Message_Ptr;
      New_Aperiodic_Message : Aperiodic_Message_Ptr;
      Ptr                   : Message_Ptr;

   BEGIN
      IF (My_Message.Message_Type = Periodic_Type)
            THEN
         New_Periodic_Message:=NEW Periodic_Message;
         New_Periodic_Message.All:=Periodic_Message_Ptr(My_Message).ALL;
         Ptr:=Message_Ptr(New_Periodic_Message);
      ELSE
         New_Aperiodic_Message:=NEW Aperiodic_Message;
         New_Aperiodic_Message.All:=Aperiodic_Message_Ptr(My_Message).
            ALL;
         Ptr:=Message_Ptr(New_Aperiodic_Message);
      END IF;

      RETURN Ptr;
   END Copy;





END Messages;


