------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Ada.Finalization;
WITH Unchecked_Deallocation;
WITH Convert_Strings;
WITH Convert_Unbounded_Strings;



PACKAGE Objects  IS




   FUNCTION Is_A_Valid_Identifier (
         Id : Unbounded_String)
     RETURN Boolean;




   TYPE Objects_Type IS
         (Buffer_Object_Type,
          Processor_Object_Type,
          Task_Object_Type,
          Resource_Object_Type,
          Address_Space_Object_Type,
          Message_Object_Type,
          Network_Object_Type);


   PROCEDURE To_Object_Type IS
   NEW Convert_Strings(Objects_Type, Task_Object_Type);
   PROCEDURE To_Object_Type IS
   NEW Convert_Unbounded_Strings(Objects_Type, Task_Object_Type);


   --------------------------------------------------

   TYPE Generic_Object IS NEW Ada.Finalization.Controlled WITH
      RECORD

         -- Object  name
         --
         Name : Unbounded_String := To_Unbounded_String ("");

         -- Objet type (task, buffer, processor, ...)
         --
         Object_Type : Objects_Type;
      END RECORD;


   TYPE Object_Ptr IS ACCESS ALL Generic_Object'Class;


   PROCEDURE Put (
         My_Object : IN     Generic_Object);

   PROCEDURE Put (
         My_Object : IN     Object_Ptr);

   PROCEDURE Put_Name (
         My_Object : IN     Object_Ptr);


   FUNCTION Copy (
         My_Object : IN     Object_Ptr)
     RETURN Object_Ptr;


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Generic_Object'Class, Object_Ptr);



END Objects;





