------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Framework_Config;
USE Framework_Config;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;

WITH Tables;



PACKAGE Parameters IS



   -- User defined parameters can have several types : boolean, integer, double or string
   -- 	
   TYPE Parameter_Type IS
         (Boolean_Parameter,
          Integer_Parameter,
          Double_Parameter,
          String_Parameter);

   -- Definition of a user defined parameter
   --
   TYPE Parameter
      (Ptype : Parameter_Type) IS
   RECORD

      -- Name of the parameter
      --
      Name : Unbounded_String;

      -- Type and value of the parameter
      --
      CASE Ptype IS
         WHEN Boolean_Parameter =>
            Boolean_Value : Boolean;
         WHEN Integer_Parameter =>
            Integer_Value : Integer;
         WHEN Double_Parameter =>
            Double_Value : Double;
         WHEN String_Parameter =>
            String_Value : Unbounded_String;
      END CASE;
   END RECORD;

   TYPE Parameter_Ptr IS ACCESS Parameter;


   PROCEDURE Initialize (
         My_Parameter :    OUT Parameter_Ptr);

   PROCEDURE Put (
         My_Parameter : Parameter_Ptr);

   ----------------------------------------------------------------------
   -- Table of user's defined parameters
   ----------------------------------------------------------------------

   PACKAGE User_Defined_Parameters_Table_Package IS NEW Tables(
      Parameter_Ptr,
      Max_User_Defined_Task_Parameters,Put);


   -- Range and type of user defined parameter tables
   --
   SUBTYPE User_Defined_Parameters_Range IS
      User_Defined_Parameters_Table_Package.Table_Range;
   SUBTYPE User_Defined_Parameters_Table IS
      User_Defined_Parameters_Table_Package.Table;


   -- Empty user defined parameter table
   --
   No_User_Defined_Parameter : User_Defined_Parameters_Table;


   ----------------------------------------------------------------------
   -- Table of framework parameters
   ----------------------------------------------------------------------
   PACKAGE Framework_Parameters_Table_Package IS NEW Tables(Parameter_Ptr,
      Max_Framework_Parameters,Put);


   -- Range and type of framework parameter tables
   --
   SUBTYPE Framework_Parameters_Range IS
      Framework_Parameters_Table_Package.
      Table_Range;
   SUBTYPE Framework_Parameters_Table IS
      Framework_Parameters_Table_Package.
      Table;


   -- Empty framework parameter table
   --
   No_Framework_Parameter : Framework_Parameters_Table;



END Parameters;



