------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded.Text_Io;
USE Ada.Strings.Unbounded.Text_Io;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Text_Io;
USE Text_Io;



PACKAGE BODY Simulations IS


   FUNCTION To_Unbounded_String (
         S    : IN     Simulation_Value_Ptr;
         From,
         To   : IN     Natural)
     RETURN Unbounded_String IS

      Msg       : Unbounded_String := Empty_String;
      Safe_From : Natural          := 0;
      Safe_To   : Natural          := 0;

   BEGIN

      IF (S/=NULL) THEN


         IF (S.Ptype = Simulation_Time_Unit_Array_Random)
               OR (S.Ptype = Simulation_Random)
               OR (S.Ptype = Simulation_Array_Random)
               THEN
            RETURN Empty_String;
         END IF;


         IF ((S.Ptype=Simulation_Time_Unit_Array_Boolean)
               OR (S.Ptype=Simulation_Time_Unit_Array_Integer)
               OR (S.Ptype=Simulation_Time_Unit_Array_Double)
               OR (S.Ptype=Simulation_Time_Unit_Array_String))
               THEN
            IF Integer(Time_Unit_Range'First)<=Integer(From)
                  THEN
               Safe_From:=From;
            ELSE
               Safe_From:=Natural(Time_Unit_Range'First);
            END IF;
            IF Integer(Time_Unit_Range'Last)>=To
                  THEN
               Safe_To:=To;
            ELSE
               Safe_To:=Natural(Time_Unit_Range'Last);
            END IF;
         END IF;

         IF ((S.Ptype=Simulation_Array_Boolean)
               OR (S.Ptype=Simulation_Array_Integer)
               OR (S.Ptype=Simulation_Array_String)
               OR (S.Ptype=Simulation_Array_Double))
               THEN

            IF Integer(Simulations_Range'First)<=Integer(From)
                  THEN
               Safe_From:=From;
            ELSE
               Safe_From:=Natural(Simulations_Range'First);
            END IF;
            IF Integer(Simulations_Range'Last)>=To
                  THEN
               Safe_To:=To;
            ELSE
               Safe_To:=Natural(Simulations_Range'Last);
            END IF;
         END IF;

         CASE S.Ptype IS
            WHEN Simulation_Time_Unit_Array_Random =>
               NULL;
            WHEN Simulation_Random =>
               NULL;
            WHEN  Simulation_Array_Random =>
               NULL;
            WHEN   Simulation_Integer =>
               RETURN To_Unbounded_String(S.Integer_Value'Img);
            WHEN  Simulation_Boolean =>
               RETURN To_Unbounded_String(S.Boolean_Value'Img);
            WHEN Simulation_Double  =>
               RETURN To_Unbounded_String(S.Double_Value'Img);
            WHEN Simulation_String =>
               RETURN S.String_Value;
            WHEN Simulation_Time_Unit_Array_Boolean=>
               FOR I IN Time_Unit_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Boolean_Time_Unit_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Time_Unit_Array_Integer=>
               FOR I IN Time_Unit_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Integer_Time_Unit_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Time_Unit_Array_Double=>
               FOR I IN Time_Unit_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Double_Time_Unit_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Time_Unit_Array_String=>
               FOR I IN Time_Unit_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.String_Time_Unit_Table_Value(I) & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Array_Boolean=>
               FOR I IN Simulations_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Boolean_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Array_Integer=>
               FOR I IN Simulations_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Integer_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Array_Double=>
               FOR I IN Simulations_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.Double_Table_Value(I)'Img & ' ';
                  END IF;
               END LOOP;
            WHEN Simulation_Array_String=>
               FOR I IN Simulations_Range LOOP
                  IF (Natural(I)>=Safe_From) AND (Natural(I) <= Safe_To)  THEN
                     Msg:=Msg & S.String_Table_Value(I) & ' ';
                  END IF;
               END LOOP;
         END CASE;
      END IF;
      RETURN Msg;

   END To_Unbounded_String;



   FUNCTION To_Unbounded_String (
         S : Simulation_Value_Ptr)
     RETURN Unbounded_String IS

   BEGIN
      RETURN To_Unbounded_String(S,0,0);
   END To_Unbounded_String;



   PROCEDURE Put (
         S : Simulation_Value_Ptr) IS
   BEGIN
      Put(To_Unbounded_String(S));
   END Put;



   FUNCTION Copy (
         My_Simulation : IN     Simulation_Value_Ptr)
     RETURN Simulation_Value_Ptr IS


      New_Simulation : Simulation_Value_Ptr;

   BEGIN
      New_Simulation := NEW Simulation_Value(My_Simulation.Ptype);

      CASE My_Simulation.Ptype IS

         WHEN Simulation_Time_Unit_Array_Random =>
            New_Simulation.Random_Value:=My_Simulation.Random_Value;
         WHEN Simulation_Random =>
            New_Simulation.Random_Table_Value:=My_Simulation.
               Random_Table_Value;
         WHEN  Simulation_Array_Random =>
            New_Simulation.Random_Time_Unit_Table_Value:=My_Simulation.
               Random_Time_Unit_Table_Value;
         WHEN Simulation_Boolean=>
            New_Simulation.Boolean_Value:=My_Simulation.Boolean_Value;
         WHEN Simulation_Integer=>
            New_Simulation.Integer_Value:=My_Simulation.Integer_Value;
         WHEN Simulation_Double=>
            New_Simulation.Double_Value:=My_Simulation.Double_Value;
         WHEN Simulation_String=>
            New_Simulation.String_Value:=My_Simulation.String_Value;

         WHEN Simulation_Array_Boolean=>
            New_Simulation.Boolean_Table_Value:=My_Simulation.
               Boolean_Table_Value;
         WHEN Simulation_Array_Integer=>
            New_Simulation.Integer_Table_Value:=My_Simulation.
               Integer_Table_Value;
         WHEN Simulation_Array_Double=>
            New_Simulation.Double_Table_Value:=My_Simulation.
               Double_Table_Value;
         WHEN Simulation_Array_String=>
            New_Simulation.String_Table_Value:=My_Simulation.
               String_Table_Value;

         WHEN Simulation_Time_Unit_Array_Boolean=>
            New_Simulation.Boolean_Time_Unit_Table_Value:=
               My_Simulation.Boolean_Time_Unit_Table_Value;
         WHEN Simulation_Time_Unit_Array_Integer=>
            New_Simulation.Integer_Time_Unit_Table_Value:=
               My_Simulation.Integer_Time_Unit_Table_Value;
         WHEN Simulation_Time_Unit_Array_Double=>
            New_Simulation.Double_Time_Unit_Table_Value:=My_Simulation.
               Double_Time_Unit_Table_Value;
         WHEN Simulation_Time_Unit_Array_String=>
            New_Simulation.String_Time_Unit_Table_Value:=My_Simulation.
               String_Time_Unit_Table_Value;

      END CASE;

      RETURN New_Simulation;

   END Copy;


   PROCEDURE Put (
         S : IN     Law_Type) IS
   BEGIN
      CASE S IS
         WHEN Uniform_Law =>
            Put(" uniform ");
         WHEN Exponential_Law =>
            Put(" exponential ");
      END CASE;
   END Put;




END Simulations;



