------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;

WITH Ada.Strings.Unbounded.Text_Io;
USE Ada.Strings.Unbounded.Text_Io;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Ada.Exceptions;
USE Ada.Exceptions;
WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Translate;
USE Translate;

WITH Simulations;
USE Simulations;

WITH Time_Units;
USE Time_Units;

WITH Ada.Numerics.Float_Random;
USE Ada.Numerics.Float_Random;





PACKAGE BODY Statements IS




   FUNCTION Find_Set (
         Var : IN     Sets_Table_Type;
         S   : IN     Unbounded_String)
     RETURN Sets_Range IS

   BEGIN
      FOR I IN 0..Var.Nb_Entries-1 LOOP
         IF Var.Entries(I).Set_Id=S
               THEN
            RETURN I;
         END IF;
      END LOOP;

      Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
            S & Lb_Comma & Lb_Undeclared_Identifier(Current_Language) ));

   END Find_Set;



   PROCEDURE Check_Set_Declaration (
         Var : IN     Sets_Table_Type;
         S   : IN     Unbounded_String) IS

      Dummy : CONSTANT Sets_Range := Find_Set (Var, S);
   BEGIN
      NULL;
   END Check_Set_Declaration;


   --------------------------------------------------------------

   PROCEDURE Initialize (
         S : IN OUT Put_Statement) IS
   BEGIN
      S.Statement_Type:=Put_Statement_Type;
      S.Expression_To_Be_Displayed:=NULL;
      S.Put_To:=NULL;
      S.Put_From:=NULL;
   END Initialize;

   PROCEDURE Initialize (
         S : IN OUT Nop_Statement) IS
   BEGIN
      S.Statement_Type:=Nop_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT If_Statement) IS
   BEGIN
      S.Statement_Type:=If_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT Random_Initialize_Statement) IS
   BEGIN
      S.Statement_Type:=Random_Initialize_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT Affectation_Statement) IS
   BEGIN
      S.Statement_Type:=Affectation_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT For_Statement) IS
   BEGIN
      S.Statement_Type:=For_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT While_Statement) IS
   BEGIN
      S.Statement_Type:=While_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT Return_Statement) IS
   BEGIN
      S.Statement_Type:=Return_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT Set_Statement) IS
   BEGIN
      S.Statement_Type:=Set_Statement_Type;
   END Initialize;


   PROCEDURE Initialize (
         S : IN OUT Subprogram_Call_Statement) IS
   BEGIN
      S.Statement_Type:=Subprogram_Call_Statement_Type;
   END Initialize;



   PROCEDURE Initialize (
         S : IN OUT Subprogram_Statement) IS
   BEGIN
      S.Statement_Type:=Subprogram_Statement_Type;
   END Initialize;


   --------------------------------------------------------------



   PROCEDURE Recursive_Put (
         S : IN     Statement_Ptr) IS

   BEGIN

      Put(S.All);

      IF (S.Statement_Type = While_Statement_Type) THEN
         IF (While_Statement_Ptr(S).Included_Statement /= NULL) THEN
            Put("Included statement : ");
            New_Line;
            Recursive_Put(While_Statement_Ptr(S).Included_Statement);
         END IF;
      END IF;
      IF (S.Statement_Type = For_Statement_Type) THEN
         IF (For_Statement_Ptr(S).Included_Statement /= NULL) THEN
            Put("Included statement : ");
            New_Line;
            Recursive_Put(For_Statement_Ptr(S).Included_Statement);
         END IF;
      END IF;

      IF (S.Statement_Type = If_Statement_Type)
            THEN
         IF (If_Statement_Ptr(S).Then_Statement /= NULL) THEN
            Put("Then statement : ");
            New_Line;
            New_Line;
            Recursive_Put(If_Statement_Ptr(S).Then_Statement);
         END IF;
         IF (If_Statement_Ptr(S).Else_Statement /= NULL) THEN
            Put("Else statement : ");
            New_Line;
            New_Line;
            Recursive_Put(If_Statement_Ptr(S).Else_Statement);
         END IF;

      ELSE
         IF (S.Next_Statement /= NULL) THEN
            Put("Next statement  : ");
            New_Line;
            New_Line;
            Recursive_Put(S.Next_Statement);
         END IF;

      END IF;


   END Recursive_Put;


   PROCEDURE Put (
         S : IN     Generic_Statement) IS

   BEGIN
      Put_Line("Type : " & S.Statement_Type'Img);
      Put_Line("Line number : " & S.Line_Number'Img);
   END Put;



   PROCEDURE Put (
         S : IN     Subprogram_Statement) IS

   BEGIN
      Put_Line("Subprogram statement :");
      Put(Generic_Statement(S));
      Put("Is a function : ");
      Put(S.Is_A_Function'Img);
      New_Line;
      Put("Subprogram name  : ");
      Put(S.Subprogram_Name);
      New_Line;
   END Put;



   PROCEDURE Put (
         S : IN     Subprogram_Call_Statement) IS

   BEGIN
      Put_Line("Subprogram call statement :");
      Put(Generic_Statement(S));
      Put("Is a function : ");
      Put(S.Is_A_Function'Img);
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Nop_Statement) IS

   BEGIN
      Put_Line("Nop statement :");
      Put(Generic_Statement(S));
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Put_Statement) IS

   BEGIN
      Put_Line("Put statement :");
      Put(Generic_Statement(S));
      Put("Expression  : ");
      Put(S.Expression_To_Be_Displayed.All);
      New_Line;
      IF (S.Put_From/=NULL) AND (S.Put_To/=NULL)
            THEN
         Put_Line("From/to : " );
         Put(S.Put_From.All);
         Put(" / ");
         Put(S.Put_To.All);
         New_Line;
      END IF;
   END Put;



   PROCEDURE Put (
         S : IN     If_Statement) IS

   BEGIN
      Put_Line("If statement :");
      Put(Generic_Statement(S));
      Put("Bool expression : ");
      Put(S.Bool_Expression.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     For_Statement) IS

   BEGIN
      Put_Line("For statement :");
      Put(Generic_Statement(S));
      Put("For index : ");
      Put(S.For_Index.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     While_Statement) IS

   BEGIN
      Put_Line("While statement :");
      Put(Generic_Statement(S));
      Put("Condition : ");
      Put(S.Condition.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Affectation_Statement) IS

   BEGIN
      Put_Line("Affectation statement :");
      Put(Generic_Statement(S));
      Put("Variable name/index : ");
      New_Line;
      Put("Lvalue : ");
      Put(S.Lvalue.All);
      New_Line;
      Put("Rvalue : ");
      Put(S.Rvalue.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Random_Initialize_Statement) IS

   BEGIN
      Put_Line("Random Initialize statement :");
      Put(Generic_Statement(S));
      Put("Variable name/index : ");
      New_Line;
      Put("Lvalue : ");
      Put(To_String(S.Lvalue));
      New_Line;
      Put("Law : ");
      Put(S.Law);
      New_Line;
      Put("Parameter1 : ");
      Put(S.Parameter1.All);
      New_Line;
      Put("Parameter2 : ");
      Put(S.Parameter2.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Return_Statement) IS

   BEGIN
      Put_Line("Return statement :");
      Put(Generic_Statement(S));
      Put("Return value: ");
      Put(S.Return_Value.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Set_Statement) IS

   BEGIN
      Put_Line("Set statement :");
      Put(Generic_Statement(S));
      Put("Set value: ");
      Put(S.Set_Value.All);
      New_Line;
      New_Line;
   END Put;


   PROCEDURE Put (
         S : IN     Set_Statement_Ptr) IS
   BEGIN
      Put(S.All);
   END Put;

   --------------------------------------------------------------



   FUNCTION Copy (
         S : IN     Statement_Ptr)
     RETURN Statement_Ptr IS

      New_If_Statement                : If_Statement_Ptr;
      New_For_Statement               : For_Statement_Ptr;
      New_Return_Statement            : Return_Statement_Ptr;
      New_Affectation_Statement       : Affectation_Statement_Ptr;
      New_Put_Statement               : Put_Statement_Ptr;
      New_Nop_Statement               : Nop_Statement_Ptr;
      New_While_Statement             : While_Statement_Ptr;
      New_Set_Statement               : Set_Statement_Ptr;
      New_Subprogram_Statement        : Subprogram_Statement_Ptr;
      New_Subprogram_Call_Statement   : Subprogram_Call_Statement_Ptr;
      New_Random_Initialize_Statement : Random_Initialize_Statement_Ptr;

      Ptr : Statement_Ptr;

   BEGIN

      CASE S.Statement_Type IS
         WHEN Affectation_Statement_Type   =>
            New_Affectation_Statement       := NEW Affectation_Statement;
            New_Affectation_Statement.All   := Affectation_Statement_Ptr(
               S).ALL;
            Ptr:= Statement_Ptr(New_Affectation_Statement);
         WHEN For_Statement_Type   =>
            New_For_Statement       := NEW For_Statement;
            New_For_Statement.All   := For_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_For_Statement);
         WHEN While_Statement_Type   =>
            New_While_Statement       := NEW While_Statement;
            New_While_Statement.All   := While_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_While_Statement);
         WHEN If_Statement_Type   =>
            New_If_Statement      := NEW If_Statement;
            New_If_Statement.All  := If_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_If_Statement);
         WHEN Return_Statement_Type   =>
            New_Return_Statement        := NEW Return_Statement;
            New_Return_Statement.All    := Return_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_Return_Statement);
         WHEN Put_Statement_Type =>
            New_Put_Statement     := NEW Put_Statement;
            New_Put_Statement.All := Put_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(
               New_Put_Statement);
         WHEN Nop_Statement_Type =>
            New_Nop_Statement     := NEW Nop_Statement;
            New_Nop_Statement.All := Nop_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_Nop_Statement);
         WHEN Set_Statement_Type   =>
            New_Set_Statement        := NEW Set_Statement;
            New_Set_Statement.All    := Set_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(New_Set_Statement);
         WHEN Random_Initialize_Statement_Type =>
            New_Random_Initialize_Statement:= NEW
               Random_Initialize_Statement;
            New_Random_Initialize_Statement.All    :=
               Random_Initialize_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(
               New_Random_Initialize_Statement);
         WHEN Subprogram_Statement_Type   =>
            New_Subprogram_Statement        := NEW Subprogram_Statement;
            New_Subprogram_Statement.All    := Subprogram_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(
               New_Subprogram_Statement);
         WHEN Subprogram_Call_Statement_Type   =>
            New_Subprogram_Call_Statement        := NEW
               Subprogram_Call_Statement;
            New_Subprogram_Call_Statement.All    :=
               Subprogram_Call_Statement_Ptr(S).ALL;
            Ptr                     := Statement_Ptr(
               New_Subprogram_Call_Statement);
      END CASE;

      RETURN Ptr;
   END Copy;





   --------------------------------------------------------------

   PROCEDURE If_Dispatch (
         Current         : IN     If_Statement_Ptr;
         Current_Section : IN     Sections;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Simu_Ptr : Simulation_Value_Ptr;

   BEGIN

      Simu_Ptr:= Value_Of (Current.Bool_Expression.
         ALL, Variables_Table);

      IF Simu_Ptr.Ptype/=Simulation_Boolean
            THEN
         Raise_Exception (Type_Error'Identity, "If statement, line " &
            Next.Line_Number'Img
            & To_String( Lb_Comma & Lb_Uncompatible_Type_Error(
                  Current_Language)) );
      END IF;

      IF Simu_Ptr.Boolean_Value
            THEN
         Dispatch(Current.Then_Statement, Current_Section, Variables_Table,
            Msg);
      ELSE
         Dispatch(Current.Else_Statement, Current_Section, Variables_Table,
            Msg);
      END IF;


      Free(Simu_Ptr);


      Next:=Current.Next_Statement;




   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);

   END If_Dispatch;



   PROCEDURE Random_Initialize_Dispatch (
         Current         : IN     Random_Initialize_Statement_Ptr;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Var  : Variables_Range;
      Val1,
      Val2 : Simulation_Value_Ptr;
      Seed : Generator;

   BEGIN

      Var:=Find_Variable(Variables_Table, Current.Lvalue,
         Current.Line_Number);

      IF  Current.Law = Uniform_Law
            THEN
         Val1:=Value_Of(Current.Parameter1.All, Variables_Table);
         Val2:=Value_Of(Current.Parameter2.All, Variables_Table);
      ELSE
         Val1:=Value_Of(Current.Parameter1.All, Variables_Table);
         Val2:= NEW  Simulation_Value(Simulation_Integer);
      END IF;


      IF Val1.Ptype/=Simulation_Integer
            THEN
         Raise_Exception (Type_Error'Identity, "Random  statement, line " &
            Next.Line_Number'Img
            & To_String( Lb_Comma & Lb_Uncompatible_Type_Error(
                  Current_Language)) );
      END IF;
      IF Val2.Ptype/=Simulation_Integer
            THEN
         Raise_Exception (Type_Error'Identity, "Random  statement, line " &
            Next.Line_Number'Img
            & To_String( Lb_Comma & Lb_Uncompatible_Type_Error(
                  Current_Language)) );
      END IF;



      -- Store random law parameters  in simulation data
      -- and initialize seed
      --
      IF (Variables_Table.Entries(Var).Simulation.Ptype =
            Simulation_Array_Random)
            THEN
         FOR I IN Simulations_Range LOOP
            Reset(Seed);
            Save(Seed, Variables_Table.Entries(Var).Simulation.
               Random_Table_Value(I));
         END LOOP;
         Variables_Table.Entries(Var).Simulation.
            Random_Table_Law_Parameters.Law:=Current.Law;
         Variables_Table.Entries(Var).Simulation.
            Random_Table_Law_Parameters.Parameter1:=Val1.Integer_Value;
         Variables_Table.Entries(Var).Simulation.
            Random_Table_Law_Parameters.Parameter2:=Val2.Integer_Value;

      ELSE

         IF (Variables_Table.Entries(Var).Simulation.Ptype =
               Simulation_Time_Unit_Array_Random)
               THEN
            FOR I IN Time_Unit_Range LOOP
               Reset(Seed);
               Save(Seed, Variables_Table.Entries(Var).Simulation.
                  Random_Time_Unit_Table_Value(I));
            END LOOP;
            Variables_Table.Entries(Var).Simulation.
               Random_Time_Unit_Table_Law_Parameters.Law:=Current.Law;
            Variables_Table.Entries(Var).Simulation.
               Random_Time_Unit_Table_Law_Parameters.Parameter1:=Val1.
               Integer_Value;
            Variables_Table.Entries(Var).Simulation.
               Random_Time_Unit_Table_Law_Parameters.Parameter2:=Val2.
               Integer_Value;

         ELSE

            Reset(Seed);
            Save(Seed, Variables_Table.Entries(Var).Simulation.
               Random_Value);
            Variables_Table.Entries(Var).
               Simulation.Random_Law_Parameters.
               Law:=Current.Law;
            Variables_Table.Entries(Var).
               Simulation.Random_Law_Parameters.
               Parameter1:=Val1.Integer_Value;
            Variables_Table.Entries(Var).
               Simulation.Random_Law_Parameters.
               Parameter2:=Val2.Integer_Value;
         END IF;
      END IF;

      Free(Val1);
      Free(Val2);

      Next:=Current.Next_Statement;



   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);


   END Random_Initialize_Dispatch;





   PROCEDURE Affectation_Dispatch (
         Current         : IN     Affectation_Statement_Ptr;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Left_Var      : Variables_Range;
      Fully_Indexed : Boolean              := False;
      Val,
      Index         : Simulation_Value_Ptr;

   BEGIN

      IF Current.Lvalue.Expression_Type=Array_Variable_Type
            THEN
         IF Array_Variable_Expr_Ptr(Current.Lvalue).Array_Index/=NULL
               THEN
            Fully_Indexed:=True;
         END IF;
      END IF;



      -- Affectation from a scalar to another scalar
      --
      IF NOT Fully_Indexed THEN

         Left_Var:=Find_Variable(Variables_Table,
            Variable_Expr_Ptr(Current.Lvalue).Name, Current.Line_Number);
         Val:=Value_Of(Current.Rvalue.All, Variables_Table);


         -- We can not initialize the left part when its type is
         -- scalar and when the right part type is vectorial
         -- The only exception is when the operator is a min_to_index
         -- or a max_to_index operator
         --

         IF (
               (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Integer)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Boolean)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Random)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_String)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Double)  )
               AND (
               (Val.Ptype=Simulation_Array_Boolean)
               OR (Val.Ptype=Simulation_Array_Integer)
               OR (Val.Ptype=Simulation_Array_Double)
               OR (Val.Ptype=Simulation_Array_String)
               OR (Val.Ptype=Simulation_Array_Random)  )
               THEN
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line " &
               Current.Line_Number'Img & To_String(
                  Lb_Comma &
                  Lb_Variable_Size(Current_Language) ));
         END IF;


         IF (
               (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Array_Integer)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Array_Boolean)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Array_Random)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Array_String)
               OR (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Array_Double)  )
               AND (
               (Val.Ptype=Simulation_Time_Unit_Array_Double)
               OR (Val.Ptype=Simulation_Time_Unit_Array_Random)
               OR (Val.Ptype=Simulation_Time_Unit_Array_Double)
               OR (Val.Ptype=Simulation_Time_Unit_Array_String)
               OR (Val.Ptype=Simulation_Time_Unit_Array_Integer) )
               THEN
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line " &
               Current.Line_Number'Img & To_String(
                  Lb_Comma &
                  Lb_Variable_Size(Current_Language) ));
         END IF;


         -- Start to compute expression evaluation
         --

         IF (Variables_Table.Entries(Left_Var).Simulation.Ptype =
               Simulation_Array_Integer) AND   (Val.Ptype=
               Simulation_Integer)
               THEN
            FOR I IN Simulations_Range LOOP
               Variables_Table.Entries(Left_Var).Simulation.
                  Integer_Table_Value(I)
                  :=Val.Integer_Value;
            END LOOP;

         ELSE

            IF (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                  Simulation_Time_Unit_Array_Integer) AND   (Val.Ptype=
                  Simulation_Integer)
                  THEN
               FOR I IN Time_Unit_Range LOOP
                  Variables_Table.Entries(Left_Var).Simulation.
                     Integer_Time_Unit_Table_Value(I)
                     :=Val.Integer_Value;
               END LOOP;


            ELSE

               IF (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                     Simulation_Time_Unit_Array_Double) AND   (Val.Ptype=
                     Simulation_Double)
                     THEN
                  FOR I IN Time_Unit_Range LOOP
                     Variables_Table.Entries(Left_Var).Simulation.
                        Double_Time_Unit_Table_Value(I)
                        :=Val.Double_Value;
                  END LOOP;


               ELSE
                  IF (Variables_Table.Entries(Left_Var).Simulation.Ptype =
                        Simulation_Time_Unit_Array_Boolean) AND   (
                        Val.Ptype=
                        Simulation_Boolean)
                        THEN
                     FOR I IN Time_Unit_Range LOOP
                        Variables_Table.Entries(Left_Var).Simulation.
                           Boolean_Time_Unit_Table_Value(I)
                           :=Val.Boolean_Value;
                     END LOOP;


                  ELSE
                     IF (Variables_Table.Entries(Left_Var).
                           Simulation.Ptype =
                           Simulation_Time_Unit_Array_String) AND   (
                           Val.Ptype=
                           Simulation_String)
                           THEN
                        FOR I IN Time_Unit_Range LOOP
                           Variables_Table.Entries(Left_Var).Simulation.
                              String_Time_Unit_Table_Value(I)
                              :=Val.String_Value;
                        END LOOP;


                     ELSE

                        IF (Variables_Table.Entries(Left_Var).Simulation.
                              Ptype =
                              Simulation_Array_Boolean) AND   (Val.Ptype=
                              Simulation_Boolean)
                              THEN
                           FOR I IN Simulations_Range LOOP
                              Variables_Table.Entries(Left_Var).
                                 Simulation.
                                 Boolean_Table_Value(I)
                                 :=Val.Boolean_Value;
                           END LOOP;

                        ELSE
                           IF (Variables_Table.Entries(Left_Var).
                                 Simulation.
                                 Ptype =
                                 Simulation_Array_Double) AND   (
                                 Val.Ptype=
                                 Simulation_Double)
                                 THEN
                              FOR I IN Simulations_Range LOOP
                                 Variables_Table.Entries(Left_Var).
                                    Simulation.
                                    Double_Table_Value(I)
                                    :=Val.Double_Value;
                              END LOOP;

                           ELSE

                              IF (Variables_Table.Entries(Left_Var).
                                    Simulation.
                                    Ptype =
                                    Simulation_Array_String) AND   (
                                    Val.Ptype=
                                    Simulation_String)
                                    THEN
                                 FOR I IN Simulations_Range LOOP
                                    Variables_Table.Entries(Left_Var).
                                       Simulation.
                                       String_Table_Value(I)
                                       :=Val.String_Value;
                                 END LOOP;

                              ELSE

                                 Variables_Table.Entries(Left_Var).
                                    Simulation:=
                                    Value_Of(Current.Rvalue.All,
                                    Variables_Table);
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;


         Free(Val);


      ELSE



         -- Indexed affectation : the left side AND the right side are indexed
         --

         Index:=Value_Of(Array_Variable_Expr_Ptr(
               Current.Lvalue).
            Array_Index.All,
            Variables_Table);


         IF Index.Ptype/=Simulation_Integer THEN
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line " &
               Current.Line_Number'Img & To_String(
                  Lb_Comma &
                  Lb_Index_Error(Current_Language) ));
         END IF;


         Left_Var:=Find_Variable(Variables_Table,
            Variable_Expr_Ptr(Current.Lvalue).Name, Current.Line_Number);


         Val:=Value_Of(Current.Rvalue.All, Variables_Table);



         IF (Val.Ptype/=Simulation_Integer)
               AND (Val.Ptype/=Simulation_Boolean)
               AND (Val.Ptype/=Simulation_Double)
               AND (Val.Ptype/=Simulation_String)
               THEN
            Raise_Exception (Type_Error'Identity,
               "Affectation statement, line "
               & Current.Line_Number'Img &
               ", rvalue " & To_String(
                  Lb_Uncompatible_Type_Error(
                     Current_Language)) );
         END IF;



         IF ( (Val.Ptype=Simulation_Integer) AND
               (Variables_Table.Entries(Left_Var).Simulation.
                  Ptype=Simulation_Array_Integer) )

               THEN
            Variables_Table.Entries(Left_Var).Simulation.
               Integer_Table_Value(Simulations_Range(
                  Index.Integer_Value)):=
               Val.Integer_Value;

         ELSE
            IF   ( (Val.Ptype=Simulation_Double) AND
                  (Variables_Table.Entries(Left_Var).
                     Simulation.
                     Ptype=Simulation_Array_Double) )

                  THEN
               Variables_Table.Entries(Left_Var).Simulation.
                  Double_Table_Value(Simulations_Range(
                     Index.
                     Integer_Value)):=
                  Val.Double_Value;
            ELSE
               IF  ( (Val.Ptype=Simulation_Boolean) AND
                     (Variables_Table.Entries(Left_Var).
                        Simulation.
                        Ptype=Simulation_Array_Boolean) )

                     THEN

                  Variables_Table.Entries(Left_Var).Simulation.
                     Boolean_Table_Value(Simulations_Range(
                        Index.
                        Integer_Value)):=
                     Val.Boolean_Value;

               ELSE
                  IF  ( (Val.Ptype=Simulation_String) AND
                        (Variables_Table.Entries(Left_Var).
                           Simulation.
                           Ptype=Simulation_Array_String) )

                        THEN

                     Variables_Table.Entries(Left_Var).Simulation.
                        String_Table_Value(Simulations_Range(
                           Index.
                           Integer_Value)):=
                        Val.String_Value;

                  ELSE

                     IF ( (Val.Ptype=Simulation_Integer) AND
                           (Variables_Table.Entries(Left_Var).Simulation.
                              Ptype=Simulation_Time_Unit_Array_Integer) )

                           THEN
                        Variables_Table.Entries(Left_Var).Simulation.
                           Integer_Time_Unit_Table_Value(Time_Unit_Range(
                              Index.Integer_Value)):=
                           Val.Integer_Value;

                     ELSE
                        IF   ( (Val.Ptype=Simulation_Double) AND
                              (Variables_Table.Entries(Left_Var).
                                 Simulation.
                                 Ptype=Simulation_Time_Unit_Array_Double) )

                              THEN
                           Variables_Table.Entries(Left_Var).Simulation.
                              Double_Time_Unit_Table_Value(
                              Time_Unit_Range(
                                 Index.
                                 Integer_Value)):=
                              Val.Double_Value;
                        ELSE
                           IF  ( (Val.Ptype=Simulation_Boolean) AND
                                 (Variables_Table.Entries(Left_Var).
                                    Simulation.
                                    Ptype=
                                    Simulation_Time_Unit_Array_Boolean) )

                                 THEN

                              Variables_Table.Entries(Left_Var).
                                 Simulation.
                                 Boolean_Time_Unit_Table_Value(
                                 Time_Unit_Range(
                                    Index.
                                    Integer_Value)):=
                                 Val.Boolean_Value;

                           ELSE
                              IF  ( (Val.Ptype=Simulation_String) AND
                                    (Variables_Table.Entries(Left_Var).
                                       Simulation.
                                       Ptype=
                                       Simulation_Time_Unit_Array_String) )

                                    THEN

                                 Variables_Table.Entries(Left_Var).
                                    Simulation.
                                    String_Time_Unit_Table_Value(
                                    Time_Unit_Range(
                                       Index.
                                       Integer_Value)):=
                                    Val.String_Value;
                              ELSE


                                 Raise_Exception(Type_Error'Identity,
                                    "Affectation statement, line " &
                                    Current.Line_Number'Img &
                                    To_String(Lb_Comma &
                                       Lb_Uncompatible_Type_Error(
                                          Current_Language) ));
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;

         Free(Index);
         Free(Val);


      END IF;


      Next:=Current.Next_Statement;



   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);


   END Affectation_Dispatch;




   PROCEDURE Put_Dispatch (
         Current         : IN     Put_Statement_Ptr;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Expr_Simu_Ptr : Simulation_Value_Ptr;
      From_Simu_Ptr : Simulation_Value_Ptr;
      To_Simu_Ptr   : Simulation_Value_Ptr;

   BEGIN

      IF (Current.Put_From/=NULL)
            THEN
         From_Simu_Ptr := Value_Of (Current.Put_From.All,
            Variables_Table);
      END IF;
      IF (Current.Put_To/=NULL)
            THEN
         To_Simu_Ptr := Value_Of (Current.Put_To.All,
            Variables_Table);
      END IF;
      Expr_Simu_Ptr := Value_Of (Current.Expression_To_Be_Displayed.All,
         Variables_Table);


      IF (Current.Put_To/=NULL)
            THEN
         IF (To_Simu_Ptr.Ptype/=Simulation_Integer)
               OR (From_Simu_Ptr.Ptype/=Simulation_Integer)
               THEN
            Raise_Exception(Type_Error'Identity,
               "Put parameter should be an integer, line " &
               Current.Line_Number'Img &
               To_String(Lb_Comma &
                  Lb_Uncompatible_Type_Error(
                     Current_Language) ));
         END IF;
      END IF;

      Msg:=Msg & To_Unbounded_String("- Line " & Current.Line_Number'Img &
         ", ");



      IF (Current.Put_To/=NULL) AND (Current.Put_From/=NULL)
            THEN
         Msg:=Msg & To_Unbounded_String(
            Current.Expression_To_Be_Displayed.All) & "=" &
            To_Unbounded_String(Expr_Simu_Ptr, From_Simu_Ptr.Integer_Value,
            To_Simu_Ptr.Integer_Value) & Unbounded_Lf;

         Free(From_Simu_Ptr);
         Free(To_Simu_Ptr);

      ELSE
         IF ((Expr_Simu_Ptr.Ptype=Simulation_Time_Unit_Array_Boolean)
               OR (Expr_Simu_Ptr.Ptype=Simulation_Time_Unit_Array_Integer)
               OR (Expr_Simu_Ptr.Ptype=Simulation_Time_Unit_Array_Double)
               OR (Expr_Simu_Ptr.Ptype=Simulation_Time_Unit_Array_String))
               THEN
            Msg:=Msg & To_Unbounded_String(
               Current.Expression_To_Be_Displayed.All) & "=" &
               To_Unbounded_String(Expr_Simu_Ptr, Integer(Time_Unit_Range'
                  First),
               Integer(Time_Unit_Range'Last)) & Unbounded_Lf;
         ELSE
            IF ((Expr_Simu_Ptr.Ptype=Simulation_Array_Boolean)
                  OR (Expr_Simu_Ptr.Ptype=Simulation_Array_Integer)
                  OR (Expr_Simu_Ptr.Ptype=Simulation_Array_String)
                  OR (Expr_Simu_Ptr.Ptype=Simulation_Array_Double))
                  THEN
               Msg:=Msg & To_Unbounded_String(
                  Current.Expression_To_Be_Displayed.All) & "=" &
                  To_Unbounded_String(Expr_Simu_Ptr, Integer(
                     Simulations_Range'
                     First), Integer(Simulations_Range'Last)) &
                  Unbounded_Lf;
            ELSE
               Msg:=Msg & To_Unbounded_String(
                  Current.Expression_To_Be_Displayed.All) & "=" &
                  To_Unbounded_String(Expr_Simu_Ptr) & Unbounded_Lf;
            END IF;
         END IF;
      END IF;


      Free(Expr_Simu_Ptr);


      Next:=Current.Next_Statement;


   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);

   END Put_Dispatch;



   PROCEDURE While_Dispatch (
         Current         : IN     While_Statement_Ptr;
         Current_Section : IN     Sections;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Simu_Ptr : Simulation_Value_Ptr;

   BEGIN


      -- Let's run included statements :
      -- do a loop iteration
      --
      LOOP

         Simu_Ptr := Value_Of (Current.Condition.All,
            Variables_Table);
         IF Simu_Ptr.Ptype/=Simulation_Boolean
               THEN
            Raise_Exception (Type_Error'Identity, "while statement, line " &
               Current.Line_Number'Img & ", " & To_String (
                  Lb_Uncompatible_Type_Error(Current_Language)));
         END IF;

         EXIT WHEN Simu_Ptr.Boolean_Value = False;

         Dispatch(Current.Included_Statement,
            Current_Section,  Variables_Table, Msg);

      END LOOP;


      Free(Simu_Ptr);

      Next:=Statement_Ptr(Current.Next_Statement);



   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);


   END While_Dispatch;


   PROCEDURE For_Dispatch (
         Current         : IN     For_Statement_Ptr;
         Current_Section : IN     Sections;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String;
         Next            : IN OUT Statement_Ptr) IS

      Var            : Variables_Range;
      Var_Loop_Index : Variables_Range;


   BEGIN


      -- Test is the expression included in the for
      -- statement is a variable ... raise a exception
      -- in the other cases
      --
      IF Current.For_Index.Expression_Type=Variable_Type
            THEN
         IF Current.For_Index.Variable_Type/=Simulation_Integer
               THEN
            Raise_Exception (Type_Error'Identity,
               "for statement, line " & Current.Line_Number'Img &
               ", for expression have to be an integer variable");
         END IF;
      ELSE
         Raise_Exception (Type_Error'Identity, "for statement, line " &
            Current.Line_Number'Img &
            ", for expression have to be an integer variable");
      END IF;


      -- Let's run included statements :
      -- do a loop iteration
      --

      CASE Current.For_Type IS
         WHEN Task_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_tasks");
         WHEN Processor_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_processors");
         WHEN Buffer_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_buffers");
         WHEN Time_Unit_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_time_units");
         WHEN Resource_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_resources");
         WHEN Message_Table_Type =>
            Var_Loop_Index:=Find_Variable(Variables_Table,
               "nb_messages");
      END CASE;


      FOR I IN 0..Natural(Variables_Table.Entries(Var_Loop_Index).
            Simulation.Integer_Value-1)  LOOP

         Var:=Find_Variable(Variables_Table,
            Variable_Expr_Ptr(Current.For_Index).Name, Current.Line_Number);
         Variables_Table.Entries(Var).Simulation.
            Integer_Value:=Integer(I);

         Dispatch(Current.Included_Statement,
            Current_Section, Variables_Table,Msg );

      END LOOP;


      Next:=Statement_Ptr(Current.Next_Statement);




   EXCEPTION
      WHEN Constraint_Error  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN STORAGE_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);
      WHEN PROGRAM_ERROR  =>
         Raise_Exception (Statement_Error'Identity, "line " &
            Current.Line_Number'Img & " ; Exception raised :"
            & Exception_Name & ":" & Exception_Message);

   END For_Dispatch;



   PROCEDURE Dispatch (
         Current         : IN     Statement_Ptr;
         Current_Section : IN     Sections;
         Variables_Table : IN OUT Variables_Table_Type;
         Msg             : IN OUT Unbounded_String) IS

      Next       : Statement_Ptr;
      My_Current : Statement_Ptr := Current;

   BEGIN


      IF My_Current/=NULL THEN
         LOOP


            -- Run the associated statement
            --
            CASE My_Current.Statement_Type IS

               WHEN Put_Statement_Type =>
                  Put_Dispatch(
                     Put_Statement_Ptr(My_Current),
                     Variables_Table,
                     Msg, Next);

               WHEN 	Nop_Statement_Type =>
                  Next:=My_Current.Next_Statement;

               WHEN       If_Statement_Type =>
                  If_Dispatch(
                     If_Statement_Ptr(My_Current),
                     Current_Section, Variables_Table, Msg, Next);

               WHEN   Random_Initialize_Statement_Type =>
                  Random_Initialize_Dispatch(
                     Random_Initialize_Statement_Ptr(
                        My_Current), Variables_Table, Msg, Next);

               WHEN   Affectation_Statement_Type =>
                  Affectation_Dispatch(Affectation_Statement_Ptr(
                        My_Current), Variables_Table, Msg, Next);

               WHEN       For_Statement_Type =>
                  For_Dispatch(
                     For_Statement_Ptr(
                        My_Current), Current_Section, Variables_Table,
                     Msg, Next);

               WHEN   Return_Statement_Type =>
                  -- Check if a return statement is called
                  --
                  IF (Return_Statement_Ptr(My_Current).Return_Value /= NULL)
                        AND (Current_Section /= Election)  THEN
                     Raise_Exception (Type_Error'Identity,
                        Current_Section'Img & " section, line " &
                        Current.
                        Line_Number' Img &
                        ", invalid return value from return statement");
                  END IF;
                  EXIT;

               WHEN     While_Statement_Type =>
                  While_Dispatch(
                     While_Statement_Ptr(
                        My_Current), Current_Section,Variables_Table,
                     Msg, Next);

               WHEN  Set_Statement_Type =>
                  NULL;

               WHEN  Subprogram_Statement_Type =>
                  NULL;

               WHEN  Subprogram_Call_Statement_Type =>
                  NULL;

            END CASE;


            -- Last statement
            --
            EXIT WHEN Next=NULL;


            My_Current:=Next;

         END LOOP;
      END IF;

   END Dispatch;


END Statements;

