------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Text_Io;
USE Text_Io;



PACKAGE BODY Processors IS



   PROCEDURE Initialize (
         My_Processor : IN OUT Processor) IS
   BEGIN
      My_Processor.Object_Type:=Processor_Object_Type;
   END Initialize;


   PROCEDURE Put_Name (
         A_Processor : IN     Processor_Ptr) IS
   BEGIN
      Put(To_String(A_Processor.Name));
   END Put_Name;

   PROCEDURE Put (
         A_Processor : IN     Processor_Ptr) IS
   BEGIN
      Put(A_Processor.All);
   END Put;


   PROCEDURE Put (
         My_Processor : IN     Processor) IS
   BEGIN
      Put_Line ("Processor name : " & To_String(My_Processor.Name) );
      Put(My_Processor.Scheduler.All);
   END Put;


   PROCEDURE Set_Scheduler (
         My_Processor : IN OUT Processor;
         Sched        : IN     Scheduler_Ptr) IS
   BEGIN
      My_Processor.Scheduler:=Sched;
   END Set_Scheduler;


   FUNCTION Get_Scheduler (
         My_Processor : IN     Processor)
     RETURN Scheduler_Ptr IS
   BEGIN
      RETURN My_Processor.Scheduler;
   END Get_Scheduler;


   PROCEDURE Set_Name (
         My_Processor : IN OUT Processor;
         Name         : IN     Unbounded_String) IS
   BEGIN
      My_Processor.Name:=Name;
   END Set_Name;


   FUNCTION Get_Name (
         My_Processor : IN     Processor)
     RETURN Unbounded_String IS
   BEGIN
      RETURN My_Processor.Name;
   END Get_Name;


   FUNCTION Copy (
         My_Processor : IN     Processor_Ptr)
     RETURN Processor_Ptr IS

      Ptr : Processor_Ptr;

   BEGIN

      Ptr:=NEW Processor;
      Ptr.All:=My_Processor.All;
      Ptr.Scheduler:=Copy(My_Processor.Scheduler.All);

      RETURN Ptr;

   END Copy;




END Processors;

