------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------





WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Scheduler;
USE Scheduler;

WITH Objects;
USE Objects;



WITH Unchecked_Deallocation;


PACKAGE Processors IS


   TYPE Processor IS NEW Generic_Object WITH
      RECORD
         Scheduler : Scheduler_Ptr;
         Network   : Unbounded_String;
      END RECORD;

   TYPE Processor_Ptr IS ACCESS ALL Processor'Class;



   PROCEDURE Initialize (
         My_Processor : IN OUT Processor);


   PROCEDURE Put (
         My_Processor : IN     Processor);


   PROCEDURE Put_Name (
         A_Processor : IN     Processor_Ptr);


   PROCEDURE Put (
         A_Processor : IN     Processor_Ptr);


   PROCEDURE Set_Name (
         My_Processor : IN OUT Processor;
         Name         : IN     Unbounded_String);


   FUNCTION Get_Name (
         My_Processor : IN     Processor)
     RETURN Unbounded_String;


   FUNCTION Get_Scheduler (
         My_Processor : IN     Processor)
     RETURN Scheduler_Ptr;


   PROCEDURE Set_Scheduler (
         My_Processor : IN OUT Processor;
         Sched        : IN     Scheduler_Ptr);


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Processor'Class, Processor_Ptr);


   FUNCTION Copy (
         My_Processor : IN     Processor_Ptr)
     RETURN Processor_Ptr;


END Processors;











