------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE BODY Qs_Tools IS

   FUNCTION Get_Rand_Parameter (
         Min  : IN     Natural;
         Max  : IN     Natural;
         Seed : IN     Generator)
     RETURN Natural IS

      Aux : Double;
      Res : Natural;

   BEGIN
      Aux := Double(Random(Seed)) * (Double(Max) - Double(Min)) + Double(
         Min);
      Res := Natural(Aux);

      IF (Res<Min) OR (Res>Max) THEN
         RAISE Constraint_Error;
      END IF;

      RETURN Res;
   END Get_Rand_Parameter;

   FUNCTION Get_Rand_Parameter (
         Min  : IN     Double;
         Max  : IN     Double;
         Seed : IN     Generator)
     RETURN Double IS

      Res : Double := 0.0;

   BEGIN
      Res := Double(Random(Seed)) * (Max - Min) + Min;

      IF (Res<Min) OR (Res>Max) THEN
         RAISE Constraint_Error;
      END IF;

      RETURN Res;
   END Get_Rand_Parameter;



   FUNCTION Get_Exponential_Time (
         Mtbe : IN     Double;
         Seed : IN     Generator)
     RETURN Double IS
      Result : Double;
   BEGIN

      Result := - Mtbe * Log(Double(Random(Seed)));

      RETURN(Result);
   END Get_Exponential_Time;

END Qs_Tools;
