------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE BODY Queueing_System.Theoritical.Mg1 IS

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Mg1_Queueing_System_Theoritical) IS
   BEGIN
      Initialize(Generic_Queueing_System_Theoritical(A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mg1;
   END Initialize;

   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Mg1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Arrival_Rate     : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries
        (0).Data;
      Avg_Service_Time : CONSTANT Double := A_Queueing_System.Avg_Service;
      Var_Service_Time : CONSTANT Double := A_Queueing_System.Var_Service;
      Taw              :          Double := 0.0;
   BEGIN
      Taw := Avg_Service_Time + (Arrival_Rate * Avg_Service_Time *
         Avg_Service_Time + Arrival_Rate* Var_Service_Time)
         /(2.0*(1.0 - Arrival_Rate * Avg_Service_Time));

      Result := Taw;
   END Qs_Average_Waiting_Time;

   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Mg1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Arrival_Rate     : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries
        (0).Data;
      Avg_Service_Time : CONSTANT Double := A_Queueing_System.Avg_Service;
      Var_Service_Time : CONSTANT Double := A_Queueing_System.Var_Service;
      Tao              :          Double := 0.0;
   BEGIN
      Tao:= Avg_Service_Time + (Arrival_Rate * Avg_Service_Time *
         Avg_Service_Time + Arrival_Rate* Var_Service_Time)
         /(2.0*(1.0 - Arrival_Rate * Avg_Service_Time));
      Tao := Tao * Arrival_Rate;

      Result := Tao;
   END Qs_Average_Number_Customer;


   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Mg1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      Result := 0.0;
      --      raise Not_Implemented;
   END Qs_Maximum_Number_Customer;

   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Mg1_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double IS
   BEGIN
      RAISE Not_Implemented;
      RETURN 0.0;
   END Get_Probability_Of_State;

   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Mg1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      Result := 0.0;
      --      raise Not_Implemented;
   END Qs_Maximum_Waiting_Time;

END Queueing_System.Theoritical.Mg1;



