------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;


PACKAGE BODY Queueing_System.Theoritical.Mm1 IS


   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Mm1_Queueing_System_Theoritical) IS
   BEGIN
      Initialize(Generic_Queueing_System_Theoritical(A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mm1;
   END Initialize;

   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Mm1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Arrival_Rate : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries
        (0).Data;
      Service_Rate : CONSTANT Double := A_Queueing_System.Service_Rate.Entries
        (0).Data;
   BEGIN
      Result :=  1.0/ (Service_Rate * (1.0 - Arrival_Rate /
            Service_Rate));
   END Qs_Average_Waiting_Time;

   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Mm1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Rho : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries (0).Data
        / A_Queueing_System.Service_Rate.Entries (0).Data;
   BEGIN
      Result := Rho / (1.0 - Rho);
   END Qs_Average_Number_Customer;

   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Mm1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      Result := 0.0;
      --      raise Not_Implemented;
   END Qs_Maximum_Number_Customer;


   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Mm1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      Result := 0.0;
      --      raise Not_Implemented;
   END Qs_Maximum_Waiting_Time;

   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Mm1_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double IS

      Rau    : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries
        (0).Data / A_Queueing_System.Service_Rate.Entries (0).Data;
      Result :          Double;
   BEGIN
      Result := Pow(Rau,Double(N));
      Result := Result * (1.0 - Rau);

      RETURN (Result);
   END Get_Probability_Of_State;

END Queueing_System.Theoritical.Mm1;



