------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE BODY Queueing_System.Theoritical.Mms IS

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Mms_Queueing_System_Theoritical) IS
   BEGIN
      Initialize(Generic_Queueing_System_Theoritical(A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mms;
   END Initialize;

   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Arrival_Rate : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries
        (0).Data;
      Service_Rate : CONSTANT Double := A_Queueing_System.Service_Rate.Entries
        (0).Data;
   BEGIN
      Result := 1.0 / (Service_Rate - Arrival_Rate) - (1.0 / Service_Rate);
   END Qs_Average_Waiting_Time;

   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Rau : CONSTANT Double := A_Queueing_System.Arrival_Rate.Entries (0).Data
        / A_Queueing_System.Service_Rate.Entries (0).Data;
   BEGIN
      Result := Rau / (1.0 - Rau);
   END Qs_Average_Number_Customer;

   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      RAISE Not_Implemented;
   END Qs_Maximum_Waiting_Time;

   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      RAISE Not_Implemented;
   END Qs_Maximum_Number_Customer;

   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double IS
      Utilisation : Double;
      Result      : Double;
   BEGIN
      Utilisation := A_Queueing_System.Arrival_Rate.Entries(0).Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;
      Result := Pow(Utilisation,Double(N));
      Result := Result * (1.0 - Utilisation);
      RETURN (Result);
   END;

   FUNCTION Get_Qs_Nb_Tasks (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical)
     RETURN Double IS
   BEGIN
      RETURN  A_Queueing_System.Nb_Tasks;
   END;

   PROCEDURE Set_Qs_Nb_Tasks (
         A_Queueing_System : IN OUT Mms_Queueing_System_Theoritical;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Nb_Tasks := Value;
   END;

   FUNCTION Get_Probability_Of_Queueing (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical)
     RETURN Double IS
      Utilisation  :          Double;
      Adivs        :          Double;
      Upper_Result :          Double := 0.0;
      Lower_Result :          Double := 0.0;
      Lambda       : CONSTANT Double := 0.0;

      FUNCTION Factorial (
            Value : IN     Double)
        RETURN Double IS
         N      : Double := Value;
         Result : Double := Value;
      BEGIN
         WHILE (N > 1.0) LOOP
            --            Put(" " & N'Img);
            N := N - 1.0;
            Result := Result * N;
         END LOOP;
         RETURN Result;
      END Factorial;

   BEGIN

      Utilisation := A_Queueing_System.Arrival_Rate.Entries(0).
         Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;

      Adivs :=  Lambda /
         A_Queueing_System.Service_Rate.Entries(0).Data /
         A_Queueing_System.Nb_Tasks;

      Upper_Result := (1.0 / Factorial(A_Queueing_System.Nb_Tasks)) *
         (Pow(Adivs,A_Queueing_System.Nb_Tasks)) *
         (1.0 / (1.0 - Utilisation));

      Lower_Result := Lower_Result + 1.0;
      FOR N IN 1 .. Natural(A_Queueing_System.Nb_Tasks)-1 LOOP
         Lower_Result := Lower_Result +
            (1.0 / Factorial(Double(N)) * Pow(Adivs,Double(N)));
      END LOOP;

      Lower_Result := Lower_Result +
         (1.0 / Factorial(A_Queueing_System.Nb_Tasks)) *
         (Pow(Adivs,A_Queueing_System.Nb_Tasks)) *
         (1.0 / (1.0 - Utilisation));

      RETURN (Upper_Result / Lower_Result);
   END;

END Queueing_System.Theoritical.Mms;



