------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE Queueing_System.Theoritical.Mms IS

   TYPE Mms_Queueing_System_Theoritical IS NEW Generic_Queueing_System_Theoritical WITH
      RECORD
         Nb_Tasks : Double;
      END RECORD;
   TYPE Mms_Queueing_System_Theoritical_Ptr IS ACCESS ALL Generic_Queueing_System_Theoritical'Class;

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Mms_Queueing_System_Theoritical);

   -- return the theoric average waiting time of one customer in the queue
   --
   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double);

   -- return the theoric average number of customer in the queue
   --
   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double);


   -- return the theoric maximum waiting time of one customer in the queue
   --
   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double);

   -- return the theoric maximum number of customer in the queue
   --
   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         Result            : IN OUT Double);

   -- return the theoric probability to have N customer in the qs
   --
   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double;

   -- return the number of tasks for mms queueing system
   --
   FUNCTION Get_Qs_Nb_Tasks (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical)
     RETURN Double;

   -- set the number of tasks for mms queueing system
   --
   PROCEDURE Set_Qs_Nb_Tasks (
         A_Queueing_System : IN OUT Mms_Queueing_System_Theoritical;
         Value             :        Double);

   --P[queueing], Eq. 2.148, Robertazzi
   --
   FUNCTION Get_Probability_Of_Queueing (
         A_Queueing_System : IN     Mms_Queueing_System_Theoritical)
     RETURN Double;

END Queueing_System.Theoritical.Mms;
