------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE BODY Queueing_System.Theoritical.Pp1 IS

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Pp1_Queueing_System_Theoritical) IS
   BEGIN
      Initialize(Generic_Queueing_System_Theoritical(A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Pp1;
   END Initialize;

   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Pp1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Service_Rate : CONSTANT Double := A_Queueing_System.Service_Rate.Entries
        (0).Data;
      Nb_Arrival   : CONSTANT Double := A_Queueing_System.Nb_Arrival;
   BEGIN
      IF A_Queueing_System.Harmonic  THEN
         Result := (2.0 * Nb_Arrival) * (1.0/Service_Rate);
      ELSE
         Result := (2.0 * Nb_Arrival + 1.0) * (1.0/Service_Rate);
      END IF;
   END Qs_Maximum_Waiting_Time;

   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Pp1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS

      Nb_Arrival : CONSTANT Double  := A_Queueing_System.Nb_Arrival;
      Harmonic   : CONSTANT Boolean := A_Queueing_System.Harmonic;
   BEGIN
      IF Harmonic  THEN
         Result := 2.0 * Nb_Arrival;
      ELSE
         Result := 2.0 * Nb_Arrival + 1.0;
      END IF;
   END Qs_Maximum_Number_Customer;


   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Pp1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      RAISE Not_Implemented;
   END Qs_Average_Waiting_Time;

   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Pp1_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   BEGIN
      RAISE Not_Implemented;
   END Qs_Average_Number_Customer;

   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Pp1_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double IS
   BEGIN
      RAISE Not_Implemented;
      RETURN 0.0;
   END Get_Probability_Of_State;

END Queueing_System.Theoritical.Pp1;



