------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Text_Io;
USE Text_Io;

PACKAGE BODY Queueing_System.Theoritical IS

   PROCEDURE Theoritical_Results (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical'Class;
         Message           : IN     Unbounded_String) IS

      Awt,
      Anc,
      Mwt,
      Mnc : Double := 0.0;
   BEGIN
      -- criteria computation
      Qs_Average_Waiting_Time(A_Queueing_System,Awt);
      Qs_Average_Number_Customer (A_Queueing_System,Anc);
      Qs_Maximum_Waiting_Time (A_Queueing_System,Mwt);
      Qs_Maximum_Number_Customer (A_Queueing_System,Mnc);

      -- criteria display
      Put_Line("Theoritical Results");
      New_Line;
      Put_Line("AW" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Awt'Img);
      Put_Line("AO" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Anc'Img);
      Put_Line("MW" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Mwt'Img);
      Put_Line("MO" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Mnc'Img);
      New_Line;
   END Theoritical_Results ;

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Generic_Queueing_System_Theoritical) IS

   BEGIN
      Initialize(Generic_Queueing_System(A_Queueing_System));
   END Initialize;

   PROCEDURE Reset (
         A_Queueing_System : IN OUT Generic_Queueing_System_Theoritical) IS

   BEGIN
      Initialize (Generic_Queueing_System_Theoritical(A_Queueing_System));
   END Reset;

   FUNCTION Get_Probability_Density (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical'Class;
         N                 : IN     Natural                                   := 0)
     RETURN Double IS
      State  : Natural;
      Result : Double  := 0.0;
   BEGIN
      State := N;
      IF State > A_Queueing_System.Max_State THEN
         RETURN (-1.0);
      END IF;
      IF State=0 THEN
         State := A_Queueing_System.Max_State;
      END IF;

      FOR I IN 0 .. State LOOP
         Result := Result + Get_Probability_Of_State(A_Queueing_System,I);
      END LOOP;

      RETURN Result;
   END;

   FUNCTION Get_Probability_Of_Full_Buffer (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double IS
   BEGIN
      RAISE Not_Implemented;
      RETURN 0.0;
   END;

   FUNCTION Get_Probability_Of_Task_Block (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double IS
   BEGIN
      RAISE Not_Implemented;
      RETURN 0.0;
   END;

   FUNCTION Get_Probability_Of_Queueing (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double IS
   BEGIN
      RAISE Not_Implemented;
      RETURN 0.0;
   END;

END Queueing_System.Theoritical;
