------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

PACKAGE Queueing_System.Theoritical IS

   TYPE Generic_Queueing_System_Theoritical IS ABSTRACT NEW Generic_Queueing_System WITH NULL RECORD;
   TYPE Queueing_System_Theoritical_Ptr IS ACCESS ALL Generic_Queueing_System_Theoritical'Class;


   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Generic_Queueing_System_Theoritical);

   -- reset caracteristics of the qs (has to be done each time a qs is used)
   --
   PROCEDURE Reset (
         A_Queueing_System : IN OUT Generic_Queueing_System_Theoritical);

   -- return the theoric average waiting time of one customer in the queue
   -- based on theoric service time
   --
   PROCEDURE Qs_Average_Waiting_Time (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   ABSTRACT;

   -- return the theoric average number of customer in the queue
   -- based on theoric service time
   --
   PROCEDURE Qs_Average_Number_Customer (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   ABSTRACT;

   -- return the theoric maximum waiting time of one customer in the queue
   --
   PROCEDURE Qs_Maximum_Waiting_Time (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   ABSTRACT;

   -- return the theoric maximum number of customer in the queue
   --
   PROCEDURE Qs_Maximum_Number_Customer (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical;
         Result            : IN OUT Double) IS
   ABSTRACT;

   -- return theoritical results on qs (waiting time, number of customer)
   --
   PROCEDURE Theoritical_Results (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical'Class;
         Message           : IN     Unbounded_String);

   -- return the probability to be in a certain state
   --
   FUNCTION Get_Probability_Of_State (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical;
         N                 : IN     Natural)
     RETURN Double IS
   ABSTRACT;

   -- return the probability of buffer will be full
   --
   FUNCTION Get_Probability_Of_Full_Buffer (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double;

   -- return the probability of all tasks are busy
   -- Eq. 2.153, Robertazzi: Queueing theory
   --
   FUNCTION Get_Probability_Of_Task_Block (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double;

   --P[queueing], Eq. 2.148, Robertazzi
   --
   FUNCTION Get_Probability_Of_Queueing (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical)
     RETURN Double;

   --Get probability density of states from 0->N
   --
   FUNCTION Get_Probability_Density (
         A_Queueing_System : IN     Generic_Queueing_System_Theoritical'Class;
         N                 : IN     Natural                                   := 0)
     RETURN Double;

END Queueing_System.Theoritical;




