------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Text_Io;
USE Text_Io;

WITH Natural_Util;
USE Natural_Util;

WITH Double_Util;
USE Double_Util;

PACKAGE BODY Queueing_System IS




   PROCEDURE Set_Constant_Consumer_Response_Time (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Response_Time     : IN     Double) IS
   BEGIN
      A_Queueing_System.Consumer_Resp_Time.Entries(0).Data:=Response_Time;
      A_Queueing_System.Consumer_Resp_Time.Nb_Entries:=1;
   END Set_Constant_Consumer_Response_Time;




   FUNCTION Qs_To_Display_String (
         A_Qs : IN     Queueing_Systems_Type)
     RETURN Unbounded_String IS
      S : Unbounded_String;
   BEGIN

      CASE A_Qs IS
         WHEN Qs_Pp1 =>
            S:=To_Unbounded_String("P/P/1");
         WHEN     Qs_Mm1 =>
            S:=To_Unbounded_String("M/M/1");
         WHEN  Qs_Md1 =>
            S:=To_Unbounded_String("M/D/1");
         WHEN Qs_Mp1   =>
            S:=To_Unbounded_String("M/P/1");
         WHEN         Qs_Mg1   =>
            S:=To_Unbounded_String("M/G/1");
         WHEN     Qs_Mms  =>
            S:=To_Unbounded_String("M/M/s");
         WHEN   Qs_Mds  =>
            S:=To_Unbounded_String("M/D/s");
         WHEN          Qs_Mps  =>
            S:=To_Unbounded_String("M/P/s");
         WHEN        Qs_Mgs   =>
            S:=To_Unbounded_String("M/G/s");
         WHEN      Qs_Mm1n  =>
            S:=To_Unbounded_String("M/M/1/n");
         WHEN       Qs_Md1n  =>
            S:=To_Unbounded_String("M/D/1/n");
         WHEN       Qs_Mp1n =>
            S:=To_Unbounded_String("M/P/1/n");
         WHEN      Qs_Mg1n =>
            S:=To_Unbounded_String("M/G/1/n");
         WHEN          Qs_Mmsn  =>
            S:=To_Unbounded_String("M/M/s/n");
         WHEN      Qs_Mdsn =>
            S:=To_Unbounded_String("M/D/s/n") ;
         WHEN    Qs_Mpsn  =>
            S:=To_Unbounded_String("M/P/s/n");
         WHEN       Qs_Mgsn =>
            S:=To_Unbounded_String("M/G/s/n");
      END CASE;
      RETURN S;
   END Qs_To_Display_String;

   PROCEDURE Display_String_To_Qs (
         From : IN     Unbounded_String;
         To   :    OUT Queueing_Systems_Type;
         Ok   :    OUT Boolean) IS
   BEGIN
      Ok:=False;

      IF From=To_Unbounded_String("M/M/1")
            THEN
         To:=Qs_Mm1;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/D/1")
            THEN
         To:=Qs_Md1;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/P/1")
            THEN
         To:=Qs_Mp1;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/G/1")
            THEN
         To:=Qs_Mg1;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/M/s")
            THEN
         To:=Qs_Mms;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/D/s")
            THEN
         To:=Qs_Mds;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/P/s")
            THEN
         To:=Qs_Mps;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/G/s")
            THEN
         To:=Qs_Mgs;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/M/1/n")
            THEN
         To:=Qs_Mm1n;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/D/1/n")
            THEN
         To:=Qs_Md1n;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/P/1/n")
            THEN
         To:=Qs_Mp1n;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/G/1/n")
            THEN
         To:=Qs_Mg1n;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/M/s/n")
            THEN
         To:=Qs_Mmsn;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/D/s/n")
            THEN
         To:=Qs_Mdsn;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/P/s/n")
            THEN
         To:=Qs_Mpsn;
         Ok:=True;
      END IF;

      IF From=To_Unbounded_String("M/G/s/n")
            THEN
         Ok:=True;
         To:=Qs_Mgsn;
      END IF;

   END Display_String_To_Qs;


   PROCEDURE Set_Qs_Type (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Qs_Type           : IN     Queueing_Systems_Type) IS
   BEGIN
      A_Queueing_System.Queueing_System_Type := Qs_Type;
   END Set_Qs_Type;

   FUNCTION Get_Qs_Type (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Queueing_Systems_Type IS
   BEGIN
      RETURN A_Queueing_System.Queueing_System_Type;
   END Get_Qs_Type;

   PROCEDURE Set_Qs_Arrival_Rate (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Arrival_Rate.
         Entries(A_Queueing_System.Arrival_Rate.Nb_Entries).
         Data :=
         Value;
      A_Queueing_System.Arrival_Rate.Nb_Entries :=
         A_Queueing_System.Arrival_Rate.Nb_Entries + 1;
   END Set_Qs_Arrival_Rate;

   PROCEDURE Set_Qs_Arrival_Rate (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Place             :        Arrival_Rate_Range;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Arrival_Rate.
         Entries(Place).Data := Value;
   END Set_Qs_Arrival_Rate;

   FUNCTION Get_Qs_Arrival_Rate (
         A_Queueing_System : IN     Generic_Queueing_System'Class;
         Place             :        Arrival_Rate_Range)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Arrival_Rate.Entries(Place).
         Data;
   END Get_Qs_Arrival_Rate;

   PROCEDURE Set_Qs_Service_Rate (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Service_Rate.
         Entries(A_Queueing_System.Service_Rate.Nb_Entries).
         Data :=
         Value;
      A_Queueing_System.Service_Rate.Nb_Entries :=
         A_Queueing_System.Service_Rate.Nb_Entries + 1;
   END Set_Qs_Service_Rate;

   PROCEDURE Set_Qs_Service_Rate (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Place             :        Service_Rate_Range;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Service_Rate.
         Entries(Place).Data := Value;
   END Set_Qs_Service_Rate;

   FUNCTION Get_Qs_Service_Rate (
         A_Queueing_System : IN     Generic_Queueing_System'Class;
         Place             :        Service_Rate_Range)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Service_Rate.Entries(Place).
         Data;
   END;

   PROCEDURE Set_Qs_Utilisation (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Utilisation := Value;
   END;

   FUNCTION Get_Qs_Utilisation (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Utilisation;
   END;


   PROCEDURE Set_Qs_Nb_Of_Messages (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Nb_Of_Messages := Value;
   END;

   FUNCTION Get_Qs_Nb_Of_Messages (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Nb_Of_Messages;
   END;

   PROCEDURE Set_Qs_Waiting_Time (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS
   BEGIN
      A_Queueing_System.Waiting_Time := Value;
   END;

   FUNCTION Get_Qs_Waiting_Time (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Waiting_Time;
   END;

   PROCEDURE Set_Qs_Max_State (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Natural) IS
   BEGIN
      A_Queueing_System.Max_State := Value;
   END;

   FUNCTION Get_Qs_Max_State (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Natural IS
   BEGIN
      RETURN A_Queueing_System.Max_State;
   END;

   PROCEDURE Set_Qs_Nb_Arrival (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS

   BEGIN
      A_Queueing_System.Nb_Arrival := Value;
   END;

   FUNCTION Get_Qs_Nb_Arrival (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Nb_Arrival;
   END;


   PROCEDURE Set_Qs_Nb_Server (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Double) IS

   BEGIN
      A_Queueing_System.Nb_Server := Value;
   END;


   FUNCTION Get_Qs_Nb_Server (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Double IS
   BEGIN
      RETURN A_Queueing_System.Nb_Server;
   END;


   PROCEDURE Set_Qs_Harmonic (
         A_Queueing_System : IN OUT Generic_Queueing_System'Class;
         Value             :        Boolean) IS

   BEGIN
      A_Queueing_System.Harmonic := Value;
   END;


   FUNCTION Get_Qs_Harmonic (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Boolean IS
   BEGIN
      RETURN A_Queueing_System.Harmonic;
   END;

   PROCEDURE Initialize (
         A_Queueing_System : IN OUT Generic_Queueing_System) IS
   BEGIN
      Initialize(A_Queueing_System.Arrival_Rate);
      Initialize(A_Queueing_System.Service_Rate);
      A_Queueing_System.Nb_Of_Messages := 0.0;
      A_Queueing_System.Waiting_Time := 0.0;
      A_Queueing_System.Utilisation := 0.0;
      A_Queueing_System.Max_State :=0;

      A_Queueing_System.Var_Service := 0.0;
      A_Queueing_System.Avg_Service := 0.0;

      A_Queueing_System.Nb_Arrival := 0.0;
      A_Queueing_System.Nb_Server  := 0.0;
      A_Queueing_System.Harmonic   := False;

      A_Queueing_System.Consumer_Resp_Time := NEW
         Resp_Time_Consumer_Table;
      A_Queueing_System.Consumer_Resp_Time_Cpt := 0;
   END Initialize;

   PROCEDURE Reset (
         A_Queueing_System : IN OUT Generic_Queueing_System) IS
   BEGIN
      Initialize(A_Queueing_System);
   END Reset;

   PROCEDURE Put (
         A_Queueing_System : IN     Generic_Queueing_System) IS


      Arrival_Period : CONSTANT Double := 1.0 / A_Queueing_System.Arrival_Rate.Entries
        (0).Data;
      Service_Period : CONSTANT Double := 1.0 / A_Queueing_System.Service_Rate.Entries
        (0).Data;
      Rho            : CONSTANT Double := A_Queueing_System.Utilisation;

      PACKAGE Double_Io IS NEW Text_Io.Float_Io(Double);
      USE Double_Io;

   BEGIN
      Put_Line("# Name " & To_String(Get_Name(A_Queueing_System)));

      Put("# Arrival rate (lambda) = 1/");
      Put(Arrival_Period, Aft=>4, Exp => 0);
      New_Line;

      Put("# Service rate (mu) = 1/");
      Put(Service_Period, Aft=>4, Exp => 0);
      New_Line;

      Put("# Utilisation (rho) = ");
      Put(Rho, Aft=>4, Exp => 0);
      New_Line;
      New_Line;

   END Put;


   FUNCTION Get_Name (
         A_Queueing_System : IN     Generic_Queueing_System'Class)
     RETURN Unbounded_String IS

      Result : Unbounded_String;
   BEGIN
      IF A_Queueing_System.Queueing_System_Type = Qs_Mm1 THEN
         Result := To_Unbounded_String("MM1");
      ELSE
         IF A_Queueing_System.Queueing_System_Type = Qs_Md1 THEN
            Result := To_Unbounded_String("MD1");
         ELSE
            IF A_Queueing_System.Queueing_System_Type = Qs_Mp1 THEN
               Result := To_Unbounded_String("MP1");
            ELSE
               IF A_Queueing_System.Queueing_System_Type = Qs_Mg1 THEN
                  Result := To_Unbounded_String("MG1");
               ELSE
                  IF A_Queueing_System.Queueing_System_Type = Qs_Mms THEN
                     Result := To_Unbounded_String("MMS");
                  ELSE
                     IF A_Queueing_System.Queueing_System_Type = Qs_Mds THEN
                        Result := To_Unbounded_String("MDS");
                     ELSE
                        IF A_Queueing_System.Queueing_System_Type = Qs_Mps THEN
                           Result := To_Unbounded_String("MPS");
                        ELSE
                           IF A_Queueing_System.Queueing_System_Type =
                                 Qs_Mgs THEN
                              Result := To_Unbounded_String("MGS");
                           ELSE
                              IF A_Queueing_System.Queueing_System_Type =
                                    Qs_Mm1n THEN
                                 Result := To_Unbounded_String("MM1N");
                              ELSE
                                 IF A_Queueing_System.Queueing_System_Type =
                                       Qs_Md1n THEN
                                    Result := To_Unbounded_String("MD1N");
                                 ELSE
                                    IF
                                          A_Queueing_System.Queueing_System_Type =
                                          Qs_Mp1n THEN
                                       Result := To_Unbounded_String(
                                          "MP1N");
                                    ELSE
                                       IF
                                             A_Queueing_System.Queueing_System_Type =
                                             Qs_Mg1n THEN
                                          Result := To_Unbounded_String(
                                             "MG1N");
                                       ELSE
                                          IF
                                                A_Queueing_System.Queueing_System_Type =
                                                Qs_Mmsn THEN
                                             Result := To_Unbounded_String(
                                                "MMSN");
                                          ELSE
                                             IF
                                                   A_Queueing_System.Queueing_System_Type =
                                                   Qs_Mdsn THEN
                                                Result :=
                                                   To_Unbounded_String(
                                                   "MDSN");
                                             ELSE
                                                IF
                                                      A_Queueing_System.Queueing_System_Type =
                                                      Qs_Mpsn THEN
                                                   Result :=
                                                      To_Unbounded_String(
                                                      "MPSN");
                                                ELSE
                                                   IF
                                                         A_Queueing_System.Queueing_System_Type =
                                                         Qs_Mgsn THEN
                                                      Result :=
                                                         To_Unbounded_String(
                                                         "MGSN");
                                                   ELSE
                                                      Result :=
                                                         To_Unbounded_String(
                                                         "?");
                                                   END IF;
                                                END IF;
                                             END IF;
                                          END IF;
                                       END IF;
                                    END IF;
                                 END IF;
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;

      RETURN Result;
   END Get_Name;

   PROCEDURE Set_Qs_Basic_Parameters (
         A_Queueing_System        : IN OUT Generic_Queueing_System'Class;
         A_Source_Queueing_System : IN     Generic_Queueing_System) IS

   BEGIN

      Set_Qs_Arrival_Rate(A_Queueing_System,0,
         Get_Qs_Arrival_Rate(A_Source_Queueing_System,0));

      Set_Qs_Service_Rate(A_Queueing_System,0,
         Get_Qs_Service_Rate(A_Source_Queueing_System,0));

      Set_Qs_Utilisation(A_Queueing_System,
         Get_Qs_Utilisation(A_Source_Queueing_System));

      Set_Qs_Nb_Arrival(A_Queueing_System,
         Get_Qs_Nb_Arrival(A_Source_Queueing_System));

      Set_Qs_Nb_Server(A_Queueing_System,
         Get_Qs_Nb_Server(A_Source_Queueing_System));

      Set_Qs_Max_State(A_Queueing_System,
         Get_Qs_Max_State(A_Source_Queueing_System));

      Set_Qs_Nb_Of_Messages(A_Queueing_System,
         Get_Qs_Nb_Of_Messages(A_Source_Queueing_System));

   END Set_Qs_Basic_Parameters;

END Queueing_System;
