------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Numerics.Float_Random;
USE Ada.Numerics.Float_Random;

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;

PACKAGE Random_Tools IS

   -- return a random parameter between min and max
   FUNCTION Get_Rand_Parameter (
         Min  : IN     Natural;
         Max  : IN     Natural;
         Seed : IN     Generator)
     RETURN Natural;

   FUNCTION Get_Rand_Parameter (
         Min  : IN     Double;
         Max  : IN     Double;
         Seed : IN     Generator)
     RETURN Double;

   -- generate poisson inter arrival time (different from 0)
   --
   FUNCTION Get_Exponential_Time (
         Arrival_Rate : IN     Double;
         Seed         : IN     Generator)
     RETURN Double;

   -- tirage aleatoire d'une valeur
   -- uniformement repartie entre 0 et 1
   --
   FUNCTION Drand48 RETURN Long_Float;
   PRAGMA Import(C, Drand48);


   -- Initialize la graine a "value"
   --
   PROCEDURE Srand48 (
         Value : IN     Integer);
   PRAGMA Import(C, Srand48);


END Random_Tools;
