------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Text_Io;
USE Text_Io;

PACKAGE BODY Resources IS


   PROCEDURE Put_Name (
         A_Resource : IN     Resource_Ptr) IS
   BEGIN
      Put(To_String(A_Resource.Name));
   END Put_Name;

   PROCEDURE Put (
         A_Resource : IN     Resource_Ptr) IS
   BEGIN
      Put(A_Resource.All);
   END Put;


   PROCEDURE Put (
         My_Resource : IN     Generic_Resource) IS

   BEGIN

      Put("Name: "& To_String(My_Resource.Name));
      Put("; Cpu : "& To_String(My_Resource.Cpu_Name));
      Put("; Address space : "& To_String(My_Resource.Address_Space_Name));
      Put("; Protocol : "& My_Resource.Protocol'Img);
      Put_Line("; State : "& To_String(To_Unbounded_String(Natural'Image(
                  My_Resource.State))));

      Put_Line("Affected Tasks :");
      Put_Line("Task_Name   Begin   End");
      Put(My_Resource.Task_Tab);


   END Put;



   FUNCTION Copy (
         My_Resource : IN     Resource_Ptr)
     RETURN Resource_Ptr IS

      New_Np_Resource   : Np_Resource_Ptr;
      New_Pcp_Resource  : Pcp_Resource_Ptr;
      New_Pip_Resource  : Pip_Resource_Ptr;
      New_IPcp_Resource : IPcp_Resource_Ptr;
      Ptr               : Resource_Ptr;

   BEGIN
      IF (My_Resource.Protocol = No_Protocol)
            THEN
         New_Np_Resource:=NEW Np_Resource;
         New_Np_Resource.All:=Np_Resource_Ptr(My_Resource).ALL;
         Ptr:=Resource_Ptr(New_Np_Resource);
      ELSE
         IF (My_Resource.Protocol = Priority_Ceiling_Protocol)
               THEN
            New_Pcp_Resource:=NEW Pcp_Resource;
            New_Pcp_Resource.All:=Pcp_Resource_Ptr(My_Resource).ALL;
            Ptr:=Resource_Ptr(New_Pcp_Resource);
         ELSE
            IF (My_Resource.Protocol = Immediate_Priority_Ceiling_Protocol)
                  THEN
               New_IPcp_Resource:=NEW IPcp_Resource;
               New_IPcp_Resource.All:=IPcp_Resource_Ptr(My_Resource).ALL;
               Ptr:=Resource_Ptr(New_IPcp_Resource);
            ELSE
               New_Pip_Resource:=NEW Pip_Resource;
               New_Pip_Resource.All:=Pip_Resource_Ptr(My_Resource).ALL;
               Ptr:=Resource_Ptr(New_Pip_Resource);
            END IF;
         END IF;
      END IF;

      RETURN Ptr;
   END Copy;


   PROCEDURE Put (
         A : IN     Affected_Task_List) IS
   BEGIN
      Put(A.Task_Begin'Img & "/" & A.Task_End'Img);
   END Put;

   PROCEDURE Initialize (
         A :    OUT Affected_Task_List) IS
   BEGIN
      A.Task_Begin:=0;
      A.Task_End:=0;
   END Initialize;


   PROCEDURE Initialize (
         A_Resource : IN OUT Generic_Resource) IS
   BEGIN
      A_Resource.Object_Type:=Resource_Object_Type;
   END Initialize;


   PROCEDURE Initialize (
         A_Resource : IN OUT Pip_Resource) IS
   BEGIN
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=Priority_Inheritance_Protocol;
   END Initialize;



   PROCEDURE Initialize (
         A_Resource : IN OUT Np_Resource) IS
   BEGIN
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=No_Protocol;
   END Initialize;



   PROCEDURE Initialize (
         A_Resource : IN OUT Pcp_Resource) IS
   BEGIN
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=Priority_Ceiling_Protocol;
   END Initialize;

   PROCEDURE Initialize (
         A_Resource : IN OUT IPcp_Resource) IS
   BEGIN
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=Immediate_Priority_Ceiling_Protocol;
   END Initialize;



END Resources;



