------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Framework_Config;
USE Framework_Config;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Ada.Strings.Unbounded.Text_Io;
USE Ada.Strings.Unbounded.Text_Io;

WITH Objects;
USE Objects;


WITH Convert_Strings;
WITH Convert_Unbounded_Strings;
WITH Indexed_Tables;
WITH Unchecked_Deallocation;


PACKAGE Resources IS


   TYPE Resources_Type IS
         (No_Protocol,
          Priority_Ceiling_Protocol,
          Priority_Inheritance_Protocol,
          Immediate_Priority_Ceiling_Protocol);


   PROCEDURE To_Protocol IS
   NEW Convert_Strings(Resources_Type,No_Protocol);
   PROCEDURE To_Protocol IS
   NEW Convert_Unbounded_Strings(Resources_Type,No_Protocol);


   TYPE Affected_Task_List IS
      RECORD
         Task_Begin : Natural := 0;
         Task_End   : Natural := 0;
      END RECORD;


   PROCEDURE Put (
         A : IN     Affected_Task_List);
   PROCEDURE Initialize (
         A :    OUT Affected_Task_List);

   PACKAGE Task_Lists IS NEW Indexed_Tables(
      Affected_Task_List, Unbounded_String,
      Framework_Config.Max_Tasks_For_A_Resource, 0,
      Put, Initialize, Put);
   USE Task_Lists;

   SUBTYPE Task_List IS Indexed_Table;
   SUBTYPE Task_List_Range IS Indexed_Table_Range;



   TYPE Start_End_Table IS ARRAY (Task_List_Range) OF Affected_Task_List;



   TYPE Generic_Resource IS NEW Generic_Object WITH
      RECORD
         State    : Integer          := 0;
         Protocol : Resources_Type;
         Task_Tab : Task_List;
         Cpu_Name : Unbounded_String := To_Unbounded_String ("");

         Address_Space_Name : Unbounded_String := To_Unbounded_String ("");
      END RECORD;

   TYPE Resource_Ptr IS ACCESS ALL Generic_Resource'Class;


   PROCEDURE Initialize (
         A_Resource : IN OUT Generic_Resource);

   PROCEDURE Put (
         My_Resource : IN     Generic_Resource);

   PROCEDURE Put_Name (
         A_Resource : IN     Resource_Ptr);

   PROCEDURE Put (
         A_Resource : IN     Resource_Ptr);

   FUNCTION Copy (
         My_Resource : IN     Resource_Ptr)
     RETURN Resource_Ptr;


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Generic_Resource'Class, Resource_Ptr);


   TYPE Np_Resource IS NEW Generic_Resource WITH PRIVATE;
   TYPE Np_Resource_Ptr IS ACCESS ALL Np_Resource'Class;

   PROCEDURE Initialize (
         A_Resource : IN OUT Np_Resource);



   TYPE Pcp_Resource IS NEW Generic_Resource WITH PRIVATE;
   TYPE Pcp_Resource_Ptr IS ACCESS ALL Pcp_Resource'Class;

   PROCEDURE Initialize (
         A_Resource : IN OUT Pcp_Resource);



   TYPE Pip_Resource IS NEW Generic_Resource WITH PRIVATE;
   TYPE Pip_Resource_Ptr IS ACCESS ALL Pip_Resource'Class;

   PROCEDURE Initialize (
         A_Resource : IN OUT Pip_Resource);

   TYPE IPcp_Resource IS NEW Generic_Resource WITH PRIVATE;
   TYPE IPcp_Resource_Ptr IS ACCESS ALL IPcp_Resource'Class;

   PROCEDURE Initialize (
         A_Resource : IN OUT IPcp_Resource);


PRIVATE

   TYPE Pip_Resource IS NEW Generic_Resource WITH NULL RECORD;
   TYPE Pcp_Resource IS NEW Generic_Resource WITH NULL RECORD;
   TYPE Np_Resource IS NEW Generic_Resource WITH NULL RECORD;
   TYPE IPcp_Resource IS NEW Generic_Resource WITH NULL RECORD;



END Resources;



