------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Framework_Config;
USE Framework_Config;
WITH Xml_Tag;
USE Xml_Tag;
WITH Double_Util;
USE Double_Util;
WITH Translate;
USE Translate;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Unbounded_Strings;
USE Unbounded_Strings;





WITH Text_Io;
USE Text_Io;

WITH Tasks;
USE Tasks;

WITH Scheduler;
USE Scheduler;


WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;







PACKAGE BODY Scheduler.Dynamic_Priority.Edf IS


   PROCEDURE Initialize (
         A_Scheduler : IN OUT Edf_Scheduler) IS
   BEGIN
      Reset(A_Scheduler);
      A_Scheduler.Name:= Earliest_Deadline_First_Protocol;
   END Initialize;



   FUNCTION Copy (
         A_Scheduler : IN     Edf_Scheduler)
     RETURN Scheduler_Ptr IS

      Ptr : Edf_Scheduler_Ptr;

   BEGIN

      Ptr:= NEW Edf_Scheduler;

      Ptr.Name:=A_Scheduler.Name;
      Ptr.Is_Preemptive:=A_Scheduler.Is_Preemptive;
      Ptr.Quantum:=A_Scheduler.Quantum;
      Ptr.Previously_Elected:=A_Scheduler.Previously_Elected;

      RETURN Scheduler_Ptr(Ptr);

   END Copy;


   PROCEDURE Do_Election (
         My_Scheduler      : IN OUT Edf_Scheduler;
         Si                : IN OUT Scheduling_Information;
         Result            : IN OUT Scheduling_Sequence_Ptr;
         Msg               : IN OUT Unbounded_String;
         Current_Time      : IN     Natural;
         Processor_Name    : IN     Unbounded_String;
         My_Dependencies   : IN     Tasks_Dependencies_Ptr;
         With_Offsets      : IN     Boolean;
         With_Precedencies : IN     Boolean;
         With_Resources    : IN     Boolean;
         Event_To_Generate : IN     Time_Unit_Event_Type_Boolean_Table;
         Elected           : IN OUT Tasks_Range;
         No_Task           : IN OUT Boolean) IS

      Smallest_Deadline : Natural     := Natural'Last;
      I                 : Tasks_Range := 0;
   BEGIN

      LOOP
         IF (Si.Tcbs(I).Tsk.Cpu_Name = Processor_Name) THEN
            IF (Si.Tcbs(I).Wake_Up_Time <= Current_Time) AND
                  (Dynamic_Priority_Tcb_Ptr(Si.Tcbs(I)).Dynamic_Deadline <
                  Smallest_Deadline) AND
                  (Si.Tcbs(I).Rest_Of_Capacity /= 0)
                  THEN
               IF (With_Offsets=False) OR
                     Check_Offset(Si.Tcbs(I), Current_Time)
                     THEN

                  IF (With_Precedencies=False) OR
                        Check_Precedencies(Si, My_Dependencies,
                        Current_Time,
                        Si.Tcbs(I))
                        THEN

                     Smallest_Deadline:=Dynamic_Priority_Tcb_Ptr(Si.Tcbs(
                           I)).
                        Dynamic_Deadline;
                     Elected:=I;
                  END IF;
               END IF;
            END IF;
         END IF;

         I:=I+1;
         EXIT WHEN Si.Tcbs(I)=NULL;
      END LOOP;


      IF Smallest_Deadline= Natural'Last
            THEN
         No_Task:=True;
      ELSE
         No_Task:=False;

         -- last time unit : compute dynamic deadline for periodic task
         --
         IF (Si.Tcbs(Elected).Rest_Of_Capacity = 1)
               AND (Si.Tcbs(Elected).Tsk.Task_Type = Periodic_Type)
               THEN
            Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).Dynamic_Deadline:=
               Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).
               Dynamic_Deadline
               +Periodic_Task_Ptr(Si.Tcbs(Elected).Tsk).Period;
         END IF;

      END IF;

   END Do_Election;


   PROCEDURE Utilization_Factor_Feasibility_Test (
         My_Scheduler   : IN     Edf_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Unbounded_String) IS

      Tmp               : Tasks_Set;
      Bound             : Double           := 0.0;
      Utildi            : Double           := 0.0;
      Msg               : Unbounded_String := Empty_String;
      Verif             : Boolean          := True;
      A_Task            : Task_Ptr;
      My_Task_Iterator  : Tasks_Iterator;
      A_Task2           : Task_Ptr;
      My_Task_Iterator2 : Tasks_Iterator;
      Calcul            : Double           := 0.0;
      L_Debut           : Natural;
      L_Dernier         : Natural;
      Ech_Sur_Req       : Boolean          := True;
      Pbconstraints     : Boolean          := False;
      Msg2              : Unbounded_String := Empty_String;
      Msg3              : Unbounded_String := Empty_String;


   BEGIN

      CASE Get_Preemptive(My_Scheduler) IS

         WHEN Preemptive =>
            Bound_On_Processor_Utilization(My_Scheduler, My_Tasks,
               Processor_Name, Bound, Msg);

            Utildi:=Processor_Utilization_Over_Deadline(My_Tasks,
               Processor_Name);


            Result := Result & Lb_Utilization_Bound1(Current_Language);
            Result := Result & Lb_Edf(Current_Language);

            IF  Utildi <= Bound THEN
               Result:= Result &  Lb_Sched_Explanation1(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Utildi) &
                  End_Fact_Bound ;
               Result:= Result & Lb_Sched_Explanation12(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Bound) &
                  End_Fact_Bound ;
            ELSE
               Result:= Result & Lb_Sched_Explanation2(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Utildi) &
                  End_Fact_Bound ;
               Result:= Result & Lb_Sched_Explanation22(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Bound) &
                  End_Fact_Bound ;
            END IF;
            Result := Result & Start_Ref & Msg  & End_Ref &
               Unbounded_Lf;

         WHEN Not_Preemptive =>

            Current_Processor_Name:=Processor_Name;
            Select_And_Copy( My_Tasks, Tmp, Select_Cpu'ACCESS );


            Msg2:= Empty_String;

            --Check if the system is in stricts constraints
            --The test works only on stricts constraints !
            --
            Reset_Iterator(Tmp, My_Task_Iterator);
            LOOP
               Current_Element(Tmp, A_Task, My_Task_Iterator);
               IF (A_Task.Cpu_Name = Processor_Name) THEN
                  IF (Periodic_Task_Ptr(A_Task).Period /= A_Task.Deadline) THEN
                     Ech_Sur_Req:= False;
                  END IF;
               END IF;
               EXIT WHEN Is_Last_Element(Tmp, My_Task_Iterator);
               Next_Element(Tmp, My_Task_Iterator);
            END LOOP;


            Sort(Tmp, Increasing_Deadline'ACCESS);
            Reset_Iterator(Tmp, My_Task_Iterator);
            Current_Element(Tmp, A_Task, My_Task_Iterator);
            L_Debut:=Periodic_Task_Ptr(A_Task).Period+1;
            Verif:=True;

            -- Main loop of all tasks
            --
            IF Ech_Sur_Req THEN

               Result := Result & Lb_Utilization_Bound11(Current_Language) &
                  Lb_Edf(Current_Language);
               LOOP
                  Msg3:=Empty_String;
                  IF (A_Task.Cpu_Name = Processor_Name) THEN
                     L_Dernier:=Periodic_Task_Ptr(A_Task).Period-1;
                     FOR L_Courant IN L_Debut..L_Dernier LOOP
                        Reset_Iterator(Tmp, My_Task_Iterator2);

                        -- Second loop until the current task
                        --
                        Calcul:=0.0;
                        LOOP
                           Current_Element(Tmp, A_Task2, My_Task_Iterator2);
                           EXIT WHEN A_Task.Name=A_Task2.Name;
                           Calcul:=Calcul+Double((Integer((L_Courant-1)/
                                    Periodic_Task_Ptr(A_Task2).Period))*
                              A_Task2.Capacity);
                           Next_Element(Tmp, My_Task_Iterator2);
                        END LOOP;

                        -- Checking
                        --
                        Calcul:=Calcul+ Double(A_Task.Capacity);
                        IF Calcul > Double(L_Courant) THEN
                           Verif:=False;
                           Msg3:= A_Task.Name& " ";
                        END IF;
                     END LOOP;
                  END IF;
                  Msg2:=Msg2 & Msg3;
                  EXIT WHEN Is_Last_Element(Tmp, My_Task_Iterator);
                  Next_Element(Tmp, My_Task_Iterator);
                  Current_Element(Tmp, A_Task, My_Task_Iterator);
               END LOOP;
            ELSE
               Result:=Result & Lb_Pb_Strict_Constraints(Current_Language)  &
                  Unbounded_Lf;
               Pbconstraints:=True;
            END IF;

            -- Checks the result
            --
            IF Pbconstraints=False THEN
               IF Verif THEN
                  Result:=Result & Lb_Sched_Explanation5(Current_Language);
                  Result:=Result & Lb_Sched_Explanation52(
                     Current_Language);
                  Result:=Result & To_Unbounded_String(" (") & Lb_See(
                     Current_Language) & "[8].)"
                     & Unbounded_Lf;
               ELSE
                  Result:=Result & Lb_Sched_Explanation6(Current_Language);
                  Result:=Result & Lb_Sched_Explanation62(
                     Current_Language) &
                     To_Unbounded_String(" (") & Lb_See(Current_Language) &
                     "[8])."
                     & Unbounded_Lf & Lb_Sched_Explanation7(
                     Current_Language)&
                     Msg2 & Unbounded_Lf;

               END IF;
            END IF;

      END CASE;


   END Utilization_Factor_Feasibility_Test;









END Scheduler.Dynamic_Priority.Edf ;


