------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Framework_Config;
USE Framework_Config;
WITH Xml_Tag;
USE Xml_Tag;
WITH Double_Util;
USE Double_Util;
WITH Translate;
USE Translate;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Tasks;
USE Tasks;

WITH Scheduler;
USE Scheduler;

WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;




PACKAGE BODY Scheduler.Dynamic_Priority.Llf IS


   PROCEDURE Initialize (
         A_Scheduler : IN OUT Llf_Scheduler) IS
   BEGIN
      Reset(A_Scheduler);
      A_Scheduler.Name:= Least_Laxity_First_Protocol;
   END Initialize;


   FUNCTION Copy (
         A_Scheduler : IN     Llf_Scheduler)
     RETURN Scheduler_Ptr IS

      Ptr : Llf_Scheduler_Ptr;

   BEGIN

      Ptr:= NEW Llf_Scheduler;

      Ptr.Name:=A_Scheduler.Name;
      Ptr.Is_Preemptive:=A_Scheduler.Is_Preemptive;
      Ptr.Quantum:=A_Scheduler.Quantum;
      Ptr.Previously_Elected:=A_Scheduler.Previously_Elected;

      RETURN Scheduler_Ptr(Ptr);

   END Copy;


   PROCEDURE Do_Election (
         My_Scheduler      : IN OUT Llf_Scheduler;
         Si                : IN OUT Scheduling_Information;
         Result            : IN OUT Scheduling_Sequence_Ptr;
         Msg               : IN OUT Unbounded_String;
         Current_Time      : IN     Natural;
         Processor_Name    : IN     Unbounded_String;
         My_Dependencies   : IN     Tasks_Dependencies_Ptr;
         With_Offsets      : IN     Boolean;
         With_Precedencies : IN     Boolean;
         With_Resources    : IN     Boolean;
         Event_To_Generate : IN     Time_Unit_Event_Type_Boolean_Table;
         Elected           : IN OUT Tasks_Range;
         No_Task           : IN OUT Boolean) IS

      Smallest_Laxity : Natural     := Natural'Last;
      Laxity          : Natural;
      I               : Tasks_Range := 0;
   BEGIN

      LOOP
         IF (Si.Tcbs(I).Tsk.Cpu_Name = Processor_Name) THEN

            Laxity:=Dynamic_Priority_Tcb_Ptr(Si.Tcbs(I)).Dynamic_Deadline -
               Si.Tcbs(I).
               Rest_Of_Capacity;

            IF (Si.Tcbs(I).Wake_Up_Time <= Current_Time) AND
                  (Laxity < Smallest_Laxity) AND
                  (Si.Tcbs(I).Rest_Of_Capacity /= 0)
                  THEN
               IF (With_Offsets=False) OR
                     Check_Offset(Si.Tcbs(I), Current_Time)
                     THEN
                  IF (With_Precedencies=False) OR
                        Check_Precedencies(Si, My_Dependencies,
                        Current_Time,
                        Si.Tcbs(I))
                        THEN

                     Smallest_Laxity:=Laxity;
                     Elected:=I;
                  END IF;
               END IF;
            END IF;
         END IF;

         I:=I+1;
         EXIT WHEN Si.Tcbs(I)=NULL;
      END LOOP;



      IF Smallest_Laxity= Natural'Last
            THEN
         No_Task:=True;
      ELSE
         No_Task:=False;

         -- last time unit : compute dynamic deadline for periodic task
         --
         IF (Si.Tcbs(Elected).Rest_Of_Capacity = 1)
               AND (Si.Tcbs(Elected).Tsk.Task_Type = Periodic_Type)
               THEN
            Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).Dynamic_Deadline:=
               Dynamic_Priority_Tcb_Ptr(Si.Tcbs(Elected)).
               Dynamic_Deadline
               +Periodic_Task_Ptr(Si.Tcbs(Elected).Tsk).Period;
         END IF;

      END IF;

   END Do_Election;



   PROCEDURE Utilization_Factor_Feasibility_Test (
         My_Scheduler   : IN     Llf_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Unbounded_String) IS


      Bound  : Double           := 0.0;
      Utildi : Double           := 0.0;
      Msg    : Unbounded_String := Empty_String;


   BEGIN

      CASE Get_Preemptive(My_Scheduler) IS

         WHEN Preemptive =>

            Utildi:=Processor_Utilization_Over_Deadline(My_Tasks,
               Processor_Name);

            Bound_On_Processor_Utilization(My_Scheduler, My_Tasks,
               Processor_Name, Bound, Msg);

            Result := Result & Lb_Utilization_Bound1(Current_Language);
            Result := Result & Lb_Llf(Current_Language);

            IF  Utildi <= Bound THEN
               Result:= Result & Lb_Sched_Explanation1(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Utildi) &
                  End_Fact_Bound ;
               Result := Result &  Lb_Sched_Explanation12(
                  Current_Language);
               Result := Result & Start_Fact_Bound & Format(Bound) &
                  End_Fact_Bound ;
            ELSE
               Result:= Result & Lb_Sched_Explanation2(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Utildi) &
                  End_Fact_Bound ;
               Result := Result & Lb_Sched_Explanation22(Current_Language);
               Result := Result & Start_Fact_Bound & Format(Bound) &
                  End_Fact_Bound ;
            END IF;
            Result := Result & Start_Ref & To_Unbounded_String(" (") &
               Lb_See(
               Current_Language) &
               To_Unbounded_String("[7]).") & End_Ref &
               Unbounded_Lf;

         WHEN Not_Preemptive =>

            Result:=Result
               & Lb_Pb_Sched_Unknown(Current_Language)
               & Unbounded_Lf;

      END CASE;


   END Utilization_Factor_Feasibility_Test;





END Scheduler.Dynamic_Priority.Llf ;


