------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;



package Scheduler.Hierarchical.Parametric is


   type Hierarchical_Parametric_Scheduler is new Hierarchical_Scheduler with private; 
   type Hierarchical_Parametric_Scheduler_Ptr is access all Hierarchical_Parametric_Scheduler'Class; 



   procedure Initialize (
         A_Scheduler : in out Hierarchical_Parametric_Scheduler ); 


   function Copy (
         A_Scheduler : in     Hierarchical_Parametric_Scheduler ) 
     return Scheduler_Ptr; 


   function Get_String_Behavior (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler ) 
     return Unbounded_String; 


   function Get_String_Behavior (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler ) 
     return Unbounded_String_Ptr; 


   procedure Put (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler ); 


   procedure Set_Behavior_File_Name (
         My_Scheduler : in out Hierarchical_Parametric_Scheduler; 
         To_Set       : in     Unbounded_String                   ); 


   function Get_Behavior_File_Name (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler ) 
     return Unbounded_String; 


   procedure Set_String_Behavior (
         My_Scheduler : in out Hierarchical_Parametric_Scheduler; 
         To_Set       : in     Unbounded_String                   ); 


   procedure Check_Before_Scheduling (
         My_Scheduler   : in     Hierarchical_Parametric_Scheduler; 
         My_Tasks       : in     Tasks_Set;                         
         Processor_Name : in     Unbounded_String                   ); 





   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------

   function Export_Aadl_Properties (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler; 
         Number_Of_Ht : in     Natural                            ) 
     return Unbounded_String; 

   function Export_Xml (
         My_Scheduler : in     Hierarchical_Parametric_Scheduler ) 
     return Unbounded_String; 


private

   type Hierarchical_Parametric_Scheduler is new Hierarchical_Scheduler with 
      record 

         -- Scheduler behavior in string format
         --
         String_Behavior : Unbounded_String_Ptr;  

         -- Scheduler behavior in string format
         --
         Behavior_File_Name : Unbounded_String;  

         --         -- Statements for each scheduling
         --         -- section (pointers given by the parser)
         --         --
         --         Root_Statement_Pointer : Section_Pointers;
         --         -- Variables table for the parsed scheduler
         --         -- (given by the parser)
         --         --
         --         Variables_Table : Variables_Table_Type;

         --         -- Table of set statements given by the parser
         --         --
         --         Sets_Table : Sets_Table_Type;
      end record; 


end Scheduler.Hierarchical.Parametric ;




