------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Task_Set;
USE Task_Set;




PACKAGE BODY Scheduler.Nothing IS








   PROCEDURE Initialize (
         A_Scheduler : IN OUT No_Scheduler) IS
   BEGIN
      Reset(A_Scheduler);
      A_Scheduler.Name:= No_Scheduling_Protocol;
   END Initialize;




   FUNCTION Copy (
         A_Scheduler : IN     No_Scheduler)
     RETURN Scheduler_Ptr IS

      Ptr : No_Scheduler_Ptr;

   BEGIN
      RAISE Invalid_Scheduler;
      RETURN NULL;
   END Copy;


   PROCEDURE Bound_On_Processor_Utilization (
         My_Scheduler   : IN     No_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Double;
         Msg            : IN OUT Unbounded_String) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Bound_On_Processor_Utilization;


   PROCEDURE Compute_Response_Time (
         My_Scheduler   : IN     No_Scheduler;
         My_Tasks       : IN OUT Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Response_Time  :    OUT Response_Time_Table) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Compute_Response_Time;





   PROCEDURE Put (
         My_Scheduler : IN     No_Scheduler) IS
   BEGIN
      Put(Generic_Scheduler(My_Scheduler));
   END Put;




   PROCEDURE Check_Before_Scheduling (
         My_Scheduler   : IN     No_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Check_Before_Scheduling;




   PROCEDURE Do_Election (
         My_Scheduler      : IN OUT No_Scheduler;
         Si                : IN OUT Scheduling_Information;
         Result            : IN OUT Scheduling_Sequence_Ptr;
         Msg               : IN OUT Unbounded_String;
         Current_Time      : IN     Natural;
         Processor_Name    : IN     Unbounded_String;
         My_Dependencies   : IN     Tasks_Dependencies_Ptr;
         With_Offsets      : IN     Boolean;
         With_Precedencies : IN     Boolean;
         With_Resources    : IN     Boolean;
         Event_To_Generate : IN     Time_Unit_Event_Type_Boolean_Table;
         Elected           : IN OUT Tasks_Range;
         No_Task           : IN OUT Boolean) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Do_Election;



   PROCEDURE Specific_Scheduler_Initialization (
         My_Scheduler   : IN OUT No_Scheduler;
         Si             : IN OUT Scheduling_Information;
         Processor_Name : IN     Unbounded_String;
         My_Tasks       : IN OUT Tasks_Set;
         My_Resources   : IN OUT Resources_Set;
         My_Buffers     : IN OUT Buffers_Set;
         My_Messages    : IN     Messages_Set;
         Msg            : IN OUT Unbounded_String) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Specific_Scheduler_Initialization;


   PROCEDURE Compute_Activation_Time (
         My_Scheduler : IN     No_Scheduler;
         Si           : IN OUT Scheduling_Information;
         Elected      : IN     Tasks_Range;
         Value        : IN OUT Natural) IS


   BEGIN
      RAISE Invalid_Scheduler;
   END Compute_Activation_Time;


   PROCEDURE Compute_Blocking_Time (
         My_Scheduler   : IN     No_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         My_Resources   : IN     Resources_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Blocking_Time  :    OUT Blocking_Time_Table) IS
   BEGIN
      RAISE Invalid_Scheduler;
   END Compute_Blocking_Time;



   PROCEDURE Utilization_Factor_Feasibility_Test (
         My_Scheduler   : IN     No_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Unbounded_String) IS

   BEGIN
      RAISE Invalid_Scheduler;
   END Utilization_Factor_Feasibility_Test;



END Scheduler.Nothing;


