------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Events;
USE Events;

WITH Framework_Config;
USE Framework_Config;

WITH Natural_Util;
USE Natural_Util;

WITH Tasks;
USE Tasks;
WITH Resources;
USE Resources;
WITH Buffers;
USE Buffers;
WITH Messages;
USE Messages;



WITH Indexed_Tables;
WITH Lists;
WITH Unchecked_Deallocation;




PACKAGE Time_Units IS



   TYPE Time_Unit_Event
      (Event_Type : Time_Unit_Event_Type) IS
   RECORD
      CASE Event_Type IS
         WHEN Start_Of_Task_Capacity =>
            Start_Task : Task_Ptr;

         WHEN End_Of_Task_Capacity =>
            End_Task : Task_Ptr;

         WHEN Write_To_Buffer =>
            Write_Buffer : Buffer_Ptr;
            Write_Task   : Task_Ptr;
            Write_Size   : Natural;

         WHEN Read_From_Buffer =>
            Read_Buffer : Buffer_Ptr;
            Read_Task   : Task_Ptr;
            Read_Size   : Natural;

         WHEN Running_Task =>
            Running_Task : Task_Ptr;
            Current_Priority : Priority_Range;

         WHEN Task_Activation =>
            Activation_Task : Task_Ptr;

         WHEN Allocate_Resource =>
            Allocate_Resource : Resource_Ptr;
            Allocate_Task     : Task_Ptr;

         WHEN Release_Resource =>
            Release_Resource : Resource_Ptr;
            Release_Task     : Task_Ptr;

         WHEN Wait_For_Resource =>
            Wait_Resource : Resource_Ptr;
            Wait_Task     : Task_Ptr;

         WHEN Send_Message =>
            Send_Message : Message_Ptr;
            Send_Task    : Task_Ptr;

         WHEN Receive_Message =>
            Receive_Message : Message_Ptr;
            Receive_Task    : Task_Ptr;

      END CASE;
   END RECORD;



   TYPE Time_Unit_Event_Ptr IS ACCESS Time_Unit_Event;


   PROCEDURE Put (
         E : IN     Time_Unit_Event_Ptr);
   PROCEDURE Put_Line (
         E : IN     Time_Unit_Event_Ptr);
   PROCEDURE Initialize (
         E : IN OUT Time_Unit_Event_Ptr);




   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Time_Unit_Event, Time_Unit_Event_Ptr);




   PACKAGE Time_Unit_List IS NEW Lists(Time_Unit_Event_Ptr, Put,
      Free);
   USE Time_Unit_List;



   PACKAGE A_Time_Unit IS NEW Indexed_Tables(Natural, Time_Unit_Event_Ptr,
      Framework_Config.Max_Scheduling_Period, 0,
      Natural_Util.Put, Natural_Util.Initialize, Put);
   USE A_Time_Unit;

   SUBTYPE Time_Unit_Table IS Indexed_Table;
   SUBTYPE Time_Unit_Range IS Indexed_Table_Range;




END Time_Units;



