------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Text_Io;
USE Ada.Text_Io;
WITH Sax.Readers;
USE Sax.Readers;
WITH Sax.Exceptions;
USE Sax.Exceptions;
WITH Sax.Locators;
USE Sax.Locators;
WITH Sax.Attributes;
USE Sax.Attributes;
WITH Unicode.Ces;
USE Unicode.Ces;
WITH Unicode;
USE Unicode;


WITH Task_Offsets;
USE Task_Offsets;
USE Task_Offsets.Offsets_Table_Package;
WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Strings;
USE Strings;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Dependencies;
USE Dependencies;
WITH Resources;
USE Resources;
USE Resources.Task_Lists;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Event_Analyzer_Set;
USE Event_Analyzer_Set;
USE Event_Analyzer_Set.Generic_Event_Analyzer_Set;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Buffers;
USE Buffers;
USE Buffers.Buffer_Roles;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Queueing_System;
USE Queueing_System;

WITH Events;
USE Events;
WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;

WITH Scheduling_Sequences;
USE Scheduling_Sequences;
WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;


WITH Scheduler;
USE Scheduler;
WITH Task_Dependencies;
USE Task_Dependencies;

WITH Network_Set;
USE Network_Set;

PACKAGE BODY Xml_Parsers IS





   -------------------------------------------------------------
   -------------------------------------------------------------
   --    Generic XML parser
   -------------------------------------------------------------
   -------------------------------------------------------------




   -- Status information (store parameters of each tag)
   --
   Current_Parameter      :          Integer;
   Max_Xml_Tag_Parameters : CONSTANT Natural                                                 :=
     10000;
   Parameter_List         :          ARRAY (1 .. Max_Xml_Tag_Parameters)
     OF Unbounded_String;








   PROCEDURE Characters (
         Handler : IN OUT Xml_Generic_Parser;
         Ch      :        Unicode.Ces.Byte_Sequence) IS

   BEGIN

      Parameter_List(Current_Parameter):=To_Unbounded_String(Ch);
      Current_Parameter:=Current_Parameter+1;

   END Characters;




   PROCEDURE Set_Document_Locator (
         Handler : IN OUT Xml_Generic_Parser;
         Loc     : ACCESS Sax.Locators.Locator'Class) IS
   BEGIN
      Handler.Locator := Locator_Access (Loc);
   END Set_Document_Locator;



   PROCEDURE Warning (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class) IS
   BEGIN
      Put_Line ("Xml warning ("
         & Get_Message (Except) & ", at "
         & To_String (Get_Locator (Except)) & ')');
   END Warning;



   PROCEDURE Error (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class) IS
   BEGIN
      Put_Line ("Xml parser error ("
         & Get_Message (Except) & ", at "
         & To_String (Get_Locator (Except)) & ')');
   END Error;



   PROCEDURE Fatal_Error (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class) IS
   BEGIN
      Put_Line ("Xml parser fatal error (" & Get_Message (Except) & ')');
      Fatal_Error (Reader (Handler), Except);
   END Fatal_Error;







   -------------------------------------------------------------
   -------------------------------------------------------------
   --    XML project file parser
   -------------------------------------------------------------
   -------------------------------------------------------------


   -- Information retrieved from the xml parser
   --
   Name               : Unbounded_String;
   X,
   Y                  : Natural;
   Deadline           : Integer;
   Start_Time         : Integer;
   Blocking_Time      : Integer;
   State              : Integer;
   Period             : Integer;
   Criticality        : Integer;
   Capacity           : Integer;
   Cpu_Name           : Unbounded_String;
   Address_Space_Name : Unbounded_String;
   Jitter             : Natural;
   Policy             : Policies;
   Task_Type          : Tasks_Type;
   Parametric_Rule    : Unbounded_String;
   Seed               : Integer;
   Predictable        : Boolean;
   Quantum            : Integer;
   Priority           : Integer;
   A_Scheduler        : Scheduler_Type;
   A_Network          : Unbounded_String;
   Size               : Natural;
   Protocol           : Resources_Type;
   Is_Preemptive      : Preemptive_Type;
   File_Name          : Unbounded_String;
   Response_Time      : Natural;
   Communication_Time : Natural;
   A_Qs               : Queueing_Systems_Type;

   Text_Memory_Size  : Integer;
   Heap_Memory_Size  : Integer;
   Data_Memory_Size  : Integer;
   Stack_Memory_Size : Integer;

   Roles  : Buffer_Roles_Table;
   A_Role : Buffer_Role;

   Offset   : Offsets_Table;
   A_Offset : Offset_Type;

   A_Task_List_Entry : Affected_Task_List;
   A_Task_List       : Task_List;

   Param   : User_Defined_Parameters_Table;
   A_Param : Parameter_Ptr;

   From_Type : Unbounded_String;
   To_Type   : Unbounded_String;

   Ok : Boolean;



   -- Initialize all data
   --
   PROCEDURE Initialize_Project_Parser (
         Handler : IN OUT Xml_Project_Parser) IS
   BEGIN

      Current_Parameter:=1;

      X:=0;
      Y:=0;
      State:=0;
      Task_Type:=Aperiodic_Type;
      Parametric_Rule:=To_Unbounded_String("");
      Seed := 0;
      Predictable:=True;
      Priority:=1;
      Name:=To_Unbounded_String("");
      Deadline:=0;
      Criticality:=0;
      Period:=0;
      Protocol:=No_Protocol;
      Cpu_Name:=To_Unbounded_String("");
      Address_Space_Name:=To_Unbounded_String("");
      Jitter:=0;
      Initialize(Offset);
      Initialize(Param);
      Quantum:=0;
      A_Scheduler:=No_Scheduling_Protocol;
      A_Network := To_Unbounded_String("");
      Is_Preemptive:=Preemptive;
      File_Name:=To_Unbounded_String("");
      Size:=0;
      Initialize(Roles);
      A_Role.The_Role:=Consumer;
      Initialize(A_Task_List_Entry);
      Initialize(A_Task_List);
      Text_Memory_Size:=0;
      Heap_Memory_Size:=0;
      Data_Memory_Size:=0;
      Stack_Memory_Size:=0;
      A_Qs:=Qs_Mm1;
   END Initialize_Project_Parser;




   PROCEDURE Start_Document (
         Handler : IN OUT Xml_Project_Parser) IS
   BEGIN
      Initialize_Project_Parser(Handler);
      Initialize(Handler.Parsed_System);
   END Start_Document;





   PROCEDURE Start_Element (
         Handler       : IN OUT Xml_Project_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence       := "";
         Qname         :        Unicode.Ces.Byte_Sequence       := "";
         Atts          :        Sax.Attributes.Attributes'Class) IS
   BEGIN


      Current_Parameter:=1;


      -- Store attributes
      --
      IF Get_Length (Atts)>0 THEN

         IF Qname="scheduler"
               THEN

            FOR J IN 0 .. Get_Length (Atts) - 1 LOOP


               IF Get_Qname (Atts, J)="quantum"
                     THEN
                  To_Integer(Get_Value (Atts, J), Quantum, Ok);
                  IF NOT Ok THEN
                     Put_Line("Warning : Error on data type. From " &
                        To_String (Handler.Locator.All));
                  END IF;
               END IF;


               IF Get_Qname (Atts, J)="is_preemptive"
                     THEN
                  To_Preemptive(Get_Value (Atts, J), Is_Preemptive, Ok);
                  IF NOT Ok THEN
                     Put_Line("Warning : Error on data type. From " &
                        To_String (Handler.Locator.All));
                  END IF;
               END IF;


               IF Get_Qname (Atts, J)="parametric_file_name"
                     THEN
                  File_Name:=To_Unbounded_String(Get_Value (
                        Atts,
                        J));
               END IF;


            END LOOP;

         ELSE
            IF Qname="task"
                  THEN

               FOR J IN 0 .. Get_Length (Atts) - 1 LOOP


                  IF Get_Qname (Atts, J)="task_type"
                        THEN
                     To_Task_Type(Get_Value (Atts, J),Task_Type,Ok);
                     IF NOT Ok THEN
                        Put_Line("Warning : Error on data type. From " &
                           To_String (Handler.Locator.All));
                     END IF;
                  END IF;

                  IF Get_Qname (Atts, J)="x"
                        THEN
                     To_Natural(Get_Value (Atts, J),X,Ok);
                     IF NOT Ok THEN
                        Put_Line("Warning : Error on data type. From " &
                           To_String (Handler.Locator.All));
                     END IF;
                  END IF;


                  IF Get_Qname (Atts, J)="y"
                        THEN
                     To_Natural(Get_Value (Atts, J),Y,Ok);
                     IF NOT Ok THEN
                        Put_Line("Warning : Error on data type. From " &
                           To_String (Handler.Locator.All));
                     END IF;
                  END IF;
               END LOOP;

            ELSE

               IF Qname="event_analyzer"
                     THEN

                  FOR J IN 0 .. Get_Length (Atts) - 1 LOOP

                     IF Get_Qname (Atts, J)="parametric_file_name"
                           THEN
                        File_Name:=To_Unbounded_String(Get_Value(Atts, J));
                     END IF;

                  END LOOP;



               ELSE

                  IF Qname="buffer"
                        THEN

                     FOR J IN 0 .. Get_Length (Atts) - 1 LOOP


                        IF Get_Qname (Atts, J)="x"
                              THEN
                           To_Natural(Get_Value (Atts, J),X,Ok);
                           IF NOT Ok THEN
                              Put_Line(
                                 "Warning : Error on data type. From " &
                                 To_String (Handler.Locator.All));
                           END IF;
                        END IF;


                        IF Get_Qname (Atts, J)="y"
                              THEN
                           To_Natural(Get_Value (Atts, J),Y,Ok);
                           IF NOT Ok THEN
                              Put_Line(
                                 "Warning : Error on data type. From " &
                                 To_String (Handler.Locator.All));
                           END IF;
                        END IF;


                     END LOOP;

                  ELSE
                     IF Qname="message"
                           THEN

                        FOR J IN 0 .. Get_Length (Atts) - 1 LOOP


                           IF Get_Qname (Atts, J)="x"
                                 THEN
                              To_Natural(Get_Value (Atts, J),X,Ok);
                              IF NOT Ok THEN
                                 Put_Line(
                                    "Warning : Error on data type. From " &
                                    To_String (Handler.Locator.All));
                              END IF;
                           END IF;


                           IF Get_Qname (Atts, J)="y"
                                 THEN
                              To_Natural(Get_Value (Atts, J),Y,Ok);
                              IF NOT Ok THEN
                                 Put_Line(
                                    "Warning : Error on data type. From " &
                                    To_String (Handler.Locator.All));
                              END IF;
                           END IF;


                        END LOOP;

                     ELSE
                        IF Qname="buffer_user"
                              THEN

                           FOR J IN 0 .. Get_Length (Atts) - 1 LOOP

                              IF Get_Qname (Atts, J)="buffer_role"
                                    THEN
                                 IF (Get_Value (Atts, J)/="consumer")
                                       AND (Get_Value (Atts, J)/=
                                       "producer")
                                       THEN
                                    Put_Line(
                                       "Warning : Error on data type. From " &
                                       To_String (
                                          Handler.Locator.All));
                                 END IF;
                                 IF Get_Value (Atts, J)="consumer"
                                       THEN
                                    A_Role.The_Role:=Consumer;
                                 ELSE
                                    A_Role.The_Role:=Producer;
                                 END IF;
                              END IF;

                           END LOOP;

                        ELSE
                           IF Qname="parameter"
                                 THEN

                              FOR J IN 0 .. Get_Length (Atts) - 1 LOOP

                                 IF Get_Qname (Atts, J)="parameter_type"
                                       THEN

                                    IF (Get_Value (Atts, J)/="integer")
                                          AND (Get_Value (Atts, J)/=
                                          "double")
                                          AND (Get_Value (Atts, J)/=
                                          "string")
                                          AND (Get_Value (Atts, J)/=
                                          "boolean")
                                          THEN
                                       Put_Line(
                                          "Warning : Error on data type. From " &
                                          To_String (
                                             Handler.Locator.All));
                                    END IF;

                                    IF Get_Value (Atts, J)="integer"
                                          THEN
                                       A_Param:= NEW Parameter(
                                          Integer_Parameter);
                                    END IF;
                                    IF Get_Value (Atts, J)="boolean"
                                          THEN
                                       A_Param:= NEW Parameter(
                                          Boolean_Parameter);
                                    END IF;
                                    IF Get_Value (Atts, J)="string"
                                          THEN
                                       A_Param:= NEW Parameter(
                                          String_Parameter);
                                    END IF;
                                    IF Get_Value (Atts, J)="double"
                                          THEN
                                       A_Param:= NEW Parameter(
                                          Double_Parameter);
                                    END IF;
                                 END IF;

                              END LOOP;

                           ELSE
                              IF Qname="dependency"
                                    THEN

                                 FOR J IN 0 .. Get_Length (Atts) - 1 LOOP


                                    IF Get_Qname (Atts, J)="from_type"
                                          THEN
                                       IF (Get_Value (Atts, J)/="buffer")
                                             AND (Get_Value (Atts, J)/=
                                             "message")
                                             AND (Get_Value (Atts, J)/=
                                             "task")
                                             THEN
                                          Put_Line(
                                             "Warning : Error on data type. From " &
                                             To_String (
                                                Handler.Locator.All));
                                       END IF;
                                       From_Type:=To_Unbounded_String(
                                          Get_Value (Atts, J));
                                    END IF;


                                    IF Get_Qname (Atts, J)="to_type"
                                          THEN
                                       IF (Get_Value (Atts, J)/="buffer")
                                             AND (Get_Value (Atts, J)/=
                                             "message")
                                             AND (Get_Value (Atts, J)/=
                                             "task")
                                             THEN
                                          Put_Line(
                                             "Warning : Error on data type. From " &
                                             To_String (
                                                Handler.Locator.All));
                                       END IF;
                                       To_Type:=To_Unbounded_String(
                                          Get_Value (Atts, J));
                                    END IF;

                                 END LOOP;

                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;

   END Start_Element;



   PROCEDURE End_Element (
         Handler       : IN OUT Xml_Project_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence := "";
         Qname         :        Unicode.Ces.Byte_Sequence := "") IS


      Buff_Iterator : Buffers_Iterator;
      Mess_Iterator : Messages_Iterator;
      A_Buff        : Buffer_Ptr;
      A_Mess        : Message_Ptr;
      Task1,
      Task2         : Task_Ptr;

   BEGIN


      -----------------------------------
      -- Element with one parameter
      -----------------------------------


      IF Qname="name"
            THEN
         Name:=Parameter_List(1);
      END IF;

      IF Qname="address_space_name"
            THEN
         Address_Space_Name:=Parameter_List(1);
      END IF;

      IF Qname="cpu_name"
            THEN
         Cpu_Name:=Parameter_List(1);
      END IF;

      IF Qname="network_link"
            THEN
         A_Network := Parameter_List(1);
      END IF;

      IF Qname="scheduler"
            THEN
         To_Scheduler(Parameter_List(1), A_Scheduler, Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " &
               To_String (Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="activation_rule"
            THEN
         Parametric_Rule:=Parameter_List(1);
      END IF;

      IF Qname="period"
            THEN
         To_Integer(Parameter_List(1),Period,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="size"
            THEN
         To_Natural(Parameter_List(1),Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="criticality"
            THEN
         To_Integer(Parameter_List(1),Criticality,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="response_time"
            THEN
         To_Natural(Parameter_List(1),Response_Time,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="communication_time"
            THEN
         To_Natural(Parameter_List(1),Communication_Time,Ok);
         IF NOT Ok THEN
            Put_Line("warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="deadline"
            THEN
         To_Integer(Parameter_List(1),Deadline,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="state"
            THEN
         To_Integer(Parameter_List(1),State,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="seed"
            THEN
         To_Integer(Parameter_List(1),Seed,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="predictable_seed"
            THEN
         To_Boolean(Parameter_List(1),Predictable,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="protocol"
            THEN
         To_Protocol(Parameter_List(1),Protocol,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;

      END IF;


      IF Qname="qs"
            THEN
         To_Qs(Parameter_List(1),A_Qs,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;


      IF Qname="policy"
            THEN
         To_Policy(Parameter_List(1),Policy,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="jitter"
            THEN
         To_Integer(Parameter_List(1),Jitter,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="capacity"
            THEN
         To_Integer(Parameter_List(1),Capacity,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="priority"
            THEN
         To_Integer(Parameter_List(1),Priority,Ok);
         IF NOT Ok THEN
            Priority:=1;
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="start_time"
            THEN
         To_Integer(Parameter_List(1),Start_Time,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="blocking_time"
            THEN
         To_Integer(Parameter_List(1),Blocking_Time,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="text_memory_size"
            THEN
         To_Integer(Parameter_List(1),Text_Memory_Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="heap_memory_size"
            THEN
         To_Integer(Parameter_List(1),Heap_Memory_Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="data_memory_size"
            THEN
         To_Integer(Parameter_List(1),Data_Memory_Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      IF Qname="stack_memory_size"
            THEN
         To_Integer(Parameter_List(1),Stack_Memory_Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
      END IF;

      -----------------------------------
      -- Element with several parameters
      -----------------------------------



      IF Qname="resource_user"
            THEN
         To_Natural(Parameter_List(2),A_Task_List_Entry.Task_Begin,Ok);
         IF NOT Ok
               THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         To_Natural(Parameter_List(3),A_Task_List_Entry.Task_End,Ok);
         IF NOT Ok
               THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         Add(A_Task_List,Parameter_List(1),A_Task_List_Entry);
      END IF;

      IF Qname="buffer_user"
            THEN
         To_Natural(Parameter_List(2),A_Role.Size,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;

         To_Natural(Parameter_List(3),A_Role.Time,Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;


         Add(Roles,Parameter_List(1),A_Role);
      END IF;

      IF Qname="offset"
            THEN
         To_Natural(Parameter_List(1),A_Offset.Activation,Ok);
         IF NOT Ok
               THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         To_Natural(Parameter_List(2),A_Offset.Value,Ok);
         IF NOT Ok
               THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         Add(Offset, A_Offset);
      END IF;



      IF Qname="parameter"
            THEN
         A_Param.Name:=Parameter_List(2);
         IF A_Param.Ptype=String_Parameter
               THEN
            A_Param.String_Value:=Parameter_List(1);
         END IF;
         IF A_Param.Ptype=Boolean_Parameter
               THEN
            To_Boolean(Parameter_List(1),A_Param.Boolean_Value,Ok);
            IF NOT Ok
                  THEN
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.All));
            END IF;
         END IF;
         IF A_Param.Ptype=Integer_Parameter
               THEN
            To_Integer(Parameter_List(1),A_Param.Integer_Value,Ok);
            IF NOT Ok
                  THEN
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.All));
            END IF;
         END IF;
         IF A_Param.Ptype=Double_Parameter
               THEN
            To_Double(Parameter_List(1),A_Param.Double_Value,Ok);
            IF NOT Ok
                  THEN
               Put_Line("Warning : Error on data type. From " & To_String (
                     Handler.Locator.All));
            END IF;
         END IF;
         Add(Param, A_Param);
      END IF;


      ----------------------------------------
      -- End of Element is the end of an objet
      ----------------------------------------

      IF Qname="address_space"
            THEN
         Add_Address_Space(Handler.Parsed_System.Address_Spaces, Name,
            Cpu_Name,
            Text_Memory_Size, Stack_Memory_Size, Data_Memory_Size,
            Heap_Memory_Size, Is_Preemptive, Quantum, File_Name,
            A_Scheduler);
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="event_analyzer"
            THEN
         Add_Event_Analyzer(Handler.Parsed_System.Event_Analyzers,
            File_Name);
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="network"
            THEN
         Add_Network(Handler.Parsed_System.Networks, Name, Parameter_List(
               1));
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="processor"
            THEN
         Add_Processor(Handler.Parsed_System.Processors, Name,
            Is_Preemptive, Quantum, File_Name,
            A_Scheduler, A_Network);
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="buffer"
            THEN
         Add_Buffer(Handler.Parsed_System.Buffers, A_Buff, Name, Size,
            Cpu_Name, Address_Space_Name, A_Qs,
            Roles);
         A_Buff.X:=X;
         A_Buff.Y:=Y;
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="message"
            THEN
         Add_Message(Handler.Parsed_System.Messages, A_Mess, Name, Size,
            Period,
            Deadline,Jitter,Param,Response_Time,Communication_Time);
         A_Mess.X:=X;
         A_Mess.Y:=Y;
         Initialize_Project_Parser(Handler);
      END IF;

      IF Qname="resource"
            THEN
         Add_Resource(Handler.Parsed_System.Resources, Name, State,
            Cpu_Name, Address_Space_Name,
            Protocol, A_Task_List);
         Initialize_Project_Parser(Handler);
      END IF;


      IF Qname="task"
            THEN
         Add_Task(Handler.Parsed_System.Tasks, Task1, Name, Cpu_Name,
            Address_Space_Name,
            Task_Type,
            Start_Time, Capacity, Period, Deadline, Jitter,
            Blocking_Time, Priority, Criticality, Policy,
            Offset, Stack_Memory_Size, Text_Memory_Size, Param,
            Parametric_Rule, Seed, Predictable);
         Task1.X:=X;
         Task1.Y:=Y;
         Initialize_Project_Parser(Handler);
      END IF;



      IF Qname="dependency"
            THEN
         IF (From_Type=To_Unbounded_String("task"))
               AND  (To_Type=To_Unbounded_String("task"))
               THEN
            Task1:=Search_Task(Handler.Parsed_System.Tasks,  Parameter_List(
                  1));
            Task2:=Search_Task(Handler.Parsed_System.Tasks,  Parameter_List(
                  2));
            Add_One_Task_Dependency(Handler.Parsed_System.Dependencies,
               Task1,
               Task2);
         ELSE
            IF (From_Type=To_Unbounded_String("buffer"))
                  OR  (To_Type=To_Unbounded_String("buffer"))
                  THEN

               Reset_Iterator(Handler.Parsed_System.Buffers, Buff_Iterator);

               LOOP
                  Current_Element(Handler.Parsed_System.Buffers, A_Buff ,
                     Buff_Iterator);

                  IF A_Buff.Name = Parameter_List(1)
                        THEN
                     Task2:=Search_Task(Handler.Parsed_System.Tasks,
                        Parameter_List(2));
                     Add_One_Task_Dependency(Handler.Parsed_System.
                        Dependencies,
                        Task2,
                        A_Buff, From_Object_To_Task);
                     EXIT;
                  END IF;

                  IF A_Buff.Name = Parameter_List(2)
                        THEN
                     Task1:=Search_Task(Handler.Parsed_System.Tasks,
                        Parameter_List(1));
                     Add_One_Task_Dependency(Handler.Parsed_System.
                        Dependencies,
                        Task1,
                        A_Buff, From_Task_To_Object);
                     EXIT;
                  END IF;

                  IF Is_Last_Element(Handler.Parsed_System.Buffers,
                        Buff_Iterator)
                        THEN
                     RAISE Buffer_Set.Buffer_Not_Found;
                  END IF;

                  Next_Element(Handler.Parsed_System.Buffers,
                     Buff_Iterator);
               END LOOP;

            ELSE
               IF (From_Type=To_Unbounded_String("message"))
                     OR  (To_Type=To_Unbounded_String("message"))
                     THEN

                  Reset_Iterator(Handler.Parsed_System.Messages,
                     Mess_Iterator);

                  LOOP
                     Current_Element(Handler.Parsed_System.Messages,
                        A_Mess ,
                        Mess_Iterator);

                     IF A_Mess.Name = Parameter_List(1)
                           THEN
                        Task2:=Search_Task(Handler.Parsed_System.Tasks,
                           Parameter_List(2));
                        Add_One_Task_Dependency(Handler.Parsed_System.
                           Dependencies,
                           Task2,
                           A_Mess, From_Object_To_Task);
                        EXIT;
                     END IF;


                     IF A_Mess.Name = Parameter_List(2)
                           THEN
                        Task1:=Search_Task(Handler.Parsed_System.Tasks,
                           Parameter_List(1));
                        Add_One_Task_Dependency(Handler.Parsed_System.
                           Dependencies,
                           Task1,
                           A_Mess, From_Task_To_Object);
                        EXIT;
                     END IF;

                     IF Is_Last_Element(Handler.Parsed_System.Messages,
                           Mess_Iterator)
                           THEN
                        RAISE Message_Set.Message_Not_Found;
                     END IF;

                     Next_Element(Handler.Parsed_System.Messages,
                        Mess_Iterator);
                  END LOOP;


               END IF;
            END IF;
         END IF;
         Initialize_Project_Parser(Handler);
      END IF;

   END End_Element;



   FUNCTION Get_Parsed_System (
         Handler : IN     Xml_Project_Parser)
     RETURN System IS
   BEGIN
      RETURN Handler.Parsed_System;
   END Get_Parsed_System;


   -------------------------------------------------------------
   -------------------------------------------------------------
   --    XML event table parser
   -------------------------------------------------------------
   -------------------------------------------------------------



   Processor_Name       : Unbounded_String;
   Task_Name            : Unbounded_String;
   Resource_Name        : Unbounded_String;
   Message_Name         : Unbounded_String;
   Buffer_Name          : Unbounded_String;
   Number               : Natural;
   Time_Value           : Natural;
   New_Processor_Events : Scheduling_Result_Ptr;
   New_Event            : Time_Unit_Event_Ptr;




   FUNCTION Get_Parsed_Event_Table (
         Handler : IN     Xml_Event_Table_Parser)
     RETURN Scheduling_Table_Ptr IS
   BEGIN
      RETURN Handler.Parsed_Event_Table;
   END Get_Parsed_Event_Table;



   PROCEDURE Set_Scheduled_System (
         Handler          : IN OUT Xml_Event_Table_Parser;
         Scheduled_System : IN     System) IS
   BEGIN
      Handler.Scheduled_System:=Scheduled_System;
   END Set_Scheduled_System;




   PROCEDURE Initialize_Event_Table_Parser (
         Handler : IN OUT Xml_Event_Table_Parser) IS
   BEGIN
      Current_Parameter:=1;
   END Initialize_Event_Table_Parser;



   PROCEDURE Start_Document (
         Handler : IN OUT Xml_Event_Table_Parser) IS
   BEGIN
      Initialize_Event_Table_Parser(Handler);
      Handler.Parsed_Event_Table := NEW Scheduling_Table;
   END Start_Document;



   PROCEDURE Start_Element (
         Handler       : IN OUT Xml_Event_Table_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence       := "";
         Qname         :        Unicode.Ces.Byte_Sequence       := "";
         Atts          :        Sax.Attributes.Attributes'Class) IS
   BEGIN
      Current_Parameter:=1;
   END Start_Element;


   PROCEDURE Read_Time_Value (
         Handler : IN OUT Xml_Event_Table_Parser) IS
   BEGIN
      To_Natural(Parameter_List(1), Number, Ok);
      IF NOT Ok THEN
         Put_Line("Warning : Error on data type. From " & To_String (
               Handler.Locator.All));
      END IF;
      Time_Value:=Number;
   END Read_Time_Value;




   PROCEDURE End_Element (
         Handler       : IN OUT Xml_Event_Table_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence := "";
         Qname         :        Unicode.Ces.Byte_Sequence := "") IS


      A_Buffer    : Buffer_Ptr;
      A_Message   : Message_Ptr;
      A_Task      : Task_Ptr;
      A_Resource  : Resource_Ptr;
      A_Processor : Processor_Ptr;

   BEGIN

      IF Qname="name"
            THEN
         Processor_Name:=Parameter_List(1);
         A_Processor:=Search_Processor(Handler.Scheduled_System.
            Processors, Processor_Name);
         New_Processor_Events:= NEW Scheduling_Result;
         New_Processor_Events.Has_Error:=False;
         New_Processor_Events.Scheduling_Msg:=Empty_String;
         New_Processor_Events.Error_Msg:=Empty_String;
         New_Processor_Events.Result:= NEW Scheduling_Sequence;
         Add(Handler.Parsed_Event_Table.All, A_Processor,
            New_Processor_Events.All);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="write_to_buffer"
            THEN
         Buffer_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         A_Buffer:=Search_Buffer(Handler.Scheduled_System.Buffers,
            Buffer_Name);
         To_Natural(Parameter_List(4), Number, Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Write_To_Buffer);
         New_Event.Write_Buffer:=A_Buffer;
         New_Event.Write_Task:=A_Task;
         New_Event.Write_Size:=Number;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="read_from_buffer"
            THEN
         Buffer_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         A_Buffer:=Search_Buffer(Handler.Scheduled_System.Buffers,
            Buffer_Name);
         To_Natural(Parameter_List(4), Number, Ok);
         IF NOT Ok THEN
            Put_Line("Warning : Error on data type. From " & To_String (
                  Handler.Locator.All));
         END IF;
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Read_From_Buffer);
         New_Event.Read_Buffer:=A_Buffer;
         New_Event.Read_Task:=A_Task;
         New_Event.Read_Size:=Number;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="send_message"
            THEN
         Message_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Message:=Search_Message(Handler.Scheduled_System.Messages,
            Message_Name);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Send_Message);
         New_Event.Send_Task:=A_Task;
         New_Event.Send_Message:=A_Message;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="receive_message"
            THEN
         Message_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Message:=Search_Message(Handler.Scheduled_System.Messages,
            Message_Name);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Receive_Message);
         New_Event.Receive_Task:=A_Task;
         New_Event.Receive_Message:=A_Message;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="allocate_resource"
            THEN
         Resource_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Resource:=Search_Resource(Handler.Scheduled_System.Resources,
            Resource_Name);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Allocate_Resource);
         New_Event.Allocate_Task:=A_Task;
         New_Event.Allocate_Resource:=A_Resource;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="release_resource"
            THEN
         Resource_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Resource:=Search_Resource(Handler.Scheduled_System.Resources,
            Resource_Name);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Release_Resource);
         New_Event.Release_Task:=A_Task;
         New_Event.Release_Resource:=A_Resource;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="wait_for_resource"
            THEN
         Resource_Name:=Parameter_List(2);
         Task_Name:=Parameter_List(3);
         A_Resource:=Search_Resource(Handler.Scheduled_System.Resources,
            Resource_Name);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Wait_For_Resource);
         New_Event.Wait_Task:=A_Task;
         New_Event.Wait_Resource:=A_Resource;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="task_activation"
            THEN
         Task_Name:=Parameter_List(2);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Task_Activation);
         New_Event.Activation_Task:=A_Task;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="running_task"
            THEN
         Task_Name:=Parameter_List(2);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Running_Task);
         New_Event.Running_Task:=A_Task;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="start_of_task_capacity"
            THEN
         Task_Name:=Parameter_List(2);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(Start_Of_Task_Capacity);
         New_Event.Start_Task:=A_Task;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


      IF Qname="end_of_task_capacity"
            THEN
         Task_Name:=Parameter_List(2);
         A_Task:=Search_Task(Handler.Scheduled_System.Tasks, Task_Name);
         Read_Time_Value(Handler);

         New_Event:= NEW Time_Unit_Event(End_Of_Task_Capacity);
         New_Event.End_Task:=A_Task;
         Add(New_Processor_Events.Result.All, New_Event, Time_Value);

         Initialize_Event_Table_Parser(Handler);
      END IF;


   END End_Element;







END Xml_Parsers;
