------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gnat.Command_Line;
use Gnat.Command_Line;
with  Gnat.Os_Lib;
use Gnat.Os_Lib;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;
with  Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;

with Version;
use Version;

with Text_Io;
use Text_Io;

with Systems;
use Systems;

with Call_Framework;
use Call_Framework;
use Call_Framework.Framework_Response_Package;
use Call_Framework.Framework_Request_Package;

with Call_Memory_Framework;
use Call_Memory_Framework;
use Call_Memory_Framework.Buffer_Result_Package;
with Call_Scheduling_Framework;
use Call_Scheduling_Framework;

with Parameters;
use Parameters;

with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;


procedure Compute is 

   Sys          : System;  
   Ok           : Boolean;  
   File_Name    : Unbounded_String;  
   Number       : Natural          := 0;  

   Operation     : Framework_Statement_Type;  
   Response_List : Framework_Response_Table;  
   Request_List  : Framework_Request_Table;  
   A_Request     : Framework_Request;  
   A_Param       : Parameter_Ptr;  




   procedure Usage is 
   begin
      Put_Line (
         "compute is a program which shows how the framework can be called for a XML project");
      New_Line;
      Put_Line(
         "Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : compute [switch] file-to-proceed operation");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line("            -v verbose mode ");
      Put_Line(
         "            -s Update the XML project if it was modified during computation ");
      New_Line;
   end Usage;


   -- Is the "compute" command should run in verbose mode ?
   --
   Verbose : Boolean := False;  

   -- Should we save the modified project ?
   --
   To_Save : Boolean := False;  


begin

   -- Get arguments 
   loop
      case Getopt ("u v s") is
         when Ascii.Nul =>
            exit;

         when 's' =>
            To_Save:=True;
         when 'v' =>
            Verbose:=True;
         when 'u' =>
            Usage;
            Os_Exit(0);

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;


   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;

         Number:=Number+1;

         if Number=1 then
            File_Name := File_Name & S;

         end if;
         if Number=2 then
            To_Framework_Statement(S,Operation,Ok);
         end if;
      end;
   end loop;


   Copyright("framework_caller");


   -- Control on switches and parameters
   --
   if (Number /= 2) then
      Usage;
      Os_Exit(0);
   end if;

   if(File_Name = Empty_String)
         then
      Usage;
      Os_Exit(0);
   end if;


   -- Reading the file
   --
   if Verbose then
      Put("Reading the file ");
      Put(To_String(File_Name) & "...." );
   end if;


   Read_From_Xml_File(Sys,File_Name);


   -- Call the framework 
   --
   if Verbose then
      Put("Calling the framework .... ");
      New_Line;
      New_Line;
      New_Line;
      New_Line;
   end if;

   A_Request.Statement:=Operation;

   case Operation  is
      when Scheduling_Simulation_Time_Line =>
         null;
      when Scheduling_Simulation_All_Response_Time =>
         null;
      when Scheduling_Simulation_Context_Switch_Number =>
         null;
      when Scheduling_Simulation_Priority_Inversion =>
         null;
      when Scheduling_Simulation_Deadlock =>
         null;
      when Scheduling_Simulation_Basics =>
         null;
      when Scheduling_Simulation_Blocking_Time =>
         null;
      when  Scheduling_Simulation_Preemption_Number =>
         null;
      when Scheduling_Simulation_Response_Time =>
         null;
      when Scheduling_Feasibility_Cpu_Utilization   =>
         null;
      when Scheduling_Feasibility_Compute_Blocking_Time   =>
         null;
      when Scheduling_Feasibility_Compute_And_Set_Blocking_Time   =>
         null;
      when Scheduling_Feasibility_Response_Time   =>
         null;
      when Scheduling_Feasibility_Basics =>
         Initialize(A_Request);
         Add(Request_List, A_Request);
         Sequential_Framework_Request (
            Sys, Request_List, Response_List);
      when Scheduling_Set_Priorities_According_To_Deadline_Monotonic =>
         null;
      when Scheduling_Set_Priorities_According_To_Rate_Monotonic =>
         null;
      when Random_Response_Time_Density    =>
         null;
      when Dependency_Compute_End_To_End_Response_Time_One_Step =>
         null;
      when Dependency_Set_End_To_End_Response_Time_One_Step   =>
         null;
      when Dependency_Compute_End_To_End_Response_Time_All_Steps =>
         null;
      when Dependency_Set_End_To_End_Response_Time_All_Steps   =>
         null;
      when Dependency_Compute_Chetto_Blazewicz_Priority  =>
         null;
      when Dependency_Compute_Chetto_Blazewicz_Deadline   =>
         null;
      when Dependency_Set_Chetto_Blazewicz_Priority  =>
         null;
      when Dependency_Set_Chetto_Blazewicz_Deadline =>
         null;
      when Buffer_Scheduling_Simulation =>
         null;
      when Scheduling_Simulation_Run_Event_Handler =>
         null;
      when Scheduling_Feasibility_First_Fit =>
         null;
      when Scheduling_Feasibility_General_Task =>
         null;
      when Scheduling_Feasibility_Next_Fit =>
         null;
      when Scheduling_Feasibility_Best_Fit =>
         null;
      when Scheduling_Feasibility_Small_Task =>
         null;
      when Global_Memory_Set_Requirement_Analysis =>
         null;
      when Global_Memory_Compute_Requirement_Analysis =>
         null;
      when Buffer_Feasibility_Tests =>
         null;
   end case;



   -- Save the project
   --
   if To_Save then
      Write_To_Xml_File(Sys,File_Name);
   end if;




   -- Display the result
   --
   Put(Response_List);
   Put_Xml(Sys);
   Put(Buff);
   Put(Sched.All);





exception
   when Gnat.Command_Line.Invalid_Switch    =>
      begin
         Put_Line ("compute.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Gnat.Command_Line.Invalid_Parameter =>
      Put_Line ("compute.adb : missing parameter for switch " &
         Full_Switch);
   when Ada.Io_Exceptions.Name_Error =>
      Put_Line(
         "compute.adb : Name error exception, can not read file to parse");
   when others =>
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line("This is an internal Cheddar bug ... sorry");
      Put_Line(
         "Please, send a bug report to singhoff@univ-brest.fr");
      Put_Line(
         "Do not forget to join the erronous Cheddar project files with your bug report");


end Compute;
