------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Io_Tools;
use Io_Tools;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;

with Parameters;
use Parameters;

with Dependencies;
use Dependencies;
with Messages;
use Messages;
with Message_Set;
use Message_Set;

with Task_Dependencies;
use Task_Dependencies;
use Task_Dependencies.Half_Dep_Set;

with Task_Offsets;
use Task_Offsets;
use Task_Offsets.Offsets_Table_Package;

with Systems;
use Systems;


procedure Write_Depend is 

   T1,  
   T2,  
   T3,  
   T4     : Task_Ptr;  
   Param  : User_Defined_Parameters_Table;  
   Sys    : System;  
   Msg1,  
   Msg2   : Message_Ptr;  
   Offset : Offsets_Table;  
   Cpu    : Unbounded_String              := To_Unbounded_String ("cpu");  
   Ea     : Unbounded_String              := To_Unbounded_String ("ea");  

begin


   Initialize(Offset);
   Initialize(Sys);

   Add_Task(Sys.Tasks, T1, To_Unbounded_String("T1"), Cpu, Ea, Periodic_Type,
      0, 5,
      12, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, T2, To_Unbounded_String("T2"), Cpu, Ea, Periodic_Type,
      0, 2,
      6,  0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, T3, To_Unbounded_String("T3"), Cpu, Ea, Periodic_Type,
      0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);
   Add_Task(Sys.Tasks, T4, To_Unbounded_String("T4"), Cpu, Ea, Periodic_Type,
      0, 5,
      24, 0, 0, 0, 1,
      Sched_Fifo, Offset);

   Add_Message(Sys.Messages, Msg1, To_Unbounded_String("m1"), 1000, 20, 0,
      0, Param, 0, 0);
   Add_Message(Sys.Messages, Msg2, To_Unbounded_String("m2"), 1000, 10, 0,
      0, Param, 0, 0);

   Put_Xml(Sys);


   Add_One_Task_Dependency(Sys.Dependencies, T1, T2);
   Add_One_Task_Dependency(Sys.Dependencies, T2, Msg1, From_Task_To_Object);
   Add_One_Task_Dependency(Sys.Dependencies, T3, Msg1, From_Object_To_Task);
   Add_One_Task_Dependency(Sys.Dependencies, T3, Msg2, From_Task_To_Object);
   Add_One_Task_Dependency(Sys.Dependencies, T4, Msg2, From_Object_To_Task);


   Put_Xml(Sys);

   Write_To_Xml_File(Sys, To_Unbounded_String("depend_cheddar.xml"));

   Delete_One_Task_Dependency(Sys.Dependencies,
      T1, T2);

   Put_Xml(Sys);


end  Write_Depend;

