------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE Gtk;
WITH Gtk.Check_Button;
USE Gtk.Check_Button;

WITH Aadl_Config;
USE Aadl_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;





PACKAGE BODY Aadl_Options_Pkg.Callbacks IS


   PROCEDURE Page_Switch (
         Notebook : ACCESS Gtk_Notebook_Record'Class;
         Page     : IN     Gtk.Gtk_Notebook_Page;
         Page_Num : IN     Gtk_Notebook) IS
      PRAGMA Warnings (Off, Page_Num);
      Old_Page : CONSTANT Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   :          Gtk_Pixmap;
   BEGIN
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);

      IF Old_Page /= NULL THEN
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Aadl_Options.Book_Closed,
            Aadl_Options.Book_Closed_Mask);
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Aadl_Options.Book_Closed,
            Aadl_Options.Book_Closed_Mask);
      END IF;
   END Page_Switch;



   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


   BEGIN

      -- Activate AADL Debug switch
      --
      IF Get_Active(Aadl_Options.Aadl_Debug)
            THEN
         Aadl_Debug:=True;
      ELSE
         Aadl_Debug:=False;
      END IF;



      -- Include system/sub-system name in Cheddar's features name
      --
      IF Get_Active(Aadl_Options.Aadl_Import_With_System_Name)
            THEN
         Aadl_Import_With_System_Name:=True;
      ELSE
         Aadl_Import_With_System_Name:=False;
      END IF;



      -- Transform event and event data port to task precedencies
      --
      IF Get_Active(Aadl_Options.Aadl_Import_Event_To_Precedencies)
            THEN
         Aadl_Import_Event_To_Precedencies:=True;
      ELSE
         Aadl_Import_Event_To_Precedencies:=False;
      END IF;
      IF Get_Active(Aadl_Options.Aadl_Import_Event_Data_To_Precedencies)
            THEN
         Aadl_Import_Event_Data_To_Precedencies:=True;
      ELSE
         Aadl_Import_Event_Data_To_Precedencies:=False;
      END IF;


      -- Transform event port and event data port to buffers and messages
      -- (it automatically includes buffer/message task dependencies)
      --
      IF Get_Active(Aadl_Options.Aadl_Import_Event_To_Buffers_Messages)
            THEN
         Aadl_Import_Event_To_Buffers_Messages:=True;
      ELSE
         Aadl_Import_Event_To_Buffers_Messages:=False;
      END IF;
      IF Get_Active(
            Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages)
            THEN
         Aadl_Import_Event_Data_To_Buffers_Messages:=True;
      ELSE
         Aadl_Import_Event_Data_To_Buffers_Messages:=False;
      END IF;


      -- Transform task precedencies to event port
      --
      IF Get_Active(
            Aadl_Options.Aadl_Export_Precedencies_To_Event)
            THEN
         Aadl_Export_Precedencies_To_Event:=True;
      ELSE
         Aadl_Export_Precedencies_To_Event:=False;
      END IF;




      -- Transform task precedencies to event data port
      --
      IF Get_Active(
            Aadl_Options.Aadl_Export_Precedencies_To_Event_Data)
            THEN
         Aadl_Export_Precedencies_To_Event_Data:=True;
      ELSE
         Aadl_Export_Precedencies_To_Event_Data:=False;
      END IF;



      -- Transform task precedencies to a Cheddar property
      --
      IF Get_Active(Aadl_Options.Aadl_Export_Precedencies_To_Properties)
            THEN
         Aadl_Export_Precedencies_To_Properties:=True;
      ELSE
         Aadl_Export_Precedencies_To_Properties:=False;
      END IF;


      -- Process second and third AADL import passes
      --

      IF Get_Active(Aadl_Options.Aadl_Process_Second_Import_Pass)
            THEN
         Aadl_Process_Second_Import_Pass:=True;
      ELSE
         Aadl_Process_Second_Import_Pass:=False;
      END IF;
      IF Get_Active(Aadl_Options.Aadl_Process_Third_Import_Pass)
            THEN
         Aadl_Process_Third_Import_Pass:=True;
      ELSE
         Aadl_Process_Third_Import_Pass:=False;
      END IF;



      Destroy(Aadl_Options);

   END On_Ok_Pressed;




   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Aadl_Options);
   END On_Cancel_Pressed;



END Aadl_Options_Pkg.Callbacks;



