------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Gdk.Bitmap;
USE Gdk.Bitmap;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Check_Button;
USE Gtk.Check_Button;
WITH Gtk.Hbutton_Box;
USE Gtk.Hbutton_Box;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Notebook;
USE Gtk.Notebook;
WITH Gtk.Pixmap;
USE Gtk.Pixmap;



PACKAGE Aadl_Options_Pkg IS

   TYPE Aadl_Options_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Aadl_Options_Access IS ACCESS ALL Aadl_Options_Record'Class;

   PROCEDURE Gtk_New (
         Aadl_Options :    OUT Aadl_Options_Access);
   PROCEDURE Initialize (
         Aadl_Options : ACCESS Aadl_Options_Record'Class);

   Aadl_Options : Aadl_Options_Access;


PRIVATE


   TYPE Aadl_Options_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Options_Vbox : Gtk_Vbox;

         Value_Table : Gtk_Table;
         Label       : Gtk_Label;

         -- Activate AADL Debug switch
         --
         Aadl_Debug : Gtk_Check_Button;

         -- Include system/sub-system name in Cheddar's features name
         --
         Aadl_Import_With_System_Name : Gtk_Check_Button;

         -- Transform event port ad event data port to task precedencies
         --
         Aadl_Import_Event_To_Precedencies      : Gtk_Check_Button;
         Aadl_Import_Event_Data_To_Precedencies : Gtk_Check_Button;

         -- Transform event port and event data port to buffers and messages
         -- (it automatically includes buffer/message task dependencies)
         --
         Aadl_Import_Event_To_Buffers_Messages      : Gtk_Check_Button;
         Aadl_Import_Event_Data_To_Buffers_Messages : Gtk_Check_Button;

         -- Transform task precedencies to a Cheddar property
         --
         Aadl_Export_Precedencies_To_Properties : Gtk_Check_Button;

         -- Transform task precedencies to event port connections
         --
         Aadl_Export_Precedencies_To_Event : Gtk_Check_Button;

         -- Transform task precedencies to event data port connections
         --
         Aadl_Export_Precedencies_To_Event_Data : Gtk_Check_Button;

         -- Process the second AADL import pass which binds software components to hardware components
         --
         Aadl_Process_Second_Import_Pass : Gtk_Check_Button;

         -- Process the third AADL import pass which build component relationships (i.e. connections)
         --
         Aadl_Process_Third_Import_Pass : Gtk_Check_Button;

         Button_Hbox   : Gtk_Hbutton_Box;
         Ok_Button     : Gtk_Button;
         Cancel_Button : Gtk_Button;
         Hseparator    : Gtk_Hseparator;

         Main_Frame : Gtk_Frame;
         Child_Page : Gtk_Frame;
         Label_Box  : Gtk_Box;
         Menu_Box   : Gtk_Box;
         Pixmap     : Gtk_Pixmap;
         Lab        : Gtk_Label;

         Book_Opened      : Gdk_Pixmap;
         Book_Opened_Mask : Gdk_Bitmap;
         Book_Closed      : Gdk_Pixmap;
         Book_Closed_Mask : Gdk_Bitmap;
         Notebook         : Gtk_Notebook;
      END RECORD;

END Aadl_Options_Pkg;
