------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE Gtk;
WITH Gdk.Types;
USE Gdk.Types;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Graph_Add_Pkg.Callbacks;
USE Graph_Add_Pkg.Callbacks;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;
WITH My_Canvas;
USE My_Canvas;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;

PACKAGE BODY Graph_Add_Pkg IS


   PROCEDURE Gtk_New (
         Add_New_Item :    OUT Add_Item_Access;
         Item         : IN     String) IS
   BEGIN
      PRAGMA Warnings (Off);

      Add_New_Item := NEW Add_Item_Record;
      Add_New_Item.Item_Type:=To_Unbounded_String(Item);
      Graph_Add_Pkg.Initialize (Add_New_Item);
   END Gtk_New;

   PROCEDURE Initialize (
         Add_New_Item : ACCESS Add_Item_Record'Class) IS

      FUNCTION Internal_Append (
            Canva : ACCESS Interactive_Canvas_Record'Class;
            Item  : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN

         IF (Add_New_Item.Item_Type="message") THEN
            IF(Get_Figure(Item) = To_Unbounded_String("message"))
                  AND THEN NOT Is_Visible(Item)  THEN
               String_List.Append (Add_New_Item.Items, To_String(Get_Name(
                        Item)));
            END IF;
         END IF;

         IF (Add_New_Item.Item_Type="buffer") THEN
            IF(Get_Figure(Item) = To_Unbounded_String("buffer"))
                  AND THEN NOT Is_Visible(Item)  THEN
               String_List.Append (Add_New_Item.Items, To_String(Get_Name(
                        Item)));
            END IF;
         END IF;

         IF (Add_New_Item.Item_Type="task") THEN
            IF(Get_Figure(Item) = To_Unbounded_String("task"))
                  AND THEN NOT Is_Visible(Item)  THEN
               String_List.Append (Add_New_Item.Items, To_String(Get_Name(
                        Item)));
            END IF;
         END IF;

         RETURN True;
      END Internal_Append;


   BEGIN
      Gtk.Window.Initialize (Add_New_Item, Window_Toplevel);
      Set_Title (Add_New_Item, To_String(Lb_New_Item(Current_Language)));
      Set_Policy (Add_New_Item, True, True, True);
      Set_Position (Add_New_Item, Win_Pos_Center_Always);
      Set_Extension_Events (Add_New_Item, Extension_Events_All);
      Set_Flags (Add_New_Item, Can_Default);
      Grab_Focus (Add_New_Item);

      Gtk_New_Vbox (Add_New_Item.Vbox, True, 0);
      Set_Border_Width (Add_New_Item.Vbox, 6);
      Add (Add_New_Item, Add_New_Item.Vbox);

      Gtk_New (Add_New_Item.Label, To_String(Lb_Choose_Item(
               Current_Language)));
      Add (Add_New_Item.Vbox, Add_New_Item.Label);

      Import_Task(Graph_Window);
      Import_Message(Graph_Window);
      Import_Buffer(Graph_Window);

      String_List.Append (Add_New_Item.Items, To_String(Lb_New(
               Current_Language)));
      For_Each_Item(Graph_Window.Buffer,Internal_Append'
         Unrestricted_Access);

      Gtk_New (Add_New_Item.Combo);
      Set_Popdown_Strings (Add_New_Item.Combo, Add_New_Item.Items);
      Set_Editable(Get_Entry(Add_New_Item.Combo),False);
      Add (Add_New_Item.Vbox, Add_New_Item.Combo);

      Gtk_New_Hseparator (Add_New_Item.Hseparator);
      Add (Add_New_Item.Vbox, Add_New_Item.Hseparator);

      Gtk_New_Hbox (Add_New_Item.Hbox, True, 0);
      Set_Border_Width (Add_New_Item.Hbox, 6);
      Add (Add_New_Item.Vbox, Add_New_Item.Hbox);

      Gtk_New (Add_New_Item.Ok_Button, "Ok");
      Pack_Start (Add_New_Item.Hbox, Add_New_Item.Ok_Button, True, True, 0);
      Button_Callback.Connect
         (Add_New_Item.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'ACCESS));

      Gtk_New_Vseparator (Add_New_Item.Vseparator);
      Set_Sensitive (Add_New_Item.Vseparator, False);
      Pack_Start (Add_New_Item.Hbox, Add_New_Item.Vseparator,  True, True,
         0);

      Gtk_New (Add_New_Item.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Add_New_Item.Hbox, Add_New_Item.Cancel_Button, True,
         True, 0);
      Button_Callback.Connect
         (Add_New_Item.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));

      Show_All(Add_New_Item);

   END Initialize;

END Graph_Add_Pkg;
