------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Gtk.Handlers;
USE Gtk.Handlers;
WITH Graph_Item_Pkg;
USE Graph_Item_Pkg;
WITH Graph_Window_Pkg ;
USE Graph_Window_Pkg;
WITH Gtk.Toolbar;
USE Gtk.Toolbar;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Main;
USE Gtk.Main;
WITH Ada.Numerics;
USE Ada.Numerics;
WITH Ada.Numerics.Elementary_Functions;
USE Ada.Numerics.Elementary_Functions;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Task_Dependencies;
USE Task_Dependencies;
WITH Task_Set;
USE Task_Set;
WITH Tasks;
USE Tasks;
USE Task_Set.Generic_Task_Set;
WITH Project;
USE Project;
WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;


PACKAGE BODY Graph_Callbacks_Pkg IS

   PROCEDURE Canvas_Zoomed (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
      FUNCTION Internal (
            Canvas : ACCESS Interactive_Canvas_Record'Class;
            Item   : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
         It : CONSTANT Item_Object_Access := Item_Object_Access (Item);
      BEGIN
         --  Change the size the item occupies on the screen.
         --
         Set_Screen_Size
            (It, To_Canvas_Coordinates (Canvas, It.W),
            To_Canvas_Coordinates (Canvas, It.H));
         RETURN True;
      END Internal;
   BEGIN
      For_Each_Item (Canvas, Internal'Unrestricted_Access);
   END Canvas_Zoomed;


   PROCEDURE Zoom_In (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
   BEGIN
      FOR J IN Zoom_Levels'First .. Zoom_Levels'Last - 1 LOOP
         IF Zoom_Levels (J) = Get_Zoom (Canvas) THEN
            Zoom (Canvas, Zoom_Levels (J + 1), 5);
         END IF;
      END LOOP;
   END Zoom_In;


   PROCEDURE Zoom_Out (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
   BEGIN
      FOR J IN Zoom_Levels'First + 1 .. Zoom_Levels'Last LOOP
         IF Zoom_Levels (J) = Get_Zoom (Canvas) THEN
            Zoom (Canvas, Zoom_Levels (J - 1), 5);
         END IF;
      END LOOP;
   END Zoom_Out;

   PROCEDURE Hide_Graph_Window (
         G : ACCESS Graph_Window_Record'Class) IS
   BEGIN
      Hide(G);
   END Hide_Graph_Window;





   PROCEDURE Open_Graph (
         Button : ACCESS Gtk_Button_Record'Class) IS

   BEGIN
      Gtk_New (Graph_Window);
   END Open_Graph;

   FUNCTION Close_Graph (
         Win : ACCESS Graph_Window_Record'Class)
     RETURN Gint IS
   BEGIN
      RETURN 1;
   END Close_Graph;


   PROCEDURE Close_Win (
         Win : ACCESS Gtk_Window_Record'Class) IS

   BEGIN
      Gtk_Exit(0);
   END Close_Win;




   FUNCTION Get_Item_Representing_Task (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         A_Task :        Task_Ptr)
     RETURN Item_Object_Access IS

      Item_Found : Item_Object_Access;

      FUNCTION Internal (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Item : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN

         IF (A_Task.Name
               = Get_Name(Item)) THEN
            Item_Found := Item_Object_Access (Item);
            RETURN False; -- Stop the For_Each_Item loop
         END IF;

         RETURN True;
      END Internal;
   BEGIN
      For_Each_Item (Canvas, Internal'Unrestricted_Access);

      RETURN Item_Found;
   END Get_Item_Representing_Task;

   PROCEDURE Disp1 (
         Win : ACCESS Graph_Window_Record'Class) IS
      Borders_Space       : CONSTANT Guint := Win.G1_Borders_Space;
      Space_Between_Items : CONSTANT Guint := Win.G1_Space_Between_Items;

      Number_Of_Items_Per_Line : Guint;
      Cur_Item                 : Guint := 0;

      Da_Width : CONSTANT Guint := Get_Allocation_Width (Win);

      FUNCTION Internal (
            Canvas : ACCESS Interactive_Canvas_Record'Class;
            Item   : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
         X,
         Y : Guint;
      BEGIN
         X := Borders_Space +
            (Guint (Graph_Window_Pkg.Item_Width) + Space_Between_Items) *
            (Cur_Item mod Number_Of_Items_Per_Line);

         Y := Borders_Space +
            (Guint (Graph_Window_Pkg.Item_Height) + Space_Between_Items) *
            (Cur_Item / Number_Of_Items_Per_Line);

         Move_To (Canvas, Item, Gint (X), Gint (Y));
         Cur_Item := Cur_Item + 1;
         RETURN True;
      END Internal;


   BEGIN

      Is_Saved:=False;

      Number_Of_Items_Per_Line := 1 +
         (Da_Width - 2 * Borders_Space - Guint (Graph_Window_Pkg.
            Item_Width)) /
         (Guint (Graph_Window_Pkg.Item_Width) + Space_Between_Items);

      For_Each_Item (Win.Buffer, Internal'Unrestricted_Access);
      Refresh_Canvas (Win.Buffer);
      Win.Cur_Graph_Type := G1;
   END Disp1;

   PROCEDURE Disp2 (
         Win : ACCESS Graph_Window_Record'Class) IS
      Borders_Space : CONSTANT Guint := Win.G2_Borders_Space;

      Angle : Float := 0.0;
      Step  : Float;

      H_Radius : Float := Float (Get_Allocation_Width (Win.Buffer) - 2 *
        Borders_Space) / 2.0;
      V_Radius : Float := Float (Get_Allocation_Height (Win.Buffer) - 2 *
        Borders_Space) / 2.0;

      FUNCTION Internal (
            Canvas : ACCESS Interactive_Canvas_Record'Class;
            Item   : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
         X,
         Y : Float;
      BEGIN
         X := Float (Borders_Space) - 5.0 + H_Radius + -- X center
            H_Radius * Cos (Angle) - Float (Graph_Window_Pkg.
            Item_Width) / 2.0;

         Y := Float (Borders_Space) - 5.0 + V_Radius + -- Y center
            V_Radius * Sin (Angle) - Float (Graph_Window_Pkg.
            Item_Height) / 2.0;

         Move_To (Canvas, Item, Gint (X), Gint (Y));

         Angle := Angle + Step;

         RETURN True;
      END Internal;

   BEGIN

      Is_Saved:=False;

      IF Win.G2_Force_Circle THEN
         H_Radius := Float'Min (H_Radius, V_Radius);
         V_Radius := Float'Min (H_Radius, V_Radius);
      END IF;

      Step := 2.0 * Pi / Float (Get_Number_Of_Items (Win.Buffer));

      For_Each_Item (Win.Buffer, Internal'Unrestricted_Access);

      Refresh_Canvas (Win.Buffer);
      Win.Cur_Graph_Type := G2;
   END Disp2;

   PROCEDURE Disp3 (
         Win : ACCESS Graph_Window_Record'Class) IS
      Borders_Space       : CONSTANT Guint := Win.G3_Borders_Space;
      Space_Between_Items : CONSTANT Guint := Win.G3_Space_Between_Items;

      An_Iterator,
      Src_Iterator : Iterator;
      A_Task,
      Src_Task     : Task_Ptr;

      Cur_Item   : Guint := 0;
      Cur_Column : Guint := 1;

      Src_Set,
      Dest_Set,
      Done_Set,
      Succ_Set,
      Pit_Set  : Tasks_Set;

   BEGIN

      Is_Saved:=False;

      IF NOT Is_Empty(Sys.Tasks) THEN
         -- Search after (a) source task(s)
         --
         LOOP
            Current_Element(Sys.Tasks, A_Task, An_Iterator);


            IF (NOT Has_Predecessor (Sys.Dependencies, A_Task) AND
                  Has_Successor (Sys.Dependencies, A_Task)) THEN
               Move_To (Win.Buffer,
                  Get_Item_Representing_Task (Win.Buffer, A_Task),
                  Gint (Borders_Space),
                  Gint (Borders_Space + Cur_Item * (Guint (
                           Graph_Window_Pkg.Item_Height) +
                        Space_Between_Items)));

               Cur_Item := Cur_Item + 1;

               Add(Done_Set, A_Task);
               Add(Src_Set, A_Task);
            END IF;


            EXIT WHEN Is_Last_Element (Sys.Tasks, An_Iterator);
            Next_Element(Sys.Tasks,  An_Iterator);
         END LOOP;

         -- Now we have found the source(s), we have to loop until we haven't any
         -- more successor.
         --
         -- Don't try to think about it : I tell you it can't be in the same loop
         -- (or it wouldn't be comprehensible :)
         --
         LOOP
            Reset (Dest_Set);
            Cur_Item := 0;

            IF NOT Is_Empty(Src_Set) THEN
               Reset_Iterator (Src_Set, Src_Iterator);
               LOOP
                  Current_Element(Src_Set, Src_Task, Src_Iterator);

                  Succ_Set := Get_Successors_List (Sys.Dependencies,
                     Src_Task);
                  EXIT WHEN Get_Number_Of_Elements (Succ_Set) = 0;

                  -- Then, for each of these successors, we have to check it hasn't been
                  -- already processed. If it hasn't been, we can lay it out depending
                  -- of it's number in the current column.
                  --
                  IF NOT Is_Empty(Succ_Set) THEN
                     Reset_Iterator (Succ_Set, An_Iterator);
                     LOOP
                        Current_Element (Succ_Set, A_Task, An_Iterator);

                        BEGIN
                           IF Search_Task (Done_Set, A_Task.Name) = NULL THEN
                              NULL;
                           END IF;

                        EXCEPTION
                           WHEN Task_Set.Task_Not_Found =>
                              IF (Has_Successor (Sys.Dependencies,
                                       A_Task)) THEN
                                 Move_To (Win.Buffer,
                                    Get_Item_Representing_Task (Win.
                                       Buffer, A_Task),
                                    Gint (Borders_Space + Cur_Column * (
                                          Guint (
                                             Graph_Window_Pkg.
                                             Item_Width) +
                                          Space_Between_Items)),
                                    Gint (Borders_Space + Cur_Item * (
                                          Guint (
                                             Graph_Window_Pkg.
                                             Item_Height) +
                                          Space_Between_Items)));

                                 Cur_Item := Cur_Item + 1;

                                 Add (Dest_Set, A_Task);
                                 Add (Done_Set, A_Task);
                              ELSE
                                 Add (Pit_Set, A_Task);
                                 Add (Done_Set, A_Task);
                              END IF;
                        END;

                        EXIT WHEN Is_Last_Element (Succ_Set,
                           An_Iterator);
                        Next_Element (Succ_Set, An_Iterator);
                     END LOOP;
                  END IF;
                  EXIT WHEN Is_Last_Element (Src_Set, Src_Iterator);
                  Next_Element (Src_Set, Src_Iterator);

               END LOOP;
            END IF;
            EXIT WHEN Get_Number_Of_Elements (Dest_Set) = 0;

            Src_Set := Dest_Set;
            Cur_Column := Cur_Column + 1;
         END LOOP;
         IF NOT Is_Empty(Pit_Set) THEN
            Reset_Iterator (Pit_Set, An_Iterator);

            FOR X IN 1..Get_Number_Of_Elements (Pit_Set) LOOP
               Current_Element (Pit_Set, A_Task, An_Iterator);

               Move_To (Win.Buffer,
                  Get_Item_Representing_Task (Win.Buffer, A_Task),
                  Gint (Borders_Space + Cur_Column * (Guint (
                           Graph_Window_Pkg.
                           Item_Width) + Space_Between_Items)),
                  Gint (Borders_Space + Cur_Item * (Guint (
                           Graph_Window_Pkg.
                           Item_Height) + Space_Between_Items)));
               Cur_Item := Cur_Item + 1;
               Next_Element (Pit_Set, An_Iterator);
            END LOOP;
         END IF;
         Refresh_Canvas (Win.Buffer);
         Win.Cur_Graph_Type := G3;
      END IF;
   END Disp3;




   PROCEDURE Callbacks_Init (
         Win : Graph_Window_Access;
         Tb  : Graph_Toolbar_Access) IS

   BEGIN

      Graph_Window_Gint_Cb.Connect(Win,"delete_event",
         Graph_Window_Gint_Cb.To_Marshaller(Close_Graph'ACCESS));

      Canvas_Cb.Connect (Win.Buffer, "zoomed",
         Canvas_Cb.To_Marshaller (Canvas_Zoomed'ACCESS));


      Canvas_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => " Zoom out ",
            Icon => Gtk_Widget(Tb.Pixmap_Zoom_Out)),
         "clicked",
         Canvas_Cb.To_Marshaller (Zoom_Out'ACCESS),
         Slot_Object => Win.Buffer);

      Canvas_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => " Zoom in ",
            Icon => Gtk_Widget(Tb.Pixmap_Zoom_In)),
         "clicked",
         Canvas_Cb.To_Marshaller (Zoom_In'ACCESS),
         Slot_Object => Win.Buffer);


      Append_Space  (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);

      Graph_Window_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => To_String(Lb_Disp1(Current_Language)),
            Icon => Gtk_Widget(Tb.Pixmap_Disp1)),
         "clicked",
         Graph_Window_Cb.To_Marshaller (Disp1'ACCESS),
         Slot_Object => Win);

      Graph_Window_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => To_String(Lb_Disp2(Current_Language)),
            Icon => Gtk_Widget(Tb.Pixmap_Disp2)),
         "clicked",
         Graph_Window_Cb.To_Marshaller (Disp2'ACCESS),
         Slot_Object => Win);

      Graph_Window_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => To_String(Lb_Disp3(Current_Language)),
            Icon => Gtk_Widget(Tb.Pixmap_Disp3)),
         "clicked",
         Graph_Window_Cb.To_Marshaller (Disp3'ACCESS),
         Slot_Object => Win);

      Append_Space  (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);
      Append_Space (Tb.Action_Toolbar);
      Graph_Window_Cb.Object_Connect
         (Append_Item (Tb.Action_Toolbar,
            Tooltip_Text => To_String(Lb_Exit(Current_Language)),
            Icon => Gtk_Widget(Tb.Pixmap_Close)),
         "clicked",
         Graph_Window_Cb.To_Marshaller (Hide_Graph_Window'ACCESS),
         Slot_Object => Win);

   END Callbacks_Init;


END Graph_Callbacks_Pkg ;
