------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gdk.Drawable;
USE Gdk.Drawable;
WITH Gdk.Gc;
USE Gdk.Gc;
WITH Gtk;
USE Gtk;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Project;
USE Project;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Mmi_Tools;
USE Mmi_Tools;
WITH Graph_Add_Pkg;
USE Graph_Add_Pkg;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Buffers;
USE Buffers;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;

WITH Text_Io;
USE Text_Io;

PACKAGE BODY Graph_Item_Pkg IS

   PROCEDURE Item_Object_New (
         Item  :    OUT Item_Object_Access;
         Fig   : IN     Unbounded_String;
         Start : IN     Boolean) IS
   BEGIN
      Item := NEW Item_Object_Record;
      Set_Figure(Item,Fig);
      IF( To_String(Fig) = "task" ) THEN
         Intialize_Task (Item,Start);
      ELSE
         IF( To_String(Fig) = "message" ) THEN
            Intialize_Message (Item,Start);
         ELSE
            IF (To_String(Fig) = "buffer" ) THEN
               Intialize_Buffer (Item,Start);
            END IF;
         END IF;
      END IF;
   END Item_Object_New;


   PROCEDURE Intialize_Task (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean) IS

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)= 0) THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         IF NOT Start THEN
            Gtk_New(Add_New_Item, "task");
         END IF;

         Item.Color := Parse("white");
         Item.W := 50;
         Item.H := 50;
         Set_Id(Item,Last_Item);
         IF (Get_Id(Item)/=0) THEN
            IF Last_Item <= Items_List'Last THEN
               Items_List (Get_Id(Item)) := Canvas_Item (Item);
            END IF;
         END IF;
         Last_Item := Last_Item + 1;
         Set_Screen_Size (Item, Item.W, Item.H);
      END IF;
   END Intialize_Task;

   PROCEDURE Intialize_Message (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean) IS
   BEGIN
      IF NOT Start THEN
         Gtk_New(Add_New_Item, "message");
      END IF;
      Item.Color := Parse("white");
      Item.W := 90;
      Item.H := 50;
      Set_Id(Item,Last_Item);
      IF (Get_Id(Item)/=0) THEN
         IF Last_Item <= Items_List'Last THEN
            Items_List (Get_Id(Item)) := Canvas_Item (Item);
         END IF;
      END IF;
      Last_Item := Last_Item + 1;
      Set_Screen_Size (Item, Item.W, Item.H);
   END Intialize_Message;

   PROCEDURE Intialize_Buffer (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)= 0) THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Item.Color := Parse("white");
         Item.W := 90;
         Item.H := 50;
         Set_Id(Item,Last_Item);
         IF (Get_Id(Item)/=0) THEN
            IF Last_Item <= Items_List'Last THEN
               Items_List (Get_Id(Item)) := Canvas_Item (Item);
            END IF;
         END IF;
         Last_Item := Last_Item + 1;
         Set_Screen_Size (Item, Item.W, Item.H);
         IF NOT Start THEN
            Gtk_New(Add_New_Item, "buffer");
         END IF;
      END IF;
   END Intialize_Buffer;

   PROCEDURE Draw (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint) IS
      Tmp : Unbounded_String;
   BEGIN
      Tmp:=Get_Figure(Item);
      IF(To_String(Tmp) = "task") THEN
         Draw_Task(Item,Canvas,Dest,Xdest,Ydest);
      END IF;
      IF(To_String(Tmp) = "message") THEN
         Draw_Message(Item,Canvas,Dest,Xdest,Ydest);
      END IF;
      IF (To_String(Tmp) = "buffer") THEN
         Draw_Buffer(Item,Canvas,Dest,Xdest,Ydest);
      END IF;
   END Draw;

   PROCEDURE Draw_Task (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint) IS

      Green_Gc : Gdk_Gc;
   BEGIN

      Set_Screen_Size(Item,To_Canvas_Coordinates(Canvas,Item.W),
         To_Canvas_Coordinates(Canvas,Item.H));
      Gdk_New(Green_Gc,Get_Window(Canvas));
      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      IF Item.Selected THEN
         Set_Foreground (Green_Gc, Parse("white"));
         Set_Foreground (Green_Gc, Parse("white"));
         Draw_Arc
            (Drawable           => Dest,
            Gc                 => Green_Gc,
            Filled             => True,
            X                  => Xdest,
            Y                  => Ydest,
            Width              => To_Canvas_Coordinates(Canvas,Item.W),
            Height             => To_Canvas_Coordinates(Canvas,Item.H),
            Angle1             => 0,
            Angle2             => 360*64);
      END IF;

      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      Draw_Arc
         (Drawable           => Dest,
         Gc                 => Green_Gc,
         Filled             => False,
         X                  => Xdest,
         Y                  => Ydest,
         Width              => To_Canvas_Coordinates(Canvas,Item.W),
         Height             => To_Canvas_Coordinates(Canvas,Item.H),
         Angle1             => 0,
         Angle2             => 360*64);

      IF(Get_Name(Item) /= "") THEN
         Load(Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
            (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 4,
            To_String(Get_Name(Item)));
      END IF;
   END Draw_Task;

   PROCEDURE Draw_Message (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint) IS

      Green_Gc : Gdk_Gc;
   BEGIN
      Set_Screen_Size(Item,To_Canvas_Coordinates(Canvas,Item.W),
         To_Canvas_Coordinates(Canvas,Item.H));
      Gdk_New(Green_Gc,Get_Window(Canvas));
      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      IF Item.Selected THEN
         Set_Foreground (Green_Gc, Parse("white"));
         Draw_Rectangle (
            Drawable => Dest,
            Gc       => Green_Gc,
            Filled   => True,
            X        => Xdest,
            Y        => Ydest,
            Width    => To_Canvas_Coordinates(Canvas,Item.W),
            Height   => To_Canvas_Coordinates(Canvas,Item.H));
      END IF;
      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      Draw_Rectangle (
         Drawable => Dest,
         Gc       => Green_Gc,
         Filled   => False,
         X        => Xdest,
         Y        => Ydest,
         Width    => To_Canvas_Coordinates(Canvas,Item.W),
         Height   => To_Canvas_Coordinates(Canvas,Item.H));

      Draw_Line (
         Drawable          => Dest,
         Gc                => Green_Gc,
         X1                => Xdest,
         Y1                => Ydest,
         X2                => Xdest + (To_Canvas_Coordinates(Canvas,Item.
               W)/2),
         Y2                => Ydest + (To_Canvas_Coordinates(Canvas,Item.
               H)/2) );
      Draw_Line (
         Drawable          => Dest,
         Gc                => Green_Gc,
         X1                => Xdest + To_Canvas_Coordinates(Canvas,Item.W),
         Y1                => Ydest,
         X2                => Xdest + (To_Canvas_Coordinates(Canvas,Item.
               W)/2),
         Y2                => Ydest + (To_Canvas_Coordinates(Canvas,Item.
               H)/2) );

      IF(Get_Name(Item) /= "") THEN
         Load(Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
            (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 5,
            To_String(Get_Name(Item)));

      END IF;

   END Draw_Message;

   PROCEDURE Draw_Buffer (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint) IS

      Green_Gc : Gdk_Gc;
   BEGIN
      Set_Screen_Size(Item,To_Canvas_Coordinates(Canvas,Item.W),
         To_Canvas_Coordinates(Canvas,Item.H));
      Gdk_New(Green_Gc, Get_Window(Canvas) );
      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      IF Item.Selected THEN
         Set_Foreground (Green_Gc,Parse("white"));
         Draw_Rectangle (
            Drawable => Dest,
            Gc       => Green_Gc,
            Filled   => True,
            X        => Xdest,
            Y        => Ydest,
            Width    => To_Canvas_Coordinates(Canvas,Item.W),
            Height   => To_Canvas_Coordinates(Canvas,Item.H));
      END IF;
      Set_Foreground (Green_Gc, Gdk.Color.Black (Gtk.Widget.
            Get_Default_Colormap));
      Draw_Rectangle (
         Drawable => Dest,
         Gc       => Green_Gc,
         Filled   => False,
         X        => Xdest,
         Y        => Ydest,
         Width    => To_Canvas_Coordinates(Canvas,Item.W),
         Height   => To_Canvas_Coordinates(Canvas,Item.H));

      IF(Get_Name(Item) /= "") THEN
         Load(Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
            (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 5,
            To_String(Get_Name(Item)));
      END IF;
   END Draw_Buffer;

   PROCEDURE Add_Canvas_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item1,
         Item2  : ACCESS Canvas_Item_Record'Class;
         Text   :        String                          := "") IS
      Link : CONSTANT Canvas_Link :=
      NEW Canvas_Link_Record;
   BEGIN
      Configure (Link, Item1, Item2, End_Arrow, Text);
      Add_Link (Canvas, Link);
      Last_Link := Last_Link + 1;
   END Add_Canvas_Link;

   PROCEDURE Remove_Link_Between (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item1,
         Item2  : ACCESS Item_Object_Record'Class) IS
      FUNCTION Remove_Internal (
            Canvas : ACCESS Interactive_Canvas_Record'Class;
            Link   : ACCESS Canvas_Link_Record'Class)
        RETURN Boolean IS
      BEGIN
         IF(Get_Src (Link) = Canvas_Item(Item1) AND THEN Get_Dest (Link) =
               Canvas_Item(Item2))
               THEN
            Remove_Link (Canvas, Link);
            RETURN False;
         END IF;
         RETURN True;
      END Remove_Internal;
   BEGIN
      For_Each_Link(Canvas, Remove_Internal'Unrestricted_Access);
      Refresh_Canvas (Canvas);
   END Remove_Link_Between;

   PROCEDURE Remove_Item (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Item_Object_Record'Class) IS
      Mes_Iterator : Messages_Iterator;
      Tas_Iterator : Tasks_Iterator;
      Buf_Iterator : Buffers_Iterator;
      A_Message    : Message_Ptr;
      A_Task       : Task_Ptr;
      A_Buffer     : Buffer_Ptr;
   BEGIN
      Remove(Canvas,Item);
      IF Get_Figure(Item) = "message" THEN
         LOOP
            Current_Element(Sys.Messages, A_Message, Mes_Iterator);

            IF A_Message.Name = Get_Name(Item)
                  THEN
               Delete(Sys.Messages,A_Message);
               Is_Saved:=False;
               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Messages, Mes_Iterator);
            Next_Element(Sys.Messages, Mes_Iterator);

         END LOOP;
      END IF;
      IF Get_Figure(Item) = "task" THEN
         LOOP
            Current_Element(Sys.Tasks, A_Task, Tas_Iterator);

            IF A_Task.Name = Get_Name(Item)
                  THEN
               Delete(Sys.Tasks,A_Task);
               Is_Saved:=False;
               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Tasks, Tas_Iterator);
            Next_Element(Sys.Tasks, Tas_Iterator);

         END LOOP;
      END IF;
      IF Get_Figure(Item) = "buffer" THEN
         LOOP
            Current_Element(Sys.Buffers, A_Buffer, Buf_Iterator);

            IF A_Buffer.Name = Get_Name(Item)
                  THEN
               Delete(Sys.Buffers,A_Buffer);
               Is_Saved:=False;
               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Buffers, Buf_Iterator);
            Next_Element(Sys.Buffers, Buf_Iterator);

         END LOOP;
      END IF;
   END Remove_Item;

   PROCEDURE Remove_Item_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Item_Object_Record'Class) IS
      FUNCTION Iternal_Remove (
            Can  : ACCESS Interactive_Canvas_Record'Class;
            Link : ACCESS Canvas_Link_Record'Class)
        RETURN Boolean IS
      BEGIN
         IF (Get_Src (Link) = Canvas_Item(Item)
               OR ELSE Get_Dest (Link) = Canvas_Item(Item)) THEN
            Remove_Link(Can,Link);
         END IF;
         RETURN True;
      END Iternal_Remove;
   BEGIN
      For_Each_Link(Canvas,Iternal_Remove'Unrestricted_Access);
   END Remove_Item_Link;

END Graph_Item_Pkg;
