------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Gdk;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH My_Canvas;
USE My_Canvas;
WITH Gdk.Color;
USE Gdk.Color;
WITH Glib;
USE Glib;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gdk.Font;
USE Gdk.Font;

PACKAGE Graph_Item_Pkg IS

   Items_List : ARRAY (1 .. 500) OF Canvas_Item;
   Last_Item  : Positive                        := 1;
   Last_Link  : Positive                        := 1;
   Font       : Gdk.Font.Gdk_Font               := Gdk.Font.Null_Font;

   TYPE Item_Object_Record IS NEW Canvas_Item_Record WITH
      RECORD
         Color    : Gdk.Color.Gdk_Color;
         W,
         H        : Gint;
         Selected : Boolean             := False;
      END RECORD;
   TYPE Item_Object_Access IS ACCESS ALL Item_Object_Record'Class;

   PROCEDURE Item_Object_New (
         Item  :    OUT Item_Object_Access;
         Fig   : IN     Unbounded_String;
         Start : IN     Boolean);


   PROCEDURE Intialize_Task (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean);
   PROCEDURE Intialize_Message (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean);
   PROCEDURE Intialize_Buffer (
         Item  : ACCESS Item_Object_Record;
         Start : IN     Boolean);

   PROCEDURE Draw (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint);

   PROCEDURE Draw_Task (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint);
   PROCEDURE Draw_Message (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint);
   PROCEDURE Draw_Buffer (
         Item   : ACCESS Item_Object_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint);

   PROCEDURE Add_Canvas_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item1,
         Item2  : ACCESS Canvas_Item_Record'Class;
         Text   :        String                          := "");

   PROCEDURE Remove_Link_Between (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item1,
         Item2  : ACCESS Item_Object_Record'Class);

   PROCEDURE Remove_Item (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Item_Object_Record'Class);

   PROCEDURE Remove_Item_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Item_Object_Record'Class);


END Graph_Item_Pkg;

