------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;

WITH Graph_Toolbar_Pkg ;
USE Graph_Toolbar_Pkg;
WITH Graph_Callbacks_Pkg;
USE Graph_Callbacks_Pkg;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Project;
USE Project;
WITH Graph_Item_Pkg;
USE Graph_Item_Pkg;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Tasks;
USE Tasks;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Messages;
USE Messages;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Buffers;
USE Buffers;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Task_Dependencies;
USE Task_Dependencies;
USE Task_Dependencies.Half_Dep_Set;
WITH Dependencies;
USE Dependencies;



PACKAGE BODY Graph_Window_Pkg IS

   PRAGMA Warnings (Off);

   PROCEDURE Rebuild_Canvas (
         Graph_Window : Graph_Window_Access) IS

      FUNCTION Internal_Append (
            Canva : ACCESS Interactive_Canvas_Record'Class;
            Item  : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN
         Remove(Graph_Window.Buffer, Item);

         RETURN True;
      END Internal_Append;

   BEGIN
      IF(Graph_Window_Pkg.Canvas_Is_Created=True) THEN
         For_Each_Item(Graph_Window.Buffer,Internal_Append'
            Unrestricted_Access);

         -- include of task,message and buffer
         -- into the window
         --
         Import_Task(Graph_Window);
         Import_Message(Graph_Window);
         Import_Buffer(Graph_Window);
         Import_Dependencies(Graph_Window);

      END IF;

   END Rebuild_Canvas;

   PROCEDURE Import_Task (
         Graph_Window : Graph_Window_Access) IS

      My_Iterator :          Tasks_Iterator;
      A_Task      :          Task_Ptr;
      Item_Tmp    :          Item_Object_Access;
      H_Position  : CONSTANT Gint               := 60;
      X_Current   :          Gint               := 10;
      Y_Current   :          Gint               := 90;

   BEGIN
      IF(NOT Is_Empty(Sys.Tasks)) THEN
         Reset_Iterator(Sys.Tasks, My_Iterator);
         LOOP
            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            IF Name_Is_New(Graph_Window.Buffer,A_Task.Name) THEN
               Item_Object_New(Item_Tmp,To_Unbounded_String("task"),True);
               Set_Name(Item_Tmp,A_Task.Name);
               Set_Visibility(Item_Tmp,False);
               Put(Graph_Window.Buffer,Item_Tmp);
               Move_To(Graph_Window.Buffer,Item_Tmp,X_Current,Y_Current);
               X_Current := X_Current + H_Position;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Tasks, My_Iterator) = True;
            Next_Element(Sys.Tasks, My_Iterator);
         END LOOP;
      END IF;
   END Import_Task;


   PROCEDURE Import_Message (
         Graph_Window : Graph_Window_Access) IS
      My_Iterator :          Messages_Iterator;
      A_Message   :          Message_Ptr;
      Item_Tmp    :          Item_Object_Access;
      H_Position  : CONSTANT Gint               := 90;
      X_Current   :          Gint               := 10;
      Y_Current   :          Gint               := 20;

   BEGIN
      IF(NOT Is_Empty(Sys.Messages)) THEN
         Reset_Iterator(Sys.Messages, My_Iterator);
         LOOP
            Current_Element(Sys.Messages, A_Message, My_Iterator);
            IF Name_Is_New(Graph_Window.Buffer,A_Message.Name) THEN
               Item_Object_New(Item_Tmp,To_Unbounded_String("message"),
                  True);
               Set_Name(Item_Tmp,A_Message.Name);
               Set_Visibility(Item_Tmp,False);
               Put(Graph_Window.Buffer,Item_Tmp);
               Move_To(Graph_Window.Buffer,Item_Tmp,X_Current,Y_Current);
               X_Current := X_Current + H_Position;
            END IF;
            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator) = True;
            Next_Element(Sys.Messages, My_Iterator);
         END LOOP;
      END IF;
   END Import_Message;



   PROCEDURE Import_Buffer (
         Graph_Window : Graph_Window_Access) IS
      My_Iterator :          Buffers_Iterator;
      A_Buffer    :          Buffer_Ptr;
      Item_Tmp    :          Item_Object_Access;
      H_Position  : CONSTANT Gint               := 90;
      X_Current   :          Gint               := 10;
      Y_Current   :          Gint               := 150;
   BEGIN
      IF(NOT Is_Empty(Sys.Buffers)) THEN
         Reset_Iterator(Sys.Buffers, My_Iterator);
         LOOP
            Current_Element(Sys.Buffers,A_Buffer,My_Iterator);
            IF Name_Is_New(Graph_Window.Buffer,A_Buffer.Name) THEN
               Item_Object_New(Item_Tmp,To_Unbounded_String("buffer"),
                  True);
               Set_Name(Item_Tmp,A_Buffer.Name);
               Set_Visibility(Item_Tmp,False);
               Put(Graph_Window.Buffer,Item_Tmp);
               Move_To(Graph_Window.Buffer,Item_Tmp,X_Current,Y_Current);
               X_Current := X_Current + H_Position;
            END IF;
            EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator) = True;
            Next_Element(Sys.Buffers, My_Iterator);
         END LOOP;
      END IF;
   END Import_Buffer;

   PROCEDURE Import_Dependencies (
         Graph_Window : Graph_Window_Access) IS

      My_Iterator       : Tasks_Dependencies_Iterator;
      A_Half_Dep        : Dependency_Ptr;
      Item1,
      Item2             : Canvas_Item;
      My_Tasks_Set      : Tasks_Set;
      My_Other_Iterator : Tasks_Iterator;
      A_Task            : Task_Ptr;

   BEGIN


      IF NOT Is_Empty(Sys.Dependencies.All) THEN
         Reset_Iterator(Sys.Dependencies.All, My_Iterator);
         LOOP
            Current_Element(Sys.Dependencies.All, A_Half_Dep, My_Iterator);

            IF A_Half_Dep.Depend_Type = Precedence_Dependency
                  THEN
               Item1 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("task"),
                  A_Half_Dep.Precedency_To_Task.Name);
            END IF;
            IF  A_Half_Dep.Depend_Type = Communication_Dependency
                  THEN
               Item1 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("task"),
                  A_Half_Dep.Communication_Dependent_Task.Name);
            END IF;
            IF A_Half_Dep.Depend_Type = Buffer_Dependency THEN
               Item1 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("task"),
                  A_Half_Dep.Buffer_Dependent_Task.Name);
            END IF;


            IF A_Half_Dep.Depend_Type = Precedence_Dependency
                  THEN
               Item2 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("task"),
                  A_Half_Dep.Precedency_From_Task.Name);
            END IF;
            IF A_Half_Dep.Depend_Type = Buffer_Dependency
                  THEN
               Item2 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("buffer"),
                  A_Half_Dep.Buffer_Dependency_Object.Name);
            END IF;
            IF A_Half_Dep.Depend_Type = Communication_Dependency
                  THEN
               Item2 := Search_Item_By_Name(Graph_Window.Buffer,
                  To_Unbounded_String("message"),
                  A_Half_Dep.Communication_Dependency_Object.Name);
            END IF;




            Set_Visibility(Item1, True);
            Set_Visibility(Item2, True);

            IF A_Half_Dep.Depend_Type = Precedence_Dependency
                  THEN
               IF NOT Has_Link_Between(Graph_Window.Buffer,Item2,Item1) THEN
                  Add_Link(Graph_Window.Buffer,Item2,Item1);
               END IF;
            END IF;

            IF A_Half_Dep.Depend_Type = Buffer_Dependency
                  THEN
               IF A_Half_Dep.Buffer_Orientation  =
                     From_Task_To_Object THEN
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item1,Item2) THEN
                     Add_Link(Graph_Window.Buffer,Item1,Item2);
                  END IF;
               ELSE
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item2,Item1) THEN
                     Add_Link(Graph_Window.Buffer,Item2,Item1);
                  END IF;
               END IF;
            END IF;

            IF A_Half_Dep.Depend_Type = Buffer_Dependency
                  THEN
               IF A_Half_Dep.Buffer_Orientation  =
                     From_Task_To_Object THEN
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item1,Item2) THEN
                     Add_Link(Graph_Window.Buffer,Item1,Item2);
                  END IF;
               ELSE
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item2,Item1) THEN
                     Add_Link(Graph_Window.Buffer,Item2,Item1);
                  END IF;
               END IF;
            END IF;

            IF A_Half_Dep.Depend_Type = Communication_Dependency
                  THEN
               IF A_Half_Dep.Communication_Orientation  =
                     From_Task_To_Object THEN
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item1,Item2) THEN
                     Add_Link(Graph_Window.Buffer,Item1,Item2);
                  END IF;
               ELSE
                  IF NOT Has_Link_Between(Graph_Window.Buffer,Item2,Item1) THEN
                     Add_Link(Graph_Window.Buffer,Item2,Item1);
                  END IF;
               END IF;
            END IF;



            EXIT WHEN Is_Last_Element(Sys.Dependencies.All, My_Iterator) =
               True;
            Next_Element(Sys.Dependencies.All, My_Iterator);
         END LOOP;
      END IF;


   END Import_Dependencies;

   PROCEDURE Gtk_New (
         Graph_Window : IN OUT Graph_Window_Access) IS

   BEGIN

      IF  Is_Hidden=False THEN
         Graph_Window := NEW Graph_Window_Record;
         Is_Hidden:=True;
         Graph_Window_Pkg.Initialize (Graph_Window);
      ELSE
         -- include of tasks, messages and buffers
         -- into the window
         --
         Import_Task(Graph_Window);
         Import_Message(Graph_Window);
         Import_Buffer(Graph_Window);
         Import_Dependencies(Graph_Window);

         Show(Graph_Window);
      END IF;

   END Gtk_New;

   PROCEDURE Initialize (
         Graph_Window : Graph_Window_Access) IS

      Toolbar : Graph_Toolbar_Access;

   BEGIN

      Graph_Window_Pkg.Canvas_Is_Created:=True;

      -- A child window
      --
      Gtk.Window.Initialize (Graph_Window,Window_Toplevel);
      Set_Policy(Graph_Window,
         Allow_Shrink=>False,
         Allow_Grow=> True,
         Auto_Shrink=>True);

      Set_Title (Graph_Window, To_String(Lb_Graph_View(Current_Language)));
      Set_Usize (Graph_Window, 600,400);


      Gtk_New_Vbox (Graph_Window.V_Box,
         Homogeneous => False,
         Spacing     => 0);
      Add(Gtk_Window(Graph_Window), Graph_Window.V_Box);

      New_Tool_Bar (Toolbar, Graph_Window);
      Add(Graph_Window.V_Box, Toolbar.Action_Toolbar);
      Add(Graph_Window.V_Box, Toolbar.Disp_Toolbar);

      -- we want to scroll with Large graphics
      --
      Gtk_New (Graph_Window.Scrolled);
      Set_Usize (Graph_Window.Scrolled, 600, 350);
      Add (Graph_Window.V_Box, Graph_Window.Scrolled);

      Gtk_New (Graph_Window.Buffer);

      Add(Graph_Window.Scrolled, Graph_Window.Buffer);
      Show_All(Graph_Window);
      Settoolbar (Graph_Window.Buffer, Toolbar);

      Gdk_New (Graph_Window.Green_Gc, Get_Window (Graph_Window.Buffer));

      -- Store the preferences
      --
      Graph_Window.G1_Borders_Space :=
         Guint (200);
      Graph_Window.G1_Space_Between_Items :=
         Guint (100);
      Graph_Window.G2_Borders_Space :=
         Guint (200);
      Graph_Window.G2_Force_Circle := True;
      Graph_Window.G3_Borders_Space :=
         Guint (100);
      Graph_Window.G3_Space_Between_Items :=
         Guint (150);

      Graph_Callbacks_Pkg.Callbacks_Init(Graph_Window,Toolbar);
      Show_All(Graph_Window);

      -- Include of task,message and buffer
      --
      Import_Task(Graph_Window);
      Import_Message(Graph_Window);
      Import_Buffer(Graph_Window);
      Import_Dependencies(Graph_Window);
      Is_Hidden:=True;

      -- Load Item x/y positions
      --
      Set_Coordinates(Graph_Window.Buffer, Sys);


   END Initialize;

END Graph_Window_Pkg;


