------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Gdk.Gc;
USE Gdk.Gc;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Viewport;
USE Gtk.Viewport;
WITH My_Canvas;
USE My_Canvas;
WITH Glib;
USE Glib;

PACKAGE Graph_Window_Pkg IS

   -- Graph type specification
   TYPE Graph_Type IS
         (G1,
          G2,
          G3);

   TYPE Graph_Window_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Scrolled : Gtk_Scrolled_Window;
         Buffer   : Interactive_Canvas;
         Green_Gc : Gdk_Gc;
         Viewport : Gtk_Viewport;
         V_Box    : Gtk_Vbox;

         Cur_Graph_Type : Graph_Type;

         G1_Borders_Space       : Guint;
         G1_Space_Between_Items : Guint;
         G2_Borders_Space       : Guint;
         G2_Force_Circle        : Boolean;
         G3_Borders_Space       : Guint;
         G3_Space_Between_Items : Guint;
      END RECORD;



   TYPE Graph_Window_Access IS ACCESS ALL Graph_Window_Record;

   PROCEDURE Rebuild_Canvas (
         Graph_Window : Graph_Window_Access);
   PROCEDURE Import_Task (
         Graph_Window : Graph_Window_Access);
   PROCEDURE Import_Message (
         Graph_Window : Graph_Window_Access);
   PROCEDURE Import_Buffer (
         Graph_Window : Graph_Window_Access);
   PROCEDURE Import_Dependencies (
         Graph_Window : Graph_Window_Access);

   PROCEDURE Gtk_New (
         Graph_Window : IN OUT Graph_Window_Access);

   PROCEDURE Initialize (
         Graph_Window : Graph_Window_Access);

   Selection_Mode    :          Boolean             := True;
   Max_Size          : CONSTANT                     := 400;
   Is_Hidden         :          Boolean             := False;
   Graph_Window      :          Graph_Window_Access;
   Canvas_Is_Created :          Boolean             := False;

   -- Default window size
   Default_Window_Width  : CONSTANT Gint := 400;
   Default_Window_Height : CONSTANT Gint := 300;

   -- Canvas item specification
   Item_Width  : CONSTANT Gint := 35;
   Item_Height : CONSTANT Gint := 35;


END Graph_Window_Pkg;



