-----------------------------------------------------------------------
--          GtkAda - Ada95 binding for the Gimp Toolkit              --
--                                                                   --
--                     Copyright (C) 1998-2000                       --
--        Emmanuel Briot, Joel Brobecker and Arnaud Charlet          --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or X1(at your option) any later version.--
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

WITH Ada.Numerics.Elementary_Functions;
USE Ada.Numerics.Elementary_Functions;
WITH Gdk.Color;
USE Gdk.Color;
WITH Gdk.Cursor;
USE Gdk.Cursor;
WITH Gdk.Drawable;
USE Gdk.Drawable;
WITH Gdk.Event;
USE Gdk.Event;
WITH Gdk.Font;
USE Gdk.Font;
WITH Gdk.Gc;
USE Gdk.Gc;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gdk.Types;
USE Gdk.Types;
WITH Gdk.Types.Keysyms;
USE Gdk.Types.Keysyms;
WITH Gdk.Rectangle;
USE Gdk.Rectangle;
WITH Gdk.Window;
USE Gdk.Window;
WITH Glib;
USE Glib;
WITH Gtk.Adjustment;
USE Gtk.Adjustment;
WITH Gtk.Drawing_Area;
USE Gtk.Drawing_Area;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Extra.Psfont;
USE Gtk.Extra.Psfont;
WITH Gtk.Handlers;
WITH Gtkada.Handlers;
USE Gtkada.Handlers;
WITH Gtk.Arguments;
USE Gtk.Arguments;
WITH Gtk.Main;
USE Gtk.Main;
PRAGMA Elaborate_All (Gtk.Main);
WITH Gtk.Style;
USE Gtk.Style;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Object;
USE Gtk.Object;
WITH Unchecked_Deallocation;
WITH Gtk.Gentry ;
USE Gtk.Gentry;
WITH Gtk.Radio_Button;
USE Gtk.Radio_Button;
WITH Graph_Item_Pkg;
USE Graph_Item_Pkg;
WITH Gtk.Combo;
USE Gtk.Combo;
WITH Interfaces.C.Strings;
USE Interfaces.C.Strings;
WITH Project;
USE Project;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH System;

WITH Dependencies;
USE Dependencies;
WITH Task_Dependencies;
USE Task_Dependencies;
WITH New_Buffer_Role_Pkg;
USE New_Buffer_Role_Pkg;
WITH Text_Io;
USE Text_Io;


WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
WITH Buffers;
USE Buffers;
WITH Buffer_Set;
USE Buffer_Set;
WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;


PACKAGE BODY My_Canvas IS

   USE TYPE Gdk_Font;
   USE TYPE Gdk_Gc;
   USE TYPE   Gdk_Window,
   Gdk_Pixmap;
   USE TYPE System.Address;

   Class_Record : System.Address := System.Null_Address;
   --  This pointer will keep a pointer to the C 'class record' for
   --  gtk. To avoid allocating memory for each widget, this may be done
   --  only once, and reused.
   --  ??? This is a global variable.
   Timeout_Between_Scrolls : CONSTANT := 10;
   --  Time between two scrollings when the mouse is in the bounding box.
   Timeout_Between_Zooms : CONSTANT := 20;
   --  Time between two zooms when smooth-scrolling the canvas
   Scrolling_Margin : CONSTANT := 10;
   --  Width and height of the surrounding box in which "infinite"
   --  scrolling is provided.
   Scrolling_Amount : CONSTANT := 4;
   --  Number of pixels to scroll while the mouse is in the surrounding box.
   Use_Double_Buffer : CONSTANT Boolean := True;
   --  Whether drawing should be done through a double buffer.
   --  The only reason to set this to False is if you don't have any link in
   --  the canvas and the items are fast to draw (for instance if they use
   --  double-buffer themselves).
   --  ??? This could be configurable at the canvas level
   --  ??? This might be set to False with gtk+2.0
   Scroll_Adj_Signal : CONSTANT                 := 4;
   Signals           : CONSTANT Chars_Ptr_Array := (1 => New_String ("background_click"),
     2 => New_String ("item_selected"), 3 => New_String ("zoomed"), Scroll_Adj_Signal
     => New_String ("set_scroll_adjustments"));
   --  Array of the signals created for this widget
   -----------------
   -- Subprograms --
   -----------------
   --  Note: Some callbacks take Gtk_Widget_Record parameters, so that we can
   --  reuse the callbacks in Gtkada.Handlers, and thus save a lot of space
   --  in the GtkAda library.
   PROCEDURE Free IS
   NEW Unchecked_Deallocation (String, String_Access);
   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Canvas_Link_List_Record, Canvas_Link_List);
   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Canvas_Link_Record'Class, Canvas_Link);
   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Canvas_Item_List_Record, Canvas_Item_List);
   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Canvas_Item_Record'Class, Canvas_Item);
   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Item_Selection_List_Record, Item_Selection_List);

   PACKAGE Canvas_Timeout IS NEW Gtk.Main.Timeout (Interactive_Canvas);


   FUNCTION Gettoolbar (
         Canvas : ACCESS Interactive_Canvas_Record'Class)
     RETURN Graph_Toolbar_Access IS
   BEGIN
      RETURN Canvas.Toolbar;
   END Gettoolbar;

   PROCEDURE Settoolbar (
         Canvas  : ACCESS Interactive_Canvas_Record'Class;
         Toolbar :        Graph_Toolbar_Access) IS
   BEGIN
      Canvas.Toolbar := Toolbar;
   END Settoolbar;

   FUNCTION Expose (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk.Event.Gdk_Event)
     RETURN Boolean;
   --  Handle the expose events for a canvas.

   PROCEDURE Realized (
         Canvas : ACCESS Gtk_Widget_Record'Class);
   --  Create all the graphic contexts required for the animation.

   PROCEDURE Canvas_Destroyed (
         Canvas : ACCESS Gtk_Widget_Record'Class);
   --  Called when the canvas is being destroyed. All the items and links
   --  are removed, and the double-buffer is freed

   PROCEDURE Update_Links (
         Canvas   : ACCESS Interactive_Canvas_Record'Class;
         Window   :        Gdk_Window;
         Gc       : IN     Gdk.Gc.Gdk_Gc;
         Selected : IN     Item_Selection_List             := NULL);
   --  Redraw all the links in the canvas.
   --  If Item is not null, only the links to or from Item are redrawn.

   FUNCTION Configure_Handler (
         Canv : ACCESS Gtk_Widget_Record'Class)
     RETURN Boolean;
   --  When the item is resized.

   FUNCTION Button_Pressed (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean;
   --  Called when the user has pressed the mouse button in the canvas.
   --  This tests whether an item was selected.

   FUNCTION Button_Release (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean;
   --  Called when the user has released the mouse button.
   --  If an item was selected, this refreshed the canvas.

   FUNCTION Button_Motion (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean;
   --  Called when the user moves the mouse while a button is pressed.
   --  If an item was selected, the item is moved.

   FUNCTION Key_Press (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean;
   --  Handle key events, to provide scrolling through Page Up, Page Down, and
   --  arrow keys.

   PROCEDURE Clip_Line (
         Rect  :        Gdk.Rectangle.Gdk_Rectangle;
         To_X  : IN     Gint;
         To_Y  : IN     Gint;
         X_Pos : IN     Gfloat;
         Y_Pos : IN     Gfloat;
         X_Out :    OUT Gint;
         Y_Out :    OUT Gint);
   --  Clip the line that goes from the middle of Rect to (To_X, To_Y).
   --  The intersection between that line and the border of Rect is returned
   --  in (X_Out, Y_Out).
   --  X_Pos and Y_Pos have the same meaning as Src_X_Pos and Src_Y_Pos in the
   --  link record.

   PROCEDURE Draw_Straight_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link);
   --  Draw Link on the screen as a straight line.
   --  This link includes both an arrow head on its destination, and an
   --  optional text displayed approximatively in its middle.

   PROCEDURE Draw_Arc_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link;
         Offset :        Gint);
   --  Draw Link on the screen.
   --  The link is drawn as a curved link (ie there is an extra handle in its
   --  middle).
   --  This link includes both an arrow head on its destination, and an
   --  optional text displayed approximatively in its middle.

   PROCEDURE Draw_Self_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link;
         Offset :        Gint);
   --  Draw a link when its source and destination items are the same

   PROCEDURE Update_Adjustments (
         Canvas : ACCESS Interactive_Canvas_Record'Class);
   --  Update the adjustments of the canvas.
   --  The bounds for the adjustments are automatically computed, given the
   --  list of items in it.

   PROCEDURE Draw_Arrow_Head (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         X,
         Y      :        Gint;
         Angle  : IN     Float);
   --  Draw an arrow head at the position (X, Y) on the canvas. The position
   --  is given in pixels, and should include zoom processing.
   --  Angle is the angle of the main axis of the arrow.

   PROCEDURE Draw_Annotation (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         X,
         Y      :        Gint;
         Str    :        String_Access);
   --  Print an annotation on the canvas.
   --  The annotation is centered around (X, Y), in pixels. These coordinates
   --  should already include zoom processing.

   PROCEDURE Set_Scroll_Adjustments (
         Canvas : ACCESS Gtk_Widget_Record'Class;
         Args   :        Gtk_Args);
   --  Change the two adjustments used for the canvas (in a callback)

   PROCEDURE Scrolled (
         Canvas : ACCESS Gtk_Widget_Record'Class);
   --  Called everytime the value of one of the adjustments is changed.

   PROCEDURE Get_Bounding_Box (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         X_Min,
         X_Max,
         Y_Min,
         Y_Max  :    OUT Gint);
   --  Find the smallest bounding box for all the items in the canvas.
   --  Note that this does not include links, which might thus be found
   --  outside of this box.
   --  The returned values are in canvas coordinates

   PROCEDURE Test_Scrolling_Box (
         Canvas   : ACCESS Gtk.Widget.Gtk_Widget_Record'Class;
         Win      :        Gdk_Window;
         X,
         Y        :        Gint;
         X_Scroll :    OUT Gint;
         Y_Scroll :    OUT Gint);
   --  We keep moving the selection (and scrolling the canvas) as long as the
   --  mouse remains in a surrounding box around the canvas. This is done even
   --  if the mouse doesn't move, so at to make it easier to move items.
   --  This subprogram tests whether (X, Y) is found in that box, and returns
   --  the extra scrolling that should be done. (0, 0) is returned if the
   --  mouse is not in that box.

   FUNCTION Scrolling_Timeout (
         Canvas : Interactive_Canvas)
     RETURN Boolean;
   --  Function called repeatedly while the mouse is in the scrolling box.
   --  This provides scrolling even when the mouse doesn't move

   FUNCTION Move_Selection (
         Canvas  : ACCESS Interactive_Canvas_Record'Class;
         Delta_X,
         Delta_Y :        Gint)
     RETURN Boolean;
   --  Moves the selected items by a given number of pixels.
   --  Return True if the selection was actually moved, False if for some
   --  reason nothing happened

   PROCEDURE Show_Item (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Canvas_Item_Record'Class;
         X,
         Y      :        Gint);
   --  Like Show_Item, but use X, Y for the coordinates instead of the
   --  ones available in Item.

   PROCEDURE Draw_Dashed_Selection (
         Canvas : ACCESS Interactive_Canvas_Record'Class);
   --  Draw all the selected items and links with dashed-lines

   FUNCTION Zoom_Timeout (
         Canvas : Interactive_Canvas)
     RETURN Boolean;
   --  Timeout function used to provide smooth zooming.

   PROCEDURE Zoom_Internal (
         Canvas  : ACCESS Interactive_Canvas_Record'Class;
         Percent :        Guint);
   --  Internal function to implement zooming

   ---------------------------
   -- To_Canvas_Coordinates --
   ---------------------------

   FUNCTION To_Canvas_Coordinates (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         X      :        Gint)
     RETURN Gint IS
   BEGIN
      RETURN X * Gint (Canvas.Zoom) / 100;
   END To_Canvas_Coordinates;

   --------------------------
   -- To_World_Coordinates --
   --------------------------

   FUNCTION To_World_Coordinates (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         X      :        Gint)
     RETURN Gint IS
   BEGIN
      RETURN X * 100 / Gint (Canvas.Zoom);
   END To_World_Coordinates;

   -------------
   -- Gtk_New --
   -------------

   PROCEDURE Gtk_New (
         Canvas :    OUT Interactive_Canvas) IS
   BEGIN
      Canvas := NEW Interactive_Canvas_Record;
      My_Canvas.Initialize (Canvas);
   END Gtk_New;

   ----------------
   -- Initialize --
   ----------------

   PROCEDURE Initialize (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
      Signal_Parameters : CONSTANT Signal_Parameter_Types := (1 => (1 =>
        Gtk.Gtk_Type_Gdk_Event, 2 => Gtk.Gtk_Type_None), 2 => (1 => Gtk.Gtk_Type_Pointer,
        2 => Gtk.Gtk_Type_None), 3 => (1 => Gtk.Gtk_Type_Uint, 2 => Gtk.Gtk_Type_None),
        4 => (1 => Gtk.Gtk_Type_Object, 2 => Gtk.Gtk_Type_Object));
      --  the parameters for the above signals.
      --  This must be defined in this function rather than at the
      --  library-level, or the value of Gtk_Type_Gdk_Event is not yet
      --  initialized.
   BEGIN
      Gtk.Drawing_Area.Initialize (Canvas);

      --  The following call is required to initialize the class record,
      --  and the new signals created for this widget.
      --  Note also that we keep Class_Record, so that the memory allocation
      --  is done only once.
      Gtk.Object.Initialize_Class_Record
         (Canvas, Signals, Class_Record, Signal_Parameters,
         Scroll_Adj_Signal);

      Return_Callback.Connect
         (Canvas, "expose_event",
         Return_Callback.To_Marshaller (Expose'ACCESS));
      Widget_Callback.Connect
         (Canvas, "realize",
         Widget_Callback.To_Marshaller (Realized'ACCESS));
      Return_Callback.Connect
         (Canvas, "button_press_event",
         Return_Callback.To_Marshaller (Button_Pressed'ACCESS));
      Return_Callback.Connect
         (Canvas, "button_release_event",
         Return_Callback.To_Marshaller (Button_Release'ACCESS));
      Return_Callback.Connect
         (Canvas, "motion_notify_event",
         Return_Callback.To_Marshaller (Button_Motion'ACCESS));
      Return_Callback.Connect
         (Canvas, "key_press_event",
         Return_Callback.To_Marshaller (Key_Press'ACCESS));
      Return_Callback.Connect
         (Canvas, "configure_event",
         Return_Callback.To_Marshaller (Configure_Handler'ACCESS));
      Widget_Callback.Connect
         (Canvas, "set_scroll_adjustments", Set_Scroll_Adjustments'ACCESS);
      Widget_Callback.Connect
         (Canvas, "destroy",
         Widget_Callback.To_Marshaller (Canvas_Destroyed'ACCESS));

      --  We want to be sure to get all the mouse events, that are required
      --  for the animation.

      Add_Events (Canvas, Gdk.Types.Button_Press_Mask
         OR Gdk.Types.Button_Motion_Mask
         OR Gdk.Types.Button_Release_Mask
         OR Gdk.Types.Key_Press_Mask
         OR Gdk.Types.Key_Release_Mask);
      Set_Flags (Canvas, Can_Focus);

      --  Configure with default values
      Configure (Canvas);
      Set_Scroll_Adjustments (Canvas, Null_Adjustment, Null_Adjustment);
      Canvas.Has_Item_Selected := False;

   END Initialize;

   ----------------------
   -- Canvas_Destroyed --
   ----------------------

   PROCEDURE Canvas_Destroyed (
         Canvas : ACCESS Gtk_Widget_Record'Class) IS
      C        : CONSTANT Interactive_Canvas := Interactive_Canvas (Canvas);
      Tmp,
      Child    :          Canvas_Item_List;
      Tmp2,
      Link_L   :          Canvas_Link_List;
      Link,
      Tmp_Link :          Canvas_Link;
   BEGIN
      IF C.Scrolling_Timeout_Id /= 0 THEN
         Timeout_Remove (C.Scrolling_Timeout_Id);
      END IF;

      Clear_Selection (C);

      Child := C.Children;
      WHILE Child /= NULL LOOP
         Tmp := Child.Next;
         Destroy (Child.Item);
         Free (Child.Item);
         Free (Child);
         Child := Tmp;
      END LOOP;

      Link_L := C.Links;
      WHILE Link_L /= NULL LOOP
         Tmp2 := Link_L.Next;
         Link := Link_L.Link;
         WHILE Link /= NULL LOOP
            Tmp_Link := Link.Sibling;
            Destroy (Link);
            Free (Link);
            Link := Tmp_Link;
         END LOOP;
         Free (Link_L);
         Link_L := Tmp2;
      END LOOP;

      Free (C.Annotation_Font);
      Unref (C.Clear_Gc);
      Unref (C.Black_Gc);
      Unref (C.Anim_Gc);
      Unref (C.Font);
      Destroy (C.Hadj);
      Destroy (C.Vadj);
      Gdk.Pixmap.Unref (C.Double_Buffer);
   END Canvas_Destroyed;

   ----------------------------
   -- Set_Scroll_Adjustments --
   ----------------------------

   PROCEDURE Set_Scroll_Adjustments (
         Canvas : ACCESS Gtk_Widget_Record'Class;
         Args   :        Gtk_Args) IS
      Hadj : CONSTANT System.Address     := To_Address (Args, 1);
      Vadj : CONSTANT System.Address     := To_Address (Args, 2);
      Canv : CONSTANT Interactive_Canvas := Interactive_Canvas (Canvas);

      Stub : Gtk_Adjustment_Record;
   BEGIN
      IF Canv.Hadj /= NULL THEN
         Unref (Canv.Hadj);
      END IF;

      IF Hadj /= System.Null_Address THEN
         Canv.Hadj := Gtk_Adjustment (Gtk.Get_User_Data (Hadj, Stub));
      ELSE
         Gtk_New (Canv.Hadj, 0.0, 0.0, 0.0, 1.0, 1.0, 10.0);
      END IF;
      Ref (Canv.Hadj);
      Sink (Canv.Hadj);

      IF Canv.Vadj /= NULL THEN
         Unref (Canv.Vadj);
      END IF;

      IF Vadj /= System.Null_Address THEN
         Canv.Vadj := Gtk_Adjustment (Gtk.Get_User_Data (Vadj, Stub));
      ELSE
         Gtk_New (Canv.Vadj, 0.0, 0.0, 0.0, 1.0, 1.0, 10.0);
      END IF;
      Ref (Canv.Vadj);
      Sink (Canv.Vadj);

      Widget_Callback.Object_Connect
         (Canv.Hadj, "value_changed",
         Widget_Callback.To_Marshaller (Scrolled'ACCESS), Canv);
      Widget_Callback.Object_Connect
         (Canv.Vadj, "value_changed",
         Widget_Callback.To_Marshaller (Scrolled'ACCESS), Canv);

      Update_Adjustments (Canv);
   END Set_Scroll_Adjustments;

   ---------------
   -- Configure --
   ---------------

   PROCEDURE Configure (
         Canvas            : ACCESS Interactive_Canvas_Record;
         Grid_Size         :        Guint                     := Default_Grid_Size;
         Annotation_Font   :        String                    := Default_Annotation_Font;
         Annotation_Height :        Gint                      := Default_Annotation_Height;
         Arc_Link_Offset   :        Gint                      := Default_Arc_Link_Offset;
         Arrow_Angle       :        Gint                      := Default_Arrow_Angle;
         Arrow_Length      :        Gint                      := Default_Arrow_Length;
         Motion_Threshold  :        Gint                      := Default_Motion_Threshold) IS
   BEGIN
      Canvas.Grid_Size := Grid_Size;

      IF Grid_Size < 2 THEN
         Canvas.Align_On_Grid := False;
      END IF;

      Free (Canvas.Annotation_Font);
      Canvas.Annotation_Font := NEW String'(Annotation_Font);
      Canvas.Annotation_Height := Annotation_Height;
      Canvas.Arc_Link_Offset := Arc_Link_Offset;
      Canvas.Arrow_Angle := Float (Arrow_Angle) * Ada.Numerics.Pi / 180.0;
      Canvas.Arrow_Length := Arrow_Length;
      Canvas.Motion_Threshold := Motion_Threshold;

      IF Canvas.Font /= Null_Font THEN
         Unref (Canvas.Font);
      END IF;

      Canvas.Font :=
         Get_Gdkfont (Canvas.Annotation_Font.All, Canvas.
         Annotation_Height);
   END Configure;

   -----------------------
   -- Configure_Handler --
   -----------------------

   FUNCTION Configure_Handler (
         Canv : ACCESS Gtk_Widget_Record'Class)
     RETURN Boolean IS
      Canvas : CONSTANT Interactive_Canvas := Interactive_Canvas (Canv);
   BEGIN
      Set_Page_Size (Canvas.Hadj, Gfloat (Get_Allocation_Width (Canvas)));
      Set_Step_Increment (Canvas.Hadj, 10.0);
      Set_Page_Increment (Canvas.Hadj, Get_Page_Size (Canvas.Hadj) / 2.0);

      Set_Page_Size (Canvas.Vadj, Gfloat (Get_Allocation_Height (Canvas)));
      Set_Step_Increment (Canvas.Vadj, 10.0);
      Set_Page_Increment (Canvas.Vadj, Get_Page_Size (Canvas.Vadj) / 2.0);

      Update_Adjustments (Canvas);

      IF Canvas.Double_Buffer /= NULL THEN
         Gdk.Pixmap.Unref (Canvas.Double_Buffer);
      END IF;
      Gdk_New
         (Canvas.Double_Buffer, Get_Window (Canvas),
         Gint (Get_Allocation_Width (Canvas)),
         Gint (Get_Allocation_Height (Canvas)));
      RETURN False;
   END Configure_Handler;

   -------------------
   -- Align_On_Grid --
   -------------------

   PROCEDURE Align_On_Grid (
         Canvas : ACCESS Interactive_Canvas_Record;
         Align  :        Boolean                   := True) IS
   BEGIN
      Canvas.Align_On_Grid := (Canvas.Grid_Size >= 2) AND THEN Align;
   END Align_On_Grid;

   ----------------------
   -- Get_Bounding_Box --
   ----------------------

   PROCEDURE Get_Bounding_Box (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         X_Min,
         X_Max,
         Y_Min,
         Y_Max  :    OUT Gint) IS
      Current : Canvas_Item_List := Canvas.Children;
      X,
      Y       : Gint;
   BEGIN
      IF Current = NULL THEN
         X_Min := 0;
         X_Max := 0;
         Y_Min := 0;
         Y_Max := 0;

      ELSE
         X_Min := Gint'Last;
         X_Max := Gint'First;
         Y_Min := Gint'Last;
         Y_Max := Gint'First;

         WHILE Current /= NULL LOOP
            IF Current.Item.Visible THEN
               X := To_Canvas_Coordinates (Canvas, Current.Item.Coord.X);
               Y := To_Canvas_Coordinates (Canvas, Current.Item.Coord.Y);
               X_Max := Gint'Max (X_Max,  X + Gint (Current.Item.Coord.
                     Width));
               X_Min := Gint'Min (X_Min, X);
               Y_Max := Gint'Max (Y_Max, Y + Gint (Current.Item.Coord.
                     Height));
               Y_Min := Gint'Min (Y_Min, Y);
            END IF;

            Current := Current.Next;
         END LOOP;
      END IF;
   END Get_Bounding_Box;

   ------------------------
   -- Update_Adjustments --
   ------------------------

   PROCEDURE Update_Adjustments (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
      X_Max,
      Y_Max,
      X_Min,
      Y_Min : Gint;
   BEGIN
      Get_Bounding_Box (Canvas, X_Min, X_Max, Y_Min, Y_Max);

      --  If the non-visible part of the canvas (left of the displayed area)
      --  contains no item, we can delete that part. However, avoid any visible
      --  scrolling, which is disturbing for the user.
      Set_Lower
         (Canvas.Hadj, Gfloat'Min (Get_Value (Canvas.Hadj), Gfloat (X_Min)));
      Set_Upper
         (Canvas.Hadj,
         Gfloat'Max (Get_Value (Canvas.Hadj) + Get_Page_Size (Canvas.Hadj),
            Gfloat (X_Max)));
      Changed (Canvas.Hadj);

      Set_Lower
         (Canvas.Vadj, Gfloat'Min (Get_Value (Canvas.Vadj), Gfloat (Y_Min)));
      Set_Upper
         (Canvas.Vadj,
         Gfloat'Max (Get_Value (Canvas.Vadj) + Get_Page_Size (Canvas.Vadj),
            Gfloat (Y_Max)));
      Changed (Canvas.Vadj);
   END Update_Adjustments;

   -------------
   -- Move_To --
   -------------

   PROCEDURE Move_To (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class;
         X,
         Y      :        Glib.Gint                 := Glib.Gint'First) IS
      Step      : Gint := Gint (Canvas.Grid_Size);
      Zoom_Step : Gint;

      FUNCTION Location_Is_Free (
            X,
            Y        : Gint;
            Return_X : Boolean)
        RETURN Gint;
      --  Return X if the location (X, Y) for the new item would be
      --  acceptable,
      --  Return the next X coordinate (if Return_X is true) or Y coordinate
      --  (otherwise) to test if (X, Y) overlaps an existing item.
      --  Keeps a space of at least Grid_Size around each item.

      ----------------------
      -- Location_Is_Free --
      ----------------------

      FUNCTION Location_Is_Free (
            X,
            Y        : Gint;
            Return_X : Boolean)
        RETURN Gint IS
         Tmp : Canvas_Item_List := Canvas.Children;

         Dest  :          Gdk.Rectangle.Gdk_Rectangle;
         Inter :          Boolean                     := False;
         W     : CONSTANT Guint                       := Item.Coord.Width;

         H : CONSTANT Guint := Item.Coord.Height;

         X_Tmp,
         Y_Tmp : Gint;
      BEGIN
         --  Keep an appropriate marging around the item.
         Item.Coord.X := X - Zoom_Step;
         Item.Coord.Y := Y - Zoom_Step;
         Item.Coord.Width := W + 2 * Guint (Zoom_Step);
         Item.Coord.Height := H + 2 * Guint (Zoom_Step);

         WHILE Tmp /= NULL LOOP
            IF Tmp.Item.Visible
                  AND THEN Tmp.Item /= Canvas_Item (Item)
                  THEN
               X_Tmp := To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.X);
               Y_Tmp := To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.Y);
               Intersect ((X_Tmp,
                     Y_Tmp,
                     Tmp.Item.Coord.Width,
                     Tmp.Item.Coord.Height),
                  Item.Coord, Dest, Inter);
               EXIT WHEN Inter;
            END IF;

            Tmp := Tmp.Next;
         END LOOP;

         Item.Coord.Width := W;
         Item.Coord.Height := H;
         IF Return_X THEN
            IF Inter THEN
               RETURN X_Tmp + Zoom_Step + Gint (Tmp.Item.Coord.Width);
            END IF;
            RETURN X;
         ELSE
            IF Inter THEN
               RETURN Y_Tmp + Zoom_Step + Gint (Tmp.Item.Coord.Height);
            END IF;
            RETURN Y;
         END IF;
      END Location_Is_Free;

      Src_Item : Canvas_Item      := NULL;
      Links    : Canvas_Link_List := Canvas.Links;
      X1,
      X2       : Gint;
      Y1       : Gint;

   BEGIN
      --  When no grid is drawn...

      IF Step = 0 THEN
         Step := Gint (Default_Grid_Size);
      END IF;
      Zoom_Step := To_Canvas_Coordinates (Canvas, Step);

      IF X /= Gint'First AND THEN Y /= Gint'First THEN
         Item.Coord.X := X;
         Item.Coord.Y := Y;
      ELSE
         --  Check if there is any link that has for destination the widget we
         --  are adding.

         WHILE Links /= NULL LOOP
            IF Links.Link.Src = Canvas_Item (Item)
                  OR ELSE Links.Link.Dest = Canvas_Item (Item)
                  THEN
               Src_Item := Links.Link.Src;
               EXIT;
            END IF;
            Links := Links.Next;
         END LOOP;

         --  The rule is the following when we have a link to an existing
         --  item: We first try to put the new item below the old
         --  one, then, if that place is already occupied, to the
         --  bottom-right, then the bottom-left, then two down, ...

         IF Src_Item /= NULL THEN
            DECLARE
               Num    :          Gint                              := 0;
               Next_Y :          ARRAY (Gint RANGE 0 .. 2) OF Gint;
               X3     : CONSTANT Gint                              := To_Canvas_Coordinates
                 (Canvas, Src_Item.Coord.X);

            BEGIN
               Y1 := To_Canvas_Coordinates (Canvas, Src_Item.Coord.Y)
                  + Gint (Src_Item.Coord.Height) + Zoom_Step;
               Next_Y := (OTHERS => Y1);

               LOOP
                  CASE Num mod 3 IS
                     WHEN 0 =>
                        X1 := X3;
                     WHEN 1 =>
                        X1 := X3 - Zoom_Step - Gint (Item.Coord.Width);
                     WHEN 2 =>
                        X1 := X3 + Zoom_Step + Gint (Src_Item.Coord.Width);
                     WHEN OTHERS =>
                        NULL;
                  END CASE;

                  Y1 := Next_Y (Num mod 3);
                  X2 := Location_Is_Free (X1, Y1, False);
                  EXIT WHEN  X2 = Y1;
                  Next_Y (Num mod 3) := X2;

                  Num := Num + 1;
               END LOOP;
            END;

            --  Else put the item in the first line, at the first possible
            --  location

         ELSE
            X1 := Gint (Get_Lower (Canvas.Hadj)) + Zoom_Step;
            Y1 := Gint (Get_Lower (Canvas.Vadj)) + Zoom_Step;

            LOOP
               X2 := Location_Is_Free (X1, Y1, True);
               EXIT WHEN X2 = X1;
               X1 := X2;
            END LOOP;
         END IF;

         Item.Coord.X := To_World_Coordinates (Canvas, X1);
         Item.Coord.Y := To_World_Coordinates (Canvas, Y1);
      END IF;

      IF Canvas.Align_On_Grid THEN
         IF Item.Coord.X mod Step /= 0 THEN
            Item.Coord.X := Item.Coord.X + Step - Item.Coord.X mod Step;
         END IF;
         IF Item.Coord.Y mod Step /= 0 THEN
            Item.Coord.Y := Item.Coord.Y + Step - Item.Coord.Y mod Step;
         END IF;
      END IF;
   END Move_To;

   ---------
   -- Put --
   ---------

   PROCEDURE Put (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class;
         X,
         Y      :        Gint                      := Gint'First) IS
   BEGIN
      Canvas.Children := NEW Canvas_Item_List_Record
         '(
         Item => Canvas_Item (Item),
         Next => Canvas.Children);
      Move_To (Canvas, Item, X, Y);
      Update_Adjustments (Canvas);
   END Put;

   --------------
   -- Realized --
   --------------

   PROCEDURE Realized (
         Canvas : ACCESS Gtk_Widget_Record'Class) IS
      Canv : CONSTANT Interactive_Canvas := Interactive_Canvas (Canvas);
   BEGIN
      --  Create all the graphic contexts if necessary.
      --  Set Exposures to False, since we want to handle the redraw
      --  events ourselves, and not have them generated automatically
      --  everytime we do a Draw_Pixmap (for optimization purposes)

      IF Canv.Black_Gc = NULL THEN
         Set_Exposures
            (Get_Background_Gc (Get_Style (Canvas), State_Normal), False);

         Gdk_New (Canv.Black_Gc, Get_Window (Canvas));
         Set_Foreground
            (Canv.Black_Gc, Black (Gtk.Widget.Get_Default_Colormap));
         Set_Exposures (Canv.Black_Gc, False);

         Gdk_New (Canv.Clear_Gc, Get_Window (Canvas));
         Set_Foreground
            (Canv.Clear_Gc, Get_Background (Get_Style (Canvas),
               State_Normal));
         Set_Exposures (Canv.Clear_Gc, False);

         --  Note: when setting the line attributes below, it is very important
         --  for the Line_Width to be 0 so has to get algorithms as fast as
         --  possible (1 is way too slow for a proper interaction with the
         --  user).

         Gdk_New (Canv.Anim_Gc, Get_Window (Canvas));
         Set_Function (Canv.Anim_Gc, Invert);
         Set_Line_Attributes
            (Canv.Anim_Gc,
            Line_Width => 0,
            Line_Style => Line_On_Off_Dash,
            Cap_Style  => Cap_Butt,
            Join_Style => Join_Miter);
         Set_Exposures (Canv.Anim_Gc, False);
      END IF;
   END Realized;

   -------------------
   -- For_Each_Item --
   -------------------

   PROCEDURE For_Each_Item (
         Canvas  : ACCESS Interactive_Canvas_Record;
         Execute :        Item_Processor) IS
      Item : Canvas_Item_List := Canvas.Children;
      Tmp  : Canvas_Item_List;
   BEGIN
      WHILE Item /= NULL LOOP
         Tmp := Item.Next;
         EXIT WHEN NOT Execute (Canvas, Item.Item);
         Item := Tmp;
      END LOOP;
   END For_Each_Item;

   ---------------
   -- Clip_Line --
   ---------------

   PROCEDURE Clip_Line (
         Rect  :        Gdk_Rectangle;
         To_X  : IN     Gint;
         To_Y  : IN     Gint;
         X_Pos : IN     Gfloat;
         Y_Pos : IN     Gfloat;
         X_Out :    OUT Gint;
         Y_Out :    OUT Gint) IS
      Src_X : CONSTANT Gint := Rect.X + Gint (Gfloat (Rect.Width) * X_Pos);

      Src_Y : CONSTANT Gint := Rect.Y + Gint (Gfloat (Rect.Height) * Y_Pos);

      Delta_X : CONSTANT Gint := To_X - Src_X;
      Delta_Y : CONSTANT Gint := To_Y - Src_Y;
      Offset  :          Gint;

   BEGIN
      --  Intersection with horizontal sides

      IF Delta_Y /= 0 THEN
         Offset := (Src_X * To_Y - To_X * Src_Y);

         IF Delta_Y < 0 THEN
            --  Intersection with north side ?
            Y_Out := Rect.Y;
         ELSE
            Y_Out := Rect.Y + Gint (Rect.Height);
         END IF;

         X_Out := (Delta_X * Y_Out + Offset) / Delta_Y;

         IF Rect.X <= X_Out
               AND THEN X_Out <= Rect.X + Gint (Rect.Width)
               THEN
            RETURN;
         END IF;
      END IF;

      --  Intersection with vertical sides

      IF Delta_X /= 0 THEN
         Offset := (To_X * Src_Y - Src_X * To_Y);

         IF Delta_X < 0 THEN
            --  Intersection with West side ?
            X_Out := Rect.X;
         ELSE
            --  Intersection with East side ?
            X_Out := Rect.X + Gint (Rect.Width);
         END IF;

         Y_Out := (Delta_Y * X_Out + Offset) / Delta_X;

         IF Rect.Y <= Y_Out
               AND THEN Y_Out <= Rect.Y + Gint (Rect.Height)
               THEN
            RETURN;
         END IF;
      END IF;

      X_Out := 0;
      Y_Out := 0;
   END Clip_Line;

   ---------------------
   -- Draw_Arrow_Head --
   ---------------------

   PROCEDURE Draw_Arrow_Head (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         X,
         Y      :        Gint;
         Angle  : IN     Float) IS
      Length : CONSTANT Float := Float (To_Canvas_Coordinates (Canvas, Canvas.Arrow_Length));











































   BEGIN
      Draw_Polygon
         (Window,
         Gc,
         Filled => True,
         Points =>
         ((X => X, Y => Y),
            (X => X + Gint (Length * Cos (Angle + Canvas.Arrow_Angle)),
               Y => Y + Gint (Length * Sin (Angle + Canvas.Arrow_Angle))),
            (X => X + Gint (Length * Cos (Angle - Canvas.Arrow_Angle)),
               Y => Y + Gint (Length * Sin (Angle - Canvas.Arrow_Angle)))));
   END Draw_Arrow_Head;

   ---------------------
   -- Draw_Annotation --
   ---------------------

   PROCEDURE Draw_Annotation (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         X,
         Y      :        Gint;
         Str    :        String_Access) IS
   BEGIN
      --  Do not draw the text in Xor mode, since this doesn't work on
      --  Windows systems, and doesn't provide any real information anyway.
      IF Gc /= Canvas.Anim_Gc AND THEN Canvas.Font /= NULL THEN
         Draw_Text
            (Window,
            Canvas.Font,
            Gc,
            X - String_Width (Canvas.Font, Str.All) / 2,
            Y - String_Height (Canvas.Font, Str.All) / 2,
            Text => Str.All);
      END IF;
   END Draw_Annotation;

   ------------------------
   -- Draw_Straight_Link --
   ------------------------

   PROCEDURE Draw_Straight_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link) IS
      X1,
      Y1,
      X2,
      Y2    :          Gint;
      Xbase : CONSTANT Gint := Gint (Get_Value (Canvas.Hadj));
      Ybase : CONSTANT Gint := Gint (Get_Value (Canvas.Vadj));
      Sx    : CONSTANT Gint := Link.Src.Coord.X;
      Sy    : CONSTANT Gint := Link.Src.Coord.Y;
      Dx    : CONSTANT Gint := Link.Dest.Coord.X;
      Dy    : CONSTANT Gint := Link.Dest.Coord.Y;
   BEGIN
      Link.Src.Coord.X := To_Canvas_Coordinates (Canvas, Sx) - Xbase;
      Link.Src.Coord.Y := To_Canvas_Coordinates (Canvas, Sy) - Ybase;
      Link.Dest.Coord.X := To_Canvas_Coordinates (Canvas, Dx) - Xbase;
      Link.Dest.Coord.Y := To_Canvas_Coordinates (Canvas, Dy) - Ybase;
      Clip_Line
         (Link.Src.Coord,
         Link.Dest.Coord.X
         + Gint (Gfloat (Link.Dest.Coord.Width) * Link.Dest_X_Pos),
         Link.Dest.Coord.Y
         + Gint (Gfloat (Link.Dest.Coord.Height) * Link.Dest_Y_Pos),
         X_Pos => Link.Src_X_Pos, Y_Pos => Link.Src_Y_Pos,
         X_Out => X1, Y_Out => Y1);
      Clip_Line
         (Link.Dest.Coord,
         Link.Src.Coord.X
         + Gint (Gfloat (Link.Src.Coord.Width) * Link.Src_X_Pos),
         Link.Src.Coord.Y
         + Gint (Gfloat (Link.Src.Coord.Height) * Link.Src_Y_Pos),
         X_Pos => Link.Dest_X_Pos, Y_Pos => Link.Dest_Y_Pos,
         X_Out => X2, Y_Out => Y2);
      Link.Src.Coord.X := Sx;
      Link.Src.Coord.Y := Sy;
      Link.Dest.Coord.X := Dx;
      Link.Dest.Coord.Y := Dy;

      --  Draw the link itself

      Draw_Line (Window, Gc, X1, Y1, X2, Y2);

      --  Draw the end arrow head

      IF Link.Arrow = End_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         IF X1 /= X2 THEN
            Draw_Arrow_Head
               (Canvas, Window, Gc, X2, Y2,
               Arctan (Float (Y1 - Y2), Float (X1 - X2)));
         ELSIF Y1 > Y2 THEN
            Draw_Arrow_Head
               (Canvas, Window, Gc, X2, Y2, Ada.Numerics.Pi / 2.0);
         ELSE
            Draw_Arrow_Head
               (Canvas, Window, Gc, X2, Y2, -Ada.Numerics.Pi / 2.0);
         END IF;
      END IF;

      --  Draw the start arrow head

      IF Link.Arrow = Start_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         IF X1 /= X2 THEN
            Draw_Arrow_Head
               (Canvas, Window, Gc, X1, Y1,
               Arctan (Float (Y2 - Y1), Float (X2 - X1)));
         ELSIF Y1 > Y2 THEN
            Draw_Arrow_Head
               (Canvas, Window, Gc, X1, Y1, -Ada.Numerics.Pi / 2.0);
         ELSE
            Draw_Arrow_Head
               (Canvas, Window, Gc, X1, Y1, +Ada.Numerics.Pi / 2.0);
         END IF;
      END IF;

      --  Draw the text if any

      IF Link.Descr /= NULL THEN
         Draw_Annotation
            (Canvas, Window, Gc,
            (X1 + X2) / 2,
            (Y1 + Y2) / 2,
            Link.Descr);
      END IF;
   END Draw_Straight_Link;

   --------------------
   -- Draw_Self_Link --
   --------------------

   PROCEDURE Draw_Self_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link;
         Offset :        Gint) IS
      Xbase       : CONSTANT Gint  := Gint (Get_Value (Canvas.Hadj));
      Ybase       : CONSTANT Gint  := Gint (Get_Value (Canvas.Vadj));
      Arc_Offset  : CONSTANT Float := Float (To_Canvas_Coordinates (Canvas,
        Canvas.Arc_Link_Offset));
      Right_Angle : CONSTANT Float := Ada.Numerics.Pi / 2.0;
      X1,
      Y1,
      X3,
      Y3,
      Xc,
      Yc,
      Radius      :          Gint;

   BEGIN
      PRAGMA Assert (Link.Src = Link.Dest);
      Xc := To_Canvas_Coordinates
         (Canvas, Link.Src.Coord.X) + Gint (Link.Src.Coord.Width) - Xbase;
      Yc := To_Canvas_Coordinates (Canvas, Link.Src.Coord.Y) - Ybase;
      Radius := Gint (Arc_Offset) / 2 * Offset;

      --  Location of the arrow and the annotation
      X3 := Xc - Radius;
      Y3 := Yc;
      X1 := Xc;
      Y1 := Yc + Radius;

      Draw_Arc (Window,
         Gc,
         Filled => False,
         X      => Xc - Radius,
         Y      => Yc - Radius,
         Width  => Radius * 2,
         Height => Radius * 2,
         Angle1 => -90 * 64,
         Angle2 => 270 * 64);

      --  Draw the arrows

      IF Link.Arrow = End_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         Draw_Arrow_Head (Canvas, Window, Gc, X3, Y3, -Right_Angle);
      END IF;

      IF Link.Arrow = Start_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         Draw_Arrow_Head (Canvas, Window, Gc, X1, Y1, 0.0);
      END IF;

      --  Draw the annotations
      IF Link.Descr /= NULL THEN
         Draw_Annotation
            (Canvas, Window, Gc, Xc + Radius / 2, Yc + Radius / 2, Link.
            Descr);
      END IF;
   END Draw_Self_Link;

   -------------------
   -- Draw_Arc_Link --
   -------------------

   PROCEDURE Draw_Arc_Link (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Window :        Gdk_Window;
         Gc     : IN     Gdk.Gc.Gdk_Gc;
         Link   : IN     Canvas_Link;
         Offset :        Gint) IS
      PROCEDURE Bezier_One_Control (
            X1,
            Y1,
            X2,
            Y2,
            X3,
            Y3   : Gint;
            Step : Float);
      --  Draw a bezier curve with one control point (X2, Y2).

      ------------------------
      -- Bezier_One_Control --
      ------------------------

      PROCEDURE Bezier_One_Control (
            X1,
            Y1,
            X2,
            Y2,
            X3,
            Y3   : Gint;
            Step : Float) IS
         T      : Float := 0.0;
         A,
         B,
         B2,
         C      : Float;
         Old_X  : Gint  := X1;
         Old_Y  : Gint  := Y1;
         Old_X2 : Gint  := X3;
         Old_Y2 : Gint  := Y3;
         New_X,
         New_Y,
         New_X2,
         New_Y2 : Gint;
      BEGIN
         --  The general formula for a quadratic bezier curve is:
         --    P(t)=(1-t)^2 * P0 + 2t(1-t) * P1 + t^2 * P2,  0 <= t <= 1
         --  For optimization purposes, we compute only for 0 <= t <= 0.5, and
         --  draw two points at once.

         WHILE T <= 0.5 LOOP
            A := (1.0 - T);
            B := 2.0 * T * A;
            B2 := B * Float (X2);
            B := B * Float (Y2);
            A := A * A;
            C := T * T;
            New_X := Gint (A * Float (X1) + B2 + C * Float (X3));
            New_Y := Gint (A * Float (Y1) + B + C * Float (Y3));
            IF Old_X /= New_X OR ELSE Old_Y /= New_Y THEN
               Draw_Line (Window, Gc, Old_X, Old_Y, New_X, New_Y);
               Old_X := New_X;
               Old_Y := New_Y;
            END IF;

            New_X2 := Gint (C * Float (X1) + B2 + A * Float (X3));
            New_Y2 := Gint (C * Float (Y1) + B + A * Float (Y3));
            IF Old_X2 /= New_X2 OR ELSE Old_Y2 /= New_Y2 THEN
               Draw_Line (Window, Gc, Old_X2, Old_Y2, New_X2, New_Y2);
               Old_X2 := New_X2;
               Old_Y2 := New_Y2;
            END IF;

            T := T + Step;
         END LOOP;
      END Bezier_One_Control;

      Angle       :          Float;
      X1,
      Y1,
      X2,
      Y2,
      X3,
      Y3          :          Gint;
      Right_Angle : CONSTANT Float := Ada.Numerics.Pi / 2.0;
      Xbase       : CONSTANT Gint  := Gint (Get_Value (Canvas.Hadj));
      Ybase       : CONSTANT Gint  := Gint (Get_Value (Canvas.Vadj));
      Arc_Offset  : CONSTANT Float := Float (To_Canvas_Coordinates (Canvas,
        Canvas.Arc_Link_Offset));
      Sx          : CONSTANT Gint  := Link.Src.Coord.X;
      Sy          : CONSTANT Gint  := Link.Src.Coord.Y;
      Dx          : CONSTANT Gint  := Link.Dest.Coord.X;
      Dy          : CONSTANT Gint  := Link.Dest.Coord.Y;
   BEGIN
      --  We will first compute the extra intermediate point between the
      --  center of the two items. Once we have this intermediate point, we
      --  will be able to use the intersection point between the two items
      --  and the two lines from the centers to the middle point. This extra
      --  point is used as a control point for the Bezier curve.

      Link.Src.Coord.X := To_Canvas_Coordinates (Canvas, Sx) - Xbase;
      Link.Src.Coord.Y := To_Canvas_Coordinates (Canvas, Sy) - Ybase;
      Link.Dest.Coord.X := To_Canvas_Coordinates (Canvas, Dx) - Xbase;
      Link.Dest.Coord.Y := To_Canvas_Coordinates (Canvas, Dy) - Ybase;

      X1 := Link.Src.Coord.X +
         Gint (Gfloat (Link.Src.Coord.Width) * Link.Src_X_Pos);
      Y1 := Link.Src.Coord.Y +
         Gint (Gfloat (Link.Src.Coord.Height) * Link.Src_Y_Pos);
      X3 := Link.Dest.Coord.X +
         Gint (Gfloat (Link.Dest.Coord.Width) * Link.Dest_X_Pos);
      Y3 := Link.Dest.Coord.Y +
         Gint (Gfloat (Link.Dest.Coord.Height) * Link.Dest_Y_Pos);

      --  Compute the middle point for the arc, and create a dummy item for it
      --  that the user can move.

      IF X1 /= X3 THEN
         Angle := Arctan (Float (Y3 - Y1), Float (X3 - X1));
      ELSIF Y3 > Y1 THEN
         Angle := Right_Angle;
      ELSE
         Angle := -Right_Angle;
      END IF;

      IF Offset < 0 THEN
         Angle := Angle - Right_Angle;
      ELSE
         Angle := Angle + Right_Angle;
      END IF;

      X2 := (X1 + X3) / 2 + ABS (Offset) * Gint (Arc_Offset * Cos (Angle));
      Y2 := (Y1 + Y3) / 2 + ABS (Offset) * Gint (Arc_Offset * Sin (Angle));

      --  Clip to the border of the boxes

      Clip_Line
         (Link.Src.Coord, X2, Y2, Link.Src_X_Pos, Link.Src_Y_Pos, X1, Y1);
      Clip_Line
         (Link.Dest.Coord, X2, Y2, Link.Dest_X_Pos, Link.Dest_Y_Pos, X3,
         Y3);

      Link.Src.Coord.X := Sx;
      Link.Src.Coord.Y := Sy;
      Link.Dest.Coord.X := Dx;
      Link.Dest.Coord.Y := Dy;

      IF Gc /= Canvas.Anim_Gc THEN
         Bezier_One_Control (X1, Y1, X2, Y2, X3, Y3, 0.005);
      ELSE
         Bezier_One_Control (X1, Y1, X2, Y2, X3, Y3, 0.01);
      END IF;

      --  Draw the arrows

      IF Link.Arrow = End_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         IF X3 /= X2 THEN
            Angle := Arctan (Float (Y2 - Y3), Float (X2 - X3));
         ELSIF Y3 > Y2 THEN
            Angle := Right_Angle;
         ELSE
            Angle := -Right_Angle;
         END IF;
         Draw_Arrow_Head (Canvas, Window, Gc, X3, Y3, Angle);
      END IF;

      IF Link.Arrow = Start_Arrow OR ELSE Link.Arrow = Both_Arrow THEN
         IF X1 /= X2 THEN
            Angle := Arctan (Float (Y2 - Y1), Float (X2 - X1));
         ELSIF Y2 > Y1 THEN
            Angle := Right_Angle;
         ELSE
            Angle := -Right_Angle;
         END IF;
         Draw_Arrow_Head (Canvas, Window, Gc, X1, Y1, Angle);
      END IF;

      --  Draw the annotations, if any, in the middle of the link
      IF Link.Descr /= NULL THEN
         X2 := Gint (0.25 * Float (X1) + 0.5 * Float (X2) + 0.25 * Float (
               X3));
         Y2 := Gint (0.25 * Float (Y1) + 0.5 * Float (Y2) + 0.25 * Float (
               Y3));
         Draw_Annotation (Canvas, Window, Gc, X2, Y2, Link.Descr);
      END IF;
   END Draw_Arc_Link;

   ------------------
   -- Update_Links --
   ------------------

   PROCEDURE Update_Links (
         Canvas   : ACCESS Interactive_Canvas_Record'Class;
         Window   :        Gdk_Window;
         Gc       : IN     Gdk.Gc.Gdk_Gc;
         Selected : IN     Item_Selection_List             := NULL) IS
      Current : Canvas_Link_List := Canvas.Links;
      Link    : Canvas_Link;
      X,
      Y       : Gint;
      Offset  : Gint;
      O       : Gint;
   BEGIN
      IF Selected /= NULL THEN
         X := Selected.Item.Coord.X;
         Y := Selected.Item.Coord.Y;
         Selected.Item.Coord.X := Selected.X;
         Selected.Item.Coord.Y := Selected.Y;
      END IF;

      WHILE Current /= NULL LOOP
         IF Is_Visible (Current.Link.Src)
               AND THEN Is_Visible (Current.Link.Dest)
               AND THEN
               (Selected = NULL
               OR ELSE Current.Link.Src = Canvas_Item (Selected.Item)
               OR ELSE Current.Link.Dest = Canvas_Item (Selected.Item))
               THEN
            --  Self-referencing links
            IF Current.Link.Src = Current.Link.Dest THEN
               Link := Current.Link;
               Offset := 1;
               WHILE Link /= NULL LOOP
                  Draw_Self_Link (Canvas, Window, Gc, Link, Offset);
                  Link := Link.Sibling;
                  Offset := Offset + 1;
               END LOOP;

            ELSE
               --  The first link in the list is always straight
               Draw_Straight_Link (Canvas, Window, Gc, Current.Link);

               Link := Current.Link.Sibling;
               Offset := 1;
               WHILE Link /= NULL LOOP

                  --  Do we need to reverse the link ? If we don't do that,
                  --  then the link will hide another link.
                  O := -Offset;
                  IF Link.Src /= Current.Link.Src THEN
                     O := Offset;
                  END IF;
                  Draw_Arc_Link (Canvas, Window, Gc, Link, O);
                  Link := Link.Sibling;

                  IF Link /= NULL THEN
                     O := Offset;
                     IF Link.Src /= Current.Link.Src THEN
                        O := -Offset;
                     END IF;
                     Draw_Arc_Link (Canvas, Window, Gc, Link, O);
                     Link := Link.Sibling;
                  END IF;
                  Offset := Offset + 1;
               END LOOP;
            END IF;
         END IF;
         Current := Current.Next;
      END LOOP;

      IF Selected /= NULL THEN
         Selected.Item.Coord.X := X;
         Selected.Item.Coord.Y := Y;
      END IF;
   END Update_Links;

   ------------
   -- Expose --
   ------------

   FUNCTION Expose (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk.Event.Gdk_Event)
     RETURN Boolean IS
      Canvas : CONSTANT Interactive_Canvas := Interactive_Canvas (Canv);

      Rect   : CONSTANT Gdk_Rectangle    := Get_Area (Event);
      Tmp    :          Canvas_Item_List := Canvas.Children;
      X,
      Y      :          Gint;
      Xmin   :          Gint;
      Dest   :          Gdk_Rectangle;
      Inters :          Boolean;
      Xbase  : CONSTANT Gint             := Gint (Get_Value (Canvas.Hadj));

      Ybase : CONSTANT Gint := Gint (Get_Value (Canvas.Vadj));

      Grid : CONSTANT Gint := To_Canvas_Coordinates (Canvas, Gint (Canvas.Grid_Size));

      Pix : Gdk_Pixmap;

   BEGIN
      IF Use_Double_Buffer THEN
         Pix := Canvas.Double_Buffer;
      ELSE
         Pix := Get_Window (Canvas);
      END IF;

      --  If the GC was not created, do not do anything

      IF Canvas.Clear_Gc = Null_Gc
            OR ELSE (Use_Double_Buffer AND THEN Canvas.Double_Buffer = NULL)
            THEN
         RETURN False;
      END IF;

      --  Clear the canvas

      Draw_Rectangle
         (Pix,
         Get_Background_Gc (Get_Style (Canvas), State_Normal),
         Filled => True,
         X => Rect.X, Y => Rect.Y,
         Width  => Gint (Rect.Width),
         Height => Gint (Rect.Height));

      Set_Clip_Rectangle (Canvas.Black_Gc, Rect);

      --  Draw the background dots.

      IF Grid >= 3 THEN
         IF (Rect.Y + Ybase) mod Grid = 0 THEN
            Y := Rect.Y;
         ELSE
            Y := Rect.Y + Grid - (Rect.Y + Ybase) mod Grid;
         END IF;
         IF (Rect.X + Xbase) mod Grid = 0 THEN
            Xmin := Rect.X;
         ELSE
            Xmin := Rect.X + Grid - (Rect.X + Xbase) mod Grid;
         END IF;

         WHILE Y <= Rect.Y + Gint (Rect.Height) LOOP
            X := Xmin;

            WHILE X <= Rect.X + Gint (Rect.Width) LOOP
               Draw_Point (Pix, Canvas.Black_Gc, X, Y);
               X := X + Grid;
            END LOOP;

            Y := Y + Grid;
         END LOOP;
      END IF;

      --  Draw the links first, so that they appear to be below the items.
      --  ??? Should redraw only the required links

      Update_Links (Canvas, Pix, Canvas.Black_Gc);

      --  Draw each of the items.

      WHILE Tmp /= NULL LOOP
         IF Tmp.Item.Visible THEN
            Intersect
               (Rect,
               (To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.X) - Xbase,
                  To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.Y) - Ybase,
                  Tmp.Item.Coord.Width,
                  Tmp.Item.Coord.Height),
               Dest, Inters);
            IF Inters THEN
               Draw
                  (Tmp.Item, Canvas, Pix,
                  To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.X) - Xbase,
                  To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.Y) - Ybase);
            END IF;
         END IF;

         Tmp := Tmp.Next;
      END LOOP;

      IF Use_Double_Buffer THEN
         Draw_Pixmap
            (Get_Window (Canvas),
            Gc     => Canvas.Black_Gc,
            Src    => Pix,
            Xsrc   => Rect.X,
            Ysrc   => Rect.Y,
            Xdest  => Rect.X,
            Ydest  => Rect.Y,
            Width  => Gint (Rect.Width),
            Height => Gint (Rect.Height));
      END IF;

      --  The dashed line (while moving items) have been deleted, and are no
      --  longer visible
      IF Canvas.Dashed_Line_Visible THEN
         Draw_Dashed_Selection (Canvas);
      END IF;

      Set_Clip_Mask (Canvas.Black_Gc, NULL);
      RETURN False;
   END Expose;

   ---------------------
   -- Set_Screen_Size --
   ---------------------

   PROCEDURE Set_Screen_Size (
         Item   : ACCESS Canvas_Item_Record;
         Width,
         Height :        Gint) IS
   BEGIN
      Item.Coord.Width  := Guint (Width);
      Item.Coord.Height := Guint (Height);
   END Set_Screen_Size;

   ---------------
   -- Key_Press --
   ---------------

   FUNCTION Key_Press (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean IS
      Canvas : CONSTANT Interactive_Canvas := Interactive_Canvas (Canv);

      Value : CONSTANT Gfloat := Get_Value (Canvas.Vadj);

      Upper : CONSTANT Gfloat := Get_Upper (Canvas.Vadj);

      Lower : CONSTANT Gfloat := Get_Lower (Canvas.Vadj);

      Page_Incr : CONSTANT Gfloat := Get_Page_Increment (Canvas.Vadj);

      Page_Size : CONSTANT Gfloat := Get_Page_Size (Canvas.Vadj);

      Step_Incr : CONSTANT Gfloat := Get_Step_Increment (Canvas.Vadj);


   BEGIN
      CASE Get_Key_Val (Event) IS
         WHEN Gdk_Home =>
            Set_Value (Canvas.Vadj, Lower);
            Changed (Canvas.Vadj);
            RETURN True;

         WHEN Gdk_End =>
            Set_Value (Canvas.Vadj, Upper - Page_Size);
            Changed (Canvas.Vadj);
            RETURN True;

         WHEN Gdk_Page_Up =>
            IF Value >= Lower + Page_Incr THEN
               Set_Value (Canvas.Vadj, Value - Page_Incr);
            ELSE
               Set_Value (Canvas.Vadj, Lower);
            END IF;

            Changed (Canvas.Vadj);
            RETURN True;

         WHEN Gdk_Page_Down =>
            IF Value + Page_Incr + Page_Size <= Upper THEN
               Set_Value (Canvas.Vadj, Value + Page_Incr);
            ELSE
               Set_Value (Canvas.Vadj, Upper - Page_Size);
            END IF;

            Changed (Canvas.Vadj);
            RETURN True;

         WHEN Gdk_Up | Gdk_Kp_Up =>
            IF Value - Step_Incr >= Lower THEN
               Set_Value (Canvas.Vadj, Value - Step_Incr);
            ELSE
               Set_Value (Canvas.Vadj, Lower);
            END IF;

            Changed (Canvas.Vadj);
            Gtk.Handlers.Emit_Stop_By_Name (Canvas, "key_press_event");
            RETURN True;

         WHEN Gdk_Down | Gdk_Kp_Down =>
            IF Value + Step_Incr + Page_Size <= Upper THEN
               Set_Value (Canvas.Vadj, Value + Step_Incr);
            ELSE
               Set_Value (Canvas.Vadj, Upper - Page_Size);
            END IF;

            Changed (Canvas.Vadj);
            Gtk.Handlers.Emit_Stop_By_Name (Canvas, "key_press_event");
            RETURN True;

         WHEN Gdk_Left | Gdk_Kp_Left =>
            IF Get_Value (Canvas.Hadj) -
                  Get_Step_Increment (Canvas.Hadj) >=
                  Get_Lower (Canvas.Hadj)
                  THEN
               Set_Value (Canvas.Hadj,
                  Get_Value (Canvas.Hadj)
                  - Get_Step_Increment (Canvas.Hadj));
            ELSE
               Set_Value (Canvas.Hadj,
                  Get_Lower (Canvas.Hadj));
            END IF;

            Changed (Canvas.Hadj);
            Gtk.Handlers.Emit_Stop_By_Name (Canvas, "key_press_event");
            RETURN True;

         WHEN Gdk_Right | Gdk_Kp_Right =>
            IF Get_Value (Canvas.Hadj) +
                  Get_Step_Increment (Canvas.Hadj) +
                  Get_Page_Size (Canvas.Hadj) <=
                  Get_Upper (Canvas.Hadj)
                  THEN
               Set_Value (Canvas.Hadj,
                  Get_Value (Canvas.Hadj) +
                  Get_Step_Increment (Canvas.Hadj));
            ELSE
               Set_Value (Canvas.Hadj,
                  Get_Upper (Canvas.Hadj) -
                  Get_Page_Size (Canvas.Hadj));
            END IF;

            Changed (Canvas.Hadj);
            Gtk.Handlers.Emit_Stop_By_Name (Canvas, "key_press_event");
            RETURN True;

         WHEN OTHERS =>
            NULL;
      END CASE;
      RETURN False;
   END Key_Press;

   --------------------
   -- Button_Pressed --
   --------------------

   FUNCTION Button_Pressed (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean IS
      Canvas : CONSTANT Interactive_Canvas := Interactive_Canvas (Canv);

      Tmp   :          Canvas_Item_List := Canvas.Children;
      Xbase : CONSTANT Gint             := Gint (Get_Value (Canvas.Hadj));

      Ybase : CONSTANT Gint := Gint (Get_Value (Canvas.Vadj));

      X : CONSTANT Gint := Gint (Get_X (Event)) + Xbase;

      Y : CONSTANT Gint := Gint (Get_Y (Event)) + Ybase;

      X2,
      Y2     : Gint;
      Cursor : Gdk.Cursor.Gdk_Cursor;
      Item   : Canvas_Item;

   BEGIN
      Grab_Focus (Canvas);
      Set_Flags (Canvas, Has_Focus);

      Clear_Selection (Canvas);

      --  Find the selected item.
      --  Note that we only keep the last item found, since this is the one
      --  on top, and thus the one the user really wanted to select
      IF(Get_Button (Event) /= 1) THEN
         RETURN False;
      END IF;

      WHILE Tmp /= NULL LOOP
         X2 := To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.X);
         Y2 := To_Canvas_Coordinates (Canvas, Tmp.Item.Coord.Y);
         IF Tmp.Item.Visible
               AND THEN X >= X2
               AND THEN X <= X2 + Gint (Tmp.Item.Coord.Width)
               AND THEN Y >= Y2
               AND THEN Y <= Y2 + Gint (Tmp.Item.Coord.Height)
               THEN
            Clear_Selection (Canvas);
            Add_To_Selection (Canvas, Tmp.Item);
         END IF;
         Tmp := Tmp.Next;
      END LOOP;

      --  If there was none, nothing to do...

      IF Canvas.Selection = NULL THEN
         Item := NEW Item_Object_Record;
         Widget_Callback.Emit_By_Name (Canvas, "background_click", Event);
         On_Button_Click (Canvas,Item, X, Y, "background_click");
         RETURN False;
      END IF;

      Set_X (Event, Gdouble (Xbase) + Get_X (Event)
         - Gdouble (To_Canvas_Coordinates
            (Canvas, Canvas.Selection.Item.Coord.X)));
      Set_Y (Event, Gdouble (Ybase) + Get_Y (Event)
         - Gdouble (To_Canvas_Coordinates
            (Canvas, Canvas.Selection.Item.Coord.Y)));

      --  Double-click events are transmitted directly to the item, and are
      --  not used to move an item.
      --  Clicks other than left mouse button are also transmitted directly

      IF Get_Event_Type (Event) = Gdk_2button_Press
            -- or else Get_Button (Event) /= 1
            THEN

         --  ??? Should do so for each item in the selection
         -- On_Button_Click (Canvas,Canvas.Selection.Item,X,Y, "item_click");
         RETURN False;
      END IF;

      --  Warn the user that a selection has been made
      --  ??? Should send one such event per item in the selection

      --   Emit_By_Name_Item
      --          (Gtk.Get_Object (Canvas), "item_selected" & ASCII.NUL,
      --           Canvas.Selection.Item);

      --  Change the cursor to give visual feedback
      Gdk_New (Cursor, Gdk.Types.Fleur);
      Set_Cursor (Get_Window (Canvas), Cursor);
      Destroy (Cursor);

      --  Initialize the move

      Canvas.Last_X_Event :=
         To_World_Coordinates (Canvas, Gint (Get_X_Root (Event)));
      Canvas.Last_Y_Event :=
         To_World_Coordinates (Canvas, Gint (Get_Y_Root (Event)));
      Canvas.Mouse_Has_Moved := False;

      --  Make sure that no other widget steal the events while we are
      --  moving an item.

      Grab_Add (Canvas);
      Deep_Copy (
         From => Event,
         To   => Canvas.Event_Press);
      RETURN False;
   END Button_Pressed;

   --------------------
   -- Button_Release --
   --------------------

   FUNCTION Button_Release (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean IS
      Canvas : CONSTANT Interactive_Canvas    := Interactive_Canvas (Canv);
      Tmp    :          Item_Selection_List;
      Cursor :          Gdk.Cursor.Gdk_Cursor;
   BEGIN
      Grab_Remove (Canvas);

      IF Canvas.Selection = NULL THEN
         Widget_Callback.Emit_By_Name (Canvas, "background_click", Event);
         RETURN False;
      END IF;

      --  restore the standard cursor
      Gdk_New (Cursor, Gdk.Types.Left_Ptr);
      Set_Cursor (Get_Window (Canvas), Cursor);
      Destroy (Cursor);

      IF Canvas.Scrolling_Timeout_Id /= 0 THEN
         Timeout_Remove (Canvas.Scrolling_Timeout_Id);
         Canvas.Scrolling_Timeout_Id := 0;
      END IF;

      IF Canvas.Mouse_Has_Moved THEN
         Tmp := Canvas.Selection;
         WHILE Tmp /= NULL LOOP
            IF Canvas.Align_On_Grid THEN
               Tmp.Item.Coord.X := Tmp.X - Tmp.X mod Gint (Canvas.
                  Grid_Size);
               Tmp.Item.Coord.Y := Tmp.Y - Tmp.Y mod Gint (Canvas.
                  Grid_Size);
            ELSE
               Tmp.Item.Coord.X := Tmp.X;
               Tmp.Item.Coord.Y := Tmp.Y;
            END IF;
            Tmp := Tmp.Next;
         END LOOP;

         Canvas.Dashed_Line_Visible := False;

         --  Scroll the canvas so as to show the first item from the selection
         Refresh_Canvas (Canvas);

         --  If the user did not move the mouse while it was pressed, this is
         --  because he only wanted to select the item.
         --  Note that even if multiple items are currently selected, only the one
         --  we actually clicked on should receive the button_click event.

      ELSE
         --  The button-press event wasn't forwarded, since we were expecting
         --  that the item would move. We thus forward it now
         --On_Button_Click (Canvas,Canvas.Selection.Item, "Item_click");

         Set_X (Event, Gdouble (Get_Value (Canvas.Hadj)) +  Get_X (Event)
            - Gdouble (To_Canvas_Coordinates
               (Canvas, Canvas.Selection.Item.Coord.X)));
         Set_Y (Event, Gdouble (Get_Value (Canvas.Vadj)) + Get_Y (Event)
            - Gdouble (To_Canvas_Coordinates
               (Canvas, Canvas.Selection.Item.Coord.Y)));
         On_Button_Click (Canvas,Canvas.Selection.Item, 0, 0, "item_click");
      END IF;


      Free (Canvas.Event_Press);

      RETURN False;
   END Button_Release;

   ------------------------
   -- Test_Scrolling_Box --
   ------------------------

   PROCEDURE Test_Scrolling_Box (
         Canvas   : ACCESS Gtk.Widget.Gtk_Widget_Record'Class;
         Win      :        Gdk_Window;
         X,
         Y        :        Gint;
         X_Scroll :    OUT Gint;
         Y_Scroll :    OUT Gint) IS
   BEGIN
      X_Scroll := 0;
      Y_Scroll := 0;

      IF Win = Get_Window (Canvas) THEN
         IF X < Scrolling_Margin THEN
            X_Scroll := -Scrolling_Amount;
         ELSIF X > Gint (Get_Allocation_Width (Canvas)) - Scrolling_Margin THEN
            X_Scroll := Scrolling_Amount;
         END IF;

         IF Y < Scrolling_Margin THEN
            Y_Scroll := -Scrolling_Amount;
         ELSIF
               Y > Gint (Get_Allocation_Height (Canvas)) -
               Scrolling_Margin
               THEN
            Y_Scroll := Scrolling_Amount;
         END IF;
      END IF;
   END Test_Scrolling_Box;

   -------------------
   -- Button_Motion --
   -------------------

   FUNCTION Button_Motion (
         Canv  : ACCESS Gtk_Widget_Record'Class;
         Event :        Gdk_Event)
     RETURN Boolean IS
      Canvas   : CONSTANT Interactive_Canvas := Interactive_Canvas (Canv);
      X_Scroll,
      Y_Scroll :          Gint;
   BEGIN
      --  Are we in the scrolling box ? If yes, do not move the item directly,
      --  but establish the timeout callbacks that will take care of the
      --  scrolling
      Test_Scrolling_Box
         (Canv, Get_Window (Event), Gint (Get_X (Event)), Gint (Get_Y (
               Event)),
         X_Scroll, Y_Scroll);

      IF X_Scroll /= 0 OR ELSE Y_Scroll /= 0 THEN
         IF Canvas.Scrolling_Timeout_Id = 0 THEN
            Canvas.Scrolling_Timeout_Id := Canvas_Timeout.Add
               (Timeout_Between_Scrolls, Scrolling_Timeout'ACCESS, Canvas);
         END IF;

      ELSE
         IF Canvas.Scrolling_Timeout_Id /= 0 THEN
            Timeout_Remove (Canvas.Scrolling_Timeout_Id);
            Canvas.Scrolling_Timeout_Id := 0;
         END IF;

         X_Scroll := To_World_Coordinates (Canvas, Gint (Get_X_Root (
                  Event)));
         Y_Scroll := To_World_Coordinates (Canvas, Gint (Get_Y_Root (
                  Event)));
         IF Move_Selection
               (Canvas, X_Scroll - Canvas.Last_X_Event,
               Y_Scroll - Canvas.Last_Y_Event)
               THEN
            Canvas.Last_X_Event := X_Scroll;
            Canvas.Last_Y_Event := Y_Scroll;
         END IF;
      END IF;
      RETURN False;
   END Button_Motion;

   -----------------------
   -- Scrolling_Timeout --
   -----------------------

   FUNCTION Scrolling_Timeout (
         Canvas : Interactive_Canvas)
     RETURN Boolean IS
      Win      : Gdk_Window;
      X,
      Y        : Gint;
      X_Scroll,
      Y_Scroll : Gint;
   BEGIN
      Window_At_Pointer (X, Y, Win);
      Test_Scrolling_Box (Canvas, Win, X, Y, X_Scroll, Y_Scroll);
      IF (X_Scroll /= 0 OR ELSE Y_Scroll /= 0)
            AND THEN Move_Selection (Canvas, X_Scroll, Y_Scroll)
            THEN
         --  Force an immediate draw, since Queue_Draw would only redraw in
         --  an idle event, and thus might not happen before the next timeout.
         --  With lots of items, this would break the scrolling.
         Draw (Canvas);
         RETURN True;
      ELSE
         Canvas.Scrolling_Timeout_Id := 0;
         RETURN False;
      END IF;
   END Scrolling_Timeout;

   ---------------------------
   -- Draw_Dashed_Selection --
   ---------------------------

   PROCEDURE Draw_Dashed_Selection (
         Canvas : ACCESS Interactive_Canvas_Record'Class) IS
      Xbase : CONSTANT Gint := Gint (Get_Value (Canvas.Hadj));

      Ybase : CONSTANT Gint := Gint (Get_Value (Canvas.Vadj));

      Selected : Item_Selection_List := Canvas.Selection;
      X,
      Y        : Gint;
   BEGIN
      WHILE Selected /= NULL LOOP
         IF Selected.Item.Visible THEN
            IF Canvas.Align_On_Grid THEN
               X := Selected.X;
               Y := Selected.Y;
               Selected.X := Selected.X -
                  Selected.X mod Gint (Canvas.Grid_Size);
               Selected.Y := Selected.Y -
                  Selected.Y mod Gint (Canvas.Grid_Size);
            END IF;

            Draw_Rectangle
               (Get_Window (Canvas),
               Gc     => Canvas.Anim_Gc,
               Filled => False,
               X      => To_Canvas_Coordinates (Canvas, Selected.X) -
               Xbase,
               Y      => To_Canvas_Coordinates (Canvas, Selected.Y) -
               Ybase,
               Width  => Gint (Selected.Item.Coord.Width),
               Height => Gint (Selected.Item.Coord.Height));

            Update_Links
               (Canvas, Get_Window (Canvas), Canvas.Anim_Gc, Selected);

            IF Canvas.Align_On_Grid THEN
               Selected.X := X;
               Selected.Y := Y;
            END IF;
         END IF;
         Selected := Selected.Next;
      END LOOP;
   END Draw_Dashed_Selection;

   --------------------
   -- Move_Selection --
   --------------------

   FUNCTION Move_Selection (
         Canvas  : ACCESS Interactive_Canvas_Record'Class;
         Delta_X,
         Delta_Y :        Gint)
     RETURN Boolean IS
      Selected : Item_Selection_List;
   BEGIN
      IF Canvas.Selection = NULL THEN
         RETURN False;
      END IF;

      IF NOT Canvas.Mouse_Has_Moved THEN
         --  Is this a motion, or simply a selection ?

         IF ABS (Delta_X) <= Canvas.Motion_Threshold
               AND THEN ABS (Delta_Y) <= Canvas.Motion_Threshold
               THEN
            RETURN False;
         END IF;

      ELSIF Canvas.Dashed_Line_Visible THEN
         --  Delete the currently dashed lines
         Draw_Dashed_Selection (Canvas);
      END IF;

      Canvas.Mouse_Has_Moved := True;
      Canvas.Dashed_Line_Visible := True;

      --  Move everything
      Selected := Canvas.Selection;
      WHILE Selected /= NULL LOOP
         Selected.X := Selected.X + Delta_X;
         Selected.Y := Selected.Y + Delta_Y;
         Selected := Selected.Next;
      END LOOP;

      Draw_Dashed_Selection (Canvas);
      Show_Item
         (Canvas, Canvas.Selection.Item,
         Canvas.Selection.X, Canvas.Selection.Y);
      RETURN True;
   END Move_Selection;

   ------------------
   -- Item_Updated --
   ------------------

   PROCEDURE Item_Updated (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Xbase : CONSTANT Gint := Gint (Get_Value (Canvas.Hadj));
      Ybase : CONSTANT Gint := Gint (Get_Value (Canvas.Vadj));
   BEGIN
      IF Item.Visible THEN
         Queue_Draw_Area
            (Canvas,
            To_Canvas_Coordinates (Canvas, Item.Coord.X) - Xbase,
            To_Canvas_Coordinates (Canvas, Item.Coord.Y) - Ybase,
            Gint (Item.Coord.Width), Gint (Item.Coord.Height));
      END IF;
   END Item_Updated;

   ------------
   -- Remove --
   ------------

   PROCEDURE Remove (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Previous      : Canvas_Item_List := NULL;
      Current       : Canvas_Item_List := Canvas.Children;
      Previous_Link : Canvas_Link_List;
      Current_Link  : Canvas_Link_List;
      Link,
      Tmp           : Canvas_Link;

   BEGIN
      WHILE Current /= NULL LOOP
         IF Current.Item = Canvas_Item (Item) THEN

            --  Remove the item itself

            IF Previous = NULL THEN
               Canvas.Children := Current.Next;
            ELSE
               Previous.Next := Current.Next;
            END IF;

            --  Remove all the links

            Previous_Link := NULL;
            Current_Link := Canvas.Links;
            WHILE Current_Link /= NULL LOOP
               IF Current_Link.Link.Src = Canvas_Item (Item)
                     OR ELSE Current_Link.Link.Dest = Canvas_Item (Item)
                     THEN
                  Link := Current_Link.Link;
                  WHILE Link /= NULL LOOP
                     Tmp := Link.Sibling;
                     Destroy (Link);
                     Free (Link);
                     Link := Tmp;
                  END LOOP;

                  IF Previous_Link = NULL THEN
                     Canvas.Links := Current_Link.Next;
                  ELSE
                     Previous_Link.Next := Current_Link.Next;
                  END IF;
               ELSE
                  Previous_Link := Current_Link;
               END IF;

               IF Previous_Link /= NULL THEN
                  Current_Link := Previous_Link.Next;
               ELSE
                  Current_Link := Canvas.Links;
               END IF;
            END LOOP;

            --  Free the memory
            Destroy (Current.Item);
            Free (Current);

            --  Have to redraw everything, since there might have been some
            --  links.
            Refresh_Canvas (Canvas);
            RETURN;
         END IF;
         Previous := Current;
         Current := Current.Next;
      END LOOP;
   END Remove;

   ---------------------
   -- On_Button_Click --
   ---------------------

   PROCEDURE On_Button_Click (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   :        Canvas_Item;
         X,
         Y      :        Gint;
         Event  :        String) IS

      Item_New : Item_Object_Access;

      PROCEDURE Emit_By_Name_Item (
            Object : IN     System.Address;
            Name   : IN     String;
            Param  : IN     Canvas_Item);
      PRAGMA Import (C, Emit_By_Name_Item, "gtk_signal_emit_by_name");

      My_Buff : Buffer_Ptr;
      My_Mess : Message_Ptr;
      Task1,
      Task2   : Task_Ptr;

   BEGIN
      IF ( Event /= "background_click") THEN

         IF(Gtk.Gentry.Get_Text(Get_Entry(Canvas.Toolbar.Combo))=
               To_String(Lb_Delete(Current_Language))) THEN
            IF(Get_Active(Canvas.Toolbar.Arrow_Radio_Button)) THEN
               IF(Canvas.Has_Item_Selected = False) THEN
                  Canvas.Item_Selection := Item;
                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     True;
                  Canvas.Has_Item_Selected := True;
               ELSE
                  -- Remove a dependency
                  --
                  Canvas.Has_Item_Selected := False;
                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     False;
                  IF(Get_Figure(Canvas.Item_Selection) =
                        To_Unbounded_String("task")) THEN
                     IF (Get_Figure(Item) = To_Unbounded_String("task")) THEN
                        Remove_Link_Between(Canvas,Item_Object_Access(
                              Canvas.Item_Selection),Item_Object_Access(
                              Item));

                        Task1:=Search_Task(Sys.Tasks, Get_Name(Canvas.
                              Item_Selection));
                        Task2:=Search_Task(Sys.Tasks, Get_Name(Item));
                        Delete_One_Task_Dependency(Sys.Dependencies,
                           Task1, Task2);

                        Is_New:=False;
                        Is_Saved:=False;

                     ELSE
                        Remove_Link_Between(Canvas,Item_Object_Access(
                              Canvas.Item_Selection),Item_Object_Access(
                              Item));
                        IF (Get_Figure(Item)= To_Unbounded_String(
                                 "buffer"))
                              THEN
                           My_Buff:=Search_Buffer(Sys.Buffers, Get_Name(
                                 Item));
                           Task1:=Search_Task(Sys.Tasks, Get_Name(Canvas.
                                 Item_Selection));
                           Delete_One_Task_Dependency(Sys.Dependencies,
                              Task1,
                              My_Buff,
                              From_Task_To_Object);

                        ELSE
                           My_Mess:=Search_Message(Sys.Messages, Get_Name(
                                 Item));
                           Task1:=Search_Task(Sys.Tasks, Get_Name(Canvas.
                                 Item_Selection));
                           Delete_One_Task_Dependency(Sys.Dependencies,
                              Task1,
                              My_Mess,
                              From_Task_To_Object);
                        END IF;

                        Is_New:=False;
                        Is_Saved:=False;

                     END IF;
                  ELSE
                     Remove_Link_Between(Canvas,Item_Object_Access(
                           Canvas.
                           Item_Selection),Item_Object_Access(Item));
                     IF (Get_Figure(Item)= To_Unbounded_String("buffer"))
                           THEN
                        My_Buff:=Search_Buffer(Sys.Buffers, Get_Name(
                              Canvas.Item_Selection));
                        Task1:=Search_Task(Sys.Tasks, Get_Name(Item));
                        Delete_One_Task_Dependency(Sys.Dependencies,
                           Task1,
                           My_Buff,
                           From_Object_To_Task);

                     ELSE
                        My_Mess:=Search_Message(Sys.Messages, Get_Name(
                              Canvas.Item_Selection));
                        Task1:=Search_Task(Sys.Tasks, Get_Name(Item));
                        Delete_One_Task_Dependency(Sys.Dependencies,
                           Task1,
                           My_Mess,
                           From_Object_To_Task);
                     END IF;


                     Is_New:=False;
                     Is_Saved:=False;

                  END IF;
               END IF;
            ELSE
               Set_Visibility(Item,False);
               -- remove dependencies
               --
               IF (Get_Figure(Item) = To_Unbounded_String("buffer")) THEN
                  My_Buff:=Search_Buffer(Sys.Buffers, Get_Name(
                        Item));
                  Delete_All_Task_Dependencies(Sys.Dependencies,
                     My_Buff);

               ELSE
                  IF (Get_Figure(Item) = To_Unbounded_String("message")) THEN
                     My_Mess:=Search_Message(Sys.Messages, Get_Name(
                           Item));
                     Delete_All_Task_Dependencies(Sys.Dependencies,
                        My_Mess);
                  ELSE
                     Task1:=Search_Task(Sys.Tasks, Get_Name(Item));
                     Delete_All_Task_Dependencies(Sys.Dependencies,
                        Task1);
                  END IF;
               END IF;

               Is_New:=False;
               Is_Saved:=False;

               Remove_Item_Link(Canvas,Item_Object_Access(Item));
               IF Canvas.Has_Item_Selected THEN
                  Canvas.Has_Item_Selected := False;
                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     False;
               END IF;
            END IF;
         END IF;
         IF(Gtk.Gentry.Get_Text(Get_Entry(Canvas.Toolbar.Combo))=
               To_String(Lb_Select(Current_Language))) THEN
            IF Canvas.Has_Item_Selected THEN
               Canvas.Has_Item_Selected := False;
               Item_Object_Access(Canvas.Item_Selection).Selected := False;
            END IF;
            Emit_By_Name_Item
               (Gtk.Get_Object (Canvas), "item_selected" & Ascii.Nul,
               Item);
         END IF;
         IF(Gtk.Gentry.Get_Text(Get_Entry(Canvas.Toolbar.Combo))=
               To_String(Lb_Create(Current_Language))) THEN
            IF(Get_Active(Canvas.Toolbar.Arrow_Radio_Button)) THEN
               IF(Canvas.Has_Item_Selected = False) THEN
                  Canvas.Item_Selection := Item;
                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     True;
                  Canvas.Has_Item_Selected := True;
               ELSE

                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     False;
                  Canvas.Has_Item_Selected := False;
                  IF(Get_Figure(Canvas.Item_Selection) =
                        To_Unbounded_String("task")) THEN
                     IF(Get_Figure(Item) = To_Unbounded_String("task")) THEN
                        IF(Get_Name(Canvas.Item_Selection) /= Get_Name(
                                 Item)) THEN
                           IF NOT Has_Link_Between(Canvas,
                                 Canvas.Item_Selection,
                                 Item) THEN

                              Task1:=Search_Task(Sys.Tasks, Get_Name(
                                    Canvas.
                                    Item_Selection));
                              Task2:=Search_Task(Sys.Tasks, Get_Name(Item));
                              Add_One_Task_Dependency(
                                 Sys.Dependencies,
                                 Task1,
                                 Task2);

                              Is_New:=False;
                              Is_Saved:=False;

                              Add_Link(Canvas,Canvas.Item_Selection,Item);
                           END IF;
                        END IF;
                     ELSE
                        IF NOT Has_Link_Between(Canvas,
                              Canvas.Item_Selection,
                              Item) THEN

                           IF(Get_Figure(Item) = To_Unbounded_String(
                                    "buffer")) THEN

                              My_Buff:=Search_Buffer(Sys.Buffers, Get_Name(
                                    Item));
                              Task1:=Search_Task(Sys.Tasks, Get_Name(
                                    Canvas.
                                    Item_Selection));
                              Add_One_Task_Dependency(Sys.Dependencies,
                                 Task1,
                                 My_Buff,
                                 From_Task_To_Object);
                           ELSE
                              IF(Get_Figure(Item) = To_Unbounded_String(
                                       "message")) THEN
                                 My_Mess:=Search_Message(Sys.Messages,
                                    Get_Name(Item));
                                 Task1:=Search_Task(Sys.Tasks, Get_Name(
                                       Canvas.
                                       Item_Selection));
                                 Add_One_Task_Dependency(Sys.Dependencies,
                                    Task1,
                                    My_Mess,
                                    From_Task_To_Object);
                              END IF;
                           END IF;

                           Is_New:=False;
                           Is_Saved:=False;

                           Add_Link(Canvas,Canvas.Item_Selection,Item);
                           IF (To_String(Get_Figure(Item))="buffer")
                                 THEN

                              Gtk_New(New_Buffer_Role,Get_Name(Canvas.
                                    Item_Selection),Get_Name(Item),
                                 "Producer");
                              Show_All(New_Buffer_Role);

                           END IF;
                        END IF;
                     END IF;
                  END IF;


                  IF(Get_Figure(Canvas.Item_Selection) =
                        To_Unbounded_String("message")) THEN
                     IF(Get_Figure(Item) = To_Unbounded_String("task")) THEN
                        IF NOT Has_Link_Between(Canvas,
                              Canvas.Item_Selection,
                              Item) THEN

                           My_Mess:=Search_Message(Sys.Messages, Get_Name(
                                 Canvas.Item_Selection));
                           Task1:=Search_Task(Sys.Tasks, Get_Name(Item));
                           Add_One_Task_Dependency(Sys.Dependencies,
                              Task1,
                              My_Mess,
                              From_Object_To_Task);

                           Is_New:=False;
                           Is_Saved:=False;

                           Add_Link(Canvas,Canvas.Item_Selection,Item);
                        END IF;
                     END IF;
                  END IF;
                  IF(Get_Figure(Canvas.Item_Selection) =
                        To_Unbounded_String("buffer")) THEN
                     IF(Get_Figure(Item) = To_Unbounded_String("task")) THEN
                        IF NOT Has_Link_Between(Canvas,
                              Canvas.Item_Selection,
                              Item) THEN

                           My_Buff:=Search_Buffer(Sys.Buffers, Get_Name(
                                 Canvas.Item_Selection));
                           Task1:=Search_Task(Sys.Tasks, Get_Name(Item));
                           Add_One_Task_Dependency(Sys.Dependencies,
                              Task1,
                              My_Buff,
                              From_Object_To_Task);

                           Is_New:=False;
                           Is_Saved:=False;

                           Add_Link(Canvas,Canvas.Item_Selection,Item);
                           Gtk_New(New_Buffer_Role,Get_Name(Item),
                              Get_Name(Canvas.Item_Selection), "Consumer");
                           Show_All(New_Buffer_Role);

                        END IF;
                     END IF;
                  END IF;
               END IF;
            ELSE
               IF Canvas.Has_Item_Selected THEN
                  Canvas.Has_Item_Selected := False;
                  Item_Object_Access(Canvas.Item_Selection).Selected :=
                     False;
               END IF;
            END IF;
         END IF;
      ELSE
         IF Canvas.Has_Item_Selected THEN
            Item_Object_Access(Canvas.Item_Selection).Selected := False;
            Canvas.Has_Item_Selected := False;
         END IF;
         IF(Gtk.Gentry.Get_Text(Get_Entry(Canvas.Toolbar.Combo))=
               To_String(Lb_Create(Current_Language))) THEN
            IF(Get_Active(Canvas.Toolbar.Arrow_Radio_Button)) THEN
               NULL;
            END IF;
            IF (Get_Active(Canvas.Toolbar.Task_Radio_Button)) THEN
               Item_Object_New(Item_New, To_Unbounded_String("task"),
                  False);
               Put(Canvas,Item_New,To_World_Coordinates (Canvas,X),
                  To_World_Coordinates (Canvas,Y));
            END IF;
            IF (Get_Active(Canvas.Toolbar.Mesg_Radio_Button)) THEN
               Item_Object_New(Item_New, To_Unbounded_String("message"),
                  False);
               Put(Canvas,Item_New,To_World_Coordinates (Canvas,X),
                  To_World_Coordinates (Canvas,Y));
            END IF;
            IF (Get_Active(Canvas.Toolbar.Buffer_Radio_Button)) THEN
               Item_Object_New(Item_New, To_Unbounded_String("buffer"),
                  False);
               Put(Canvas,Item_New,To_World_Coordinates (Canvas,X),
                  To_World_Coordinates (Canvas,Y));
            END IF;
         END IF;
      END IF;
      Refresh_Canvas(Canvas);
   END On_Button_Click;

   ---------------
   -- Get_Coord --
   ---------------

   FUNCTION Get_Coord (
         Item : ACCESS Canvas_Item_Record)
     RETURN Gdk.Rectangle.Gdk_Rectangle IS
   BEGIN
      RETURN Item.Coord;
   END Get_Coord;

   -----------------------
   -- Has_Link_beetween --
   -----------------------

   FUNCTION Has_Link_Between (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item1,
         Item2  : ACCESS Canvas_Item_Record'Class)
     RETURN Boolean IS
      Ok : Boolean := False;
      PRAGMA Warnings (Off);
      FUNCTION Search_Internal (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Link : ACCESS Canvas_Link_Record'Class)
        RETURN Boolean IS
      BEGIN
         IF(Get_Src (Link) = Canvas_Item(Item1) AND THEN Get_Dest (Link) =
               Canvas_Item(Item2))
               THEN
            Ok := True;
            RETURN False;
         END IF;
         RETURN True;
      END Search_Internal;
   BEGIN
      For_Each_Link(Canvas, Search_Internal'Unrestricted_Access);
      RETURN Ok;
   END Has_Link_Between;

   --------------
   -- Has_Link --
   --------------

   FUNCTION Has_Link (
         Canvas : ACCESS Interactive_Canvas_Record;
         From,
         To     : ACCESS Canvas_Item_Record'Class;
         Name   :        String                    := "")
     RETURN Boolean IS
      Current : Canvas_Link_List := Canvas.Links;
      Link    : Canvas_Link;
   BEGIN
      WHILE Current /= NULL LOOP
         IF Current.Link.Src = Canvas_Item (From)
               AND THEN Current.Link.Dest = Canvas_Item (To)
               THEN
            Link := Current.Link;
            WHILE Link /= NULL LOOP
               IF (Name = "" OR ELSE Link.Descr.All = Name) THEN
                  RETURN True;
               END IF;
               Link := Link.Sibling;
            END LOOP;
            RETURN False;
         END IF;
         Current := Current.Next;
      END LOOP;
      RETURN False;
   END Has_Link;

   ----------------
   -- Lower_Item --
   ----------------

   PROCEDURE Lower_Item (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Previous : Canvas_Item_List := Canvas.Children;
      Current  : Canvas_Item_List;

   BEGIN
      --  Already on top?

      IF Canvas.Children = NULL
            OR ELSE Canvas.Children.Item = Canvas_Item (Item)
            THEN
         RETURN;
      END IF;

      Current := Canvas.Children.Next;

      --  Look for the item.

      WHILE Current /= NULL LOOP
         IF Current.Item = Canvas_Item (Item) THEN
            Previous.Next := Current.Next;
            Current.Next := Canvas.Children;
            Canvas.Children := Current;
            EXIT;
         END IF;

         Previous := Current;
         Current := Current.Next;
      END LOOP;

      Queue_Draw (Canvas);
   END Lower_Item;

   ----------------
   -- Raise_Item --
   ----------------

   PROCEDURE Raise_Item (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Previous : Canvas_Item_List := NULL;
      Current  : Canvas_Item_List := Canvas.Children;
      To_Move  : Canvas_Item_List := NULL;

   BEGIN
      --  A single item => Nothing to do
      IF Canvas.Children.Next = NULL THEN
         RETURN;
      END IF;

      WHILE Current /= NULL LOOP
         IF Current.Item = Canvas_Item (Item) THEN
            To_Move := Current;

            IF Previous = NULL THEN
               Canvas.Children := Current.Next;
            ELSE
               Previous.Next := Current.Next;
            END IF;
         ELSE
            Previous := Current;
         END IF;
         Current := Current.Next;
      END LOOP;

      IF Previous /= NULL AND THEN To_Move /= NULL THEN
         Previous.Next := To_Move;
         To_Move.Next := NULL;
      END IF;

      --  Have to redraw everything, since there might have been some links.
      Queue_Draw (Canvas);
   END Raise_Item;

   ---------------
   -- Is_On_Top --
   ---------------

   FUNCTION Is_On_Top (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class)
     RETURN Boolean IS
      Current : Canvas_Item_List := Canvas.Children;
   BEGIN
      IF Current = NULL THEN
         RETURN False;
      END IF;

      WHILE Current.Next /= NULL LOOP
         Current := Current.Next;
      END LOOP;

      RETURN Current.Item = Canvas_Item (Item);
   END Is_On_Top;

   ---------------
   -- Show_Item --
   ---------------

   PROCEDURE Show_Item (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Item   : ACCESS Canvas_Item_Record'Class;
         X,
         Y      :        Gint) IS
      X1 : CONSTANT Gint := To_Canvas_Coordinates (Canvas, X);
      Y1 : CONSTANT Gint := To_Canvas_Coordinates (Canvas, Y);
      X2 : CONSTANT Gint := X1 + Gint (Item.Coord.Width);
      Y2 : CONSTANT Gint := Y1 + Gint (Item.Coord.Height);
   BEGIN
      --  Do we need to scroll the canvas to the right to show the item?

      IF X2 > Gint (Get_Upper (Canvas.Hadj)) THEN
         Set_Upper (Canvas.Hadj, Gfloat (X2));
         Changed (Canvas.Hadj);
      END IF;

      --  In case the item is larger than the canvas, make sure we display
      --  its right-most part
      --      ----------------------------
      --     |                            |
      --     |                          ------
      --     |                         |      |
      --     |                         |      |
      --     |                          ------
      --     |                            |
      --      ----------------------------
      --     ^                            ^
      --   value                    value + page_size
      --                                      ^
      --                                      X2

      IF X2 > Gint (Get_Value (Canvas.Hadj) + Get_Page_Size (Canvas.Hadj)) THEN
         Set_Value (Canvas.Hadj, Gfloat (X2) - Get_Page_Size (Canvas.Hadj));
      END IF;

      --  Do we need to scroll the canvas to the left ?

      IF X1 < Gint (Get_Lower (Canvas.Hadj)) THEN
         Set_Lower (Canvas.Hadj, Gfloat (X1));
         Changed (Canvas.Hadj);
      END IF;

      IF X1 < Gint (Get_Value (Canvas.Hadj)) THEN
         Set_Value (Canvas.Hadj, Gfloat (X1));
      END IF;

      --  Do we need to scroll the canvas to the top to show the selection?

      IF Y2 > Gint (Get_Upper (Canvas.Vadj)) THEN
         Set_Upper (Canvas.Vadj, Gfloat (Y2));
         Changed (Canvas.Vadj);
      END IF;

      IF Y2 > Gint (Get_Value (Canvas.Vadj) + Get_Page_Size (Canvas.Vadj)) THEN
         Set_Value (Canvas.Vadj, Gfloat (Y2) - Get_Page_Size (Canvas.Vadj));
      END IF;

      --  Do we need to scroll the canvas to the bottom ?

      IF Y1 < Gint (Get_Lower (Canvas.Vadj)) THEN
         Set_Lower (Canvas.Vadj, Gfloat (Y1));
         Changed (Canvas.Vadj);
      END IF;

      IF Y1 < Gint (Get_Value (Canvas.Vadj)) THEN
         Set_Value (Canvas.Vadj, Gfloat (Y1));
      END IF;
   END Show_Item;

   ---------------
   -- Show_Item --
   ---------------

   PROCEDURE Show_Item (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
   BEGIN
      Show_Item (Canvas, Item, Item.Coord.X, Item.Coord.Y);
   END Show_Item;

   -----------------------
   -- Get_Align_On_Grid --
   -----------------------

   FUNCTION Get_Align_On_Grid (
         Canvas : ACCESS Interactive_Canvas_Record)
     RETURN Boolean IS
   BEGIN
      RETURN Canvas.Align_On_Grid;
   END Get_Align_On_Grid;

   --------------------
   -- Set_Visibility --
   --------------------

   PROCEDURE Set_Visibility (
         Item    : ACCESS Canvas_Item_Record;
         Visible :        Boolean) IS
   BEGIN
      Item.Visible := Visible;
   END Set_Visibility;

   ----------------
   -- Is_Visible --
   ----------------

   FUNCTION Is_Visible (
         Item : ACCESS Canvas_Item_Record)
     RETURN Boolean IS
   BEGIN
      RETURN Item.Visible;
   END Is_Visible;

   --------------------
   -- Refresh_Canvas --
   --------------------

   PROCEDURE Refresh_Canvas (
         Canvas : ACCESS Interactive_Canvas_Record) IS
   BEGIN
      Update_Adjustments (Canvas);
      Queue_Draw (Canvas);
   END Refresh_Canvas;

   ---------------------
   -- Clear_Selection --
   ---------------------

   PROCEDURE Clear_Selection (
         Canvas : ACCESS Interactive_Canvas_Record) IS
      Tmp : Item_Selection_List;
   BEGIN
      WHILE Canvas.Selection /= NULL LOOP
         Tmp := Canvas.Selection;
         Canvas.Selection := Canvas.Selection.Next;
         Free (Tmp);
      END LOOP;
   END Clear_Selection;

   ----------------------
   -- Add_To_Selection --
   ----------------------

   PROCEDURE Add_To_Selection (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Tmp : Item_Selection_List := Canvas.Selection;
   BEGIN
      --  Is the item already in the selection ?
      WHILE Tmp /= NULL LOOP
         IF Tmp.Item = Canvas_Item (Item) THEN
            RETURN;
         END IF;
         Tmp := Tmp.Next;
      END LOOP;

      --  No => Add it
      Canvas.Selection := NEW Item_Selection_List_Record'
         (
         Item => Canvas_Item (Item),
         X    => Item.Coord.X,
         Y    => Item.Coord.Y,
         Next => Canvas.Selection);
   END Add_To_Selection;

   ---------------------------
   -- Remove_From_Selection --
   ---------------------------

   PROCEDURE Remove_From_Selection (
         Canvas : ACCESS Interactive_Canvas_Record;
         Item   : ACCESS Canvas_Item_Record'Class) IS
      Tmp      : Item_Selection_List := Canvas.Selection;
      Previous : Item_Selection_List := NULL;
   BEGIN
      WHILE Tmp /= NULL LOOP
         IF Tmp.Item = Canvas_Item (Item) THEN
            IF Previous = NULL THEN
               Canvas.Selection := Tmp.Next;
            ELSE
               Previous.Next := Tmp.Next;
            END IF;
            Free (Tmp);
            RETURN;
         END IF;
         Previous := Tmp;
         Tmp := Tmp.Next;
      END LOOP;
   END Remove_From_Selection;

   ---------------
   -- Configure --
   ---------------

   PROCEDURE Configure (
         Link  : ACCESS Canvas_Link_Record;
         Src   : ACCESS Canvas_Item_Record'Class;
         Dest  : ACCESS Canvas_Item_Record'Class;
         Arrow : IN     Arrow_Type               := End_Arrow;
         Descr : IN     String                   := "") IS
   BEGIN
      Link.Src := Canvas_Item (Src);
      Link.Dest := Canvas_Item (Dest);
      Link.Arrow := Arrow;

      Free (Link.Descr);
      Link.Descr := NEW String'(Descr);
   END Configure;

   --------------
   -- Add_Link --
   --------------

   PROCEDURE Add_Link (
         Canvas : ACCESS Interactive_Canvas_Record;
         Link   : ACCESS Canvas_Link_Record'Class) IS
      List : Canvas_Link_List := Canvas.Links;
      L    : Canvas_Link;
   BEGIN
      --  Do we already have links between the two items ?

      WHILE List /= NULL LOOP
         IF (List.Link.Src = Link.Src AND THEN List.Link.Dest = Link.Dest)
               OR ELSE
               (List.Link.Src = Link.Dest AND THEN List.Link.Dest = Link.
               Src)
               THEN
            L := List.Link;
            WHILE L.Sibling /= NULL LOOP
               L := L.Sibling;
            END LOOP;
            L.Sibling := Canvas_Link (Link);
            RETURN;
         END IF;
         List := List.Next;
      END LOOP;

      --  If not, we simply add a new item to the list

      Canvas.Links := NEW Canvas_Link_List_Record'
         (
         Link => Canvas_Link (Link),
         Next => Canvas.Links);
   END Add_Link;

   --------------
   -- Add_Link --
   --------------

   PROCEDURE Add_Link (
         Canvas : ACCESS Interactive_Canvas_Record;
         Src    : ACCESS Canvas_Item_Record'Class;
         Dest   : ACCESS Canvas_Item_Record'Class;
         Arrow  : IN     Arrow_Type                := End_Arrow;
         Descr  : IN     String                    := "") IS
      L : Canvas_Link :=
      NEW Canvas_Link_Record;
   BEGIN
      Configure (L, Src, Dest, Arrow, Descr);
      Add_Link (Canvas, L);
   END Add_Link;

   -----------------
   -- Remove_Link --
   -----------------

   PROCEDURE Remove_Link (
         Canvas : ACCESS Interactive_Canvas_Record;
         Link   : ACCESS Canvas_Link_Record'Class) IS
      Previous : Canvas_Link_List;
      Current  : Canvas_Link_List := Canvas.Links;
      L        : Canvas_Link;
   BEGIN
      WHILE Current /= NULL LOOP
         L := Current.Link;
         IF L = Canvas_Link (Link) THEN
            IF L.Sibling = NULL THEN
               IF Previous = NULL THEN
                  Canvas.Links := Current.Next;
               ELSE
                  Previous.Next := Current.Next;
               END IF;
               Free (Current);
            ELSE
               Current.Link := L.Sibling;
               Destroy (L);
               Free (L);
            END IF;
            RETURN;
         END IF;

         WHILE L /= NULL LOOP
            IF L.Sibling = Canvas_Link (Link) THEN
               L.Sibling := Link.Sibling;
               Destroy (Link);
               L := Canvas_Link (Link);
               Free (L);
               RETURN;
            END IF;
            L := L.Sibling;
         END LOOP;
         Previous := Current;
         Current := Current.Next;
      END LOOP;
   END Remove_Link;

   -------------------
   -- For_Each_Link --
   -------------------

   PROCEDURE For_Each_Link (
         Canvas  : ACCESS Interactive_Canvas_Record;
         Execute :        Link_Processor) IS
      Current  : Canvas_Link_List := Canvas.Links;
      Tmp      : Canvas_Link_List;
      Link,
      Tmp_Link : Canvas_Link;
   BEGIN
      WHILE Current /= NULL LOOP
         Tmp := Current.Next;
         Link := Current.Link;
         WHILE Link /= NULL LOOP
            Tmp_Link := Link.Sibling;
            EXIT WHEN NOT Execute (Canvas, Link);
            Link := Tmp_Link;
         END LOOP;
         Current := Tmp;
      END LOOP;
   END For_Each_Link;

   -------------
   -- Destroy --
   -------------

   PROCEDURE Destroy (
         Link : ACCESS Canvas_Link_Record) IS
   BEGIN
      Free (Link.Descr);
   END Destroy;

   PROCEDURE Destroy (
         Item : ACCESS Canvas_Item_Record) IS
   BEGIN
      NULL;
   END Destroy;

   PROCEDURE Destroy (
         Item : ACCESS Buffered_Item_Record) IS
   BEGIN
      IF Item.Pixmap /= NULL THEN
         Gdk.Pixmap.Unref (Item.Pixmap);
      END IF;
   END Destroy;

   -------------
   -- Get_Src --
   -------------

   FUNCTION Get_Src (
         Link : ACCESS Canvas_Link_Record)
     RETURN Canvas_Item IS
   BEGIN
      RETURN Link.Src;
   END Get_Src;

   --------------
   -- Get_Dest --
   --------------

   FUNCTION Get_Dest (
         Link : ACCESS Canvas_Link_Record)
     RETURN Canvas_Item IS
   BEGIN
      RETURN Link.Dest;
   END Get_Dest;

   ---------------
   -- Get_Descr --
   ---------------

   FUNCTION Get_Descr (
         Link : ACCESS Canvas_Link_Record)
     RETURN String IS
   BEGIN
      IF Link.Descr = NULL THEN
         RETURN "";
      ELSE
         RETURN Link.Descr.All;
      END IF;
   END Get_Descr;

   -----------------
   -- Set_Src_Pos --
   -----------------

   PROCEDURE Set_Src_Pos (
         Link  : ACCESS Canvas_Link_Record;
         X_Pos,
         Y_Pos :        Gfloat             := 0.5) IS
   BEGIN
      Link.Src_X_Pos := X_Pos;
      Link.Src_Y_Pos := Y_Pos;
   END Set_Src_Pos;

   ------------------
   -- Set_Dest_Pos --
   ------------------

   PROCEDURE Set_Dest_Pos (
         Link  : ACCESS Canvas_Link_Record;
         X_Pos,
         Y_Pos :        Gfloat             := 0.5) IS
   BEGIN
      Link.Dest_X_Pos := X_Pos;
      Link.Dest_Y_Pos := Y_Pos;
   END Set_Dest_Pos;

   ------------------
   -- Zoom_Timeout --
   ------------------

   FUNCTION Zoom_Timeout (
         Canvas : Interactive_Canvas)
     RETURN Boolean IS
      Z : CONSTANT Guint := Guint (Gint (Canvas.Zoom) + Canvas.Zoom_Step);











































   BEGIN
      IF (Canvas.Zoom_Step > 0 AND THEN Z >= Canvas.Target_Zoom)
            OR ELSE (Canvas.Zoom_Step < 0 AND THEN Z <= Canvas.
            Target_Zoom)
            THEN
         Zoom_Internal (Canvas, Canvas.Target_Zoom);
         RETURN False;
      ELSE
         Zoom_Internal (Canvas, Z);

         --  Force an immediate draw, otherwise since Queue_Draw does its work
         --  in an idle loop, it might not complete before the next timeout
         Draw (Canvas);
         RETURN True;
      END IF;
   END Zoom_Timeout;

   -------------------
   -- Zoom_Internal --
   -------------------

   PROCEDURE Zoom_Internal (
         Canvas  : ACCESS Interactive_Canvas_Record'Class;
         Percent :        Guint) IS
      Z : CONSTANT Gfloat := Gfloat (Canvas.Zoom);
      H :          Gfloat := Get_Value (Canvas.Hadj) / Z * 100.0;

      V : Gfloat := Get_Value (Canvas.Vadj) / Z * 100.0;

      Lh : Gfloat := Get_Page_Size (Canvas.Hadj) / Z * 100.0;

      Lv : Gfloat := Get_Page_Size (Canvas.Vadj) / Z * 100.0;

      L         : Gfloat;
      Font_Size : Gint;
   BEGIN
      Canvas.Zoom := Percent;

      IF Canvas.Font /= NULL THEN
         Unref (Canvas.Font);
      END IF;

      Font_Size := To_Canvas_Coordinates (Canvas, Canvas.
         Annotation_Height);
      IF Font_Size >= 1 THEN
         Canvas.Font := Get_Gdkfont (Canvas.Annotation_Font.All, Font_Size);
      ELSE
         Canvas.Font := NULL;
      END IF;

      Refresh_Canvas (Canvas);

      --  Display the proper area in the canvas
      --  When zooming out, we want to keep the old area centered into the
      --  new one.
      --  When zooming in, we want to keep the same center as before
      --  (reverse of zoom out)

      IF Gfloat (Canvas.Zoom) < Z THEN  --  zoom out
         L := (Z / Gfloat (Canvas.Zoom) - 1.0) / 2.0;
         H := (H - L * Lh) * Gfloat (Canvas.Zoom) / 100.0;
         V := (V - L * Lv) * Gfloat (Canvas.Zoom) / 100.0;
         Lh := Lh * (L * 2.0 + 1.0) * Gfloat (Canvas.Zoom) / 100.0;
         Lv := Lv * (L * 2.0 + 1.0) * Gfloat (Canvas.Zoom) / 100.0;

      ELSE  --  zoom in
         L := (1.0 - Z / Gfloat (Canvas.Zoom)) / 2.0;
         H := (H + Lh * L) * Gfloat (Canvas.Zoom) / 100.0;
         V := (V + Lv * L) * Gfloat (Canvas.Zoom) / 100.0;
         Lh := Lh * Z / 100.0;
         Lv := Lv * Z / 100.0;
      END IF;

      Set_Lower (Canvas.Hadj, Gfloat'Min (H, Get_Lower (Canvas.Hadj)));
      Set_Upper (Canvas.Hadj, Gfloat'Max (H + Lh, Get_Upper (Canvas.Hadj)));

      Set_Lower (Canvas.Vadj, Gfloat'Min (V, Get_Lower (Canvas.Vadj)));
      Set_Upper (Canvas.Vadj, Gfloat'Max (V + Lv, Get_Upper (Canvas.Vadj)));

      Set_Value (Canvas.Hadj, H);
      Set_Value (Canvas.Vadj, V);

      --  Do not report the modification to the scrollbar, since this does
      --  too much flickering and slows things done a lot when the canvas is
      --  in a scrolled_window whose policy is set on automatic.

      IF Canvas.Zoom = Canvas.Target_Zoom THEN
         Changed (Canvas.Hadj);
         Changed (Canvas.Vadj);
      END IF;

      Widget_Callback.Emit_By_Name (Canvas, "zoomed", Percent);
   END Zoom_Internal;

   ----------
   -- Zoom --
   ----------

   PROCEDURE Zoom (
         Canvas  : ACCESS Interactive_Canvas_Record;
         Percent :        Guint                     := 100;
         Steps   :        Guint                     := 1) IS
      Id : Timeout_Handler_Id;
   BEGIN
      IF Canvas.Zoom = Percent THEN
         RETURN;
      END IF;
      Canvas.Target_Zoom := Percent;

      --  Do we want smooth scrolling ?
      IF Steps > 1 THEN
         Canvas.Zoom_Step := ((Gint (Percent) - Gint(Canvas.Zoom)) / Gint (
               Steps));
         IF Canvas.Zoom_Step = 0 THEN
            IF Percent > Canvas.Zoom THEN
               Canvas.Zoom_Step := 1;
            ELSE
               Canvas.Zoom_Step := -1;
            END IF;
         END IF;
         Id := Canvas_Timeout.Add
            (Timeout_Between_Zooms, Zoom_Timeout'ACCESS,
            Interactive_Canvas (Canvas));

      ELSE
         Zoom_Internal (Canvas, Percent);
      END IF;
   END Zoom;

   --------------
   -- Get_Zoom --
   --------------

   FUNCTION Get_Zoom (
         Canvas : ACCESS Interactive_Canvas_Record)
     RETURN Glib.Guint IS
   BEGIN
      RETURN Canvas.Zoom;
   END Get_Zoom;

   --------------
   -- Scrolled --
   --------------

   PROCEDURE Scrolled (
         Canvas : ACCESS Gtk_Widget_Record'Class) IS
   BEGIN
      Queue_Draw (Canvas);
   END Scrolled;

   ----------
   -- Draw --
   ----------

   PROCEDURE Draw (
         Item   : ACCESS Buffered_Item_Record;
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Dest   :        Gdk.Pixmap.Gdk_Pixmap;
         Xdest,
         Ydest  :        Glib.Gint) IS
   BEGIN
      Draw_Pixmap (Dest, Canvas.Black_Gc, Item.Pixmap, 0, 0, Xdest, Ydest);
   END Draw;

   --------------------------------
   -- Set_Screen_Size_And_Pixmap --
   --------------------------------

   PROCEDURE Set_Screen_Size_And_Pixmap (
         Item   : ACCESS Buffered_Item_Record;
         Win    :        Gdk.Window.Gdk_Window;
         Width,
         Height :        Glib.Gint) IS
   BEGIN
      Set_Screen_Size (Item, Width, Height);

      IF Item.Pixmap /= NULL THEN
         Gdk.Pixmap.Unref (Item.Pixmap);
      END IF;

      Gdk_New (Item.Pixmap, Win, Width, Height);
   END Set_Screen_Size_And_Pixmap;

   ------------
   -- Pixmap --
   ------------

   FUNCTION Pixmap (
         Item : ACCESS Buffered_Item_Record)
     RETURN Gdk_Pixmap IS
   BEGIN
      RETURN Item.Pixmap;
   END Pixmap;

   FUNCTION Get_Id (
         Item : ACCESS Canvas_Item_Record'Class)
     RETURN Integer IS
   BEGIN
      RETURN Item.Id;
   END  Get_Id;

   PROCEDURE Set_Id (
         Item : ACCESS Canvas_Item_Record'Class;
         Val  :        Integer) IS
   BEGIN
      Item.Id := Val;
   END  Set_Id;


   FUNCTION Get_Figure (
         Item : ACCESS Canvas_Item_Record'Class)
     RETURN Unbounded_String IS

   BEGIN
      RETURN Item.Figure;
   END  Get_Figure;

   PROCEDURE Set_Figure (
         Item : ACCESS Canvas_Item_Record'Class;
         Val  :        Unbounded_String) IS
   BEGIN
      Item.Figure:=Val;
   END  Set_Figure;

   FUNCTION Get_Selected (
         Canvas : ACCESS Interactive_Canvas_Record'Class)
     RETURN Canvas_Item IS
   BEGIN
      RETURN Canvas.Selection.Item;
   END Get_Selected;

   PROCEDURE Set_Name (
         Item : ACCESS Canvas_Item_Record'Class;
         Nam  :        Unbounded_String) IS
   BEGIN
      Item.Name := Nam;
   END  Set_Name;

   FUNCTION Get_Name (
         Item : ACCESS Canvas_Item_Record'Class)
     RETURN Unbounded_String IS
   BEGIN
      RETURN Item.Name;
   END Get_Name;

   FUNCTION Name_Is_New (
         Canvas : ACCESS Interactive_Canvas_Record;
         Val    :        Unbounded_String)
     RETURN Boolean IS
      Ok : Boolean := True;
      PRAGMA Warnings(Off);
      FUNCTION Compare (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Item : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN
         IF Get_Name(Item) = Val THEN
            Ok := False;
            RETURN False;
         END IF;
         RETURN True;
      END Compare;
   BEGIN
      For_Each_Item(Canvas,Compare'Unrestricted_Access);
      RETURN Ok;
   END Name_Is_New;

   FUNCTION Get_Number_Of_Items (
         Canvas : ACCESS Interactive_Canvas_Record'Class)
     RETURN Guint IS

      Items_Number : Guint := 0;
      PRAGMA Warnings(Off);
      FUNCTION Internal (
            Canvas : ACCESS Interactive_Canvas_Record'Class;
            Item   : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN
         Items_Number := Items_Number + 1;
         RETURN True;
      END Internal;
   BEGIN
      For_Each_Item (Canvas, Internal'Unrestricted_Access);

      RETURN Items_Number;
   END Get_Number_Of_Items;


   FUNCTION Search_Item_By_Name (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Fig    :        Unbounded_String;
         Val    :        Unbounded_String)
     RETURN Canvas_Item IS
      Result : Canvas_Item;
      PRAGMA Warnings(Off);
      FUNCTION Internal (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Item : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS
      BEGIN
         IF(Get_Figure(Item) = Fig) AND THEN (Get_Name(Item) = Val) THEN
            Result := Canvas_Item(Item);
            RETURN False;
         END IF;
         RETURN True;
      END Internal;
   BEGIN
      For_Each_Item(Canvas,Internal'Unrestricted_Access);
      RETURN Result;
   END Search_Item_By_Name;



   PROCEDURE Set_Coordinates (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Sys    : IN     Systems.System) IS


      FUNCTION Set_An_Item (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Item : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS


         A_Task : Task_Ptr;
         A_Mess : Message_Ptr;
         A_Buff : Buffer_Ptr;


      BEGIN


         IF To_String(Get_Figure(Item))="buffer"
               THEN

            A_Buff:=Search_Buffer(Sys.Buffers, Get_Name(Item));
            Item.Coord.X:=Gint(A_Buff.X);
            Item.Coord.Y:=Gint(A_Buff.Y);
         ELSE

            IF To_String(Get_Figure(Item))="message"
                  THEN

               A_Mess:=Search_Message(Sys.Messages, Get_Name(Item));
               Item.Coord.X:=Gint(A_Mess.X);
               Item.Coord.Y:=Gint(A_Mess.Y);

            ELSE

               A_Task:=Search_Task(Sys.Tasks, Get_Name(Item));
               Item.Coord.X:=Gint(A_Task.X);
               Item.Coord.Y:=Gint(A_Task.Y);
            END IF;
         END IF;


         RETURN True;
      END Set_An_Item;

   BEGIN

      For_Each_Item(Canvas,Set_An_Item'Unrestricted_Access);

   END Set_Coordinates;




   PROCEDURE Get_Coordinates (
         Canvas : ACCESS Interactive_Canvas_Record'Class;
         Sys    : IN OUT Systems.System) IS

      FUNCTION Get_An_Item (
            Canv : ACCESS Interactive_Canvas_Record'Class;
            Item : ACCESS Canvas_Item_Record'Class)
        RETURN Boolean IS


         A_Task : Task_Ptr;
         A_Mess : Message_Ptr;
         A_Buff : Buffer_Ptr;


      BEGIN

         IF To_String(Get_Figure(Item))="buffer"
               THEN

            A_Buff:=Search_Buffer(Sys.Buffers, Get_Name(Item));
            A_Buff.X:=Integer(Item.Coord.X);
            A_Buff.Y:=Integer(Item.Coord.Y);

         ELSE

            IF To_String(Get_Figure(Item))="message"
                  THEN

               A_Mess:=Search_Message(Sys.Messages, Get_Name(Item));
               A_Mess.X:=Integer(Item.Coord.X);
               A_Mess.Y:=Integer(Item.Coord.Y);

            ELSE

               A_Task:=Search_Task(Sys.Tasks, Get_Name(Item));
               A_Task.X:=Integer(Item.Coord.X);
               A_Task.Y:=Integer(Item.Coord.Y);

            END IF;
         END IF;

         RETURN True;

      END Get_An_Item;

   BEGIN

      For_Each_Item(Canvas,Get_An_Item'Unrestricted_Access);

   END Get_Coordinates;



END My_Canvas;
