------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Buffers;
USE Buffers;
USE Buffers.Buffer_Roles;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Project;
USE Project;
WITH Mmi_Tools;
USE Mmi_Tools;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;


PACKAGE BODY New_Buffer_Role_Pkg.Callbacks IS


   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok          : Boolean;
      Role        : Unbounded_String;
      Task_Name   : Unbounded_String;
      Size_String : Unbounded_String;
      Size        : Natural;
      My_Iterator : Buffers_Iterator;
      A_Buffer    : Buffer_Ptr;

   BEGIN
      Task_Name:=To_Unbounded_String(Get_Text(New_Buffer_Role.
            Task_Name_Entry));
      Role:=To_Unbounded_String(Get_Text(New_Buffer_Role.Role_Entry));
      Size_String:=To_Unbounded_String(Get_Text(New_Buffer_Role.
            Size_Entry));


      IF(Size_String=To_Unbounded_String(""))
            THEN
         Size:=0;
      ELSE
         To_Natural(Size_String,Size,Ok);
         IF NOT Ok
               THEN
            Show_Message_Box(Lb_Size(Current_Language)
               & Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      IF (Size<=0)
            THEN
         Show_Message_Box(Lb_Size(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Buffers, My_Iterator);
      LOOP
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         IF (New_Buffer_Role.Buffer_Name = A_Buffer.Name) THEN

            A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Data.Size:=
               Size;
            A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Item:=
               Task_Name;
            IF  (Role="Producer")
                  THEN
               A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Data.
                  The_Role:=Producer;
            ELSE
               A_Buffer.Roles.Entries(A_Buffer.Roles.Nb_Entries).Data.
                  The_Role:=Consumer;
            END IF;
            A_Buffer.Roles.Nb_Entries:=A_Buffer.Roles.Nb_Entries+1;

         END IF;
         EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);
      END LOOP;


      Hide(New_Buffer_Role);
   END On_Ok_Pressed;



END New_Buffer_Role_Pkg.Callbacks;
