------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;
WITH Gtk;
USE Gtk;
WITH Gdk.Types;
USE Gdk.Types;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Display_Address_Spaces_Set_Pkg.Callbacks;
USE Display_Address_Spaces_Set_Pkg.Callbacks;

WITH Scheduler;
USE Scheduler;
WITH Project;
USE Project;
WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;





PACKAGE BODY Display_Address_Spaces_Set_Pkg IS

   PROCEDURE Gtk_New (
         Display_Address_Spaces_Set :    OUT Display_Address_Spaces_Set_Access) IS
   BEGIN
      Display_Address_Spaces_Set := NEW Display_Address_Spaces_Set_Record;
      Display_Address_Spaces_Set_Pkg.Initialize (
         Display_Address_Spaces_Set);
   END Gtk_New;

   PROCEDURE Initialize (
         Display_Address_Spaces_Set : ACCESS Display_Address_Spaces_Set_Record'Class) IS


      An_Address_Space : Address_Space_Ptr;
      My_Iterator      : Iterator;
      Ligne            : Guint             := 1;
      Heigth           : Gint;

   BEGIN
      Gtk.Window.Initialize (Display_Address_Spaces_Set, Window_Toplevel);
      Set_Title (Display_Address_Spaces_Set,To_String(
            Lb_Address_Spaces_List(
               Current_Language)));
      Set_Policy (Display_Address_Spaces_Set, True, True, True);
      Heigth:=Gint(3+Get_Number_Of_Elements(Sys.Address_Spaces))*50;
      IF Heigth > 400
            THEN
         Heigth:=400;
      END IF;
      Set_Default_Size (Display_Address_Spaces_Set, 700, Heigth);
      Set_Position (Display_Address_Spaces_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Address_Spaces_Set, True);
      Set_Extension_Events (Display_Address_Spaces_Set,
         Extension_Events_All);
      Set_Flags (Display_Address_Spaces_Set, Can_Default);

      Grab_Focus (Display_Address_Spaces_Set);


      Gtk_New (Display_Address_Spaces_Set.Scroll);
      Set_Border_Width (Display_Address_Spaces_Set.Scroll, Border_Width =>
         10);
      Set_Policy (Display_Address_Spaces_Set.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Display_Address_Spaces_Set, Display_Address_Spaces_Set.Scroll);


      Gtk_New (Display_Address_Spaces_Set.Table, Guint(3+
            Get_Number_Of_Elements(
               Sys.Address_Spaces)), 2, True);
      Set_Row_Spacings (Display_Address_Spaces_Set.Table, 1);
      Set_Col_Spacings (Display_Address_Spaces_Set.Table, 1);


      Add_With_Viewport (Display_Address_Spaces_Set.Scroll,
         Display_Address_Spaces_Set.
         Table);
      Set_Focus_Hadjustment (Display_Address_Spaces_Set.Table,
         Get_Hadjustment (
            Display_Address_Spaces_Set.Scroll));
      Set_Focus_Vadjustment (Display_Address_Spaces_Set.Table,
         Get_Vadjustment (
            Display_Address_Spaces_Set.Scroll));


      Gtk_New (Display_Address_Spaces_Set.Lab, To_String(
            Lb_Address_Space_Name(
               Current_Language)));
      Set_Alignment (Display_Address_Spaces_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Address_Spaces_Set.Lab, 0, 0);
      Set_Justify (Display_Address_Spaces_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Address_Spaces_Set.Lab, False);
      Attach (Display_Address_Spaces_Set.Table,
         Display_Address_Spaces_Set.
         Lab, 0, 1, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Address_Spaces_Set.Lab, To_String(Lb_Processor(
               Current_Language)));
      Set_Alignment (Display_Address_Spaces_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Address_Spaces_Set.Lab, 0, 0);
      Set_Justify (Display_Address_Spaces_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Address_Spaces_Set.Lab, False);
      Attach (Display_Address_Spaces_Set.Table,
         Display_Address_Spaces_Set.
         Lab, 1, 2,
         0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Display_Address_Spaces_Set.Lab, To_String(Lb_Scheduler(
               Current_Language)));
      Set_Alignment (Display_Address_Spaces_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Address_Spaces_Set.Lab, 0, 0);
      Set_Justify (Display_Address_Spaces_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Address_Spaces_Set.Lab, False);
      Attach (Display_Address_Spaces_Set.Table,
         Display_Address_Spaces_Set.
         Lab, 2, 3,
         0,
         1,
         0, 0,
         0, 0);



      Reset_Iterator(Sys.Address_Spaces, My_Iterator);

      LOOP
         Current_Element(Sys.Address_Spaces, An_Address_Space, My_Iterator);

         Gtk_New (Display_Address_Spaces_Set.Entry_Name);
         Set_Editable (Display_Address_Spaces_Set.Entry_Name, False);
         Set_Max_Length (Display_Address_Spaces_Set.Entry_Name, 0);
         Set_Text (Display_Address_Spaces_Set.Entry_Name, To_String(
               An_Address_Space.Name) );
         Set_Visibility (Display_Address_Spaces_Set.Entry_Name, True);
         Attach (Display_Address_Spaces_Set.Table,
            Display_Address_Spaces_Set.Entry_Name, 0,
            1,
            Ligne,
            Ligne+1,
            Expand OR Fill, 0,
            0, 0);

         Gtk_New (Display_Address_Spaces_Set.Entry_Processor);
         Set_Editable (Display_Address_Spaces_Set.Entry_Processor, False);
         Set_Max_Length (Display_Address_Spaces_Set.Entry_Processor, 0);
         Set_Text (Display_Address_Spaces_Set.Entry_Processor, To_String(
               An_Address_Space.
               Cpu_Name));
         Set_Visibility (Display_Address_Spaces_Set.Entry_Processor, True);
         Attach (Display_Address_Spaces_Set.Table,
            Display_Address_Spaces_Set.
            Entry_Processor, 1, 2,
            Ligne,
            Ligne+1,
            Expand OR Fill, 0,
            0, 0);



         Gtk_New (Display_Address_Spaces_Set.Entry_Scheduler);
         Set_Editable (Display_Address_Spaces_Set.Entry_Scheduler, False);
         Set_Max_Length (Display_Address_Spaces_Set.Entry_Scheduler, 0);
         Set_Text (Display_Address_Spaces_Set.Entry_Scheduler, To_String(
               Get_Name(An_Address_Space.
                  Scheduler.All)));
         Set_Visibility (Display_Address_Spaces_Set.Entry_Scheduler, True);
         Attach (Display_Address_Spaces_Set.Table,
            Display_Address_Spaces_Set.
            Entry_Scheduler, 2, 3,
            Ligne,
            Ligne+1,
            Expand OR Fill, 0,
            0, 0);


         Ligne:=Ligne+1;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces, My_Iterator);

         Next_Element(Sys.Address_Spaces, My_Iterator);

      END LOOP;



      Gtk_New_Hseparator (Display_Address_Spaces_Set.Separator);
      Attach (Display_Address_Spaces_Set.Table,
         Display_Address_Spaces_Set.
         Separator, 0,
         3, Ligne, Ligne+1,
         Fill, Expand OR Fill,
         0, 0);

      Ligne:=Ligne+1;
      Gtk_New (Display_Address_Spaces_Set.Close_Button, To_String(
            Lb_Close(
               Current_Language)));
      Set_Usize (Display_Address_Spaces_Set.Close_Button, 70, 25);
      Attach (Display_Address_Spaces_Set.Table,
         Display_Address_Spaces_Set.
         Close_Button, 0, 3,
         Ligne, Ligne+1,
         0, 0,
         0, 0);
      Button_Callback.Connect
         (Display_Address_Spaces_Set.Close_Button, "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));



   END Initialize;

END Display_Address_Spaces_Set_Pkg;
