------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;


WITH Project;
USE Project;

WITH Buffers;
USE Buffers;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
USE Buffers.Buffer_Roles;




PACKAGE BODY Display_Buffers_Set_Pkg.Callbacks IS


   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Display_Buffers_Set);
   END On_Ok_Pressed;



   PROCEDURE Show_Roles (
         A_Buffer_Name : Unbounded_String) IS

      A_Buffer    : Buffer_Ptr;
      My_Iterator : Iterator;
      Index       : Integer                  := 0;
      Line        : Gint;
      Role_Share  : Chars_Ptr_Array (1 .. 4);
      Role_Type   : Unbounded_String;

   BEGIN
      Reset_Iterator(Sys.Buffers, My_Iterator);
      Clear(Display_Buffers_Set.Role_List);

      LOOP
         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         IF (Natural(A_Buffer.Roles.Nb_Entries) >= 1) THEN
            IF A_Buffer.Name = A_Buffer_Name THEN
               FOR I IN 0..Buffer_Roles_Range(Natural(A_Buffer.Roles.
                        Nb_Entries) -1) LOOP
                  IF ( A_Buffer.Roles.Entries(I).Data.The_Role = Producer) THEN
                     Role_Type:=To_Unbounded_String("Producer");
                  ELSE
                     Role_Type:=To_Unbounded_String("Consumer");
                  END IF;
                  Role_Share:= To_String(A_Buffer.Roles.Entries(I).Item)
                     + To_String(To_Unbounded_String(Natural'Image(
                           A_Buffer.Roles.Entries(I).Data.Size)))
                     + To_String(Role_Type)
                     + To_String(To_Unbounded_String(Natural'Image(
                           A_Buffer.Roles.Entries(I).Data.Time)))
                     ;
                  Display_Buffers_Set.Current_Role_Index:=
                     Indexed_Table_Range(
                     Index);
                  Index:=Index+1;
                  Line:= Append(Display_Buffers_Set.Role_List, Role_Share);
               END LOOP;
            END IF;
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);
      END LOOP;
   END Show_Roles;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Buffer_Name : Unbounded_String;
      Row         : Gint;

   BEGIN
      Row:=Get_Focus_Row(Display_Buffers_Set.Buffer_List);
      Buffer_Name:= To_Unbounded_String(Get_Text(Display_Buffers_Set.
            Buffer_List,Row,0));

      Show_Roles(Buffer_Name);
   END On_Click_List;

END Display_Buffers_Set_Pkg.Callbacks;
