------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtk;
USE Gtk;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Display_Messages_Set_Pkg.Callbacks;
USE Display_Messages_Set_Pkg.Callbacks;
WITH Project;
USE Project;
WITH Glib;
USE Glib;

PACKAGE BODY Display_Messages_Set_Pkg IS


   PROCEDURE Gtk_New (
         Display_Messages_Set :    OUT Display_Messages_Set_Access) IS
   BEGIN


      Display_Messages_Set := NEW Display_Messages_Set_Record;
      Display_Messages_Set_Pkg.Initialize (Display_Messages_Set);

   END Gtk_New;


   PROCEDURE Initialize (
         Display_Messages_Set : ACCESS Display_Messages_Set_Record'Class) IS


      A_Message          : Message_Ptr;
      My_Iterator        : Iterator;
      The_Name           : Unbounded_String;
      The_Jitter         : Natural;
      The_Deadline       : Natural;
      The_Period         : Natural;
      The_Size           : Natural;
      The_Delay          : Natural;
      A_Line             : Chars_Ptr_Array (1 .. 6);
      Line               : Gint;
      Tmp_Jitter         : Unbounded_String;
      Tmp_Deadline       : Unbounded_String;
      Tmp_Period         : Unbounded_String;
      Tmp_Size           : Unbounded_String;
      Tmp_Delay          : Unbounded_String;
      A_Periodic_Message : Periodic_Message_Ptr;

   BEGIN

      Gtk.Window.Initialize(Display_Messages_Set, Window_Toplevel);
      Set_Policy (Display_Messages_Set, True, True, True);
      Set_Title (Display_Messages_Set, To_String(Lb_Messages_List(
               Current_Language)));
      Set_Position (Display_Messages_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Messages_Set, True);
      Grab_Focus(Display_Messages_Set);
      Gtk.Box.Gtk_New_Vbox (Display_Messages_Set.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Messages_Set.Hbox_Up,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Messages_Set.Hbox_Down,False,10);

      Add(Display_Messages_Set,Display_Messages_Set.Vbox_Main);
      Add(Display_Messages_Set.Vbox_Main,Display_Messages_Set.Hbox_Up);
      Add(Display_Messages_Set.Vbox_Main,Display_Messages_Set.Hbox_Down);

      Gtk.Scrolled_Window.Gtk_New(Display_Messages_Set.Scroll_List, NULL, NULL);

      Add(Display_Messages_Set.Hbox_Up,Display_Messages_Set.Scroll_List);

      Display_Messages_Set.List_Titles:= To_String(Lb_Name(
            Current_Language))
         +To_String(Lb_Jitter(Current_Language))
         +To_String(Lb_Deadline(Current_Language))
         +To_String(Lb_Period(Current_Language))
         +To_String(Lb_Size(Current_Language))
         +To_String(Lb_Response_Time_Message(Current_Language));

      Gtk.Clist.Gtk_New(Display_Messages_Set.List,6,Display_Messages_Set.
         List_Titles);
      Set_Usize(Display_Messages_Set.List,300,200);
      Add(Display_Messages_Set.Scroll_List,Display_Messages_Set.List);

      Gtk.Button.Gtk_New(Display_Messages_Set.Button_Close, To_String(
            Lb_Close(Current_Language)));

      Button_Callback.Connect (Display_Messages_Set.Button_Close,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));

      Add(Display_Messages_Set.Hbox_Down,Display_Messages_Set.
         Button_Close);



      -- Put a Line in the Clist
      --
      Reset_Iterator(Sys.Messages, My_Iterator);
      LOOP

         Current_Element(Sys.Messages, A_Message, My_Iterator);

         The_Name  := A_Message.Name;
         The_Deadline := A_Message.Deadline;
         The_Size     := A_Message.Size;
         The_Delay    := A_Message.Response_Time;

         IF(A_Message.Message_Type=Periodic_Type) THEN
            A_Periodic_Message := Periodic_Message_Ptr(A_Message);
            The_Jitter   := A_Periodic_Message.Jitter;
            The_Period  := A_Periodic_Message.Period;
         ELSE
            The_Jitter   := 0;
            The_Period   := 0;
         END IF;
         Tmp_Jitter   := To_Unbounded_String(Natural'Image(The_Jitter));
         Tmp_Deadline := To_Unbounded_String(Natural'Image(The_Deadline));
         Tmp_Period   := To_Unbounded_String(Natural'Image(The_Period));
         Tmp_Size     := To_Unbounded_String(Natural'Image(The_Size));
         Tmp_Delay    := To_Unbounded_String(Natural'Image(The_Delay));

         -- Create the line
         --
         A_Line := To_String(The_Name)+To_String(Tmp_Jitter)+To_String(
            Tmp_Deadline)+To_String(Tmp_Period)+To_String(Tmp_Size)+
            To_String(Tmp_Delay);

         -- Display the Line
         --
         Line := Append(Display_Messages_Set.List, A_Line);

         EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);

         Next_Element(Sys.Messages, My_Iterator);

      END LOOP;

   END Initialize;


END Display_Messages_Set_Pkg;

