------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtk;
USE Gtk;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Display_Networks_Set_Pkg.Callbacks;
USE Display_Networks_Set_Pkg.Callbacks;
WITH Project;
USE Project;
WITH Glib;
USE Glib;

PACKAGE BODY Display_Networks_Set_Pkg IS


   PROCEDURE Gtk_New (
         Display_Networks_Set :    OUT Display_Networks_Set_Access) IS
   BEGIN

      Display_Networks_Set := NEW Display_Networks_Set_Record;
      Display_Networks_Set_Pkg.Initialize (Display_Networks_Set);

   END Gtk_New;


   PROCEDURE Initialize (
         Display_Networks_Set : ACCESS Display_Networks_Set_Record'Class) IS


      A_Network   : Network_Ptr;
      My_Iterator : Iterator;
      The_Name    : Unbounded_String;
      The_Type    : Unbounded_String;
      A_Line      : Chars_Ptr_Array (1 .. 2);
      Line        : Gint;

   BEGIN

      Gtk.Window.Initialize(Display_Networks_Set, Window_Toplevel);

      Set_Policy (Display_Networks_Set, True, True, True);
      Set_Title (Display_Networks_Set, To_String(Lb_Networks_List(
               Current_Language)));
      Set_Position (Display_Networks_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Networks_Set, True);
      Grab_Focus(Display_Networks_Set);
      Gtk.Box.Gtk_New_Vbox (Display_Networks_Set.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Networks_Set.Hbox_Up,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Networks_Set.Hbox_Down,False,10);

      Add(Display_Networks_Set,Display_Networks_Set.Vbox_Main);
      Add(Display_Networks_Set.Vbox_Main,Display_Networks_Set.Hbox_Up);
      Add(Display_Networks_Set.Vbox_Main,Display_Networks_Set.Hbox_Down);

      Gtk.Scrolled_Window.Gtk_New(Display_Networks_Set.Scroll_List, NULL, NULL);

      Add(Display_Networks_Set.Hbox_Up,Display_Networks_Set.Scroll_List);

      Display_Networks_Set.List_Titles:= To_String(Lb_Networks_Name(
            Current_Language))
         +To_String(Lb_View_Networks_Type(Current_Language));

      Gtk.Clist.Gtk_New(Display_Networks_Set.List,2,
         Display_Networks_Set.List_Titles);
      Set_Column_Width(Display_Networks_Set.List, 0, 150);
      Set_Usize(Display_Networks_Set.List,300,200);
      Add(Display_Networks_Set.Scroll_List,Display_Networks_Set.List);

      Gtk.Button.Gtk_New(Display_Networks_Set.Button_Close, To_String(
            Lb_Close(Current_Language)));

      Button_Callback.Connect (Display_Networks_Set.Button_Close,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));

      Add(Display_Networks_Set.Hbox_Down,Display_Networks_Set.
         Button_Close);

      Reset_Iterator(Sys.Networks, My_Iterator);

      LOOP

         Current_Element(Sys.Networks, A_Network, My_Iterator);

         The_Name := A_Network.Name;
         The_Type := To_Unbounded_String(A_Network.Network_Type'Img);

         -- Create the line
         --
         A_Line := To_String(The_Name)+To_String(The_Type);

         -- Display the Line
         --
         Line := Append(Display_Networks_Set.List, A_Line);

         EXIT WHEN Is_Last_Element(Sys.Networks, My_Iterator);

         Next_Element(Sys.Networks, My_Iterator);

      END LOOP;

   END Initialize;


END Display_Networks_Set_Pkg;
