------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtk;
USE Gtk;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Resources;
USE Resources;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Display_Resources_Set_Pkg.Callbacks;
USE Display_Resources_Set_Pkg.Callbacks;
WITH Project;
USE Project;
WITH Glib;
USE Glib;

PACKAGE BODY Display_Resources_Set_Pkg IS


   PROCEDURE Gtk_New (
         Display_Resources_Set :    OUT Display_Resources_Set_Access) IS
   BEGIN


      Display_Resources_Set := NEW Display_Resources_Set_Record;
      Display_Resources_Set_Pkg.Initialize (Display_Resources_Set);

   END Gtk_New;


   PROCEDURE Initialize (
         Display_Resources_Set : ACCESS Display_Resources_Set_Record'Class) IS


      A_Resource   : Resource_Ptr;
      My_Iterator  : Iterator;
      The_Cpu      : Unbounded_String;
      The_Name     : Unbounded_String;
      The_State    : Integer;
      The_Protocol : Unbounded_String;
      A_Line       : Chars_Ptr_Array (1 .. 4);
      Line         : Gint;

   BEGIN

      Gtk.Window.Initialize(Display_Resources_Set, Window_Toplevel);

      Set_Policy (Display_Resources_Set, True, True, True);
      Set_Title (Display_Resources_Set, To_String(Lb_Resources_List(
               Current_Language)));
      Set_Position (Display_Resources_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Resources_Set, True);
      Grab_Focus(Display_Resources_Set);
      Gtk.Box.Gtk_New_Vbox (Display_Resources_Set.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Resources_Set.Hbox_Up,False,10);
      Gtk.Box.Gtk_New_Hbox (Display_Resources_Set.Hbox_Down,False,10);

      Add(Display_Resources_Set,Display_Resources_Set.Vbox_Main);
      Add(Display_Resources_Set.Vbox_Main,Display_Resources_Set.Hbox_Up);
      Add(Display_Resources_Set.Vbox_Main,Display_Resources_Set.Hbox_Down);

      Gtk.Scrolled_Window.Gtk_New(Display_Resources_Set.Scroll_List, NULL, NULL);

      Add(Display_Resources_Set.Hbox_Up,Display_Resources_Set.Scroll_List);

      Display_Resources_Set.List_Titles:= To_String(Lb_Resource_Name(
            Current_Language))
         +To_String(Lb_State(Current_Language))
         +To_String(Lb_Protocol(Current_Language))
         +To_String(Lb_Processor_Name(Current_Language));
      Gtk.Clist.Gtk_New(Display_Resources_Set.List,4,
         Display_Resources_Set.List_Titles);
      Set_Column_Width(Display_Resources_Set.List, 0, 150);
      Set_Usize(Display_Resources_Set.List,300,200);
      Add(Display_Resources_Set.Scroll_List,Display_Resources_Set.List);

      Gtk.Button.Gtk_New(Display_Resources_Set.Button_Close, To_String(
            Lb_Close(Current_Language)));

      Button_Callback.Connect (Display_Resources_Set.Button_Close,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));

      Add(Display_Resources_Set.Hbox_Down,Display_Resources_Set.
         Button_Close);



      -- Put a Line in the Clist
      --
      Reset_Iterator(Sys.Resources, My_Iterator);
      LOOP

         Current_Element(Sys.Resources, A_Resource, My_Iterator);

         The_Cpu := A_Resource.Cpu_Name;
         The_Name := A_Resource.Name;
         The_State := A_Resource.State;

         -- Name of the protocol
         --
         The_Protocol := To_Unbounded_String(A_Resource.Protocol'Img);

         -- Create the line
         --
         A_Line := To_String(The_Name)+To_String(To_Unbounded_String(
               Integer'Image(The_State)))+To_String(The_Protocol)+
            To_String(The_Cpu);

         -- Display the Line
         --
         Line := Append(Display_Resources_Set.List, A_Line);

         EXIT WHEN Is_Last_Element(Sys.Resources, My_Iterator);

         Next_Element(Sys.Resources, My_Iterator);

      END LOOP;





   END Initialize;


END Display_Resources_Set_Pkg;
