------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Framework_Config;
USE Framework_Config;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Translate;
USE Translate;


WITH Project;
USE Project;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Processors;
USE Processors;
WITH Glib;
USE Glib;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Gtk.Button;
USE Gtk.Button;


WITH Text_Tools;
USE Text_Tools;

WITH Call_Framework;
USE Call_Framework;
USE Call_Framework.Framework_Request_Package;
USE Call_Framework.Framework_Response_Package;


WITH Gtk.Check_Button;
USE Gtk.Check_Button;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Glib;
USE Glib;

WITH Text_Io;
USE Text_Io;





PACKAGE BODY Do_Scheduling_Feasibility_Pkg.Callbacks IS




   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Test1       : Boolean       := False;
      Test2       : Boolean       := False;
      A_Processor : Processor_Ptr;
      Cpu_Index   : Gint;
      My_Iterator : Iterator;


      PROCEDURE Call_Feasibility (
            Processor_Name : IN     Unbounded_String) IS

         Response_List : Framework_Response_Table;
         Request_List  : Framework_Request_Table;
         A_Request     : Framework_Request;

      BEGIN
         Initialize(Response_List);
         Initialize(Request_List);

         IF Test1 THEN
            Initialize(A_Request);
            A_Request.Target:=Processor_Name;
            A_Request.Statement:=Scheduling_Feasibility_Cpu_Utilization;
            Add(Request_List, A_Request);
         END IF;

         IF Test2 THEN
            Initialize(A_Request);
            A_Request.Target:=Processor_Name;
            A_Request.Statement:=Scheduling_Feasibility_Response_Time;
            Add(Request_List, A_Request);
         END IF;


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title(Do_Scheduling_Feasibility.Text, Response_List.
            Entries(0).Title);
         FOR J IN 0..Response_List.Nb_Entries-1 LOOP
            Write_Text(Do_Scheduling_Feasibility.Text, Response_List.
               Entries(J).Text);
         END LOOP;

      END Call_Feasibility;


   BEGIN

      IF Get_Active(Do_Scheduling_Feasibility.Test1)
            THEN
         Test1:=True;
      END IF;

      IF Get_Active(Do_Scheduling_Feasibility.Test2)
            THEN
         Test2:=True;
      END IF;


      IF (Test1=False) AND (Test2=False)
            THEN
         Show_Message_Box(Lb_Select_Almost_One_Feasilibity_Test(
               Current_Language) );
         RETURN;
      END IF;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (
               Do_Scheduling_Feasibility.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors, A_Processor, Processors_Range(
            Cpu_Index));


      IF NOT Get_Active(Do_Scheduling_Feasibility.
            Schedule_All_Processeurs)
            THEN
         Call_Feasibility(A_Processor.Name);
      ELSE

         Reset_Iterator(Sys.Processors, My_Iterator);

         LOOP
            Current_Element(Sys.Processors, A_Processor, My_Iterator);

            Call_Feasibility(A_Processor.Name);

            EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);

            Next_Element(Sys.Processors, My_Iterator);

         END LOOP;

      END IF;

      Destroy(Do_Scheduling_Feasibility);

   END On_Ok_Pressed;




   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Do_Scheduling_Feasibility);
   END On_Cancel_Pressed;




END Do_Scheduling_Feasibility_Pkg.Callbacks;



