------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;

WITH Gtk.Check_Button;
USE Gtk.Check_Button;


WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Project;
USE Project;
WITH Glib;
USE Glib;

WITH Text_Io;
USE Text_Io;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Generic_Select_File_Pkg;



PACKAGE BODY Do_Scheduling_Simulation_Pkg.Callbacks IS




   PROCEDURE Read_Sc_File (
         File_Name : IN     Unbounded_String) IS

   BEGIN
      Set_Text(Do_Scheduling_Simulation.
         Event_Table_File_Name,
         To_String(File_Name) );
   END Read_Sc_File;



   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;

   PACKAGE Read_Sc_File_Package IS NEW Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   USE Read_Sc_File_Package;



   PROCEDURE On_Read_From_File_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

   BEGIN

      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Sc_File_Package.Generic_Select_File);


   END On_Read_From_File_Pressed;



   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      A_Processor : Processor_Ptr;
      Cpu_Index   : Gint;


   BEGIN


      -- Get customization provided by the user
      --
      Context_Switch_From_Simulation :=
         Get_Active(Do_Scheduling_Simulation.Context_Switch);
      Preemption_From_Simulation:=
         Get_Active(Do_Scheduling_Simulation.Number_Preemption);
      Run_Event_Analyzer_From_Simulation:=
         Get_Active(Do_Scheduling_Simulation.Event_Analyzer);
      Schedule_And_Display_Event_Table:=
         Get_Active(Do_Scheduling_Simulation.
         Schedule_And_Display_Event_Table);
      Schedule_And_Export_Event_Table:=
         Get_Active(Do_Scheduling_Simulation.
         Schedule_And_Export_Event_Table);
      Event_Table_File_Name:=To_Unbounded_String(Get_Text(
            Do_Scheduling_Simulation.Event_Table_File_Name));
      Schedule_All_Processeurs:=
         Get_Active(Do_Scheduling_Simulation.Schedule_All_Processeurs);
      Xml_Event_Table_File_Name:= To_Unbounded_String(Get_Text(
            Do_Scheduling_Simulation.
            Event_Table_File_Name));


      FOR I IN Data_From_Simulation_Table'RANGE LOOP
         Blocking_Time_From_Simulation(I):=
            Get_Active(Do_Scheduling_Simulation.Blocking_Time(I));
         Response_Time_From_Simulation(I):=
            Get_Active(Do_Scheduling_Simulation.Response_Time(I));
      END LOOP;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (
               Do_Scheduling_Simulation.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors, A_Processor, Processors_Range(
            Cpu_Index));
      Last_Selected_Processor :=A_Processor.Name;



      Do_Scheduling_Simulation.Call.All;


      Hide(Do_Scheduling_Simulation);

   END On_Ok_Pressed;





   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Hide(Do_Scheduling_Simulation);
   END On_Cancel_Pressed;




END Do_Scheduling_Simulation_Pkg.Callbacks;



