------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Gdk;
WITH Gdk.Event;
USE Gdk.Event;
WITH Gtk;
USE Gtk;
WITH Double_Buffer;
USE Double_Buffer;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gdk.Color;
USE Gdk.Color;
WITH Gdk.Font;
USE Gdk.Font;
WITH Glib;
USE Glib;
WITH Gdk.Gc;
USE Gdk.Gc;

WITH Buffers;
USE Buffers;
WITH Scheduling_Sequences;
USE Scheduling_Sequences;
WITH Time_Units;
USE Time_Units;


PACKAGE Draw_Buffer_Pkg IS



   TYPE Draw_Buffer_Record IS NEW Gtk_Double_Buffer_Record WITH
      RECORD
         Font : Gdk_Font;

         Buff             : Buffer_Size_Table_Ptr;
         Last_Time_Unit   : Natural;
         Time_Unit_Number : Time_Unit_Range;
         Max_Buff         : Natural;

         White_Gc  : Gdk.Gc.Gdk_Gc;
         Black_Gc  : Gdk.Gc.Gdk_Gc;
         Red_Color : Gdk_Color;

         Left_Margin     : Gint := 60;
         High_Margin     : Gint := 20;
         Buffer_Margin   : Gint := 20;
         Timeunit_Margin : Gint := 20;

         First_Width    : Gint := 800;
         First_Height   : Gint := 300;
         Current_Width  : Gint;
         Current_Height : Gint;

         Have_Colors : Boolean := False;
         Initialized : Boolean := False;

         Buff_Ptr : Buffer_Ptr;
      END RECORD;

   TYPE Draw_Buffer_Access IS ACCESS ALL Draw_Buffer_Record'Class;

   PROCEDURE Gtk_New (
         Draw_Buffer :    OUT Draw_Buffer_Access);
   PROCEDURE Initialize (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class);

   PROCEDURE Redraw (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class);

   PROCEDURE Draw (
         Draw_Buffer       : ACCESS Draw_Buffer_Record'Class;
         Buff              :        Buffer_Ptr;
         Buff1             :        Buffer_Size_Table;
         Last_Time_Unit1   :        Natural;
         Time_Unit_Number1 :        Time_Unit_Range);

   PROCEDURE Clear (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class);

   FUNCTION Configure (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class;
         Event       :        Gdk.Event.Gdk_Event)
     RETURN Boolean;

   PROCEDURE Color_Allocation (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class);

   PROCEDURE Size_Allocate (
         Draw_Buffer : ACCESS Draw_Buffer_Record'Class;
         Allocation  :        Gtk_Allocation_Access);

   Draw_Buffer : Draw_Buffer_Access;

   PRAGMA Inline (Redraw);
   PRAGMA Inline (Color_Allocation);
   PRAGMA Inline (Clear);



END Draw_Buffer_Pkg;
