------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;
WITH Gdk;
USE Gdk;
WITH Gdk.Event;
USE Gdk.Event;
WITH Gdk.Font;
USE Gdk.Font;
WITH Gdk.Gc;
USE Gdk.Gc;
WITH Gdk.Color;
USE Gdk.Color;
WITH Gtk;
USE Gtk;
WITH Double_Buffer;
USE Double_Buffer;
WITH Gtk.Widget;
USE Gtk.Widget;

WITH Buffer_Set;
USE Buffer_Set;
WITH Task_Set;
USE Task_Set;
WITH Resource_Set;
USE Resource_Set;
WITH Message_Set;
USE Message_Set;
WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;


PACKAGE Draw_Scheduling_Pkg IS


   TYPE Draw_Scheduling_Record IS TAGGED
      RECORD
         Font   : Gdk_Font;
         Buffer : Gtk_Double_Buffer;

         Tasks      : Tasks_Set;
         Resources  : Resources_Set;
         Messages   : Messages_Set;
         Buffers    : Buffers_Set;
         Period     : Natural;
         Start_Draw : Natural;
         Sched      : Scheduling_Table_Ptr;

         White_Gc   : Gdk.Gc.Gdk_Gc;
         Black_Gc   : Gdk.Gc.Gdk_Gc;
         Red_Color  : Gdk_Color;
         Blue_Color : Gdk_Color;

         Left_Margin     : Gint := 0;
         Right_Margin    : Gint := 20;
         High_Margin     : Gint := 20;
         Interligne      : Gint := 50;
         Timeunit_Margin : Gint := 20;

         First_Width    : Gint := 800;
         First_Height   : Gint := 300;
         Current_Width  : Gint;
         Current_Height : Gint;

         Have_Colors : Boolean := False;
         Initialized : Boolean := False;
      END RECORD;

   TYPE Draw_Scheduling_Access IS ACCESS ALL Draw_Scheduling_Record'Class;

   PROCEDURE Gtk_New (
         Draw_Scheduling :    OUT Draw_Scheduling_Access);
   PROCEDURE Initialize (
         Draw_Scheduling : ACCESS Draw_Scheduling_Record'Class);

   PROCEDURE Redraw_Time_Line (
         Draw_Scheduling : ACCESS Draw_Scheduling_Record'Class);

   PROCEDURE Draw_Time_Line (
         Draw_Scheduling : ACCESS Draw_Scheduling_Record;
         Task1           :        Tasks_Set;
         Resource1       :        Resources_Set;
         Message1        :        Messages_Set;
         Buffer1         :        Buffers_Set;
         Period1         :        Natural;
         Start_Draw1     :        Natural;
         Sched1          :        Scheduling_Table_Ptr);

   PROCEDURE Clear_Time_Line (
         Draw_Scheduling : ACCESS Draw_Scheduling_Record);

   PROCEDURE Color_Allocation (
         Draw_Scheduling : ACCESS Draw_Scheduling_Record);

   FUNCTION Configure (
         Buffer : ACCESS Gtk_Double_Buffer_Record'Class;
         Event  :        Gdk.Event.Gdk_Event)
     RETURN Boolean;
   PROCEDURE Size_Allocate (
         Buffer     : ACCESS Gtk_Double_Buffer_Record'Class;
         Allocation :        Gtk_Allocation_Access);


   Draw_Scheduling : Draw_Scheduling_Access;


END Draw_Scheduling_Pkg;









