------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;


WITH Call_Framework;
USE Call_Framework;
USE Call_Framework.Framework_Request_Package;
USE Call_Framework.Framework_Response_Package;

WITH Parameters;
USE Parameters;
USE Parameters.Framework_Parameters_Table_Package;

WITH Call_Scheduling_Framework;
USE Call_Scheduling_Framework;

WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;

WITH Framework_Config;
USE Framework_Config;

WITH Project;
USE Project;

WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Translate;
USE Translate;

WITH Text_Io;
USE Text_Io;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;


PACKAGE BODY Draw_Time_Line_From_To_Pkg.Callbacks IS



   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Draw_Time_Line_From_To);
   END On_Cancel_Pressed;




   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      Draw_From,
      Draw_To   : Unbounded_String;

      Draw_From_Time : Integer := 0;
      Draw_To_Time   : Integer := 0;

      Ok : Boolean;

   BEGIN


      Draw_From:=To_Unbounded_String(Get_Text(Draw_Time_Line_From_To.
            Draw_From_Entry));
      Draw_To:=To_Unbounded_String(Get_Text(Draw_Time_Line_From_To.
            Draw_To_Entry));



      To_Integer(Draw_From, Draw_From_Time, Ok);
      IF NOT Ok
            THEN
         Show_Message_Box(Lb_Double_Quote & Lb_Draw_From(Current_Language) &
            Lb_Double_Quote &
            Lb_Must_Be_Numeric(Current_Language));
         RETURN;
      END IF;


      To_Integer(Draw_To, Draw_To_Time, Ok);
      IF NOT Ok
            THEN
         Show_Message_Box(Lb_Double_Quote & Lb_Draw_To(
               Current_Language) & Lb_Double_Quote &
            Lb_Must_Be_Numeric(Current_Language));
         RETURN;
      END IF;


      IF(Draw_To_Time <= 0)
            THEN
         Show_Message_Box(Lb_Double_Quote & Lb_Draw_To(
               Current_Language) & Lb_Double_Quote &
            Lb_Must_Be(Current_Language) & To_Unbounded_String(" > 0"));
         RETURN;
      END IF;

      IF(Draw_From_Time < 0)
            THEN
         Show_Message_Box(Lb_Double_Quote & Lb_Draw_From(Current_Language) &
            Lb_Double_Quote & Lb_Must_Be(Current_Language) &
            To_Unbounded_String(" >= 0"));
         RETURN;
      END IF;


      IF(Draw_From_Time > Draw_To_Time)
            THEN
         Show_Message_Box(Lb_Double_Quote & Lb_Draw_From(Current_Language) &
            Lb_Must_Be(
               Current_Language) & Lb_Double_Quote & To_Unbounded_String(
               " <= ")
            & Lb_Double_Quote & Lb_Draw_To(Current_Language) &
            Lb_Double_Quote );
         RETURN;
      END IF;

      IF (Draw_To_Time - Draw_From_Time > 1500)
            THEN
         Show_Message_Box(Lb_Draw_Time_Line_Too_Large(Current_Language));
         RETURN;
      END IF;


      Draw_Time_Line (Draw_Time_Line_From_To.Draw_Scheduling,
         Sys.Tasks,
         Sys.Resources,
         Sys.Messages,
         Sys.Buffers,
         Draw_To_Time,
         Draw_From_Time,
         Call_Scheduling_Framework.Sched);


      Destroy(Draw_Time_Line_From_To);


   END  On_Ok_Pressed;


END Draw_Time_Line_From_To_Pkg.Callbacks;

