------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Project;
USE Project;

WITH Text_Io;
USE Text_Io;

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;

WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Buffers;
USE Buffers;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;

WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Resources;
USE Resources;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;


WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;


WITH Task_Dependencies;
USE Task_Dependencies;


PACKAGE BODY Duplicate_Pkg.Callbacks IS



   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Duplicate);
   END On_Cancel_Pressed;


   PROCEDURE On_Duplicate_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Object_Index : Gint;
      Ok,
      Already      : Boolean;
      Old_Object,
      New_Object   : Unbounded_String;

      A_Resource,
      A_New_Resource      : Resource_Ptr;
      Resource_Ite        : Resources_Iterator;
      A_Task,
      A_New_Task          : Task_Ptr;
      Task_Ite            : Tasks_Iterator;
      A_Processor,
      A_New_Processor     : Processor_Ptr;
      Processor_Ite       : Processors_Iterator;
      A_Buffer,
      A_New_Buffer        : Buffer_Ptr;
      Buffer_Ite          : Buffers_Iterator;
      A_Message,
      A_New_Message       : Message_Ptr;
      Message_Ite         : Messages_Iterator;
      A_Network,
      A_New_Network       : Network_Ptr;
      Network_Ite         : Network_Iterator;
      An_Address_Space,
      A_New_Address_Space : Address_Space_Ptr;
      Address_Space_Ite   : Address_Spaces_Iterator;

   BEGIN

      Old_Object:=To_Unbounded_String(Get_Text(Duplicate.Old_Entry));
      New_Object:=To_Unbounded_String(Get_Text(Duplicate.New_Entry));


      Object_Index:=  Gint(Selected_Button (Duplicate.Clist_Object_Menu));

      -- First of all : check if sets are empty
      --
      Ok:=False;


      IF Object_Index=0 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.Networks)>
               0) THEN
            Ok:=True;
         ELSE
            Show_Message_Box( Lb_Define_Networks_Before(
                  Current_Language));
         END IF;
      END IF;

      IF Object_Index=1 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.Messages)>0)
               THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Messages_Before(Current_Language));
         END IF;
      END IF;

      IF Object_Index=2 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.Buffers)>0)
               THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Buffers_Before(Current_Language));
         END IF;
      END IF;

      IF Object_Index=3 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.Resources)>0)
               THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Resources_Before(Current_Language));
         END IF;
      END IF;

      IF Object_Index=4 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.Tasks)>0)
               THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Tasks_Before(Current_Language));
         END IF;
      END IF;

      IF Object_Index=5 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.
                  Address_Spaces)>
               0) THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Address_Spaces_Before(
                  Current_Language));
         END IF;
      END IF;

      IF Object_Index=6 THEN
         IF (Get_Number_Of_Elements(Duplicate.Sys.
                  Processors)>
               0) THEN
            Ok:=True;
         ELSE
            Show_Message_Box(
               Lb_Define_Processors_Before(
                  Current_Language));
         END IF;
      END IF;




      -- Now, looking for the object and copy itself
      --
      IF Ok THEN

         Ok:=False;
         Already:=False;


         IF Object_Index=0 THEN
            Reset_Iterator(Duplicate.Sys.Networks, Network_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Networks,
                  A_Network, Network_Ite);

               IF A_Network.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Network.Name=Old_Object THEN
                  A_New_Network:=Copy(A_Network);
                  A_New_Network.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.
                  Networks, Network_Ite);

               Next_Element(Duplicate.Sys.Networks,
                  Network_Ite);
            END LOOP;
            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Network_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Network(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Networks, A_New_Network);
               END IF;
            END IF;
         END IF;


         IF Object_Index=1 THEN
            Reset_Iterator(Duplicate.Sys.Messages, Message_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Messages, A_Message,
                  Message_Ite);

               IF A_Message.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Message.Name=Old_Object THEN
                  A_New_Message:=Copy(A_Message);
                  A_New_Message.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.Messages,
                  Message_Ite);

               Next_Element(Duplicate.Sys.Messages, Message_Ite);
            END LOOP;

            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Message_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Message(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Messages, A_New_Message);
               END IF;
            END IF;
         END IF;


         IF Object_Index=2 THEN
            Reset_Iterator(Duplicate.Sys.Buffers, Buffer_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Buffers, A_Buffer,
                  Buffer_Ite);

               IF A_Buffer.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Buffer.Name=Old_Object THEN
                  A_New_Buffer:=Copy(A_Buffer);
                  A_New_Buffer.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.Buffers,
                  Buffer_Ite);

               Next_Element(Duplicate.Sys.Buffers, Buffer_Ite);
            END LOOP;

            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Buffer_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Buffer(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Buffers, A_New_Buffer);

                  -- Update dependencies :
                  --
                  Add_All_Task_Dependencies(Duplicate.Sys.Dependencies,
                     Duplicate.Sys.Tasks, A_New_Buffer);
               END IF;
            END IF;
         END IF;

         IF Object_Index=3 THEN
            Reset_Iterator(Duplicate.Sys.Resources, Resource_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Resources, A_Resource,
                  Resource_Ite);

               IF A_Resource.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Resource.Name=Old_Object THEN
                  A_New_Resource:=Copy(A_Resource);
                  A_New_Resource.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.Resources,
                  Resource_Ite);

               Next_Element(Duplicate.Sys.Resources, Resource_Ite);
            END LOOP;

            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Resource_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Resource(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Resources, A_New_Resource);
               END IF;
            END IF;
         END IF;


         IF Object_Index=4 THEN
            Reset_Iterator(Duplicate.Sys.Tasks, Task_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Tasks, A_Task,
                  Task_Ite);

               IF A_Task.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Task.Name=Old_Object THEN
                  A_New_Task:=Copy(A_Task);
                  A_New_Task.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.Tasks,
                  Task_Ite);

               Next_Element(Duplicate.Sys.Tasks, Task_Ite);
            END LOOP;

            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Task_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Task(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Tasks, A_New_Task);
               END IF;
            END IF;
         END IF;


         IF Object_Index=5 THEN
            Reset_Iterator(Duplicate.Sys.Address_Spaces, Address_Space_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Address_Spaces,
                  An_Address_Space, Address_Space_Ite);

               IF An_Address_Space.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF An_Address_Space.Name=Old_Object THEN
                  A_New_Address_Space:=Copy(An_Address_Space);
                  A_New_Address_Space.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.
                  Address_Spaces, Address_Space_Ite);

               Next_Element(Duplicate.Sys.Address_Spaces,
                  Address_Space_Ite);
            END LOOP;
            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Address_Space_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Address_Space(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Address_Spaces,
                     A_New_Address_Space);
               END IF;
            END IF;
         END IF;


         IF Object_Index=6 THEN
            Reset_Iterator(Duplicate.Sys.Processors, Processor_Ite);

            LOOP
               Current_Element(Duplicate.Sys.Processors,
                  A_Processor, Processor_Ite);

               IF A_Processor.Name=New_Object THEN
                  Already:=True;
               END IF;

               IF A_Processor.Name=Old_Object THEN
                  A_New_Processor:=Copy(A_Processor);
                  A_New_Processor.Name:=New_Object;
                  Ok:=True;
               END IF;

               EXIT WHEN Is_Last_Element(Duplicate.Sys.
                  Processors, Processor_Ite);

               Next_Element(Duplicate.Sys.Processors,
                  Processor_Ite);
            END LOOP;
            IF NOT Ok THEN
               Show_Message_Box(
                  Lb_Processor_Not_Found(Current_Language));

            ELSE
               IF Already THEN
                  Show_Message_Box(
                     Lb_Processor(Current_Language)
                     & Lb_Already_Defined(Current_Language));
               ELSE
                  Add(Duplicate.Sys.Processors,
                     A_New_Processor);
               END IF;
            END IF;
         END IF;


      END IF;


   EXCEPTION

      WHEN Task_Set.Generic_Task_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Tasks(
                  Current_Language));
            RETURN;
         END;
      WHEN Processor_Set.Generic_Processor_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Processors(
                  Current_Language));
            RETURN;
         END;
      WHEN Address_Space_Set.Generic_Address_Space_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Address_Spaces(
                  Current_Language));
            RETURN;
         END;
      WHEN Message_Set.Generic_Message_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Messages(
                  Current_Language));
            RETURN;
         END;
      WHEN Buffer_Set.Generic_Buffer_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Buffers(
                  Current_Language));
            RETURN;
         END;
      WHEN Resource_Set.Generic_Resource_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Resources(
                  Current_Language));
            RETURN;
         END;

      WHEN Network_Set.Generic_Network_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Networks(
                  Current_Language));
            RETURN;
         END;
   END On_Duplicate_Pressed;



   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

   BEGIN
      Sys:=Duplicate.Sys;

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Duplicate);
   END  On_Ok_Pressed;


END Duplicate_Pkg.Callbacks;

