------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Gtk.Check_Button;
USE Gtk.Check_Button;


WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Project;
USE Project;
WITH Glib;
USE Glib;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Call_Framework;
USE Call_Framework;
USE Call_Framework.Framework_Request_Package;
USE Call_Framework.Framework_Response_Package;

WITH Parameters;
USE Parameters;
USE Parameters.Framework_Parameters_Table_Package;


WITH Project;
USE Project;

WITH Text_Tools;
USE Text_Tools;

WITH Text_Io;
USE Text_Io;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;



PACKAGE BODY Event_Table_Analysis_Pkg.Callbacks IS



   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Event_Table_Analysis);
   END On_Cancel_Pressed;




   -- Now, call scheduling analysis for each processor
   --
   PROCEDURE Call_Processor_Analysis (
         Name : Unbounded_String) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;
      A_Param       : Parameter_Ptr;

   BEGIN
      Initialize(Response_List);
      Initialize(Request_List);


      IF Preemption_From_Simulation THEN
         Initialize(A_Request);
         A_Request.Target:=Name;
         A_Request.Statement:=Scheduling_Simulation_Preemption_Number;
         Add(Request_List, A_Request);
      END IF;

      IF Context_Switch_From_Simulation THEN
         Initialize(A_Request);
         A_Request.Target:=Name;
         A_Request.Statement:=
            Scheduling_Simulation_Context_Switch_Number;
         Add(Request_List, A_Request);
      END IF;

      IF  ( Response_Time_From_Simulation(Worst_Case) OR
            Response_Time_From_Simulation(Best_Case) OR
            Response_Time_From_Simulation(Average_Case) OR
            Response_Time_From_Simulation(Plot_Case)) THEN

         Initialize(A_Request);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("worst_case");
         IF Response_Time_From_Simulation(Worst_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("best_case");
         IF Response_Time_From_Simulation(Best_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("average_case");
         IF Response_Time_From_Simulation(Average_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Request.Target:=Name;


         A_Request.Statement:=Scheduling_Simulation_Response_Time;
         Add(Request_List, A_Request);
      END IF;

      IF  ( Blocking_Time_From_Simulation(Worst_Case) OR
            Blocking_Time_From_Simulation(Best_Case) OR
            Blocking_Time_From_Simulation(Average_Case) OR
            Blocking_Time_From_Simulation(Plot_Case) ) THEN

         Initialize(A_Request);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("worst_case");
         IF Blocking_Time_From_Simulation(Worst_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("best_case");
         IF Blocking_Time_From_Simulation(Best_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Param := NEW Parameter(Boolean_Parameter);
         A_Param.Name:=To_Unbounded_String("average_case");
         IF Blocking_Time_From_Simulation(Average_Case) THEN
            A_Param.Boolean_Value:=True;
         ELSE
            A_Param.Boolean_Value:=False;
         END IF;
         Add(A_Request.Param, A_Param);

         A_Request.Target:=Name;

         A_Request.Statement:=Scheduling_Simulation_Blocking_Time;
         Add(Request_List, A_Request);
      END IF;


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);


      Write_Title(Event_Table_Analysis.Text, Response_List.Entries(0).
         Title);
      FOR J IN 0..Response_List.Nb_Entries-1 LOOP
         Write_Text(Event_Table_Analysis.Text, Response_List.Entries(J).
            Text);
      END LOOP;

   END Call_Processor_Analysis;




   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      A_Processor           : Processor_Ptr;
      My_Processor_Iterator : Processors_Iterator;

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

      Cpu_Index : Gint;


   BEGIN


      -- Get customization provided by the user
      --
      Context_Switch_From_Simulation :=
         Get_Active(Event_Table_Analysis.Context_Switch);
      Preemption_From_Simulation:=
         Get_Active(Event_Table_Analysis.Number_Preemption);
      Run_Event_Analyzer_From_Simulation:=
         Get_Active(Event_Table_Analysis.Event_Analyzer);
      Schedule_All_Processeurs:=
         Get_Active(Event_Table_Analysis.Schedule_All_Processeurs);



      FOR I IN Data_From_Simulation_Table'RANGE LOOP
         Blocking_Time_From_Simulation(I):=
            Get_Active(Event_Table_Analysis.Blocking_Time(I));
         Response_Time_From_Simulation(I):=
            Get_Active(Event_Table_Analysis.Response_Time(I));
      END LOOP;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (
               Event_Table_Analysis.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors, A_Processor, Processors_Range(
            Cpu_Index));
      Last_Selected_Processor :=A_Processor.Name;




      -- Call the framework now
      --
      Initialize(A_Request);

      IF Schedule_All_Processeurs=False THEN
         Call_Processor_Analysis(Last_Selected_Processor);
      ELSE

         Reset_Iterator(Sys.Processors, My_Processor_Iterator);
         LOOP
            Current_Element(Sys.Processors, A_Processor,
               My_Processor_Iterator);

            Call_Processor_Analysis(A_Processor.Name);

            EXIT WHEN Is_Last_Element(Sys.Processors,
               My_Processor_Iterator);
            Next_Element(Sys.Processors, My_Processor_Iterator);
         END LOOP;

      END IF;


      -- Now run, event analyzers,
      --

      IF Run_Event_Analyzer_From_Simulation THEN

         Initialize(Response_List);
         Initialize(Request_List);

         Initialize(A_Request);
         A_Request.Statement:=Scheduling_Simulation_Run_Event_Handler;
         Add(Request_List, A_Request);

         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Event_Table_Analysis.Text, Response_List);

      END IF;


      Destroy(Event_Table_Analysis);

   END  On_Close_Pressed;




END Event_Table_Analysis_Pkg.Callbacks;

