------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;

WITH Message_Pkg;
USE Message_Pkg;

WITH Gtk.Menu;
USE Gtk.Menu;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;







PACKAGE BODY Mmi_Tools  IS


   PROCEDURE Inner_Scheduler_Table_Name (
         Items : IN OUT Strings_Table) IS

   BEGIN
      Add(Items,To_Unbounded_String("Earliest Deadline First"));
      Add(Items,To_Unbounded_String("Least Laxity First"));
      Add(Items,To_Unbounded_String("Rate Monotonic"));
      Add(Items,To_Unbounded_String("Deadline Monotonic"));
      Add(Items,To_Unbounded_String(
            "POSIX 1003.1b/Highest Priority First"));
      Add(Items,To_Unbounded_String(
            "Time sharing based on waiting time (Linux scheduler)"));
      Add(Items,To_Unbounded_String("Round robin (with quantum)"));
      Add(Items,To_Unbounded_String(
            "Maximum Urgency First based on laxity"));
      Add(Items,To_Unbounded_String(
            "Maximum Urgency First based on deadline"));
      Add(Items,To_Unbounded_String("D-Over/Stable EDF"));
      Add(Items,To_Unbounded_String("Time sharing based on cpu usage"));
      Add(Items,To_Unbounded_String("Parametric scheduler"));
   END Inner_Scheduler_Table_Name;


   PROCEDURE Build_Local_Scheduler_Table_Name (
         Items : IN OUT Strings_Table) IS
   BEGIN
      Add(Items,To_Unbounded_String("No local scheduling protocol"));
      Inner_Scheduler_Table_Name(Items);
   END Build_Local_Scheduler_Table_Name;


   PROCEDURE Build_Global_Scheduler_Table_Name (
         Items : IN OUT Strings_Table) IS

   BEGIN
      Inner_Scheduler_Table_Name(Items);
      Add(Items,To_Unbounded_String("Hierarchical scheduling : cyclic"));
      Add(Items,To_Unbounded_String(
            "Hierarchical scheduling : round robin"));
      Add(Items,To_Unbounded_String(
            "Hierarchical scheduling : fixed priority"));
      Add(Items,To_Unbounded_String(
            "Hierarchical scheduling : parametric"));
   END Build_Global_Scheduler_Table_Name;



   PROCEDURE Show_Message_Box (
         Msg : Unbounded_String) IS
   BEGIN
      Gtk_New (Message,
         Unbounded_Lf & Msg &  Unbounded_Lf);
      Show_All(Message);
   END Show_Message_Box;



   PROCEDURE Show_Message_Box (
         Msg : String) IS
   BEGIN
      Show_Message_Box(To_Unbounded_String(Msg));
   END Show_Message_Box;


   PROCEDURE Build_Option_Menu (
         Omenu   :    OUT Gtk.Option_Menu.Gtk_Option_Menu;
         Gr      :    OUT Widget_Slist.Gslist;
         Items   :        Strings_Table;
         History :        Gint;
         Cb      :        Widget_Handler.Marshallers.Void_Marshaller.Handler) IS
      Menu      : Gtk_Menu;
      Menu_Item : Gtk_Radio_Menu_Item;

   BEGIN
      Gtk.Option_Menu.Gtk_New (Omenu);
      Gtk_New (Menu);

      FOR I IN 0..Items.Nb_Entries-1 LOOP
         IF (Items.Entries(I)/=Empty_String)
               THEN
            Gtk_New (Menu_Item, Gr, To_String(Items.Entries(I)) );
            Widget_Handler.Object_Connect (Menu_Item, "activate",
               Widget_Handler.To_Marshaller (Cb),
               Slot_Object => Menu_Item);
            Gr := Group (Menu_Item);
            Append (Menu, Menu_Item);
            IF Gint (I) = History THEN
               Set_Active (Menu_Item, True);
            END IF;
            Show (Menu_Item);
         END IF;
      END LOOP;
      Gtk.Option_Menu.Set_Menu (Omenu, Menu);
      Gtk.Option_Menu.Set_History (Omenu, History);
   END Build_Option_Menu;




END Mmi_Tools;


