------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Gtk.Button;
USE Gtk.Button;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Mmi_Tools;
USE Mmi_Tools;
WITH Objects;
USE Objects;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Network_Set;
USE Network_Set;

WITH Networks;
USE Networks;

WITH Project;
USE Project;


WITH Mmi_Tools;
USE Mmi_Tools;

WITH Generic_Select_File_Pkg;


PACKAGE BODY Networks_Parameters_Pkg.Callbacks  IS



   PROCEDURE Read_Sc_File (
         File_Name : IN     Unbounded_String) IS


   BEGIN

      Set_Text(Networks_Parameters.
         Entry_File_Name, To_String(File_Name));

   END Read_Sc_File;

   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;

   PACKAGE Read_Sc_File_Package IS NEW Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   USE Read_Sc_File_Package;


   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

   BEGIN

      Sys.Networks:=Networks_Parameters.Undo_Networks;

      Destroy(Networks_Parameters);

   END  On_Cancel_Pressed;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Destroy(Networks_Parameters);

   END On_Close_Pressed;


   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Network_Name : Unbounded_String;
      Network_Type : Unbounded_String;

   BEGIN

      Network_Name := To_Unbounded_String(Get_Text(Networks_Parameters.
            Networks_Name));

      IF Network_Name=""
            THEN
         Show_Message_Box(Lb_Networks_Name(Current_Language) &
            Lb_Mandatory(Current_Language));
         RETURN;
      END IF;

      IF NOT Is_A_Valid_Identifier(Network_Name) THEN
         Show_Message_Box(Lb_Networks_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         RETURN;
      END IF;

      Network_Type := To_Unbounded_String(Gtk.Gentry.Get_Text(Get_Entry(
               Networks_Parameters.Networks_Type)));

      IF Network_Type = "Bounded delay" THEN
         Network_Type := To_Unbounded_String(Type_Of_Network'Image(
               Bounded_Delay));
      ELSE
         Network_Type := To_Unbounded_String(Type_Of_Network'Image(
               Parametric_Delay));
      END IF;

      Add_Network(Sys.Networks, Network_Name, Network_Type);


   EXCEPTION
      WHEN  Network_Already_Defined =>
         BEGIN
            Show_Message_Box(Lb_Networks_Name(Current_Language) &
               Lb_Already_Defined(Current_Language) );
            RETURN;
         END;
      WHEN Network_Set.Generic_Network_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Networks(
                  Current_Language));
            RETURN;
         END;

   END On_Add_Pressed;





   PROCEDURE On_Read_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Network_Type : Unbounded_String;

   BEGIN

      -- if the combo selection is not parametric
      --
      Network_Type := To_Unbounded_String(Gtk.Gentry.Get_Text(Get_Entry(
               Networks_Parameters.Networks_Type)));

      IF (Network_Type /= "Parametric delay")
            THEN
         Show_Message_Box(Lb_Only_On_Parametric_Network(Current_Language));
         RETURN;
      END IF;


      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Show_All (Read_Sc_File_Package.Generic_Select_File);


   END On_Read_Pressed;

END Networks_Parameters_Pkg.Callbacks;




