------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE Gtk;
WITH Gtk.Main;
USE Gtk.Main;
WITH Gtkada.Dialogs;
USE Gtkada.Dialogs;





WITH Open_File_Selection_Pkg;
USE Open_File_Selection_Pkg;
USE Open_File_Selection_Pkg.Open_Select_File_Pkg;
WITH Save_File_Selection_Pkg;
USE Save_File_Selection_Pkg;
USE Save_File_Selection_Pkg.Save_Select_File_Pkg;

WITH Import_Aadl_Selection_Pkg;
USE Import_Aadl_Selection_Pkg;
USE Import_Aadl_Selection_Pkg.Import_Aadl_File_Pkg;
WITH Export_Aadl_Selection_Pkg;
USE Export_Aadl_Selection_Pkg;
USE Export_Aadl_Selection_Pkg.Export_Aadl_File_Pkg;


WITH Root_Window_Pkg;
USE Root_Window_Pkg;
WITH Root_Window_Pkg.Callbacks;
USE Root_Window_Pkg.Callbacks;
WITH Call_Scheduling_Framework;
USE Call_Scheduling_Framework;
WITH My_Canvas;
USE My_Canvas;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;
WITH Unbounded_Strings;
USE Unbounded_Strings;

WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;






PACKAGE BODY Project IS



   PROCEDURE Do_New IS
   BEGIN

      IF Root_Window /= NULL THEN

         IF (NOT Is_Saved)
               THEN
            IF (NOT Is_New)
                  THEN
               IF Message_Dialog (To_String(
                        Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                        Lb_Continue(Current_Language)),
                     Confirmation,
                     Button_Yes OR Button_Cancel,
                     Button_Cancel, -- default
                     "", -- help message
                     "") /= Button_Yes THEN
                  RETURN;
               END IF;
            END IF;
         END IF;
         Clear_Work_Space;

      END IF;

      Initialize(Sys);
      Xml_Project_File_Name:=To_Unbounded_String("");
      Initialize(Aadl_Project_File_Name);
      Xml_Event_Table_File_Name:=To_Unbounded_String(
         "default_event_table.xml");


      Is_Saved:=False;
      Is_New:=True;

      Event_Table_File_Name:=Empty_String;
      Initialize(Call_Scheduling_Framework.Sched.All);


   END Do_New;


   PROCEDURE Do_Open IS
   BEGIN

      IF (NOT Is_Saved)
            THEN
         IF (NOT Is_New)
               THEN
            IF Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes OR Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") /= Button_Yes THEN
               RETURN;
            END IF;
         END IF;
      END IF;

      Clear_Work_Space;

      Gtk_New (Open_Select_File_Pkg.Generic_Select_File, Lb_Open_Xml(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Open_File_Initialize;
      Show_All (Open_Select_File_Pkg.Generic_Select_File);

   END  Do_Open;

   PROCEDURE Do_Save IS
   BEGIN


      IF Graph_Window/=NULL
            THEN
         Get_Coordinates(Graph_Window.Buffer, Sys);
      END IF;

      IF (Xml_Project_File_Name /= To_Unbounded_String(""))
            THEN
         Systems.Write_To_Xml_File(Sys, Xml_Project_File_Name);

         Is_Saved:=True;
      ELSE
         Gtk_New (Save_Select_File_Pkg.Generic_Select_File, Lb_Save_Xml(
               Current_Language) & To_Unbounded_String(" ") & Lb_File(
               Current_Language));
         Save_File_Initialize;
         Show_All (Save_Select_File_Pkg.Generic_Select_File);
      END IF;

   END  Do_Save;

   PROCEDURE Do_Aadl_Export IS
   BEGIN
      Gtk_New (Export_Aadl_File_Pkg.Generic_Select_File, Lb_Save(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Export_Aadl_Initialize;
      Show_All (Export_Aadl_File_Pkg.Generic_Select_File);

   END  Do_Aadl_Export;

   PROCEDURE Do_Aadl_Import IS
   BEGIN

      IF (NOT Is_Saved)
            THEN
         IF (NOT Is_New)
               THEN
            IF Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes OR Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") /= Button_Yes THEN
               RETURN;
            END IF;
         END IF;
      END IF;

      Clear_Work_Space;

      Gtk_New (Import_Aadl_File_Pkg.Generic_Select_File, Lb_Open(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Import_Aadl_Initialize;
      Show_All (Import_Aadl_File_Pkg.Generic_Select_File);

   END  Do_Aadl_Import;


   PROCEDURE Do_Save_As IS
   BEGIN
      Gtk_New (Save_Select_File_Pkg.Generic_Select_File, Lb_Save_As_Xml(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Save_File_Initialize;
      Show_All (Save_Select_File_Pkg.Generic_Select_File);
   END  Do_Save_As;



   PROCEDURE Do_Exit IS
   BEGIN
      IF (NOT Is_Saved)
            THEN
         IF (NOT Is_New)
               THEN
            IF Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes OR Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") = Button_Yes THEN
               Gtk.Main.Gtk_Exit (0);
            ELSE
               RETURN;
            END IF;
         END IF;
      END IF;
      Gtk.Main.Gtk_Exit (0);
   END  Do_Exit;


END Project;

