------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Systems;
USE Systems;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Unbounded_String_List_Package;



PACKAGE Project IS



   -- The working project
   --
   Sys : System;


   -- List of directory where to read AADL or XML files
   --
   Project_File_Dir_List : Unbounded_String_List;


   -- File name of the working XML/AADL project files
   --
   Xml_Project_File_Name  : Unbounded_String;
   Aadl_Project_File_Name : Unbounded_String_List;


   -- File name of the working XML event table file
   --
   Xml_Event_Table_File_Name : Unbounded_String;


   -- Is the project modified ? should we have to save it ?
   --
   Is_Saved : Boolean;
   Is_New   : Boolean;


   -- Processor name on which we should compute something
   --
   Last_Selected_Processor : Unbounded_String;


   -- The name of the file in which the event table has to be saved
   --
   Event_Table_File_Name : Unbounded_String;


   -- Sub programs do project management
   --
   PROCEDURE Do_New;
   PROCEDURE Do_Open;
   PROCEDURE Do_Save;
   PROCEDURE Do_Save_As;
   PROCEDURE Do_Exit;
   PROCEDURE Do_Aadl_Export;
   PROCEDURE Do_Aadl_Import;


END Project;


