------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;



PACKAGE Question_Pkg IS


   TYPE Question_Callback_Type IS ACCESS FUNCTION
   (Value : IN Unbounded_String)
      RETURN Boolean;

   TYPE Question_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Question_Access IS ACCESS ALL Question_Record'Class;

   PROCEDURE Gtk_New (
         Question      :    OUT Question_Access;
         Title         :        Unbounded_String;
         The_Question  :        Unbounded_String;
         Initial_Value :        Unbounded_String;
         Call          :        Question_Callback_Type);

   PROCEDURE Initialize (
         Question      : ACCESS Question_Record'Class;
         Title         :        Unbounded_String;
         The_Question  :        Unbounded_String;
         Initial_Value :        Unbounded_String;
         Call          :        Question_Callback_Type);

   Question : Question_Access;

PRIVATE

   TYPE Question_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Vbox      : Gtk_Vbox;
         Hbox1     : Gtk_Hbox;
         Hbox2     : Gtk_Hbox;
         Separator : Gtk_Hseparator;

         Question_Label : Gtk_Label;
         Response_Entry : Gtk_Entry;

         Cancel_Button : Gtk_Button;
         Ok_Button     : Gtk_Button;

         Calling_Function : Question_Callback_Type;
      END RECORD;

END Question_Pkg;
