------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Framework_Config;
USE Framework_Config;

WITH Translate;
USE Translate;

WITH Project;
USE Project;

WITH References;
WITH Manual;

WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;

WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Resources;
USE Resources;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH Time_Units;
USE Time_Units;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Scheduling_Sequences;
USE Scheduling_Sequences;
USE Scheduling_Sequences.Density_Table_Package;
USE Scheduling_Sequences.Densities_Table_Package;

WITH Ada.Io_Exceptions;
USE Ada.Io_Exceptions;
WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Buffers;
USE Buffers;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;

WITH Event_Analyzers;
USE Event_Analyzers;
WITH Event_Analyzer_Set;
USE Event_Analyzer_Set;
USE Event_Analyzer_Set.Generic_Event_Analyzer_Set;

WITH Call_Framework;
USE Call_Framework;
USE Call_Framework.Framework_Request_Package;
USE Call_Framework.Framework_Response_Package;
WITH Call_Scheduling_Framework;
USE Call_Scheduling_Framework;
WITH Call_Memory_Framework;
USE Call_Memory_Framework;
USE Call_Memory_Framework.Buffer_Result_Package;
WITH Parameters;
USE Parameters;
USE Parameters.Framework_Parameters_Table_Package;


WITH Glib;
USE Glib;
WITH Gtk.Widget;
USE Gtk.Widget;


WITH Mmi_Tools;
USE Mmi_Tools;

WITH Display_Resources_Set_Pkg;
USE Display_Resources_Set_Pkg;
WITH Display_Messages_Set_Pkg;
USE Display_Messages_Set_Pkg;
WITH Display_Networks_Set_Pkg;
USE Display_Networks_Set_Pkg;
WITH Display_Tasks_Set_Pkg;
USE Display_Tasks_Set_Pkg;
WITH Display_Address_Spaces_Set_Pkg;
USE Display_Address_Spaces_Set_Pkg;
WITH Display_Networks_Set_Pkg;
USE Display_Networks_Set_Pkg;
WITH Display_Buffers_Set_Pkg;
USE Display_Buffers_Set_Pkg;
WITH Display_Processors_Set_Pkg;
USE Display_Processors_Set_Pkg;
WITH Display_Event_Analyzers_Set_Pkg;
USE Display_Event_Analyzers_Set_Pkg;

WITH Update_Event_Analyzers_Set_Pkg;
USE Update_Event_Analyzers_Set_Pkg;
WITH Update_Buffers_Set_Pkg;
USE Update_Buffers_Set_Pkg;
WITH Update_Tasks_Set_Pkg;
USE Update_Tasks_Set_Pkg;
WITH Update_Address_Spaces_Set_Pkg;
USE Update_Address_Spaces_Set_Pkg;
WITH Update_Networks_Set_Pkg;
USE Update_Networks_Set_Pkg;
WITH Update_Processors_Set_Pkg;
USE Update_Processors_Set_Pkg;
WITH Update_Resources_Set_Pkg;
USE Update_Resources_Set_Pkg;
WITH Update_Messages_Set_Pkg;
USE Update_Messages_Set_Pkg;

WITH Draw_Scheduling_Pkg;
USE Draw_Scheduling_Pkg;
WITH Scrolling_Draw_Buffer_Pkg;
USE Scrolling_Draw_Buffer_Pkg;
WITH Scrolling_Text_Pkg;
USE Scrolling_Text_Pkg;
WITH Question_Pkg;
USE Question_Pkg;
WITH Do_Scheduling_Simulation_Pkg;
USE Do_Scheduling_Simulation_Pkg;
WITH Do_Scheduling_Feasibility_Pkg;
USE Do_Scheduling_Feasibility_Pkg;
WITH Networks_Parameters_Pkg;
USE Networks_Parameters_Pkg;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;
WITH Scheduling_Options_Pkg;
USE Scheduling_Options_Pkg;
WITH Aadl_Options_Pkg;
USE Aadl_Options_Pkg;
WITH Duplicate_Pkg;
USE Duplicate_Pkg;
WITH Schedule_From_To_Pkg;
USE Schedule_From_To_Pkg;
WITH Draw_Time_Line_From_To_Pkg;
USE Draw_Time_Line_From_To_Pkg;
WITH Event_Table_Analysis_Pkg;
USE Event_Table_Analysis_Pkg;

WITH Systems;
USE Systems;

WITH Generic_Select_File_Pkg;


WITH Text_Tools;
USE Text_Tools;

WITH Xml_Parsers;
USE Xml_Parsers;

WITH Multiprocessor_Services;
USE Multiprocessor_Services;
USE Multiprocessor_Services.Scheduling_Result_Per_Processor;

WITH Text_Io;
USE Text_Io;





PACKAGE BODY Root_Window_Pkg.Callbacks IS

   USE Gtk.Arguments;


   ----------------------------------------------------------------------
   --   Callbacks which call the framework
   ----------------------------------------------------------------------


   PROCEDURE Check_Buffer IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Buffers)=0)
            THEN
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Buffer_Feasibility_Tests;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);



   END Check_Buffer;



   PROCEDURE On_Draw_Buffer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Buffers)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      ELSE

         -- Check if scheduling is computed or not
         --
         IF Call_Scheduling_Framework.Sched.Nb_Entries=0
               THEN
            Show_Message_Box(Lb_Schedule_Before(Current_Language));
            RETURN;
         END IF;

         -- For each buffer, compute and draw its size from
         -- 0 to the base period
         --
         Initialize(A_Request);
         A_Request.Statement:=Buffer_Scheduling_Simulation;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);


         Write_Title_And_Text(Root_Window.Text, Response_List);


         -- Now, draw the result on buffer utilization
         --
         FOR I IN 0..Buff.Nb_Entries-1 LOOP

            Gtk_New(Scrolling_Draw_Buffer,To_String(Buff.Entries(I).Item.
                  Name));
            Show_All(Scrolling_Draw_Buffer);
            Clear(Scrolling_Draw_Buffer);
            Draw(Scrolling_Draw_Buffer, Buff.Entries(I).Item, Buff.
               Entries(I).Data,
               Call_Scheduling_Framework.Scheduling_Base_Period,
               Time_Unit_Range (Buff.Entries(I).Data.Nb_Entries));
         END LOOP;


      END IF;
   END On_Draw_Buffer_Activate;




   -- Resource tools : blocking time evaluation
   --

   PROCEDURE On_Blocking_Time_Display_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Resources)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Scheduling_Feasibility_Compute_Blocking_Time;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);

      END IF;
   END On_Blocking_Time_Display_Callback;


   PROCEDURE On_Blocking_Time_Update_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Resources)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Scheduling_Feasibility_Compute_And_Set_Blocking_Time;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      END IF;
   END On_Blocking_Time_Update_Callback;








   -- Precedency tools : deadline, jitter and priority modifications
   --
   PROCEDURE On_Chetto_Priority_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END  On_Chetto_Priority_Callback;


   PROCEDURE On_Chetto_Priority_Display_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_Chetto_Blazewicz_Priority;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);

      END IF;
   END  On_Chetto_Priority_Display_Callback;


   PROCEDURE On_Chetto_Priority_Update_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=Dependency_Set_Chetto_Blazewicz_Priority;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;

      END IF;
   END  On_Chetto_Priority_Update_Callback;


   PROCEDURE On_Chetto_Deadline_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END  On_Chetto_Deadline_Callback;


   PROCEDURE On_Chetto_Deadline_Display_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_Chetto_Blazewicz_Deadline;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


      END IF;
   END  On_Chetto_Deadline_Display_Callback;


   PROCEDURE On_Chetto_Deadline_Update_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=Dependency_Set_Chetto_Blazewicz_Deadline;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      END IF;
   END  On_Chetto_Deadline_Update_Callback;


   PROCEDURE On_Jitter_From_Response_Time_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END  On_Jitter_From_Response_Time_Callback;


   PROCEDURE On_Jitter_From_Response_Time_Display_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_End_To_End_Response_Time_All_Steps;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


      END IF;
   END  On_Jitter_From_Response_Time_Display_Callback;


   PROCEDURE On_Jitter_From_Response_Time_Update_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Set_End_To_End_Response_Time_All_Steps;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      END IF;
   END  On_Jitter_From_Response_Time_Update_Callback;


   PROCEDURE On_Jitter_From_Response_Time_Display_One_Step_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Compute_End_To_End_Response_Time_One_Step;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);

      END IF;
   END  On_Jitter_From_Response_Time_Display_One_Step_Callback;


   PROCEDURE On_Jitter_From_Response_Time_Update_One_Step_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));

      ELSE
         Initialize(A_Request);
         A_Request.Statement:=
            Dependency_Set_End_To_End_Response_Time_One_Step;
         Add(Request_List, A_Request);


         Sequential_Framework_Request (
            Sys, Request_List, Response_List);

         Write_Title_And_Text(Root_Window.Text, Response_List);


         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New:=False;
         Is_Saved:=False;


      END IF;
   END  On_Jitter_From_Response_Time_Update_One_Step_Callback;





   -- Set priorities according to a fixed priority scheduler
   --

   FUNCTION Rm_Priorities_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS

      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;
      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      LOOP

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         IF A_Processor.Name = Value
               THEN

            Initialize(A_Request);
            A_Request.Target:=Value;
            A_Request.Statement:=
               Scheduling_Set_Priorities_According_To_Rate_Monotonic;
            Add(Request_List, A_Request);


            Sequential_Framework_Request (
               Sys, Request_List, Response_List);

            Write_Title_And_Text(Root_Window.Text, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New:=False;
            Is_Saved:=False;


            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);
         Next_Element(Sys.Processors, My_Iterator);

      END LOOP;

      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      RETURN False;


   END Rm_Priorities_Callback;



   FUNCTION Dm_Priorities_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS

      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;
      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;


   BEGIN

      LOOP

         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         IF A_Processor.Name = Value
               THEN

            Initialize(A_Request);
            A_Request.Target:=Value;
            A_Request.Statement:=
               Scheduling_Set_Priorities_According_To_Deadline_Monotonic;
            Add(Request_List, A_Request);


            Sequential_Framework_Request (
               Sys, Request_List, Response_List);

            Write_Title_And_Text(Root_Window.Text, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);
         Next_Element(Sys.Processors, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      RETURN False;

   END Dm_Priorities_Callback;


   PROCEDURE On_Dm_Priorities_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Set_Priorities_Dm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Dm_Priorities_Callback'ACCESS);
         Show_All(Question);
      END IF;

   END On_Dm_Priorities_Activate;




   PROCEDURE On_Rm_Priorities_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Set_Priorities_Rm(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Rm_Priorities_Callback'ACCESS);
         Show_All(Question);
      END IF;

   END On_Rm_Priorities_Activate;


   -- Random services
   --


   PROCEDURE On_Compute_Response_Time_Density_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS


      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;


      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Random_Response_Time_Density;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);


   END On_Compute_Response_Time_Density_Activate;





   ----------------------------------------------------------------------
   --   Menu callbacks without calls to the framework
   ----------------------------------------------------------------------



   PROCEDURE Clear_Work_Space IS
      Size  : Guint;
      Dummy : Boolean;
   BEGIN
      Size:=Get_Length(Root_Window.Text);
      Set_Point(Root_Window.Text,0);
      Dummy:= Forward_Delete(Root_Window.Text,Size);
      Clear_Time_Line(Draw_Scheduling);
   END Clear_Work_Space;



   --  Delete functions : called by delete callbacks
   --

   FUNCTION Delete_Resource_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator : Resources_Iterator;
      A_Resource  : Resource_Ptr;
   BEGIN
      LOOP

         Current_Element(Sys.Resources, A_Resource, My_Iterator);

         IF A_Resource.Name = Value
               THEN

            Delete(Sys.Resources, A_Resource);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Resources, My_Iterator);
         Next_Element(Sys.Resources, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Resource_Not_Found(Current_Language));
      RETURN False;
   END Delete_Resource_Callback;


   FUNCTION Delete_Network_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator : Network_Iterator;
      A_Network   : Network_Ptr;
   BEGIN
      LOOP

         Current_Element(Sys.Networks, A_Network, My_Iterator);

         IF A_Network.Name = Value
               THEN

            Delete(Sys.Networks,A_Network);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Networks, My_Iterator);
         Next_Element(Sys.Networks, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Network_Not_Found(Current_Language));
      RETURN False;
   END Delete_Network_Callback;


   FUNCTION Delete_Buffer_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator : Buffers_Iterator;
      A_Buffer    : Buffer_Ptr;

   BEGIN

      LOOP

         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);

         IF A_Buffer.Name = Value
               THEN

            Delete_Buffer(Sys, A_Buffer);
            Rebuild_Canvas(Graph_Window);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      RETURN False;
   END Delete_Buffer_Callback;


   FUNCTION Delete_Message_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator : Messages_Iterator;
      A_Message   : Message_Ptr;
   BEGIN
      LOOP

         Current_Element(Sys.Messages, A_Message, My_Iterator);

         IF A_Message.Name = Value
               THEN

            Delete_Message(Sys, A_Message);
            Rebuild_Canvas(Graph_Window);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);
         Next_Element(Sys.Messages, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Buffer_Not_Found(Current_Language));
      RETURN False;
   END Delete_Message_Callback;


   FUNCTION Delete_Task_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator : Tasks_Iterator;
      A_Task      : Task_Ptr;
   BEGIN

      LOOP
         Current_Element(Sys.Tasks, A_Task, My_Iterator);

         IF A_Task.Name = Value
               THEN

            Delete_Task(Sys, A_Task);
            Rebuild_Canvas(Graph_Window);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Tasks, My_Iterator);
         Next_Element(Sys.Tasks, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Task_Not_Found(Current_Language));
      RETURN False;
   END Delete_Task_Callback;


   FUNCTION Delete_Processor_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS

      Iterator1   : Processors_Iterator;
      A_Processor : Processor_Ptr;

   BEGIN
      Reset_Iterator(Sys.Processors, Iterator1);

      LOOP

         Current_Element(Sys.Processors, A_Processor, Iterator1);

         IF A_Processor.Name = Value
               THEN

            Delete_Processor(Sys, A_Processor);
            Rebuild_Canvas(Graph_Window);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Processors, Iterator1);
         Next_Element(Sys.Processors, Iterator1);

      END LOOP;

      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      RETURN False;
   END Delete_Processor_Callback;


   FUNCTION On_Root_Window_Delete_Event (
         Object : ACCESS Gtk_Widget_Record'Class;
         Params :        Gtk.Arguments.Gtk_Args)
     RETURN Boolean IS
   BEGIN
      Do_Exit;
      RETURN True;
   END On_Root_Window_Delete_Event;


   PROCEDURE On_File_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_File_Activate;


   PROCEDURE On_New_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   END On_New_Activate;


   PROCEDURE On_Open_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Open;
   END On_Open_Activate;


   PROCEDURE On_Save_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Save;
   END On_Save_Activate;


   PROCEDURE On_Save_As_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Save_As;
   END On_Save_As_Activate;


   PROCEDURE On_Exit_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Exit;
   END On_Exit_Activate;


   PROCEDURE On_Add_Task_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
      ELSE
         Gtk_New (Update_Tasks_Set, Lb_Add_Task(Current_Language));
         Show_Advanced (Update_Tasks_Set, False);
      END IF;
   END On_Add_Task_Activate;


   PROCEDURE On_Update_Task_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
      ELSE
         Gtk_New (Update_Tasks_Set, Lb_Update_Task(Current_Language));
         Show_Advanced (Update_Tasks_Set,True);
      END IF;
   END On_Update_Task_Activate;


   PROCEDURE On_Add_Processor_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New (Update_Processors_Set, Lb_Add_Processor(Current_Language));
      Show_Advanced (Update_Processors_Set,False);
   END On_Add_Processor_Activate;


   PROCEDURE On_Update_Processor_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Gtk_New (Update_Processors_Set, Lb_Update_Processor(
               Current_Language));
         Show_Advanced (Update_Processors_Set,True);
      END IF;
   END On_Update_Processor_Activate;

   PROCEDURE On_Update_Network_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Networks)=0)
            THEN
         Show_Message_Box(Lb_Define_Networks_Before(Current_Language));
      ELSE
         Gtk_New (Update_Networks_Set);
         Show_All (Update_Networks_Set);
      END IF;
   END On_Update_Network_Activate;

   PROCEDURE On_Add_Network_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New (Networks_Parameters);
      Show_All (Networks_Parameters);
   END On_Add_Network_Activate;


   PROCEDURE On_Tasks_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      ELSE
         Gtk_New (Display_Tasks_Set);
         Show_All (Display_Tasks_Set);
      END IF;
   END On_Tasks_List_Activate;


   PROCEDURE On_Processors_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Gtk_New (Display_Processors_Set);
         Show_All (Display_Processors_Set);
      END IF;
   END On_Processors_List_Activate;


   PROCEDURE On_Resource_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Resources)=0) THEN
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      ELSE
         Gtk_New (Display_Resources_Set);
         Show_All (Display_Resources_Set);
      END IF;
   END On_Resource_List_Activate;

   PROCEDURE On_Network_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Networks)=0) THEN
         Show_Message_Box(Lb_Define_Networks_Before(Current_Language));
      ELSE
         Gtk_New (Display_Networks_Set);
         Show_All (Display_Networks_Set);
      END IF;
   END On_Network_List_Activate;

   PROCEDURE On_Message_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Messages)=0) THEN
         Show_Message_Box(Lb_Define_Messages_Before(Current_Language));
      ELSE
         Gtk_New (Display_Messages_Set);
         Show_All (Display_Messages_Set);
      END IF;
   END On_Message_List_Activate;


   PROCEDURE On_Buffer_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Buffers)=0) THEN
         Show_Message_Box(Lb_Define_Buffers_Before(Current_Language));
      ELSE
         Gtk_New (Display_Buffers_Set);
         Show_All (Display_Buffers_Set);
      END IF;
   END On_Buffer_List_Activate;


   PROCEDURE On_Tools_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_Tools_Activate;


   PROCEDURE On_Clear_Work_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Clear_Work_Space;
   END On_Clear_Work_Space_Activate;


   PROCEDURE Call_Schedule_From_To_After_Do_Scheduling IS
   BEGIN

      Gtk_New(Schedule_From_To, Root_Window.Text, Draw_Scheduling,
         Sys, True );
      Show_All(Schedule_From_To);

   END Call_Schedule_From_To_After_Do_Scheduling;


   PROCEDURE On_Draw_Scheduling_Graph_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      Gtk_New(Do_Scheduling_Simulation,
         Call_Schedule_From_To_After_Do_Scheduling'ACCESS);
      Show_All(Do_Scheduling_Simulation);
   END On_Draw_Scheduling_Graph_Activate;


   PROCEDURE On_Options_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New(Scheduling_Options);
      Show_All(Scheduling_Options);
   END On_Options_Activate;


   PROCEDURE On_Check_Scheduling_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      Gtk_New(Do_Scheduling_Feasibility, Root_Window.Text);
      Show_All(Do_Scheduling_Feasibility);
   END On_Check_Scheduling_Activate;


   PROCEDURE On_Help_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_Help_Activate;


   PROCEDURE On_Update_Resource_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Resources)=0) THEN
         Show_Message_Box(Lb_Define_Resources_Before(Current_Language));
      ELSE
         Gtk_New(Update_Resources_Set, Lb_Update_Resource(
               Current_Language));
         Show_Advanced (Update_Resources_Set, True);
      END IF;
   END On_Update_Resource_Activate;


   PROCEDURE On_Add_Resource_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
      ELSE
         IF (Get_Number_Of_Elements(Sys.Tasks)=0)
               THEN
            Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         ELSE
            Gtk_New(Update_Resources_Set, Lb_Add_Resource(
                  Current_Language));
            Show_Advanced (Update_Resources_Set, False);
         END IF;
      END IF;
   END On_Add_Resource_Activate;


   PROCEDURE On_About_Cheddar_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN
      Show_Message_Box(Lb_About_Text(Current_Language));
   END On_About_Cheddar_Activate;


   PROCEDURE On_Manual_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New(Scrolling_Text,Lb_Manual(Current_Language));
      Set_Usize(Scrolling_Text,690,425);


      Write_Title_Small(Scrolling_Text,Manual.Ustr_File_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_File_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Edit_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Edit_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_View_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_View_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Tools_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Tools_Menu_Content);

      Write_Title_Small(Scrolling_Text,Manual.Ustr_Help_Menu);
      Write_Text(Scrolling_Text,Manual.Ustr_Help_Menu_Content);

      Show_All(Scrolling_Text);
   END On_Manual_Activate;


   PROCEDURE On_Bibliography_References_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New(Scrolling_Text,Lb_Ref(Current_Language));
      Set_Usize(Scrolling_Text,535, 440);
      Write_Text(Scrolling_Text,References.Ref_Text);
      Show_All(Scrolling_Text);
   END On_Bibliography_References_Activate;


   PROCEDURE On_Delete_Task_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Delete_Task(Current_Language),
            Lb_Task_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Task_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Task_Activate;


   PROCEDURE On_Delete_Resource_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Resources)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Resources_Before(Current_Language));
      ELSE
         Gtk_New(Question,Lb_Delete_Resource(Current_Language),
            Lb_Resource_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Resource_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Resource_Activate;

   PROCEDURE On_Delete_Network_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Networks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Networks_Before(Current_Language));
      ELSE
         Gtk_New(Question,Lb_Delete_Network(Current_Language),
            Lb_Networks_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Network_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Network_Activate;

   PROCEDURE On_Delete_Processor_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Delete_Processor(Current_Language),
            Lb_Processor_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Processor_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Processor_Activate;


   PROCEDURE On_Add_Message_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New (Update_Messages_Set, To_Unbounded_String("cheddar_window"));
      Show_Advanced (Update_Messages_Set, False);
   END On_Add_Message_Activate;


   PROCEDURE On_Delete_Message_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Messages)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Delete_Message(Current_Language),
            Lb_Message_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Message_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Message_Activate;


   PROCEDURE On_Update_Message_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Messages)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Messages_Before(Current_Language));
      ELSE
         Gtk_New(Update_Messages_Set, To_Unbounded_String(
               "cheddar_window"));
         Show_All(Update_Messages_Set);
      END IF;
   END On_Update_Message_Activate;


   PROCEDURE On_Add_Buffer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(
               Current_Language));
      ELSE
         Gtk_New (Update_Buffers_Set, Lb_Add_Buffer(Current_Language),
            To_Unbounded_String("cheddar_window"));
         Show_Advanced (Update_Buffers_Set, False);
      END IF;
   END On_Add_Buffer_Activate;


   PROCEDURE On_Delete_Buffer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Buffers)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Buffers_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Delete_Buffer(Current_Language),
            Lb_Buffer_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Buffer_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Buffer_Activate;


   PROCEDURE On_Update_Buffer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(
               Current_Language));
      ELSE
         Gtk_New (Update_Buffers_Set, Lb_Update_Buffer(Current_Language),
            To_Unbounded_String("cheddar_window"));
         Show_Advanced (Update_Buffers_Set, True);
      END IF;

   END On_Update_Buffer_Activate;


   PROCEDURE On_Check_Buffer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Check_Buffer;
   END On_Check_Buffer_Activate;


   PROCEDURE On_Blocking_Time_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_Blocking_Time_Callback;


   PROCEDURE On_Duplicate_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN
      Gtk_New(Duplicate_Pkg.Duplicate);
      Show_All(Duplicate_Pkg.Duplicate);
   END On_Duplicate_Activate;








   ----------------------------------------------------------------------
   --   Pixmap callbacks without calls to the framework
   ----------------------------------------------------------------------


   PROCEDURE On_Pixmap_Draw_Scheduling_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      Gtk_New(Schedule_From_To, Root_Window.Text, Draw_Scheduling,
         Sys );
      Show_All(Schedule_From_To);


   END On_Pixmap_Draw_Scheduling_Clicked;


   PROCEDURE On_Pixmap_New_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas(Graph_Window);
   END On_Pixmap_New_Clicked;


   PROCEDURE On_Pixmap_Open_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Do_Open;
   END On_Pixmap_Open_Clicked;


   PROCEDURE On_Pixmap_Save_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Do_Save;
   END On_Pixmap_Save_Clicked;


   PROCEDURE On_Pixmap_Save_As_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Do_Save_As;
   END On_Pixmap_Save_As_Clicked;


   PROCEDURE On_Pixmap_Exit_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Do_Exit;
   END On_Pixmap_Exit_Clicked;


   PROCEDURE On_Pixmap_Clear_Work_Space_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Clear_Work_Space;
   END On_Pixmap_Clear_Work_Space_Clicked;


   PROCEDURE On_Pixmap_Dependencies_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS
   BEGIN
      Gtk_New(Graph_Window);
      Show(Graph_Window);
   END On_Pixmap_Dependencies_Clicked;


   PROCEDURE On_Pixmap_Check_Scheduling_Clicked (
         Object : ACCESS Gtk_Toolbar_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_Basics;
      A_Request.Target:=To_Unbounded_String("");
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Pixmap_Check_Scheduling_Clicked;


   PROCEDURE On_Deadlock_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Simulation_Deadlock;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Deadlock_Callback;



   PROCEDURE On_Priority_Inversion_Callback (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Simulation_Priority_Inversion;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Priority_Inversion_Callback;


   PROCEDURE On_Add_Event_Analyzer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New (Update_Event_Analyzers_Set, Lb_Add_Event_Analyzer(
            Current_Language));
      Show_Advanced (Update_Event_Analyzers_Set,False);
   END On_Add_Event_Analyzer_Activate ;


   FUNCTION Delete_Event_Analyzer_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator       : Event_Analyzers_Iterator;
      An_Event_Analyzer : Event_Analyzer_Ptr;
   BEGIN
      LOOP

         Current_Element(Sys.Event_Analyzers, An_Event_Analyzer,
            My_Iterator);

         IF An_Event_Analyzer.Behavior_File_Name = Value
               THEN

            Delete(Sys.Event_Analyzers, An_Event_Analyzer);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Event_Analyzers, My_Iterator);
         Next_Element(Sys.Event_Analyzers, My_Iterator);

      END LOOP;


      Show_Message_Box(Lb_Event_Analyzer_Not_Found(Current_Language));
      RETURN False;
   END Delete_Event_Analyzer_Callback;


   PROCEDURE On_Update_Event_Analyzer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New (Update_Event_Analyzers_Set, Lb_Update_Event_Analyzer(
            Current_Language));
      Show_Advanced (Update_Event_Analyzers_Set,True);
   END On_Update_Event_Analyzer_Activate ;


   PROCEDURE On_Delete_Event_Analyzer_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Event_Analyzers)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Event_Analyzers_Before(Current_Language));
      ELSE
         Gtk_New(Question,Lb_Delete_Event_Analyzer(Current_Language),
            Lb_Event_Analyzer_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Event_Analyzer_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Event_Analyzer_Activate ;


   PROCEDURE On_Event_Analyzer_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Event_Analyzers)=0) THEN
         Show_Message_Box(Lb_Define_Event_Analyzers_Before(
               Current_Language));
      ELSE
         Gtk_New (Display_Event_Analyzers_Set);
         Show_All (Display_Event_Analyzers_Set);
      END IF;
   END On_Event_Analyzer_List_Activate;

   PROCEDURE On_Partition_Small_Task_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;

      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_Small_Task;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Partition_Small_Task_Activate;

   PROCEDURE On_Partition_Best_Fit_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_Best_Fit;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Partition_Best_Fit_Activate;

   PROCEDURE On_Partition_Next_Fit_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_Next_Fit;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Partition_Next_Fit_Activate;


   PROCEDURE On_Partition_General_Task_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_General_Task;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Partition_General_Task_Activate;

   PROCEDURE On_Partition_First_Fit_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Scheduling_Feasibility_First_Fit;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);

   END On_Partition_First_Fit_Activate;



   PROCEDURE Load_Event_Table_File (
         File_Name : IN     Unbounded_String) IS

   BEGIN

      Xml_Event_Table_File_Name := File_Name;

      Read_From_Xml_File (Call_Scheduling_Framework.Sched, Sys, File_Name);

   EXCEPTION
      WHEN Xml_Read_Error =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Xml Fatal Error"));
      WHEN Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      WHEN Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      WHEN Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      WHEN Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      WHEN Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      WHEN Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", End_Error"));
      WHEN Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      WHEN Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      WHEN OTHERS =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language)
            & " ; Exception name    : " & Exception_Name & " ; "
            & "Exception message : " & Exception_Message);


   END Load_Event_Table_File;



   PROCEDURE Save_Event_Table_File (
         File_Name : IN     Unbounded_String) IS

   BEGIN

      Xml_Event_Table_File_Name := File_Name;

      Write_To_Xml_File (Call_Scheduling_Framework.Sched, Sys, File_Name);

   EXCEPTION
      WHEN Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      WHEN Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      WHEN Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      WHEN Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      WHEN Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      WHEN Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", End_Error"));
      WHEN Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      WHEN Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      WHEN OTHERS =>
         Show_Message_Box(Lb_Can_Not_Open_File(Current_Language)
            & " ; Exception name    : " & Exception_Name & " ; "
            & "Exception message : " & Exception_Message);

   END Save_Event_Table_File;


   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;


   PACKAGE Read_Xml_File_Package IS NEW Generic_Select_File_Pkg
      (Load_Event_Table_File, Dummy);
   USE Read_Xml_File_Package;
   PACKAGE Write_Xml_File_Package IS NEW Generic_Select_File_Pkg
      (Save_Event_Table_File, Dummy);
   USE Write_Xml_File_Package;


   PROCEDURE Read_Event_Table_File_Initialize IS
   BEGIN
      NULL;
   END Read_Event_Table_File_Initialize;


   PROCEDURE Write_Event_Table_File_Initialize IS
   BEGIN
      NULL;
   END Write_Event_Table_File_Initialize;



   PROCEDURE On_Load_Event_Table_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      Gtk_New (Read_Xml_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Xml_File_Package.Generic_Select_File);
      Read_Event_Table_File_Initialize;
   END On_Load_Event_Table_Activate;


   PROCEDURE On_Save_Event_Table_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN

      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Gtk_New (Write_Xml_File_Package.Generic_Select_File,  Lb_Save(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Write_Event_Table_File_Initialize;
      Show_All (Write_Xml_File_Package.Generic_Select_File);

   END On_Save_Event_Table_Activate;


   PROCEDURE On_Event_Table_Services_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_Event_Table_Services_Activate;



   PROCEDURE On_Draw_Time_Line_From_Event_Table_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Gtk_New (Draw_Time_Line_From_To, Draw_Scheduling, Sys);
      Show_All(Draw_Time_Line_From_To);
   END On_Draw_Time_Line_From_Event_Table_Activate;




   PROCEDURE On_Compute_Scheduling_And_Generate_Event_Table_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;

   END On_Compute_Scheduling_And_Generate_Event_Table_Activate;


   PROCEDURE On_Run_Analysis_On_Event_Table_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN


      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;


      -- Check if scheduling is computed or not
      --
      IF Call_Scheduling_Framework.Sched.Nb_Entries=0
            THEN
         Show_Message_Box(Lb_Schedule_Before(Current_Language));
         RETURN;
      END IF;


      Gtk_New(Event_Table_Analysis, Root_Window.Text, Sys);
      Show_All(Event_Table_Analysis);


   END On_Run_Analysis_On_Event_Table_Activate;


   PROCEDURE On_Add_Address_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Gtk_New (Update_Address_Spaces_Set, Lb_Add_Address_Space(
               Current_Language));
         Show_Advanced (Update_Address_Spaces_Set, False);
      END IF;
   END On_Add_Address_Space_Activate;


   PROCEDURE On_Update_Address_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Processors)=0)
            THEN
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      ELSE
         Gtk_New (Update_Address_Spaces_Set, Lb_Update_Address_Space(
               Current_Language));
         Show_Advanced (Update_Address_Spaces_Set, True);
      END IF;
   END On_Update_Address_Space_Activate;


   FUNCTION Delete_Address_Space_Callback (
         Value : Unbounded_String)
     RETURN Boolean IS
      My_Iterator      : Address_Spaces_Iterator;
      An_Address_Space : Address_Space_Ptr;
   BEGIN

      LOOP
         Current_Element(Sys.Address_Spaces, An_Address_Space, My_Iterator);

         IF An_Address_Space.Name = Value
               THEN

            Delete_Address_Space(Sys, An_Address_Space);
            Rebuild_Canvas(Graph_Window);

            Is_New:=False;
            Is_Saved:=False;

            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Address_Spaces, My_Iterator);
         Next_Element(Sys.Address_Spaces,
            My_Iterator);
      END LOOP;


      Show_Message_Box(Lb_Address_Space_Not_Found(Current_Language));
      RETURN False;
   END Delete_Address_Space_Callback;


   PROCEDURE On_Delete_Address_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(
            Lb_Define_Address_Spaces_Before(Current_Language));
      ELSE
         Gtk_New(Question, Lb_Delete_Address_Space(Current_Language),
            Lb_Address_Space_Name(Current_Language) & " : ",
            To_Unbounded_String(""),
            Delete_Address_Space_Callback'ACCESS);
         Show_All(Question);
      END IF;
   END On_Delete_Address_Space_Activate;


   PROCEDURE On_Address_Spaces_List_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
      ELSE
         Gtk_New (Display_Address_Spaces_Set);
         Show_All (Display_Address_Spaces_Set);
      END IF;
   END On_Address_Spaces_List_Activate;




   PROCEDURE On_Aadl_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      NULL;
   END On_Aadl_Activate;

   PROCEDURE On_Export_Aadl_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Aadl_Export;
   END On_Export_Aadl_Activate;

   PROCEDURE On_Import_Aadl_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Do_Aadl_Import;
   END On_Import_Aadl_Activate;




   PROCEDURE On_Check_Address_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Global_Memory_Set_Requirement_Analysis;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);


   END On_Check_Address_Space_Activate;

   PROCEDURE On_Check_And_Update_Address_Space_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   BEGIN

      IF (Get_Number_Of_Elements(Sys.Tasks)=0)
            THEN
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         RETURN;
      END IF;
      IF (Get_Number_Of_Elements(Sys.Address_Spaces)=0)
            THEN
         Show_Message_Box(Lb_Define_Address_Spaces_Before(
               Current_Language));
         RETURN;
      END IF;


      Initialize(A_Request);
      A_Request.Statement:=Global_Memory_Compute_Requirement_Analysis;
      Add(Request_List, A_Request);


      Sequential_Framework_Request (
         Sys, Request_List, Response_List);

      Write_Title_And_Text(Root_Window.Text, Response_List);


   END On_Check_And_Update_Address_Space_Activate;



   PROCEDURE On_Export_Aadl_Standard_Properties_Set_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Write_Aadl_Standard_Properties_Set_To_File(Sys);
   END  On_Export_Aadl_Standard_Properties_Set_Activate;


   PROCEDURE On_Export_Cheddar_Property_Sets_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Write_Cheddar_Property_Sets_To_File(Sys);
   END  On_Export_Cheddar_Property_Sets_Activate;

   PROCEDURE On_Aadl_Options_Activate (
         Object : ACCESS Gtk_Menu_Item_Record'Class) IS
   BEGIN
      Gtk_New(Aadl_Options);
      Show_All(Aadl_Options);
   END On_Aadl_Options_Activate;



END Root_Window_Pkg.Callbacks;





