------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Translate;
USE Translate;

WITH Project;
USE Project;
WITH Systems;
USE Systems;

WITH Ada.Io_Exceptions;
USE Ada.Io_Exceptions;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Mmi_Tools;
USE Mmi_Tools;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;
WITH My_Canvas;
USE My_Canvas;
WITH Framework_Config;
USE Framework_Config;

WITH Text_Io;
USE Text_Io;


WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;



PACKAGE BODY Save_File_Selection_Pkg IS


   PROCEDURE Save_File_Ok_Pressed (
         File : IN     Unbounded_String) IS

   BEGIN

      IF Graph_Window/=NULL
            THEN
         Get_Coordinates(Graph_Window.Buffer, Sys);
      END IF;

      Xml_Project_File_Name := File;
      Systems.Write_To_Xml_File(Sys, Xml_Project_File_Name);


      Is_Saved:=True;
      Is_New:=False;


   EXCEPTION
      WHEN Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      WHEN Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      WHEN Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      WHEN Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      WHEN Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      WHEN Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", End_Error"));
      WHEN Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      WHEN Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      WHEN OTHERS =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language)
            & " ; Exception name    : " & Exception_Name & " ; "
            & "Exception message : " & Exception_Message);



   END Save_File_Ok_Pressed;



   PROCEDURE Save_File_Cancel_Pressed IS
   BEGIN
      NULL;
   END Save_File_Cancel_Pressed;



   PROCEDURE Save_File_Initialize IS

   BEGIN
      NULL;
   END Save_File_Initialize;



END Save_File_Selection_Pkg;
