------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE Gtk;
WITH Gtk.Check_Button;
USE Gtk.Check_Button;

WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Editor_Config;
USE Editor_Config;


WITH Project;
USE Project;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Strings;
USE Strings;

WITH Ada.Numerics.Float_Random;
USE Ada.Numerics.Float_Random;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;


PACKAGE BODY Scheduling_Options_Pkg.Callbacks IS


   PROCEDURE Page_Switch (
         Notebook : ACCESS Gtk_Notebook_Record'Class;
         Page     : IN     Gtk.Gtk_Notebook_Page;
         Page_Num : IN     Gtk_Notebook) IS
      PRAGMA Warnings (Off, Page_Num);
      Old_Page : CONSTANT Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   :          Gtk_Pixmap;
   BEGIN
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);

      IF Old_Page /= NULL THEN
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Scheduling_Options.Book_Closed,
            Scheduling_Options.Book_Closed_Mask);
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Scheduling_Options.Book_Closed,
            Scheduling_Options.Book_Closed_Mask);
      END IF;
   END Page_Switch;



   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok : Boolean;

   BEGIN

      IF Get_Active(Scheduling_Options.With_Offsets)
            THEN
         Schedule_With_Offsets:=True;
      ELSE
         Schedule_With_Offsets:=False;
      END IF;

      IF Get_Active(Scheduling_Options.With_Precedencies)
            THEN
         Schedule_With_Precedencies:=True;
      ELSE
         Schedule_With_Precedencies:=False;
      END IF;

      IF Get_Active(Scheduling_Options.With_Resources)
            THEN
         Schedule_With_Resources:=True;
      ELSE
         Schedule_With_Resources:=False;
      END IF;



      IF Get_Active(Scheduling_Options.Seed_Task)
            THEN
         Has_Global_Seed:=False;
      ELSE
         Has_Global_Seed:=True;
      END IF;

      IF Get_Active(Scheduling_Options.Seed_Predictable)
            THEN
         Is_Global_Predictable_Seed:=True;
      ELSE
         Is_Global_Predictable_Seed:=False;
      END IF;

      IF Get_Active(Scheduling_Options.Seed_Unpredictable)
            THEN
         Is_Global_Predictable_Seed:=False;
      ELSE
         Is_Global_Predictable_Seed:=True;
      END IF;



      To_Integer(Get_Text(Scheduling_Options.Seed_Entry),
         Global_Seed_Value,
         Ok);
      IF NOT Ok THEN
         IF Get_Text(Scheduling_Options.Seed_Entry) /= "" THEN
            Show_Message_Box(Lb_Seed(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;


      FOR I IN Time_Unit_Event_Type'Range LOOP
         IF (I/=Task_Activation) AND (I/=Running_Task) THEN
            IF Get_Active(Scheduling_Options.Event_Check_Button(I))
                  THEN
               Generate_Events(I):=True;
            ELSE
               Generate_Events(I):=False;
            END IF;
         END IF;
      END LOOP;



      Destroy(Scheduling_Options);

   END On_Ok_Pressed;




   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Destroy(Scheduling_Options);
   END On_Cancel_Pressed;

   PROCEDURE On_Seed_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Seed : Generator;
      Nmb  : Integer;
   BEGIN
      -- Give random value for the schedulers seed
      --
      Reset(Seed);
      -- range from integer'first .. integer'last
      Nmb := Integer(Double(Random(Seed))*Double(Integer'Last-Integer'
            First)+
         Double(Integer'First));
      Gtk.Gentry.Set_Text(Scheduling_Options.Seed_Entry,Nmb'Img);

   END On_Seed_Pressed;


END Scheduling_Options_Pkg.Callbacks;



