------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;



WITH Gtk;
USE Gtk;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Scrolling_Editor_Pkg.Callbacks;
USE Scrolling_Editor_Pkg.Callbacks;


WITH Mmi_Tools;
USE Mmi_Tools;

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;



PACKAGE BODY Scrolling_Editor_Pkg IS


   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            :        Unbounded_String) IS

      Dummy : CONSTANT Unbounded_String_Ptr :=
      NEW Unbounded_String;

   BEGIN
      Scrolling_Editor := NEW Scrolling_Editor_Record;
      Scrolling_Editor_Pkg.Initialize (Scrolling_Editor, Title,
         Dummy, True, False);
   END Gtk_New;



   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            :        Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr) IS


   BEGIN
      Scrolling_Editor := NEW Scrolling_Editor_Record;
      Scrolling_Editor_Pkg.Initialize (Scrolling_Editor, Title,
         Edited_Text, False, False);
   END Gtk_New;


   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            :        Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr;
         Modify_Callback  : IN     Modify_Callback_Function) IS

   BEGIN
      Scrolling_Editor := NEW Scrolling_Editor_Record;
      Scrolling_Editor_Pkg.Initialize (Scrolling_Editor, Title,
         Edited_Text, False, True, Modify_Callback);
   END Gtk_New;


   PROCEDURE Initialize (
         Scrolling_Editor : ACCESS Scrolling_Editor_Record'Class;
         Title            : IN     Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr;
         Read_Only        : IN     Boolean;
         Stay_Opened      : IN     Boolean;
         Modify_Callback  : IN     Modify_Callback_Function      := NULL) IS

   BEGIN

      Scrolling_Editor.Is_Displayed:=True;
      Scrolling_Editor.Modify_Callback:=Modify_Callback;
      Scrolling_Editor.Edited_Text:=Edited_Text;
      Scrolling_Editor.Read_Only:=Read_Only;
      Scrolling_Editor.Stay_Opened:=Stay_Opened;


      Gtk.Window.Initialize (Scrolling_Editor, Window_Toplevel);
      Set_Title (Scrolling_Editor, To_String(Title));
      Set_Policy (Scrolling_Editor, True, True, True);
      Set_Position (Scrolling_Editor, Win_Pos_Center_Always);

      IF NOT Stay_Opened THEN

         Set_Modal (Scrolling_Editor, True);
         Set_Flags (Scrolling_Editor, Can_Default);
         Grab_Focus (Scrolling_Editor);
      END IF;

      Gtk_New
         (Scrolling_Editor.Alignment, 0.5, 0.5, 1.0,
         1.0);
      Set_Sensitive (Scrolling_Editor.Alignment, True);
      Add (Scrolling_Editor, Scrolling_Editor.Alignment);

      Gtk_New_Vbox (Scrolling_Editor.Vbox, False, 0);
      Add (Scrolling_Editor.Alignment, Scrolling_Editor.Vbox);

      Gtk_New (Scrolling_Editor.Scrolledwindow);
      Set_Usize (Scrolling_Editor.Scrolledwindow, 300, 370);
      Set_Policy (Scrolling_Editor.Scrolledwindow, Policy_Automatic,
         Policy_Automatic);
      Pack_Start (Scrolling_Editor.Vbox, Scrolling_Editor.Scrolledwindow,
         True,
         True, 0);
      Gtk_New (Scrolling_Editor.Text);
      Insert (Scrolling_Editor.Text, Scrolling_Editor.Font_R,
         Scrolling_Editor.
         Black, Null_Color, To_String( Scrolling_Editor.Edited_Text.All));

      Set_Editable (Scrolling_Editor.Text, NOT Read_Only);

      Add (Scrolling_Editor.Scrolledwindow, Scrolling_Editor.Text);


      Gtk_New_Hseparator (Scrolling_Editor.Separator);
      Pack_Start (Scrolling_Editor.Vbox, Scrolling_Editor.Separator, True,
         True, 0);

      Gtk_New_Hbox (Scrolling_Editor.Hbox, True, 6);
      Set_Border_Width (Scrolling_Editor.Hbox, 15);
      Pack_Start (Scrolling_Editor.Vbox, Scrolling_Editor.Hbox, True, True,
         0);

      IF NOT Stay_Opened THEN

         Gtk_New (Scrolling_Editor.Ok_Button, "Ok");
         Set_Usize (Scrolling_Editor.Ok_Button, 70, 25);
         Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.Ok_Button,
            False,
            False, 0);

         Button_Callback.Connect
            (Scrolling_Editor.Ok_Button, "clicked",
            Button_Callback.To_Marshaller (On_Ok_Pressed'ACCESS));

         IF NOT Read_Only THEN



            Gtk_New_Vseparator (Scrolling_Editor.Vseparator1);
            Set_Sensitive (Scrolling_Editor.Vseparator1, False);
            Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.
               Vseparator1,  True, True, 0);


            Gtk_New (Scrolling_Editor.Cancel_Button, To_String(Lb_Cancel(
                     Current_Language)));
            Set_Usize (Scrolling_Editor.Cancel_Button, 70, 25);
            Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.
               Cancel_Button, False,
               False, 0);

            Button_Callback.Connect
               (Scrolling_Editor.Cancel_Button, "clicked",
               Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));
         END IF;

      ELSE

         Gtk_New (Scrolling_Editor.Modify_Button, To_String(Lb_Modify(
                  Current_Language)));
         Set_Usize (Scrolling_Editor.Modify_Button, 70, 25);
         Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.
            Modify_Button, False,
            False, 0);

         Button_Callback.Connect
            (Scrolling_Editor.Modify_Button, "clicked",
            Button_Callback.To_Marshaller (On_Modify_Pressed'ACCESS));


         Gtk_New_Vseparator (Scrolling_Editor.Vseparator1);
         Set_Sensitive (Scrolling_Editor.Vseparator1, False);
         Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.
            Vseparator1,  True, True, 0);


         Gtk_New (Scrolling_Editor.Close_Button, To_String(Lb_Close(
                  Current_Language)));
         Set_Usize (Scrolling_Editor.Close_Button, 70, 25);
         Pack_Start (Scrolling_Editor.Hbox, Scrolling_Editor.Close_Button,
            False,
            False, 0);

         Button_Callback.Connect
            (Scrolling_Editor.Close_Button, "clicked",
            Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));

      END IF;


      Load (Scrolling_Editor.Font_R,
         "-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
      Load (Scrolling_Editor.Font_Bs,
         "-adobe-courier-bold-r-*-*-*-160-*-*-*-*-*-*");
      Load (Scrolling_Editor.Font_B,
         "-adobe-courier-bold-r-*-*-*-200-*-*-*-*-*-*");

      Set_Rgb (Scrolling_Editor.Red, Text_Colors (2).Red, Text_Colors (2).
         Green,Text_Colors (2).Blue);
      Set_Rgb (Scrolling_Editor.Black, Text_Colors (0).Red, Text_Colors (
            0).
         Green,Text_Colors (0).Blue);


   END Initialize;


   PROCEDURE Write_Text (
         Scrolling_Editor : Scrolling_Editor_Access;
         Msg              : Unbounded_String) IS
   BEGIN
      Insert (Scrolling_Editor.Text, Scrolling_Editor.Font_R,
         Scrolling_Editor.
         Black,
         Null_Color,
         To_String(Msg));
   END Write_Text;


   PROCEDURE Write_Title (
         Scrolling_Editor : Scrolling_Editor_Access;
         Msg              : Unbounded_String) IS
   BEGIN
      Insert (Scrolling_Editor.Text, Scrolling_Editor.Font_B,
         Scrolling_Editor.
         Red,
         Null_Color,
         To_String(Msg));
   END Write_Title;

   PROCEDURE Write_Title_Small (
         Scrolling_Editor : Scrolling_Editor_Access;
         Msg              : Unbounded_String) IS
   BEGIN
      Insert (Scrolling_Editor.Text, Scrolling_Editor.Font_Bs,
         Scrolling_Editor.
         Red,
         Null_Color,
         To_String(Msg));
   END Write_Title_Small;



END Scrolling_Editor_Pkg;
