------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;


WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Alignment;
USE Gtk.Alignment;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Gtk.Text;
USE Gtk.Text;
WITH Gdk.Color;
USE Gdk.Color;
WITH Gdk.Font;
USE Gdk.Font;




PACKAGE Scrolling_Editor_Pkg IS



   -- Modify callback type
   --  The procedure is called each time the modify button is pressed
   --
   TYPE Modify_Callback_Function IS ACCESS PROCEDURE
   (Edited_Text : IN Unbounded_String);




   TYPE Scrolling_Editor_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Is_Displayed   : Boolean             := False;
         Alignment      : Gtk_Alignment;
         Vbox           : Gtk_Vbox;
         Separator      : Gtk_Hseparator;
         Hbox           : Gtk_Hbox;
         Scrolledwindow : Gtk_Scrolled_Window;
         Text           : Gtk_Text;
         Vseparator1    : Gtk_Vseparator;

         Ok_Button     : Gtk_Button;
         Cancel_Button : Gtk_Button;
         Close_Button  : Gtk_Button;
         Modify_Button : Gtk_Button;

         Font_R  : Gdk_Font;
         Font_B  : Gdk_Font;
         Font_Bs : Gdk_Font;
         Red     : Gdk_Color;
         Black   : Gdk_Color;

         Read_Only   : Boolean;
         Stay_Opened : Boolean;
         Edited_Text : Unbounded_String_Ptr;

         Modify_Callback : Modify_Callback_Function;
      END RECORD;


   TYPE Scrolling_Editor_Access IS ACCESS ALL Scrolling_Editor_Record'Class;




   -- Read only text : the text can not be edited
   --
   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            : IN     Unbounded_String);

   -- Write text : the text can be edited
   -- The result is stored in "edited_result"
   --
   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            : IN     Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr);

   -- Write text : the text can be edited and the window
   -- stays opened.
   -- The result is stored in "edited_result"
   -- and the procedure "Modify_Callback" is called
   --
   PROCEDURE Gtk_New (
         Scrolling_Editor :    OUT Scrolling_Editor_Access;
         Title            : IN     Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr;
         Modify_Callback  : IN     Modify_Callback_Function);


   PROCEDURE Initialize (
         Scrolling_Editor : ACCESS Scrolling_Editor_Record'Class;
         Title            : IN     Unbounded_String;
         Edited_Text      : IN     Unbounded_String_Ptr;
         Read_Only        : IN     Boolean;
         Stay_Opened      : IN     Boolean;
         Modify_Callback  : IN     Modify_Callback_Function      := NULL);


   PROCEDURE Write_Text (
         Scrolling_Editor : IN     Scrolling_Editor_Access;
         Msg              : IN     Unbounded_String);
   PROCEDURE Write_Title (
         Scrolling_Editor : IN     Scrolling_Editor_Access;
         Msg              : IN     Unbounded_String);
   PROCEDURE Write_Title_Small (
         Scrolling_Editor : IN     Scrolling_Editor_Access;
         Msg              : IN     Unbounded_String);


   Scrolling_Editor : Scrolling_Editor_Access;


   PRAGMA Inline (Write_Text);
   PRAGMA Inline (Write_Title);
   PRAGMA Inline (Write_Title_Small);

END Scrolling_Editor_Pkg;
