------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Project;
USE Project;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Objects;
USE Objects;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Mmi_Tools;
USE Mmi_Tools;


WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;


PACKAGE BODY Select_Time_Line_Pkg.Callbacks IS



   PROCEDURE On_Ok_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      An_Object                   : Object_Ptr;
      A_Name                      : Unbounded_String;
      A_Type                      : Unbounded_String;
      Number_Of_Object_To_Display : Natural          := 0;

   BEGIN

      Reset(Time_Line_To_Display);


      -- First, check we require not too much object to be displayed
      --
      FOR I IN 0..Select_Time_Line.Nb_Entries-1 LOOP
         IF Get_Active(Select_Time_Line.Entries(I).Display_Time_Line)
               THEN
            Number_Of_Object_To_Display:=Number_Of_Object_To_Display+1;
         END IF;
      END LOOP;

      IF Number_Of_Object_To_Display>
            Editor_Config.Max_Time_Line_To_Display
            THEN
         Show_Message_Box(Lb_Can_Not_Display_So_Much_Time_Line(
               Current_Language));
         RETURN;
      END IF;

      IF Number_Of_Object_To_Display=0
            THEN
         Show_Message_Box(Lb_Select_At_Least_One_Time_Line(
               Current_Language));
         RETURN;
      END IF;


      -- Scan the widget to store each task/message/buffer/resource to display
      --
      FOR I IN 0..Select_Time_Line.Nb_Entries-1 LOOP
         IF Get_Active(Select_Time_Line.Entries(I).Display_Time_Line)
               THEN
            A_Name:=To_Unbounded_String(Get_Text(Select_Time_Line.Entries(
                     I).Object_Name));
            A_Type:=To_Unbounded_String(Get_Text(Select_Time_Line.Entries(
                     I).Object_Type));

            IF (A_Type=Lb_Task(Current_Language))
                  THEN
               An_Object:=Object_Ptr(Search_Task(Sys.Tasks, A_Name));
            ELSE
               IF (A_Type=Lb_Buffer(Current_Language))
                     THEN
                  An_Object:=Object_Ptr(Search_Buffer(Sys.Buffers, A_Name));
               ELSE
                  IF (A_Type=Lb_Message(Current_Language))
                        THEN
                     An_Object:=Object_Ptr(Search_Message(Sys.Messages,
                           A_Name));
                  ELSE
                     IF (A_Type=Lb_Resource(Current_Language))
                           THEN
                        An_Object:=Object_Ptr(Search_Resource(Sys.
                              Resources, A_Name));
                     END IF;
                  END IF;
               END IF;
            END IF;
            Add(Time_Line_To_Display, An_Object);
         END IF;
      END LOOP;


      -- Display Time line
      --
      Select_Time_Line.To_Be_Called.All;

      Destroy(Select_Time_Line);

   END On_Ok_Pressed;




END Select_Time_Line_Pkg.Callbacks;
