------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;

WITH Glib;
USE Glib;
WITH Gdk.Types;
USE Gdk.Types;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Gtk.Widget;
USE Gtk.Widget;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Project;
USE Project;

WITH Select_Time_Line_Pkg.Callbacks;
USE Select_Time_Line_Pkg.Callbacks;

WITH Framework_Config;
USE Framework_Config;
WITH Translate;
USE Translate;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Tasks;
USE Tasks;

WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Resources;
USE Resources;

WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Messages;
USE Messages;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Buffers;
USE Buffers;




PACKAGE BODY Select_Time_Line_Pkg IS


   PROCEDURE Gtk_New (
         Select_Time_Line :    OUT Select_Time_Line_Access;
         A_System         : IN     System;
         Call             : IN     Select_Time_Line_Callback_Type) IS
   BEGIN
      Select_Time_Line := NEW Select_Time_Line_Record;
      Select_Time_Line_Pkg.Initialize (Select_Time_Line, Sys, Call);
   END Gtk_New;



   PROCEDURE Initialize (
         Select_Time_Line : ACCESS Select_Time_Line_Record'Class;
         A_System         : IN     System;
         Call             : IN     Select_Time_Line_Callback_Type) IS

      My_Task_Iterator     : Tasks_Iterator;
      My_Resource_Iterator : Resources_Iterator;
      My_Buffer_Iterator   : Buffers_Iterator;
      My_Message_Iterator  : Messages_Iterator;
      A_Task               : Task_Ptr;
      A_Message            : Message_Ptr;
      A_Resource           : Resource_Ptr;
      A_Buffer             : Buffer_Ptr;
      Line                 : Guint              := 1;


      PROCEDURE New_Object (
            An_Object : Object_Ptr) IS
      BEGIN


         Gtk_New (Select_Time_Line.Entries(Select_Time_Line.Nb_Entries).
            Object_Name);
         Set_Editable (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Name, False);
         Set_Max_Length (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Name, 0);
         Set_Text (Select_Time_Line.Entries(Select_Time_Line.Nb_Entries).
            Object_Name,
            To_String((An_Object.Name)));
         Set_Visibility (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Name, True);
         Attach (Select_Time_Line.Table, Select_Time_Line.
            Entries(Select_Time_Line.Nb_Entries).Object_Name, 0, 1, Line,
            Line+1,
            Expand OR Fill, 0,
            0, 0);

         Gtk_New (Select_Time_Line.Entries(Select_Time_Line.Nb_Entries).
            Object_Type);
         Set_Editable (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Type, False);
         Set_Max_Length (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Type, 0);


         CASE An_Object.Object_Type IS
            WHEN Address_Space_Object_Type =>
               NULL;
            WHEN Processor_Object_Type =>
               NULL;
            WHEN Network_Object_Type =>
               NULL;
            WHEN Message_Object_Type =>
               Set_Text (Select_Time_Line.Entries(Select_Time_Line.
                     Nb_Entries).
                  Object_Type, To_String(Lb_Message(Current_Language)));
            WHEN Task_Object_Type =>
               Set_Text (Select_Time_Line.Entries(Select_Time_Line.
                     Nb_Entries).
                  Object_Type, To_String(Lb_Task(Current_Language)));
            WHEN Buffer_Object_Type =>
               Set_Text (Select_Time_Line.Entries(Select_Time_Line.
                     Nb_Entries).
                  Object_Type, To_String(Lb_Buffer(Current_Language)));
            WHEN Resource_Object_Type =>
               Set_Text (Select_Time_Line.Entries(Select_Time_Line.
                     Nb_Entries).
                  Object_Type, To_String(Lb_Resource(Current_Language)));
         END CASE;


         Set_Visibility (Select_Time_Line.Entries(Select_Time_Line.
               Nb_Entries).
            Object_Type, True);
         Attach (Select_Time_Line.Table, Select_Time_Line.
            Entries(Select_Time_Line.Nb_Entries).Object_Type, 1, 2,  Line,
            Line+1,
            Expand OR Fill, 0,
            0, 0);

         Gtk_New (Select_Time_Line.Entries(Select_Time_Line.Nb_Entries).
            Display_Time_Line,
            "");
         Set_Active(Select_Time_Line.Entries(Select_Time_Line.Nb_Entries).
            Display_Time_Line, False);
         Attach (Select_Time_Line.Table, Select_Time_Line.
            Entries(Select_Time_Line.Nb_Entries).Display_Time_Line, 2,  3,
            Line,
            Line+1,
            Expand OR Fill, 0,
            0, 0);


         Line:=Line+1;
         Select_Time_Line.Nb_Entries:=Select_Time_Line.Nb_Entries+1;
      END New_Object;


   BEGIN

      Reset(Time_Line_To_Display);
      Select_Time_Line.Nb_Entries:=0;
      Select_Time_Line.To_Be_Called:=Call;

      Gtk.Window.Initialize (Select_Time_Line, Window_Toplevel);
      Set_Title (Select_Time_Line, To_String(Lb_Select_Time_Line(
               Current_Language)));
      Set_Policy (Select_Time_Line, True, True, True);
      Set_Position (Select_Time_Line, Win_Pos_Center_Always);
      Set_Modal (Select_Time_Line, True);
      Set_Default_Size (Select_Time_Line, 530, 550);
      Set_Extension_Events (Select_Time_Line, Extension_Events_All);
      Set_Flags (Select_Time_Line, Can_Default);
      Grab_Focus (Select_Time_Line);


      Gtk_New (Select_Time_Line.Scroll);
      Set_Border_Width (Select_Time_Line.Scroll, Border_Width => 10);
      Set_Policy (Select_Time_Line.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Select_Time_Line, Select_Time_Line.Scroll);


      Gtk_New (Select_Time_Line.Table, 0, 0, True);
      Set_Row_Spacings (Select_Time_Line.Table, 8);
      Set_Col_Spacings (Select_Time_Line.Table, 7);


      Add_With_Viewport (Select_Time_Line.Scroll, Select_Time_Line.
         Table);
      Set_Focus_Hadjustment (Select_Time_Line.Table, Get_Hadjustment (
            Select_Time_Line.Scroll));
      Set_Focus_Vadjustment (Select_Time_Line.Table, Get_Vadjustment (
            Select_Time_Line.Scroll));




      Gtk_New (Select_Time_Line.Lab, To_String(
            Lb_Name(Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach (Select_Time_Line.Table, Select_Time_Line.
         Lab, 0, 1, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Select_Time_Line.Lab, To_String(
            Lb_Type(Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach (Select_Time_Line.Table, Select_Time_Line.
         Lab, 1, 2, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Select_Time_Line.Lab, To_String(
            Lb_To_Display(Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach (Select_Time_Line.Table, Select_Time_Line.
         Lab, 2, 3, 0, 1,
         0, 0,
         0, 0);



      Reset_Iterator(Sys.Tasks, My_Task_Iterator);
      LOOP
         Current_Element(Sys.Tasks, A_Task, My_Task_Iterator);
         New_Object (Object_Ptr(A_Task));
         EXIT WHEN Is_Last_Element(Sys.Tasks, My_Task_Iterator);
         Next_Element(Sys.Tasks, My_Task_Iterator);
      END LOOP;
      IF NOT Is_Empty(Sys.Buffers) THEN
         Reset_Iterator(Sys.Buffers, My_Buffer_Iterator);
         LOOP
            Current_Element(Sys.Buffers, A_Buffer, My_Buffer_Iterator);
            New_Object (Object_Ptr(A_Buffer));
            EXIT WHEN Is_Last_Element(Sys.Buffers, My_Buffer_Iterator);
            Next_Element(Sys.Buffers, My_Buffer_Iterator);
         END LOOP;
      END IF;
      IF NOT Is_Empty(Sys.Messages) THEN
         Reset_Iterator(Sys.Messages, My_Message_Iterator);
         LOOP
            Current_Element(Sys.Messages, A_Message, My_Message_Iterator);
            New_Object (Object_Ptr(A_Message));
            EXIT WHEN Is_Last_Element(Sys.Messages, My_Message_Iterator);
            Next_Element(Sys.Messages, My_Message_Iterator);
         END LOOP;
      END IF;
      IF NOT Is_Empty(Sys.Resources) THEN
         Reset_Iterator(Sys.Resources, My_Resource_Iterator);
         LOOP
            Current_Element(Sys.Resources, A_Resource,
               My_Resource_Iterator);
            New_Object (Object_Ptr(A_Resource));
            EXIT WHEN Is_Last_Element(Sys.Resources, My_Resource_Iterator);
            Next_Element(Sys.Resources, My_Resource_Iterator);
         END LOOP;
      END IF;



      Gtk_New_Hseparator (Select_Time_Line.Hseparator);
      Attach (Select_Time_Line.Table, Select_Time_Line.
         Hseparator, 0, 3, Line, Line+1,
         Fill, Expand OR Fill,
         0, 0);


      Gtk_New_Hbox (Select_Time_Line.Hbox, True, 0);
      Set_Border_Width (Select_Time_Line.Hbox, 6);
      Attach (Select_Time_Line.Table, Select_Time_Line.Hbox,
         0,
         3, Line+1, Line+2,
         Fill, Expand OR Fill,
         0, 0);


      Gtk_New (Select_Time_Line.Ok_Button, "Ok");
      Pack_Start (Select_Time_Line.Hbox, Select_Time_Line.
         Ok_Button, True,
         True, 0);

      Button_Callback.Connect
         (Select_Time_Line.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'ACCESS));


   END Initialize;


   FUNCTION To_Display (
         Searched_Name : IN     Unbounded_String;
         Searched_Type : IN     Objects_Type)
     RETURN Boolean IS

      Ite       : Time_Line_Object_Iterator;
      An_Object : Object_Ptr;

   BEGIN
      IF Is_Empty(Time_Line_To_Display)
            THEN
         RETURN True;
      END IF;

      Reset_Iterator(Time_Line_To_Display, Ite);
      LOOP
         Current_Element(Time_Line_To_Display, An_Object, Ite);

         IF  (An_Object.Name=Searched_Name) AND (An_Object.Object_Type=
               Searched_Type)
               THEN
            RETURN True;
         END IF;

         EXIT WHEN Is_Last_Element(Time_Line_To_Display, Ite);
         Next_Element(Time_Line_To_Display, Ite);
      END LOOP;

      RETURN False;
   END To_Display;



END Select_Time_Line_Pkg;
