------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk;
USE Gtk;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Check_Button;
USE Gtk.Check_Button;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Objects;
USE Objects;
WITH Systems;
USE Systems;

WITH Editor_Config;
USE Editor_Config;

WITH Sets;
WITH Unchecked_Deallocation;


PACKAGE Select_Time_Line_Pkg IS

   TYPE Select_Time_Line_Callback_Type IS ACCESS PROCEDURE
   ;


   TYPE Select_Time_Line_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Select_Time_Line_Access IS ACCESS ALL Select_Time_Line_Record'Class;

   PROCEDURE Gtk_New (
         Select_Time_Line :    OUT Select_Time_Line_Access;
         A_System         : IN     System;
         Call             : IN     Select_Time_Line_Callback_Type);
   PROCEDURE Initialize (
         Select_Time_Line : ACCESS Select_Time_Line_Record'Class;
         A_System         : IN     System;
         Call             : IN     Select_Time_Line_Callback_Type);


   Select_Time_Line : Select_Time_Line_Access;


   PACKAGE Generic_Time_Line_Object_Set IS NEW Sets (
      Max_Element => Editor_Config.Max_Time_Line,
      Element     => Object_Ptr,
      Free        => Free,
      Copy        => Copy,
      Put         => Put);
   USE Generic_Time_Line_Object_Set;

   TYPE Time_Line_Object_Set IS NEW Generic_Time_Line_Object_Set.Set WITH NULL RECORD;
   SUBTYPE Time_Line_Object_Range IS Generic_Time_Line_Object_Set.
      Element_Range;
   SUBTYPE Time_Line_Object_Iterator IS   Generic_Time_Line_Object_Set.
      Iterator;

   FUNCTION To_Display (
         Searched_Name : IN     Unbounded_String;
         Searched_Type : IN     Objects_Type)
     RETURN Boolean;
   Time_Line_To_Display : Time_Line_Object_Set;

PRIVATE

   TYPE Object_Widget IS
      RECORD
         Object_Name       : Gtk_Entry;
         Object_Type       : Gtk_Entry;
         Display_Time_Line : Gtk_Check_Button;
      END RECORD;

   TYPE Object_Widget_Table IS ARRAY (Time_Line_Object_Range) OF Object_Widget;

   TYPE Select_Time_Line_Record IS NEW Gtk_Window_Record WITH
      RECORD
         To_Be_Called : Select_Time_Line_Callback_Type;
         Scroll       : Gtk_Scrolled_Window;
         Table        : Gtk_Table;
         Lab          : Gtk_Label;
         Entries      : Object_Widget_Table;
         Nb_Entries   : Time_Line_Object_Range;
         Ok_Button    : Gtk_Button;
         Hseparator   : Gtk_Hseparator;
         Vseparator   : Gtk_Vseparator;
         Hbox         : Gtk_Hbox;
      END RECORD;

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Select_Time_Line_Record'Class, Select_Time_Line_Access);







END Select_Time_Line_Pkg;
