------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Glib;
USE Glib;
WITH Gdk;
WITH Gdk.Drawable;
USE Gdk.Drawable;
WITH Gdk.Event;
USE Gdk.Event;
WITH Gdk.Font;
USE Gdk.Font;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gdk.Rectangle;
USE Gdk.Rectangle;
WITH Gdk.Types;
USE Gdk.Types;
WITH Gdk.Window;
USE Gdk.Window;
WITH Gtk;
USE Gtk;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Drawing_Area;
USE Gtk.Drawing_Area;
WITH Gtk.Main;
USE Gtk.Main;
WITH Gtk.Handlers;
USE Gtk.Handlers;
WITH Gtk.Style;
USE Gtk.Style;
WITH Gtk.Window;
USE Gtk.Window;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Events;
USE Events;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Time_Units;
USE Time_Units;
USE Time_Units.A_Time_Unit;


PACKAGE BODY Simple_Mmi IS

   PROCEDURE Draw_Scheduling (
         Task1  : Tasks_Set;
         Period : Natural;
         Sched1 : Scheduling_Sequence) IS

      Left_Margin     : CONSTANT Natural := 60;
      High_Margin     : CONSTANT Natural := 20;
      Interligne      : CONSTANT Natural := 50;
      Timeunit_Margin : CONSTANT Natural := 20;

      Pixmap : Gdk_Pixmap;
      Font   : Gdk_Font;
      Window : Gtk_Window;

      PACKAGE Configured IS NEW Gtk.Handlers.Callback
         (Widget_Type => Gtk_Drawing_Area_Record);


      PROCEDURE Draw_Scheduling (
            Widget : ACCESS Gtk_Drawing_Area_Record'Class) IS

         My_Iterator : Tasks_Iterator;
         A_Task      : Task_Ptr;
         I           : Natural        := 0;

         Msg       : Unbounded_String;
         Task_Name : ARRAY (Tasks_Range) OF Unbounded_String;
         Task_Id   : Natural;
         Time_U    : Natural;
         Ok        : Boolean;

      BEGIN

         ---------------------------------
         -- Draw a line per task
         ---------------------------------

         Reset_Iterator(Task1, My_Iterator);

         LOOP

            Current_Element(Task1, A_Task, My_Iterator);


            IF(A_Task.Task_Type=Periodic_Type)
                  THEN
               Msg:=To_Unbounded_String("P=")
                  & Periodic_Task_Ptr(A_Task).Period'Img
                  & To_Unbounded_String("; C=")
                  & A_Task.Capacity'Img
                  & To_Unbounded_String("; D=")
                  & Periodic_Task_Ptr(A_Task).Deadline'Img
                  & To_Unbounded_String("; S=")
                  & A_Task.Start_Time'Img ;
            ELSE
               Msg:=To_Unbounded_String("C=")
                  & A_Task.Capacity'Img
                  & To_Unbounded_String("; D=")
                  & Aperiodic_Task_Ptr(A_Task).Deadline'Img
                  & To_Unbounded_String("; S=")
                  & A_Task.Start_Time'Img ;
            END IF;

            Draw_Text (Pixmap, Font, Get_Black(Get_Style(Widget)),
               Gint(Left_Margin),
               Gint(High_Margin+I*Interligne+20), To_String(Msg));

            Draw_Text (Pixmap, Font, Get_Black(Get_Style(Widget)),
               10, Gint(High_Margin+I*Interligne), To_String(A_Task.Name));

            Draw_Line (Pixmap, Get_Black(Get_Style(Widget)),
               Gint(Left_Margin),  Gint(High_Margin+I*Interligne),
               Gint(Left_Margin+Period*Timeunit_Margin), Gint(High_Margin+
                  I*Interligne));

            -- For each line, draw unit time lines
            FOR J IN 0 .. Period   LOOP
               Draw_Line (Pixmap, Get_Black(Get_Style(Widget)),
                  Gint(Left_Margin+J*Timeunit_Margin),  Gint(High_Margin+
                     I*Interligne-3),
                  Gint(Left_Margin+J*Timeunit_Margin), Gint(High_Margin+I*
                     Interligne+3));
            END LOOP;


            -- Register each task in the name table
            Task_Name(Tasks_Range(I)):=A_Task.Name;

            I:=I+1;

            EXIT WHEN Is_Last_Element(Task1, My_Iterator);
            Next_Element(Task1, My_Iterator);

         END LOOP;


         ----------------------------------------
         -- Now, draw resulting scheduling
         ----------------------------------------

         FOR K IN 0..Sched1.Nb_Entries -1 LOOP


            -- find task position
            Ok:=False;
            FOR J IN Tasks_Range  LOOP
               IF Sched1.Entries(K).Item.Event_Type = Running_Task THEN
                  IF (Task_Name(J) = Sched1.
                        Entries(K).
                        Item.Running_Task.Name)
                        THEN
                     Task_Id:=Natural(J);
                     Time_U:=Sched1.Entries(K).Data;
                     Ok:=True;
                  END IF;
               END IF;
            END LOOP;
            IF (NOT Ok)
                  THEN
               RAISE Task_Not_Found;
            END IF;

            Draw_Rectangle (Pixmap, Get_Black(Get_Style(Widget)), True,
               Gint(Time_U*Timeunit_Margin+Left_Margin),
               Gint(Task_Id*Interligne+High_Margin-3),
               Gint(Timeunit_Margin), Gint(7));

         END LOOP;



         Draw (Widget);
      END Draw_Scheduling;


      PROCEDURE Button_Press_Event (
            Widget : ACCESS Gtk_Drawing_Area_Record'Class;
            Event  : IN     Gdk_Event) IS
         USE TYPE Gdk.Pixmap.Gdk_Pixmap;
      BEGIN
         IF Get_Button(Event) = 1 AND Pixmap /= Null_Pixmap THEN
            Draw_Scheduling(Widget);
         END IF;
      END Button_Press_Event;

      ---------------------
      -- Configure_Event --
      ---------------------

      PROCEDURE Configure_Event (
            Drawing_Area : ACCESS Gtk_Drawing_Area_Record'Class) IS
         Win    : Gdk_Window;
         Width  : Gint;
         Height : Gint;

      BEGIN
         Win:=Get_Window (Drawing_Area);
         Get_Size (Win, Width, Height);

         Gdk.Pixmap.Gdk_New (Pixmap, Win, Width, Height, -1);
         Draw_Rectangle (Pixmap, Get_White(Get_Style(Drawing_Area)),
            True, 0, 0, Width, Height);
      END Configure_Event;

      ------------------
      -- Expose_Event --
      ------------------

      PROCEDURE Expose_Event (
            Drawing_Area : ACCESS Gtk_Drawing_Area_Record'Class;
            Event        : IN     Gdk.Event.Gdk_Event) IS
         Area : CONSTANT Gdk_Rectangle := Get_Area (Event);
      BEGIN
         Draw_Pixmap (Get_Window(Drawing_Area),
            Get_Fg_Gc(Get_Style(Drawing_Area),State_Normal),
            Pixmap, Area.X, Area.Y, Area.X, Area.Y,
            Gint(Area.Width), Gint(Area.Height));
      END Expose_Event;

      -------------------------
      -- Motion_Notify_Event --
      -------------------------

      PROCEDURE Motion_Notify_Event (
            Widget : ACCESS Gtk_Drawing_Area_Record'Class;
            Event  : IN     Gdk_Event) IS
         X,
         Y      : Gint;
         State  : Gdk_Modifier_Type;
         Result : Gdk_Window;

         USE TYPE Gdk.Pixmap.Gdk_Pixmap;
      BEGIN
         IF Get_Is_Hint (Event) THEN
            Get_Pointer (Get_Window(Widget),X,Y,State,Result);
         ELSE
            X := Gint(Get_X(Event));
            Y := Gint(Get_Y(Event));
            State := Get_State (Event);
         END IF;

         IF (State AND Button1_Mask) /= 0 AND Pixmap /= Null_Pixmap THEN
            Draw_Scheduling(Widget);
         END IF;
      END Motion_Notify_Event;

      Vbox         : Gtk_Box;
      Drawing_Area : Gtk_Drawing_Area;

   BEGIN
      Gtk.Main.Init;
      Gtk_New (Window, Window_Toplevel);
      Set_Title (Window, "Scheduling Result");
      Set_Border_Width (Window, Border_Width => 5);

      Gtk_New_Vbox (Vbox, Homogeneous => False, Spacing => 0);
      Add (Window, Vbox);
      Show (Vbox);

      Gtk_New(Drawing_Area);

      Size (Drawing_Area,
         Gint(Period*Timeunit_Margin+Left_Margin),
         Gint(High_Margin+Interligne*Natural(Get_Number_Of_Elements(Task1))));

      Pack_Start (
         In_Box => Vbox,
         Child  => Drawing_Area);
      Show (Drawing_Area);
      Unrealize (Drawing_Area);

      Set_Events (Drawing_Area, Exposure_Mask OR Leave_Notify_Mask OR
         Button_Press_Mask OR Pointer_Motion_Mask OR
         Pointer_Motion_Hint_Mask);

      Configured.Connect (
         Widget => Drawing_Area,
         Name   => "expose_event",
         Marsh  => Configured.To_Marshaller (Expose_Event'ACCESS));

      Configured.Connect (
         Widget => Drawing_Area,
         Name   => "motion_notify_event",
         Marsh  => Configured.To_Marshaller (Motion_Notify_Event'ACCESS));

      Configured.Connect (
         Widget => Drawing_Area,
         Name   => "configure_event",
         Marsh  => Configured.To_Marshaller (Configure_Event'ACCESS));

      Configured.Connect (
         Widget => Drawing_Area,
         Name   => "button_press_event",
         Marsh  => Configured.To_Marshaller (Button_Press_Event'ACCESS));

      Load (Font,
         "-adobe-helvetica-medium-o-normal--14-140-75-75-p-78-iso8859-1");

      Show_All (Window);
      Gtk.Main.Main;
   END Draw_Scheduling;



END Simple_Mmi;

