------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Text_Io;
USE Text_Io;


PACKAGE BODY Text_Tools  IS




   PROCEDURE Write_Text (
         A_Text : IN     Gtk_Text;
         Msg    :        Unbounded_String) IS
   BEGIN
      Freeze(A_Text);
      Insert (A_Text, Font_R, Black,
         Null_Color,
         To_String(Msg));
      Thaw(A_Text);

      Set_Position(A_Text, Gint(Get_Length(A_Text)));
   END Write_Text;


   PROCEDURE Write_Title (
         A_Text : IN     Gtk_Text;
         Msg    :        Unbounded_String) IS
   BEGIN
      Freeze(A_Text);
      Insert (A_Text, Font_B, Red,
         Null_Color,
         To_String(Msg));
      Thaw(A_Text);

      Set_Position(A_Text, Gint(Get_Length(A_Text)));
   END Write_Title;




   PROCEDURE Write_Title_And_Text (
         A_Text        : IN     Gtk_Text;
         Response_List : IN     Framework_Response_Table) IS

   BEGIN
      FOR J IN 0..Response_List.Nb_Entries-1 LOOP

         Write_Title(A_Text, Response_List.Entries(J).Title);
         Write_Text(A_Text, Response_List.Entries(J).Text);
      END LOOP;
   END Write_Title_And_Text;




   PROCEDURE Initialize IS

   BEGIN
      Load (Font_R,
         "-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
      Load (Font_B,
         "-adobe-courier-bold-r-*-*-*-200-*-*-*-*-*-*");

      Set_Rgb (Red, Text_Colors (2).Red, Text_Colors (2).
         Green,
         Text_Colors (2).Blue);
      Set_Rgb (Black, Text_Colors (0).Red, Text_Colors (0).
         Green,
         Text_Colors (0).Blue);
   END Initialize;



END Text_Tools;


