------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Glib;
USE Glib;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;



WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;


WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;




PACKAGE BODY Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks IS



   PROCEDURE Check_Integrity_Constraints (
         Update_Address_Spaces_Set : IN OUT Update_Address_Spaces_Set_Access;
         Error_Message             : IN OUT Unbounded_String;
         Ok                        : IN OUT Boolean) IS


      Address_Space_Data_String    : Unbounded_String;
      Address_Space_Text_String    : Unbounded_String;
      Address_Space_Heap_String    : Unbounded_String;
      Address_Space_Stack_String   : Unbounded_String;
      Address_Space_Quantum_String : Unbounded_String;

      Sched       : Gint;
      Cpu_Index   : Gint          := 0;
      A_Processor : Processor_Ptr;
      Integer_Ok  : Boolean;


   BEGIN

      Ok:=True;

      Address_Space_Name := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Name));

      Address_Space_Heap_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Heap_Memory_Size));
      Address_Space_Text_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Text_Memory_Size));
      Address_Space_Data_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Data_Memory_Size));
      Address_Space_Stack_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Stack_Memory_Size));


      Address_Space_Quantum_String:=To_Unbounded_String(
         Get_Text(
            Update_Address_Spaces_Set.
            Quantum));
      IF(Address_Space_Quantum_String="")
            THEN
         Address_Space_Quantum:=0;
      ELSE
         To_Integer(Address_Space_Quantum_String, Address_Space_Quantum,
            Integer_Ok);
         IF NOT Integer_Ok
               THEN
            Error_Message:="Quantum" &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      IF Get_Active(Update_Address_Spaces_Set.
            Option)
            THEN
         Address_Space_Preemptive:=Preemptive;
      ELSE
         Address_Space_Preemptive:=Not_Preemptive;
      END IF;




      Address_Space_File_Name :=To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            File_Name));



      Sched:= Gint'Val (12 -
         Gint(Selected_Button (
               Update_Address_Spaces_Set.Clist_Scheduler_Menu)));

      IF (Sched=0)
            THEN
         Address_Space_Scheduler := No_Scheduling_Protocol;
      END IF;
      IF (Sched=1)
            THEN
         Address_Space_Scheduler := Earliest_Deadline_First_Protocol;
      END IF;
      IF (Sched=2)
            THEN
         Address_Space_Scheduler := Least_Laxity_First_Protocol;
      END IF;
      IF (Sched=3)
            THEN
         Address_Space_Scheduler := Rate_Monotonic_Protocol;
      END IF;
      IF (Sched=4)
            THEN
         Address_Space_Scheduler := Deadline_Monotonic_Protocol;
      END IF;
      IF (Sched=5)
            THEN
         Address_Space_Scheduler :=
            Posix_1003_Highest_Priority_First_Protocol;
      END IF;
      IF (Sched=6)
            THEN
         Address_Space_Scheduler :=
            Time_Sharing_Based_On_Wait_Time_Protocol;
      END IF;
      IF (Sched=7)
            THEN
         Address_Space_Scheduler := Round_Robin_Protocol;
      END IF;
      IF (Sched=8)
            THEN
         Address_Space_Scheduler :=
            Maximum_Urgency_First_Based_On_Laxity_Protocol;
      END IF;
      IF (Sched=9)
            THEN
         Address_Space_Scheduler :=
            Maximum_Urgency_First_Based_On_Deadline_Protocol;
      END IF;
      IF (Sched=10)
            THEN
         Address_Space_Scheduler := D_Over_Protocol;
      END IF;
      IF (Sched=11)
            THEN
         Address_Space_Scheduler :=
            Time_Sharing_Based_On_Cpu_Usage_Protocol;
      END IF;
      IF (Sched=12)
            THEN
         Address_Space_Scheduler := Parametric_Protocol;
      END IF;




      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Address_Spaces_Set.
               Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Address_Space_Processor :=A_Processor.Name;



      To_Integer(Address_Space_Text_String,Address_Space_Text, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Address_Space_Text_String /= "" THEN
            Error_Message:=Lb_Text_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Address_Space_Data_String,Address_Space_Data, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Address_Space_Data_String /= To_Unbounded_String("") THEN
            Error_Message:=Lb_Data_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Address_Space_Heap_String,Address_Space_Heap, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Address_Space_Heap_String /= To_Unbounded_String("") THEN
            Error_Message:=Lb_Heap_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;


      To_Integer(Address_Space_Stack_String,Address_Space_Stack,
         Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Address_Space_Stack_String /= To_Unbounded_String("") THEN
            Error_Message:=Lb_Stack_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;



      Check_Address_Space (Sys.Address_Spaces, Integrity_Checks.
         Address_Space_Name,
         Integrity_Checks.Address_Space_Processor,
         Integrity_Checks.Address_Space_Text,
         Integrity_Checks.Address_Space_Stack,
         Integrity_Checks.Address_Space_Data,
         Integrity_Checks.Address_Space_Heap,
         Integrity_Checks.Address_Space_Preemptive,
         Integrity_Checks.Address_Space_Quantum,
         Integrity_Checks.Address_Space_File_Name,
         Integrity_Checks.Address_Space_Scheduler);




   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   END Check_Integrity_Constraints;




END Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks;

