------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;


WITH Scheduler;
USE Scheduler;
WITH Scheduler.Parametrics;
USE Scheduler.Parametrics;

WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;


WITH Generic_Select_File_Pkg;
WITH Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks;



PACKAGE BODY Update_Address_Spaces_Set_Pkg.Callbacks IS



   PROCEDURE Read_Sc_File (
         File_Name : IN     Unbounded_String) IS


   BEGIN
      Set_Text(Update_Address_Spaces_Set.
         File_Name,
         To_String(File_Name) );
   END Read_Sc_File;



   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;

   PACKAGE Read_Sc_File_Package IS NEW Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   USE Read_Sc_File_Package;


   PROCEDURE Show_Address_Spaces (
         Update_Address_Spaces_Set : ACCESS Update_Address_Spaces_Set_Record'Class) IS

      An_Address_Space    : Address_Space_Ptr;
      My_Iterator         : Address_Spaces_Iterator;
      Line                : Gint;
      Address_Space_Share : Chars_Ptr_Array (1 .. 6);
      Text                : Unbounded_String;
      Stack               : Unbounded_String;
      Data                : Unbounded_String;
      Heap                : Unbounded_String;


   BEGIN
      Clear(Update_Address_Spaces_Set.Address_Spaces_List);

      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      IF NOT Is_Empty(Sys.Address_Spaces) THEN
         LOOP

            Current_Element(Sys.Address_Spaces, An_Address_Space,
               My_Iterator);

            Stack := To_Unbounded_String(
               Natural'Image(An_Address_Space.Stack_Memory_Size));
            Data := To_Unbounded_String(
               Natural'Image(An_Address_Space.Data_Memory_Size));
            Heap := To_Unbounded_String(
               Natural'Image(An_Address_Space.Heap_Memory_Size));
            Text := To_Unbounded_String(
               Natural'Image(Natural(An_Address_Space.Text_Memory_Size)));


            -- Create the line
            --
            Address_Space_Share := To_String(An_Address_Space.Name)+
               To_String(An_Address_Space.Cpu_Name)+To_String(Stack)+
               To_String(Data)+To_String(Heap)+To_String(Text);

            -- Display the Line
            --
            Line := Append(Update_Address_Spaces_Set.Address_Spaces_List,
               Address_Space_Share);

            EXIT WHEN Is_Last_Element(Sys.Address_Spaces, My_Iterator);

            Next_Element(Sys.Address_Spaces, My_Iterator);

         END LOOP;
      END IF;
   END Show_Address_Spaces;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name             : Unbounded_String;
      Cpu_Name         : Unbounded_String;
      Text             : Unbounded_String;
      Stack            : Unbounded_String;
      Data             : Unbounded_String;
      Heap             : Unbounded_String;
      Row              : Gint;
      My_Iterator      : Processor_Set.Generic_Processor_Set.Iterator;
      A_Processor      : Processor_Ptr;
      Index            : Strings_Range                                :=
        Strings_Range'First;
      An_Address_Space : Address_Space_Ptr;
      My_Scheduler     : Scheduler_Ptr;

      Items    : Strings_Table;
      Sched_Id : Gint          := 0;

      Current        : Gint;
      Location_Items : Strings_Table;

   BEGIN

      Row:=Get_Focus_Row(Update_Address_Spaces_Set.Address_Spaces_List);
      Name := To_Unbounded_String(Get_Text(Update_Address_Spaces_Set.
            Address_Spaces_List,
            Row,0));

      Update_Address_Spaces_Set.Current_Address_Space := Name;
      An_Address_Space:=Search_Address_Space(Sys.Address_Spaces, Name);

      Cpu_Name      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,1));
      Stack      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,2));
      Data        := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,3));
      Heap      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,4));
      Text        := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,5));

      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Text_Memory_Size,
         To_String(
            Text));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Heap_Memory_Size,
         To_String(
            Heap));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Data_Memory_Size,
         To_String(
            Data));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Stack_Memory_Size,
         To_String(
            Stack));


      -- Local scheduler information
      --
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Quantum,
         To_String(Get_Quantum(An_Address_Space.Scheduler.All)));



      IF Get_Preemptive(An_Address_Space.Scheduler.All) = Not_Preemptive
            THEN
         Set_Active(Update_Address_Spaces_Set.Option, False);
      ELSE
         Set_Active(Update_Address_Spaces_Set.Option, True);
      END IF;



      My_Scheduler:=An_Address_Space.All.Scheduler;


      IF (Get_Name(My_Scheduler.All) = No_Scheduling_Protocol)
            THEN
         Sched_Id:=0;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Earliest_Deadline_First_Protocol)
            THEN
         Sched_Id:=1;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Least_Laxity_First_Protocol)
            THEN
         Sched_Id:=2;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Rate_Monotonic_Protocol)
            THEN
         Sched_Id:=3;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Deadline_Monotonic_Protocol)
            THEN
         Sched_Id:=4;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Posix_1003_Highest_Priority_First_Protocol)
            THEN
         Sched_Id:=5;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Time_Sharing_Based_On_Wait_Time_Protocol)
            THEN
         Sched_Id:=6;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Round_Robin_Protocol)
            THEN
         Sched_Id:=7;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Maximum_Urgency_First_Based_On_Laxity_Protocol)
            THEN
         Sched_Id:=8;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Maximum_Urgency_First_Based_On_Deadline_Protocol)
            THEN
         Sched_Id:=9;
      END IF;
      IF (Get_Name(My_Scheduler.All) = D_Over_Protocol)
            THEN
         Sched_Id:=10;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Time_Sharing_Based_On_Cpu_Usage_Protocol)
            THEN
         Sched_Id:=11;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Parametric_Protocol)
            THEN
         Sched_Id:=12;
      END IF;


      IF (Sched_Id=12)
            THEN
         Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.File_Name,
            To_String(Get_Behavior_File_Name(Parametric_Scheduler(
                     My_Scheduler.All))));
      ELSE
         Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.File_Name,
            "");
      END IF;



      Update_Address_Spaces_Set.Clist_Scheduler_Menu:= Widget_Slist.
         Null_List;
      Gtk.Option_Menu.Gtk_New(Update_Address_Spaces_Set.Scheduler_Menu);
      Build_Local_Scheduler_Table_Name(Items);
      Build_Option_Menu (Update_Address_Spaces_Set.Scheduler_Menu,
         Update_Address_Spaces_Set.Clist_Scheduler_Menu, Items,
         Sched_Id, NULL);
      Show_All(Update_Address_Spaces_Set.Scheduler_Menu);
      Attach_Defaults(Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.
         Scheduler_Menu, 1, 2, 2, 3);




      -- Rebuild The CPU Option Menu & select the right item
      --
      Initialize(Location_Items);
      Index:=0;
      Reset_Iterator(Sys.Processors, My_Iterator);

      LOOP
         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         Add(Location_Items,A_Processor.Name);
         IF A_Processor.Name =
               Cpu_Name THEN
            Current:=Gint(Index);
         END IF;
         Index:=Index+1;
         EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);
         Next_Element(Sys.Processors, My_Iterator);
      END LOOP;

      Update_Address_Spaces_Set.Clist_Processor_Menu:= Widget_Slist.
         Null_List;

      Gtk.Option_Menu.Gtk_New(Update_Address_Spaces_Set.Processor_Menu);

      Build_Option_Menu (Update_Address_Spaces_Set.Processor_Menu,
         Update_Address_Spaces_Set.Clist_Processor_Menu,
         Location_Items, Current, NULL);


      Show_All(Update_Address_Spaces_Set.Processor_Menu);
      Attach_Defaults(Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.
         Processor_Menu,1,2,1,2);


      Show_All(Update_Address_Spaces_Set);

   END On_Click_List;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Address_Spaces_Set);
   END On_Close_Pressed;



   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Address_Space_Set.Generic_Address_Space_Set.Iterator;

      An_Address_Space : Address_Space_Ptr;

   BEGIN



      IF Update_Address_Spaces_Set.Current_Address_Space = "" THEN
         Show_Message_Box(Lb_Select_Address_Space_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Address_Spaces) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      IF NOT Is_Empty(Sys.Address_Spaces) THEN
         LOOP

            Current_Element(Sys.Address_Spaces, An_Address_Space,
               My_Iterator);
            IF An_Address_Space.Name = Update_Address_Spaces_Set.
                  Current_Address_Space THEN


               Delete_Address_Space(Sys, An_Address_Space);
               Rebuild_Canvas(Graph_Window);


               -- Clear  entries
               --
               Update_Address_Spaces_Set.Current_Address_Space :=
                  To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Stack_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Data_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Heap_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Text_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Quantum, "");

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Address_Spaces, My_Iterator);
            Next_Element(Sys.Address_Spaces, My_Iterator);

         END LOOP;
      END IF;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   END On_Delete_Pressed;




   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Address_Spaces_Set.Undo, Sys);
      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Address_Spaces_Set);
   END On_Cancel_Pressed;


   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Address_Spaces_Set.Vbox_List) THEN
         Show_Advanced(Update_Address_Spaces_Set,False);
      ELSE
         Show_Advanced(Update_Address_Spaces_Set,True);
      END IF;
   END On_Advanced_Pressed;



   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Address_Spaces_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      -- Add the new item in the set
      --
      Add_Address_Space (Sys.Address_Spaces, Integrity_Checks.
         Address_Space_Name,
         Integrity_Checks.Address_Space_Processor,
         Integrity_Checks.Address_Space_Text,
         Integrity_Checks.Address_Space_Stack,
         Integrity_Checks.Address_Space_Data,
         Integrity_Checks.Address_Space_Heap,
         Integrity_Checks.Address_Space_Preemptive,
         Integrity_Checks.Address_Space_Quantum,
         Integrity_Checks.Address_Space_File_Name,
         Integrity_Checks.Address_Space_Scheduler);



      Update_Address_Spaces_Set.Current_Address_Space :=
         Integrity_Checks.Address_Space_Name;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   EXCEPTION
      WHEN Address_Space_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;

   END On_Add_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      An_Address_Space : Address_Space_Ptr;
      My_Iterator      : Address_Space_Set.Generic_Address_Space_Set.Iterator;

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;


   BEGIN


      IF Update_Address_Spaces_Set.Current_Address_Space = "" THEN
         Show_Message_Box(Lb_Select_Address_Space_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Address_Spaces) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Address_Spaces_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      --
      -- Update set
      --

      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      LOOP

         Current_Element(Sys.Address_Spaces, An_Address_Space, My_Iterator);
         IF An_Address_Space.Name = Update_Address_Spaces_Set.
               Current_Address_Space THEN

            Delete(Sys.Address_Spaces, An_Address_Space);


            Add_Address_Space (Sys.Address_Spaces, Integrity_Checks.
               Address_Space_Name,
               Integrity_Checks.Address_Space_Processor,
               Integrity_Checks.Address_Space_Text,
               Integrity_Checks.Address_Space_Stack,
               Integrity_Checks.Address_Space_Data,
               Integrity_Checks.Address_Space_Heap,
               Integrity_Checks.Address_Space_Preemptive,
               Integrity_Checks.Address_Space_Quantum,
               Integrity_Checks.Address_Space_File_Name,
               Integrity_Checks.Address_Space_Scheduler);



            Update_Address_Spaces_Set.Current_Address_Space :=
               Integrity_Checks.Address_Space_Name;
            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Address_Spaces, My_Iterator);
         Next_Element(Sys.Address_Spaces, My_Iterator);

      END LOOP;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   EXCEPTION
      WHEN Address_Space_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;



   END On_Modify_Pressed;



   PROCEDURE On_Read_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Sched : Gint;

   BEGIN

      Sched:= Gint'Val (12 - Gint(Selected_Button(
               Update_Address_Spaces_Set.
               Clist_Scheduler_Menu)));

      IF (Sched/=12)
            THEN
         Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
         RETURN;
      END IF;

      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Sc_File_Package.Generic_Select_File);

   END On_Read_Pressed;



END Update_Address_Spaces_Set_Pkg.Callbacks;
