------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;



WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;


WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;




PACKAGE BODY Update_Buffers_Set_Pkg.Callbacks.Integrity_Checks IS



   PROCEDURE Check_Integrity_Constraints (
         Update_Buffers_Set : IN OUT Update_Buffers_Set_Access;
         Error_Message      : IN OUT Unbounded_String;
         Ok                 : IN OUT Boolean) IS


      Buffer_Qs_String   : Unbounded_String;
      Buffer_Size_String : Unbounded_String;
      Integer_Ok         : Boolean;
      Buffer_Ok          : Boolean;

   BEGIN

      Ok:=True;

      Buffer_Name := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Name));


      Buffer_Processor :=To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Cpu_Name));
      Buffer_Address_Space:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Address_Space_Menu)));


      Buffer_Size_String := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.Size));

      To_Integer(Buffer_Size_String, Buffer_Size, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         Error_Message:=Lb_Size(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language);
         Ok:=False;
         RETURN;
      END IF;


      Buffer_Qs_String:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Qs_Menu)));
      Display_String_To_Qs(Buffer_Qs_String, Buffer_Qs, Buffer_Ok);



      Check_Buffer (Sys.Buffers,
         Buffer_Name, Buffer_Size,
         Buffer_Processor, Buffer_Address_Space);


   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   END Check_Integrity_Constraints;


   PROCEDURE Check_Task_Role_Integrity_Constraints (
         Update_Buffers_Set : IN OUT Update_Buffers_Set_Access;
         Error_Message      : IN OUT Unbounded_String;
         Ok                 : IN OUT Boolean) IS


      Time_String : Unbounded_String;
      Size_String : Unbounded_String;
      Type_String : Unbounded_String;
      Integer_Ok  : Boolean;
      Role_Ok     : Boolean;

   BEGIN

      Ok:=True;


      -- Task name
      --
      Buffer_Role_Task:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Affect_Task)));


      -- Role Time
      --
      Time_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affect_Roletime));

      To_Integer(Time_String,Buffer_Role_Time,Integer_Ok);
      IF NOT Integer_Ok THEN
         IF (Time_String/="") THEN
            Error_Message:=Lb_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;


      -- Role size
      --
      Size_String:=To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affect_Size));

      To_Integer(Size_String, Buffer_Role_Size, Integer_Ok);
      IF NOT Integer_Ok THEN
         IF (Size_String/="") THEN
            Error_Message:=Lb_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;


      -- Type of task access (producer or consumer ?)
      --
      Type_String:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Affect_Roletype)));
      To_Role(Type_String, Buffer_Role_Type, Role_Ok);




      Check_Buffer_Roles (Sys.Tasks,
         Integrity_Checks.Buffer_Name,
         Integrity_Checks.Buffer_Role_Task,
         Integrity_Checks.Buffer_Role_Time,
         Integrity_Checks.Buffer_Role_Size );


   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   END Check_Task_Role_Integrity_Constraints;




END Update_Buffers_Set_Pkg.Callbacks.Integrity_Checks;

