------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Task_Dependencies;
USE Task_Dependencies;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;
WITH Framework_Config;
USE Framework_Config;
WITH Buffers;
USE Buffers;
USE Buffers.Buffer_Roles;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Queueing_System;
USE Queueing_System;


WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;

WITH Systems;
USE Systems;


WITH Update_Buffers_Set_Pkg.Callbacks.Integrity_Checks;




PACKAGE BODY Update_Buffers_Set_Pkg.Callbacks IS



   PROCEDURE Show_Buffers (
         Update_Buffers_Set : ACCESS Update_Buffers_Set_Record'Class) IS

      A_Buffer     : Buffer_Ptr;
      My_Iterator  : Buffers_Iterator;
      Line         : Gint;
      Buffer_Share : Chars_Ptr_Array (1 .. 3);
      Size         : Unbounded_String;


   BEGIN
      Clear(Update_Buffers_Set.Buffers_List);

      Reset_Iterator(Sys.Buffers, My_Iterator);
      IF NOT Is_Empty(Sys.Buffers) THEN
         LOOP

            Current_Element(Sys.Buffers, A_Buffer,
               My_Iterator);

            Size := To_Unbounded_String(
               Natural'Image(A_Buffer.Size));


            -- Create the line
            --
            Buffer_Share := To_String(A_Buffer.Name)+
               To_String(A_Buffer.Cpu_Name)+To_String(Size);

            -- Display the Line
            --
            Line := Append(Update_Buffers_Set.Buffers_List,
               Buffer_Share);

            EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);

            Next_Element(Sys.Buffers, My_Iterator);

         END LOOP;
      END IF;
   END Show_Buffers;



   PROCEDURE Show_Task_Role_List (
         Update_Buffers_Set : ACCESS Update_Buffers_Set_Record'Class) IS



      Task_Name   : Unbounded_String;
      Role_Time   : Natural;
      Role_Size   : Natural;
      A_Role_Type : Role_Type;
      Line        : Gint;
      Role_Share  : Chars_Ptr_Array (1 .. 4);


   BEGIN
      Clear(Update_Buffers_Set.Affected_Role_List);


      FOR I IN 0 .. Update_Buffers_Set.My_Roles.Nb_Entries-1
            LOOP

         Task_Name:=Update_Buffers_Set.My_Roles.Entries(I).
            Item;
         Role_Size :=
            Update_Buffers_Set.My_Roles.Entries(I).
            Data.Size;
         Role_Time :=
            Update_Buffers_Set.My_Roles.Entries(I).
            Data.Time;
         A_Role_Type :=
            Update_Buffers_Set.My_Roles.Entries(I).
            Data.The_Role;

         Role_Share:= To_String(Task_Name) +
            Role_Size'Img + A_Role_Type'Img + Role_Time'Img;


         -- Show Task on Clist
         --
         Line := Append(Update_Buffers_Set.Affected_Role_List,
            Role_Share);


      END LOOP;

   END  Show_Task_Role_List;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name : Unbounded_String;
      Size : Unbounded_String;
      Row  : Gint;

      A_Buffer : Buffer_Ptr;

      A_Address_Space        : Address_Space_Ptr;
      Address_Space_Iterator : Address_Space_Set.Generic_Address_Space_Set.Iterator;
      Empty_Menu             : String_List.Glist;



   BEGIN

      Row:=Get_Focus_Row(Update_Buffers_Set.Buffers_List);
      Name := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Buffers_List,
            Row,0));
      Update_Buffers_Set.Current_Buffer := Name;
      Size      := To_Unbounded_String(Get_Text(
            Update_Buffers_Set.
            Buffers_List,Row,2));


      A_Buffer:=Search_Buffer(Sys.Buffers,
         Update_Buffers_Set.Current_Buffer);


      -- Show buffer name
      --
      Gtk.Gentry.Set_Text(Update_Buffers_Set.Name, To_String(Name));


      -- Show buffer size
      --
      Gtk.Gentry.Set_Text(Update_Buffers_Set.Size,
         To_String(
            Size));


      -- Rebuild The CPU Option Menu & select the right item
      --
      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces,
         A_Buffer.
         Address_Space_Name);
      Set_Text(Update_Buffers_Set.Cpu_Name, To_String(
            A_Address_Space.
            Cpu_Name));

      Update_Buffers_Set.Clist_Address_Space_Menu:=Empty_Menu;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name =  A_Buffer.Address_Space_Name THEN
            String_List.Append(Update_Buffers_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name /=  A_Buffer.Address_Space_Name THEN
            String_List.Append(Update_Buffers_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;

      Set_Popdown_Strings (Update_Buffers_Set.Address_Space_Menu,
         Update_Buffers_Set.Clist_Address_Space_Menu);

      -- Show queueing system
      --
      Update_Buffers_Set.Clist_Qs_Menu:=Empty_Menu;
      String_List.Append(Update_Buffers_Set.
         Clist_Qs_Menu, To_String(Qs_To_Display_String(
               A_Buffer.Queueing_System_Type)));

      FOR I IN Queueing_Systems_Type'RANGE  LOOP
         IF I/=A_Buffer.Queueing_System_Type
               THEN
            String_List.Append(Update_Buffers_Set.Clist_Qs_Menu,  To_String(
                  Qs_To_Display_String(
                     I)));
         END IF;
      END LOOP;
      Set_Popdown_Strings (Update_Buffers_Set.Qs_Menu,
         Update_Buffers_Set.Clist_Qs_Menu);



      --  Show task using the buffer
      --
      Initialize(Update_Buffers_Set.My_Roles);
      FOR I IN 0 .. A_Buffer.Roles.Nb_Entries-1
            LOOP
         Update_Buffers_Set.My_Roles.Entries(I).Item:=
            A_Buffer.Roles.Entries(I).Item;
         Update_Buffers_Set.My_Roles.Entries(I).Data :=
            A_Buffer.Roles.Entries(I).Data;
         Update_Buffers_Set.My_Roles.Nb_Entries:=
            Update_Buffers_Set.My_Roles.Nb_Entries+1;
      END LOOP;



      Show_Task_Role_List(Update_Buffers_Set);
      Show_All(Update_Buffers_Set);

   END On_Click_List;




   PROCEDURE On_Change_Cpu (
         Widget : ACCESS Gtk_Editable_Record'Class) IS

      Buffer_Address_Space : Unbounded_String;
      A_Address_Space      : Address_Space_Ptr;

   BEGIN
      Buffer_Address_Space:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Buffers_Set.Address_Space_Menu)));
      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces,
         Buffer_Address_Space);
      Set_Text(Update_Buffers_Set.Cpu_Name, To_String(A_Address_Space.
            Cpu_Name));
   END On_Change_Cpu;



   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Buffers_Set.Undo, Sys);
      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Buffers_Set);
   END On_Cancel_Pressed;


   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Buffers_Set.Vbox_List) THEN
         Show_Advanced(Update_Buffers_Set,False);
      ELSE
         Show_Advanced(Update_Buffers_Set,True);
      END IF;
   END On_Advanced_Pressed;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      A_Buff      : Buffer_Ptr;
      My_Iterator : Buffers_Iterator;

   BEGIN

      Is_New:=False;
      Is_Saved:=False;


      Reset_Iterator(Sys.Buffers, My_Iterator);
      LOOP
         Current_Element(Sys.Buffers, A_Buff, My_Iterator);

         -- For all buffers, update dependencies :
         -- Delete and then add all dependencies
         --
         Add_All_Task_Dependencies(Sys.Dependencies, Sys.Tasks, A_Buff);

         EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);
      END LOOP;

      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Buffers_Set);
   END On_Close_Pressed;


   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Buffer_Set.Generic_Buffer_Set.Iterator;
      A_Buffer    : Buffer_Ptr;

   BEGIN

      IF Update_Buffers_Set.Current_Buffer = "" THEN
         Show_Message_Box(Lb_Select_Buffer_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Buffers) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Buffers, My_Iterator);
      IF NOT Is_Empty(Sys.Buffers) THEN
         LOOP

            Current_Element(Sys.Buffers, A_Buffer,
               My_Iterator);
            IF A_Buffer.Name = Update_Buffers_Set.
                  Current_Buffer THEN

               Delete_Buffer(Sys, A_Buffer);
               Rebuild_Canvas(Graph_Window);

               -- Clear  entries
               --
               Update_Buffers_Set.Current_Buffer :=
                  To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Buffers_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Buffers_Set.
                  Size, "");

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
            Next_Element(Sys.Buffers, My_Iterator);

         END LOOP;
      END IF;

      Show_Buffers(Update_Buffers_Set);


   END On_Delete_Pressed;




   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Buffers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      -- Add the new item in the set
      --
      Add_Buffer (Sys.Buffers,
         Integrity_Checks.Buffer_Name, Integrity_Checks.Buffer_Size,
         Integrity_Checks.Buffer_Processor,
         Integrity_Checks.Buffer_Address_Space,
         Integrity_Checks.Buffer_Qs, Update_Buffers_Set.My_Roles);


      Update_Buffers_Set.Current_Buffer :=
         Integrity_Checks.Buffer_Name;

      Show_Buffers(Update_Buffers_Set);


   EXCEPTION
      WHEN Buffer_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Add_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      A_Buffer    : Buffer_Ptr;
      My_Iterator : Buffer_Set.Generic_Buffer_Set.Iterator;

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;


   BEGIN


      IF Update_Buffers_Set.Current_Buffer = "" THEN
         Show_Message_Box(Lb_Select_Buffer_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Buffers) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Buffers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      --
      -- Update set
      --

      Reset_Iterator(Sys.Buffers, My_Iterator);
      LOOP

         Current_Element(Sys.Buffers, A_Buffer, My_Iterator);
         IF A_Buffer.Name = Update_Buffers_Set.
               Current_Buffer THEN

            Delete(Sys.Buffers, A_Buffer);
            Update_Buffers_Set.Current_Buffer :=
               Integrity_Checks.Buffer_Name;
            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Buffers, My_Iterator);
         Next_Element(Sys.Buffers, My_Iterator);

      END LOOP;


      Add_Buffer (Sys.Buffers,
         Integrity_Checks.Buffer_Name, Integrity_Checks.Buffer_Size,
         Integrity_Checks.Buffer_Processor,
         Integrity_Checks.Buffer_Address_Space,
         Integrity_Checks.Buffer_Qs, Update_Buffers_Set.My_Roles);



      Show_Buffers(Update_Buffers_Set);


   EXCEPTION
      WHEN Buffer_Set.Invalid_Parameter =>

         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;

   END On_Modify_Pressed;




   PROCEDURE On_Role_Click (
         List : ACCESS Gtk_Clist_Record'Class) IS




      Name        : Unbounded_String;
      Size_String : Unbounded_String;
      Time_String : Unbounded_String;
      Type_String : Unbounded_String;
      Row         : Gint;

      A_Task     : Task_Ptr;
      Ite        : Tasks_Iterator;
      Task_Items : String_List.Glist;
      Type_Items : String_List.Glist;

   BEGIN

      Row:=Get_Focus_Row(Update_Buffers_Set.Affected_Role_List);
      Name       := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,0));
      A_Task:=Search_Task(Sys.Tasks, Name);
      Size_String := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,1));
      Type_String  := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,2));
      Time_String  := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,3));


      -- Set task name
      --
      String_List.Append (Task_Items, To_String(Name));
      Reset_Iterator(Sys.Tasks, Ite);
      LOOP
         Current_Element(Sys.Tasks, A_Task, Ite);

         IF A_Task.Name/=Name
               THEN
            String_List.Append (Task_Items,
               To_String(A_Task.Name));
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Tasks, Ite);

         Next_Element(Sys.Tasks, Ite);
      END LOOP;


      Set_Popdown_Strings (Update_Buffers_Set.Affect_Task,
         Task_Items);


      -- Set size and time
      --
      Set_Text(Update_Buffers_Set.Affect_Size, To_String(Size_String));
      Set_Text(Update_Buffers_Set.Affect_Roletime, To_String(Time_String));


      -- Set type
      --
      String_List.Append (Type_Items, To_String(Type_String));
      IF Type_String="CONSUMER"
            THEN
         String_List.Append (Type_Items, "PRODUCER");
      ELSE
         String_List.Append (Type_Items, "CONSUMER");
      END IF;

      Set_Popdown_Strings (Update_Buffers_Set.Affect_Roletype,
         Type_Items);

   END On_Role_Click;





   PROCEDURE On_Modify_Role_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok                    : Boolean          := True;
      Error_Message         : Unbounded_String;
      Affect_Task_Share     : Buffer_Role;
      Old_Affect_Task_Share : Buffer_Role;
      Name                  : Unbounded_String;
      Natural_Ok            : Boolean;
      Role_Ok               : Boolean;
      Size_String           : Unbounded_String;
      Time_String           : Unbounded_String;
      Type_String           : Unbounded_String;
      Old_Time              : Natural;
      Old_Size              : Natural;
      Old_Type              : Role_Type;
      Row                   : Gint;


   BEGIN

      Row:=Get_Focus_Row(Update_Buffers_Set.Affected_Role_List);
      Name       := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,0));
      Size_String := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,1));
      Type_String  := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,2));
      Time_String  := To_Unbounded_String(Get_Text(Update_Buffers_Set.
            Affected_Role_List,Row,3));


      To_Natural(Size_String, Old_Size, Natural_Ok);
      To_Natural(Time_String, Old_Time, Natural_Ok);
      To_Role(Type_String, Old_Type, Role_Ok);


      Old_Affect_Task_Share.Time :=
         Old_Time;
      Old_Affect_Task_Share.Size :=
         Old_Size;
      Old_Affect_Task_Share.The_Role :=
         Old_Type;



      Integrity_Checks.Check_Task_Role_Integrity_Constraints (
         Update_Buffers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;

      Affect_Task_Share.Time :=
         Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size :=
         Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role :=
         Integrity_Checks.Buffer_Role_Type;



      -- Find old values and replace them
      --
      FOR I IN 0..Update_Buffers_Set.My_Roles.Nb_Entries-1 LOOP
         IF (Update_Buffers_Set.My_Roles.Entries(I).Item=Name)
               AND (Update_Buffers_Set.My_Roles.Entries(I).
               Data=Old_Affect_Task_Share)
               THEN
            Update_Buffers_Set.My_Roles.Entries(I).Item:=
               Integrity_Checks.Buffer_Role_Task;
            Update_Buffers_Set.My_Roles.Entries(I).
               Data:=Affect_Task_Share;
            EXIT;
         END IF;
      END LOOP;


      -- Show Task on Clist
      --
      Show_Task_Role_List(Update_Buffers_Set);

   END On_Modify_Role_Pressed;



   PROCEDURE On_Delete_Role_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok                : Boolean          := True;
      Error_Message     : Unbounded_String;
      Affect_Task_Share : Buffer_Role;

   BEGIN

      Integrity_Checks.Check_Task_Role_Integrity_Constraints (
         Update_Buffers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      Affect_Task_Share.Time :=
         Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size :=
         Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role :=
         Integrity_Checks.Buffer_Role_Type;



      -- Find item and delete it
      --
      FOR I IN 0..Update_Buffers_Set.My_Roles.Nb_Entries-1 LOOP
         IF (Update_Buffers_Set.My_Roles.Entries(I).Item=
               Integrity_Checks.Buffer_Role_Task)
               AND (Update_Buffers_Set.My_Roles.Entries(I).
               Data=Affect_Task_Share)
               THEN
            Update_Buffers_Set.My_Roles.Entries(I):=
               Update_Buffers_Set.My_Roles.Entries(
               Update_Buffers_Set.My_Roles.Nb_Entries-1);
            Update_Buffers_Set.My_Roles.Nb_Entries:=
               Update_Buffers_Set.My_Roles.Nb_Entries-1;
            EXIT;
         END IF;
      END LOOP;



      -- Show Task on Clist
      --
      Show_Task_Role_List(Update_Buffers_Set);

   END On_Delete_Role_Pressed;


   PROCEDURE On_Add_Role_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Affect_Task_Share : Buffer_Role;
      Ok                : Boolean          := True;
      Error_Message     : Unbounded_String;

   BEGIN

      Integrity_Checks.Check_Task_Role_Integrity_Constraints (
         Update_Buffers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      Affect_Task_Share.Time :=
         Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size :=
         Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role :=
         Integrity_Checks.Buffer_Role_Type;

      -- Add Task on list
      --
      Update_Buffers_Set.My_Roles.Entries(
         Update_Buffers_Set.
         My_Roles.Nb_Entries).Item:=
         Integrity_Checks.Buffer_Role_Task;
      Update_Buffers_Set.My_Roles.Entries(
         Update_Buffers_Set.
         My_Roles.Nb_Entries).Data := Affect_Task_Share;
      Update_Buffers_Set.My_Roles.Nb_Entries:=
         Update_Buffers_Set.My_Roles.Nb_Entries+1;


      -- Show Task on Clist
      --
      Show_Task_Role_List(Update_Buffers_Set);


   END On_Add_Role_Pressed;






END Update_Buffers_Set_Pkg.Callbacks;
