------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Text_Io;
USE Text_Io;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;



WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;


WITH Event_Analyzer_Set;
USE Event_Analyzer_Set;
USE Event_Analyzer_Set.Generic_Event_Analyzer_Set;

WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;




PACKAGE BODY Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks IS



   PROCEDURE Check_Integrity_Constraints (
         Update_Event_Analyzers_Set : IN OUT Update_Event_Analyzers_Set_Access;
         Error_Message              : IN OUT Unbounded_String;
         Ok                         : IN OUT Boolean) IS


   BEGIN

      Ok:=True;

      Event_Analyzer_Name := To_Unbounded_String(Get_Text(
            Update_Event_Analyzers_Set.Entry_File_Name));



      Check_Event_Analyzer (Sys.Event_Analyzers, Integrity_Checks.
         Event_Analyzer_Name);


   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   END Check_Integrity_Constraints;




END Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks;

