------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Mmi_Tools;
USE Mmi_Tools;

WITH Project;
USE Project;

WITH Event_Analyzers;
USE Event_Analyzers;
WITH Event_Analyzer_Set;
USE Event_Analyzer_Set;
USE Event_Analyzer_Set.Generic_Event_Analyzer_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Glib;
USE Glib;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks;


WITH Generic_Select_File_Pkg;



PACKAGE BODY Update_Event_Analyzers_Set_Pkg.Callbacks IS





   PROCEDURE Show_Event_Analyzers (
         Update_Event_Analyzers_Set : ACCESS Update_Event_Analyzers_Set_Record'Class) IS

      An_Event_Analyzer : Event_Analyzer_Ptr;
      My_Iterator       : Event_Analyzers_Iterator;
      A_Line            : Chars_Ptr_Array (1 .. 2);
      Line              : Gint;

   BEGIN
      Clear(Update_Event_Analyzers_Set.Event_Analyzers_List);

      Reset_Iterator(Sys.Event_Analyzers, My_Iterator);
      IF NOT Is_Empty(Sys.Event_Analyzers) THEN
         LOOP

            Current_Element(Sys.Event_Analyzers, An_Event_Analyzer,
               My_Iterator);

            -- Create the line
            --
            A_Line := To_String(An_Event_Analyzer.Behavior_File_Name) + "";

            -- Display the Line
            --
            Line := Append(Update_Event_Analyzers_Set.
               Event_Analyzers_List, A_Line);


            EXIT WHEN Is_Last_Element(Sys.Event_Analyzers, My_Iterator);

            Next_Element(Sys.Event_Analyzers, My_Iterator);

         END LOOP;
      END IF;
   END Show_Event_Analyzers;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name : Unbounded_String;
      Row  : Gint;

   BEGIN

      Row:=Get_Focus_Row(Update_Event_Analyzers_Set.Event_Analyzers_List);
      Name := To_Unbounded_String(Get_Text(Update_Event_Analyzers_Set.
            Event_Analyzers_List,
            Row,0));

      Update_Event_Analyzers_Set.Current_Event_Analyzer := Name;

      Gtk.Gentry.Set_Text(Update_Event_Analyzers_Set.Entry_File_Name,
         To_String(Name));


   END On_Click_List;




   PROCEDURE Read_Sc_File (
         File_Name : IN     Unbounded_String) IS


   BEGIN

      Set_Text(Update_Event_Analyzers_Set.
         Entry_File_Name,
         To_String(File_Name) );

   END Read_Sc_File;


   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;

   PACKAGE Read_Sc_File_Package IS NEW Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   USE Read_Sc_File_Package;




   PROCEDURE On_Read_From_File_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

   BEGIN
      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Sc_File_Package.Generic_Select_File);
   END On_Read_From_File_Pressed;




   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Event_Analyzers_Set);
   END On_Close_Pressed;



   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Event_Analyzer_Set.Generic_Event_Analyzer_Set.Iterator;

      An_Event_Analyzer : Event_Analyzer_Ptr;


   BEGIN



      IF Update_Event_Analyzers_Set.Current_Event_Analyzer = "" THEN
         Show_Message_Box(Lb_Select_Event_Analyzer_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Event_Analyzers) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Event_Analyzers, My_Iterator);
      IF NOT Is_Empty(Sys.Event_Analyzers) THEN
         LOOP

            Current_Element(Sys.Event_Analyzers, An_Event_Analyzer,
               My_Iterator);
            IF An_Event_Analyzer.Behavior_File_Name =
                  Update_Event_Analyzers_Set.Current_Event_Analyzer THEN

               Delete(Sys.Event_Analyzers, An_Event_Analyzer);

               -- Clear entry
               --
               Gtk.Gentry.Set_Text(Update_Event_Analyzers_Set.
                  Entry_File_Name, "");
               Update_Event_Analyzers_Set.Current_Event_Analyzer:=
                  Empty_String;

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Event_Analyzers, My_Iterator);
            Next_Element(Sys.Event_Analyzers, My_Iterator);

         END LOOP;

      END IF;


      Show_Event_Analyzers(Update_Event_Analyzers_Set);


   END On_Delete_Pressed;


   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Event_Analyzers_Set.Undo,
         Sys);
      Destroy(Update_Event_Analyzers_Set);
   END On_Cancel_Pressed;




   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Event_Analyzers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      Add_Event_Analyzer(Sys.Event_Analyzers,
         Integrity_Checks.Event_Analyzer_Name);


      Show_Event_Analyzers(Update_Event_Analyzers_Set);



   EXCEPTION
      WHEN Event_Analyzer_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Add_Pressed;


   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Event_Analyzers_Set.Vbox_List) THEN
         Show_Advanced(Update_Event_Analyzers_Set,False);
      ELSE
         Show_Advanced(Update_Event_Analyzers_Set,True);
      END IF;

   END On_Advanced_Pressed;


   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      An_Event_Analyzer : Event_Analyzer_Ptr;
      My_Iterator       : Event_Analyzer_Set.Generic_Event_Analyzer_Set.Iterator;

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;


   BEGIN


      IF Update_Event_Analyzers_Set.Current_Event_Analyzer = "" THEN
         Show_Message_Box(Lb_Select_Event_Analyzer_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Event_Analyzers) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Event_Analyzers_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      --
      -- Update set
      --

      Reset_Iterator(Sys.Event_Analyzers, My_Iterator);
      LOOP

         Current_Element(Sys.Event_Analyzers, An_Event_Analyzer,
            My_Iterator);
         IF An_Event_Analyzer.Behavior_File_Name =
               Update_Event_Analyzers_Set.
               Current_Event_Analyzer THEN

            Delete(Sys.Event_Analyzers, An_Event_Analyzer);



            Add_Event_Analyzer(Sys.Event_Analyzers,
               Integrity_Checks.Event_Analyzer_Name);


            Update_Event_Analyzers_Set.Current_Event_Analyzer :=
               Integrity_Checks.Event_Analyzer_Name;
            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Event_Analyzers, My_Iterator);
         Next_Element(Sys.Event_Analyzers, My_Iterator);

      END LOOP;

      Show_Event_Analyzers(Update_Event_Analyzers_Set);


   EXCEPTION
      WHEN Event_Analyzer_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;



   END On_Modify_Pressed;




END Update_Event_Analyzers_Set_Pkg.Callbacks;
