------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Text_Io;
USE Text_Io;


WITH Gtk;
USE Gtk;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Update_Event_Analyzers_Set_Pkg.Callbacks;
USE Update_Event_Analyzers_Set_Pkg.Callbacks;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE  Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;
WITH Gtkada.Types;
USE Gtkada.Types;


WITH Project;
USE Project;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;



PACKAGE BODY Update_Event_Analyzers_Set_Pkg IS


   PROCEDURE Gtk_New (
         Update_Event_Analyzers_Set :    OUT Update_Event_Analyzers_Set_Access;
         Title                      :        Unbounded_String) IS
   BEGIN
      Update_Event_Analyzers_Set := NEW Update_Event_Analyzers_Set_Record;
      Update_Event_Analyzers_Set_Pkg.Initialize (
         Update_Event_Analyzers_Set, Title);
   END Gtk_New;


   PROCEDURE Show_Advanced (
         Update_Event_Analyzers_Set : IN     Update_Event_Analyzers_Set_Access;
         Advanced                   : IN     Boolean) IS

   BEGIN

      IF NOT Advanced THEN
         Hide(Update_Event_Analyzers_Set.Vbox_List);
         Hide(Update_Event_Analyzers_Set.Delete_Button);
         Hide(Update_Event_Analyzers_Set.Modify_Button);
         Show(Update_Event_Analyzers_Set.Add_Button);
         Show(Update_Event_Analyzers_Set.Close_Button);
         Show(Update_Event_Analyzers_Set.Cancel_Button);
         Show(Update_Event_Analyzers_Set.Advanced_Button);

         Show(Update_Event_Analyzers_Set.Vbox_Main);
         Show(Update_Event_Analyzers_Set.Hbox_Event);
         Show_All(Update_Event_Analyzers_Set.Vbox_Update);
         Show(Update_Event_Analyzers_Set.Hbutton_Box_Modify);
         Show(Update_Event_Analyzers_Set.
            Event_Analyzers_List);
         Show_All(Update_Event_Analyzers_Set.Frame_Event_Analyzer);

      ELSE
         Show_All(Update_Event_Analyzers_Set.Vbox_Main);
      END IF;
      Show(Update_Event_Analyzers_Set);


   END Show_Advanced;

   PROCEDURE Initialize (
         Update_Event_Analyzers_Set : ACCESS Update_Event_Analyzers_Set_Record'Class;
         Title                      :        Unbounded_String) IS



      List_Titles : CONSTANT Chars_Ptr_Array (1 .. 2) := To_String (Lb_File_Name
        (Current_Language)) + "";


   BEGIN
      Initialize(Update_Event_Analyzers_Set.Undo);
      Duplicate(Sys,Update_Event_Analyzers_Set.Undo);
      Update_Event_Analyzers_Set.Current_Event_Analyzer:=Empty_String;

      Gtk.Window.Initialize(Update_Event_Analyzers_Set, Window_Toplevel);
      Set_Policy (Update_Event_Analyzers_Set, True, True, True);
      Set_Title (Update_Event_Analyzers_Set, To_String(Title));
      Set_Position (Update_Event_Analyzers_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Event_Analyzers_Set, True);
      Grab_Focus(Update_Event_Analyzers_Set);



      Gtk.Box.Gtk_New_Vbox(Update_Event_Analyzers_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Event_Analyzers_Set.Vbox_Main, 10);
      Add(Update_Event_Analyzers_Set, Update_Event_Analyzers_Set.
         Vbox_Main);

      Gtk.Box.Gtk_New_Hbox(Update_Event_Analyzers_Set.Hbox_Event, False,
         10);
      Add(Update_Event_Analyzers_Set.Vbox_Main,
         Update_Event_Analyzers_Set.
         Hbox_Event);

      Gtk.Box.Gtk_New_Vbox(Update_Event_Analyzers_Set.Vbox_Update, False,
         10);
      Add(Update_Event_Analyzers_Set.Hbox_Event,
         Update_Event_Analyzers_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox(Update_Event_Analyzers_Set.Vbox_List, False, 10);
      Add(Update_Event_Analyzers_Set.Hbox_Event,
         Update_Event_Analyzers_Set.Vbox_List);

      Gtk.Frame.Gtk_New(Update_Event_Analyzers_Set.Frame_Event_Analyzer,
         "");
      Add(Update_Event_Analyzers_Set.Vbox_Update,
         Update_Event_Analyzers_Set.Frame_Event_Analyzer);

      Gtk.Table.Gtk_New (Update_Event_Analyzers_Set.Table,2, 3, True);
      Set_Border_Width (Update_Event_Analyzers_Set.Table, 5);
      Add(Update_Event_Analyzers_Set.Frame_Event_Analyzer,
         Update_Event_Analyzers_Set.Table);


      -- Button panel
      --
      Gtk_New(Update_Event_Analyzers_Set.Hbutton_Box_Modify);
      Set_Spacing(Update_Event_Analyzers_Set.Hbutton_Box_Modify, 10);
      Set_Layout(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size(Update_Event_Analyzers_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding(Update_Event_Analyzers_Set.Hbutton_Box_Modify, 7,
         0);
      Pack_Start(Update_Event_Analyzers_Set.Vbox_Update,
         Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         False, True, 5);


      Gtk_New (Update_Event_Analyzers_Set.Lab, To_String(
            Lb_File_Name(
               Current_Language) & Lb_Colon));
      Set_Alignment (Update_Event_Analyzers_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Event_Analyzers_Set.Lab, 0, 0);
      Set_Justify (Update_Event_Analyzers_Set.Lab, Justify_Left);
      Set_Line_Wrap (Update_Event_Analyzers_Set.Lab, False);
      Attach (Update_Event_Analyzers_Set.Table,
         Update_Event_Analyzers_Set.
         Lab,
         0, 1,
         0, 1,
         Fill, Expand OR Fill,
         0, 0);



      Gtk_New (Update_Event_Analyzers_Set.Read_From_File_Behavior,
         To_String(
            Lb_Read(Current_Language)));
      Attach (Update_Event_Analyzers_Set.Table,
         Update_Event_Analyzers_Set.
         Read_From_File_Behavior,
         1, 2,
         0, 1,
         Fill, Expand OR Fill,
         0, 0);
      Button_Callback.Connect
         (Update_Event_Analyzers_Set.Read_From_File_Behavior, "clicked",
         Button_Callback.To_Marshaller (On_Read_From_File_Pressed'ACCESS));


      Gtk_New (Update_Event_Analyzers_Set.Entry_File_Name);
      Set_Editable (Update_Event_Analyzers_Set.Entry_File_Name, True);
      Set_Max_Length (Update_Event_Analyzers_Set.Entry_File_Name, 0);
      Set_Text (Update_Event_Analyzers_Set.Entry_File_Name, "");
      Set_Visibility (Update_Event_Analyzers_Set.Entry_File_Name, True);
      Attach (Update_Event_Analyzers_Set.Table,
         Update_Event_Analyzers_Set.Entry_File_Name,
         2, 3,
         0, 1,
         Fill, Expand OR Fill,
         1, 3);


      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Close_Button,
         To_String(Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Cancel_Button,
         To_String(Lb_Cancel(Current_Language)));
      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Advanced_Button,
         To_String(Lb_Advanced(Current_Language)));
      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Delete_Button,
         To_String(Lb_Delete(Current_Language)));
      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Modify_Button,
         To_String(Lb_Modify(Current_Language)));
      Gtk.Button.Gtk_New(Update_Event_Analyzers_Set.Add_Button,
         To_String(Lb_Add(Current_Language)));

      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Close_Button);
      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Cancel_Button);
      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Advanced_Button);
      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Delete_Button);
      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Modify_Button);
      Add(Update_Event_Analyzers_Set.Hbutton_Box_Modify,
         Update_Event_Analyzers_Set.Add_Button);

      -- Spreadsheet for event_analyzer information

      Gtk.Scrolled_Window.Gtk_New(Update_Event_Analyzers_Set.Scroll_List, NULL, NULL);
      Add(Update_Event_Analyzers_Set.Vbox_List,
         Update_Event_Analyzers_Set.
         Scroll_List);

      Gtk.Clist.Gtk_New(Update_Event_Analyzers_Set.Event_Analyzers_List, 1,
         List_Titles);
      Set_Usize(Update_Event_Analyzers_Set.Event_Analyzers_List,300,100);
      Set_Sort_Column(Update_Event_Analyzers_Set.Event_Analyzers_List,0);
      Set_Sort_Type(Update_Event_Analyzers_Set.Event_Analyzers_List,
         Ascending);
      Set_Auto_Sort(Update_Event_Analyzers_Set.Event_Analyzers_List, True);
      Add(Update_Event_Analyzers_Set.Scroll_List,
         Update_Event_Analyzers_Set.Event_Analyzers_List);



      Show_Event_Analyzers(Update_Event_Analyzers_Set);





      -- Select task from spreadsheet activation
      --
      List_Callback.Connect (Update_Event_Analyzers_Set.
         Event_Analyzers_List,"select_row",
         List_Callback.To_Marshaller (On_Click_List'ACCESS));

      Button_Callback.Connect (Update_Event_Analyzers_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'ACCESS));
      Button_Callback.Connect (Update_Event_Analyzers_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));
      Button_Callback.Connect (Update_Event_Analyzers_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'ACCESS));
      Button_Callback.Connect (Update_Event_Analyzers_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));
      Button_Callback.Connect (Update_Event_Analyzers_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'ACCESS));
      Button_Callback.Connect (Update_Event_Analyzers_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'ACCESS));



   END Initialize;



END Update_Event_Analyzers_Set_Pkg;
