------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;
WITH Objects;
USE Objects;

WITH Translate;
USE Translate;

WITH Framework_Config;
USE Framework_Config;

WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;
WITH Graph_Item_Pkg;
USE Graph_Item_Pkg;
WITH Graph_Add_Pkg;
USE Graph_Add_Pkg;
WITH My_Canvas;
USE My_Canvas;

WITH Double_Util;
USE Double_Util;

WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;

WITH Update_Messages_Set_Pkg;
USE Update_Messages_Set_Pkg;


WITH Task_Dependencies;
USE Task_Dependencies;

WITH Systems;
USE Systems;


PACKAGE BODY Update_Messages_Set_Pkg.Callbacks  IS


   PROCEDURE Page_Switch (
         Notebook : ACCESS Gtk_Notebook_Record'Class;
         Page     : IN     Gtk.Gtk_Notebook_Page;
         Page_Num : IN     Gtk_Notebook) IS
      PRAGMA Warnings (Off, Page_Num);
      Old_Page : CONSTANT Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   :          Gtk_Pixmap;
   BEGIN
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Messages_Set.Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Messages_Set.Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);

      IF Old_Page /= NULL THEN
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Messages_Set.Book_Closed,
            Update_Messages_Set.Book_Closed_Mask);
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Messages_Set.Book_Closed,
            Update_Messages_Set.Book_Closed_Mask);
      END IF;
   END Page_Switch;



   PROCEDURE Show_Messages (
         A_Message_Name : Unbounded_String) IS
      A_Message          : Message_Ptr;
      My_Iterator        : Messages_Iterator;
      Line               : Gint;
      Message_Share      : Chars_Ptr_Array (1 .. 7);
      Jitter             : Unbounded_String;
      Deadline           : Unbounded_String;
      Period             : Unbounded_String;
      Size               : Unbounded_String;
      Response_Time      : Unbounded_String;
      Communication_Time : Unbounded_String;


   BEGIN
      Clear(Update_Messages_Set.Messages_List);

      Reset_Iterator(Sys.Messages, My_Iterator);
      IF NOT Is_Empty(Sys.Messages) THEN
         LOOP

            Current_Element(Sys.Messages, A_Message, My_Iterator);

            Deadline := To_Unbounded_String(
               Natural'Image(A_Message.Deadline));
            Size     := To_Unbounded_String(
               Natural'Image(A_Message.Size));
            Response_Time := To_Unbounded_String(
               Natural'Image(A_Message.Response_Time));
            Communication_Time := To_Unbounded_String(
               Natural'Image(A_Message.Communication_Time));

            CASE A_Message.Message_Type IS
               WHEN Aperiodic_Type =>
                  Jitter    := To_Unbounded_String(" 0");
                  Period    := To_Unbounded_String(" 0");
               WHEN Periodic_Type =>
                  Jitter    := To_Unbounded_String(Natural'Image(
                        Periodic_Message_Ptr(A_Message).Jitter));
                  Period    := To_Unbounded_String(Natural'Image(
                        Periodic_Message_Ptr(A_Message).Period));
               WHEN Generic_Type =>
                  Jitter    := To_Unbounded_String(" 0");
                  Period    := To_Unbounded_String(" 0");
            END CASE;

            -- Create the line
            --
            Message_Share := To_String(A_Message.Name)+
               To_String(Jitter)+To_String(Deadline)+To_String(Period)+
               To_String(Size)+To_String(Response_Time)+To_String(
               Communication_Time);

            -- Display the Line
            --
            Line := Append(Update_Messages_Set.Messages_List,
               Message_Share);

            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);

            Next_Element(Sys.Messages, My_Iterator);

         END LOOP;
      END IF;
   END Show_Messages;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name               : Unbounded_String;
      Jitter             : Unbounded_String;
      Deadline           : Unbounded_String;
      Period             : Unbounded_String;
      Size               : Unbounded_String;
      Response_Time      : Unbounded_String;
      Communication_Time : Unbounded_String;
      Row                : Gint;

      A_Message   : Message_Ptr;
      Current     : Gint;
      Message_Str : Unbounded_String;

      Parameter_Items : ARRAY (User_Defined_Parameters_Range) OF Strings_Table;



   BEGIN

      Row:=Get_Focus_Row(Update_Messages_Set.Messages_List);
      Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,
            Row,0));

      Update_Messages_Set.Current_Message := Name;

      Jitter        := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,1));
      Deadline      := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,2));
      Period        := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,3));
      Size    := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,4));
      Response_Time := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Messages_List,Row,5));
      Communication_Time := To_Unbounded_String(Get_Text(
            Update_Messages_Set.
            Messages_List,Row,6));

      Gtk.Gentry.Set_Text(Update_Messages_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Jitter, To_String(Jitter));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Deadline, To_String(
            Deadline));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Period, To_String(Period));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Size, To_String(Size));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Response_Time, To_String(
            Response_Time));
      Gtk.Gentry.Set_Text(Update_Messages_Set.Communication_Time,
         To_String(
            Communication_Time));

      Message_Str:=Update_Messages_Set.Current_Message;
      IF (Message_Str /= To_Unbounded_String("")) THEN
         A_Message:=Search_Message(Sys.Messages,Message_Str);
         Update_Messages_Set.Current_Message_Ptr := A_Message;

         Show_All(Update_Messages_Set);


         -- User defined parameters
         --

         FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP
            Set_Text(Update_Messages_Set.Param(I).Entry_Name, "");
            Set_Text(Update_Messages_Set.Param(I).Entry_Value, "");
         END LOOP;

         FOR I IN A_Message.Param.Nb_Entries ..
               User_Defined_Parameters_Range'
               Last-1 LOOP
            Initialize(Parameter_Items(I));
            Add(Parameter_Items(I),To_Unbounded_String("Integer"));
            Add(Parameter_Items(I),To_Unbounded_String("Double"));
            Add(Parameter_Items(I),To_Unbounded_String("String"));
            Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

            Destroy(Update_Messages_Set.Param(I).Parameter_Menu);
            Update_Messages_Set.Param(I).Clist_Parameter_Menu :=
               Widget_Slist.Null_List;
            Gtk.Option_Menu.Gtk_New(Update_Messages_Set.Param(I).
               Parameter_Menu);

            Build_Option_Menu (Update_Messages_Set.Param(I).
               Parameter_Menu,
               Update_Messages_Set.Param(I).Clist_Parameter_Menu,
               Parameter_Items(I),
               -1, NULL);
            Show_All(Update_Messages_Set.Param(I).Parameter_Menu);
            Attach (Update_Messages_Set.Parameter_Table,
               Update_Messages_Set.
               Param(I).Parameter_Menu, 1, 2,
               1+Guint(I), 2+Guint(I),
               Expand OR Fill, 0,
               0, 0);
         END LOOP;
         Show_All(Update_Messages_Set);


         FOR I IN 0..A_Message.Param.Nb_Entries-1 LOOP
            Set_Text(Update_Messages_Set.Param(I).Entry_Name, To_String(
                  A_Message.Param.Entries(I).Name));


            IF A_Message.Param.Entries(I).Ptype=Boolean_Parameter THEN
               Set_Text(Update_Messages_Set.Param(I).Entry_Value,
                  A_Message.
                  Param.Entries(I).Boolean_Value'Img);
               Current:=3;
            ELSE
               IF A_Message.Param.Entries(I).Ptype=Integer_Parameter THEN
                  Set_Text(Update_Messages_Set.Param(I).Entry_Value,
                     A_Message.
                     Param.Entries(I).Integer_Value'Img);
                  Current:=0;
               ELSE
                  IF A_Message.Param.Entries(I).Ptype=Double_Parameter THEN
                     Set_Text(Update_Messages_Set.Param(I).Entry_Value,
                        To_String(Format(A_Message.Param.Entries(I).
                              Double_Value)));
                     Current:=1;
                  ELSE
                     Set_Text(Update_Messages_Set.Param(I).Entry_Value,
                        To_String(A_Message.Param.Entries(I).String_Value));
                     Current:=2;
                  END IF;
               END IF;
            END IF;

            Initialize(Parameter_Items(I));
            Add(Parameter_Items(I),To_Unbounded_String("Integer"));
            Add(Parameter_Items(I),To_Unbounded_String("Double"));
            Add(Parameter_Items(I),To_Unbounded_String("String"));
            Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

            Destroy(Update_Messages_Set.Param(I).Parameter_Menu);
            Update_Messages_Set.Param(I).Clist_Parameter_Menu :=
               Widget_Slist.
               Null_List;
            Gtk.Option_Menu.Gtk_New(Update_Messages_Set.Param(I).
               Parameter_Menu);

            Build_Option_Menu (Update_Messages_Set.Param(I).
               Parameter_Menu,
               Update_Messages_Set.Param(I).Clist_Parameter_Menu,
               Parameter_Items(I),
               Current, NULL);
            Show_All(Update_Messages_Set.Param(I).Parameter_Menu);
            Attach (Update_Messages_Set.Parameter_Table,
               Update_Messages_Set.
               Param(I).Parameter_Menu, 1, 2,
               1+Guint(I), 2+Guint(I),
               Expand OR Fill, 0,
               0, 0);


         END LOOP;
         Show_All(Update_Messages_Set);

      END IF;


   END On_Click_List;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Messages_Set);
   END On_Close_Pressed;


   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok           : Boolean;
      Message_Name : Unbounded_String;

      Message_Jitter     : Integer;
      Message_Deadline   : Integer;
      Message_Period     : Integer;
      Message_Size       : Integer;
      Response_Time      : Integer;
      Communication_Time : Integer;

      Message_Size_String       : Unbounded_String;
      Message_Jitter_String     : Unbounded_String;
      Message_Deadline_String   : Unbounded_String;
      Message_Period_String     : Unbounded_String;
      Response_Time_String      : Unbounded_String;
      Communication_Time_String : Unbounded_String;

      A_Message : Message_Ptr;

      Message_Param : User_Defined_Parameters_Table;

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;

      Current      : Unbounded_String;
      A_Param      : Parameter_Ptr;
      A_Param_Type : Gint;

   BEGIN

      --
      -- First Message page : general information
      --

      Message_Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Name));


      IF Message_Name=""
            THEN
         Show_Message_Box(
            Lb_Message_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         RETURN;
      END IF;


      IF NOT Is_A_Valid_Identifier(Message_Name)
            THEN
         Show_Message_Box(Lb_Message_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         RETURN;
      END IF;



      Reset_Iterator(Sys.Messages, My_Iterator);
      IF NOT Is_Empty(Sys.Messages) THEN
         LOOP
            Current_Element(Sys.Messages, A_Message, My_Iterator);
            IF (Message_Name = A_Message.Name) THEN
               IF(A_Message.Name/=Update_Messages_Set.Current_Message) THEN
                  Show_Message_Box(Lb_Message_Name(Current_Language) &
                     Lb_Already_Defined(Current_Language));
                  RETURN;
               END IF;
            END IF;
            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);
            Next_Element(Sys.Messages, My_Iterator);
         END LOOP;
      END IF;


      Message_Jitter_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Jitter));
      Message_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Messages_Set.Deadline));
      Message_Period_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Period));
      Message_Size_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Size));
      Response_Time_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Response_Time));
      Communication_Time_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Communication_Time));

      To_Integer(Response_Time_String,Response_Time,Ok);
      IF NOT Ok
            THEN
         IF Response_Time_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Communication_Time_String,Communication_Time,Ok);
      IF NOT Ok
            THEN
         IF Communication_Time_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Jitter_String,Message_Jitter,Ok);
      IF NOT Ok
            THEN
         IF Message_Jitter_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Deadline_String,Message_Deadline,Ok);
      IF NOT Ok
            THEN
         IF Message_Deadline_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Size_String,Message_Size,Ok);
      IF NOT Ok
            THEN
         IF Message_Size_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Period_String,Message_Period,Ok);
      IF NOT Ok
            THEN
         IF Message_Period_String /= "" THEN
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      IF (Message_Size<=0)
            THEN
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         RETURN;
      END IF;

      IF (Response_Time<0)
            THEN
         Show_Message_Box(Lb_Response_Time_Message(Current_Language) &
            Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;


      IF (Communication_Time<0)
            THEN
         Show_Message_Box(Lb_Communication_Time_Message(Current_Language) &
            Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Jitter<0)
            THEN
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Period<0)
            THEN
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Deadline<0)
            THEN
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         RETURN;
      END IF;

      IF (Message_Period/=0 AND Message_Jitter>Message_Deadline)
            THEN
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         RETURN;
      END IF;

      IF (Message_Period/=0 AND Message_Jitter=0)
            THEN
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Mandatory(
               Current_Language));
         RETURN;
      END IF;



      --
      --- Second page of the widget : user defined parameters
      --

      Initialize(Message_Param);

      FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP

         Current:=To_Unbounded_String(Get_Text(Update_Messages_Set.
               Param(I).Entry_Name));


         IF Current /= Empty_String THEN


            A_Param_Type:=  Gint(Selected_Button (Update_Messages_Set.
                  Param(I).Clist_Parameter_Menu));


            IF A_Param_Type = 3
                  THEN
               A_Param:= NEW Parameter(Integer_Parameter);
            ELSE
               IF A_Param_Type = 2
                     THEN
                  A_Param:= NEW Parameter(Double_Parameter);
               ELSE
                  IF A_Param_Type = 1
                        THEN
                     A_Param:= NEW Parameter(String_Parameter);
                  ELSE
                     A_Param:= NEW Parameter(Boolean_Parameter);
                  END IF;
               END IF;
            END IF;


            A_Param.Name:=Current;

            Current:=To_Unbounded_String(Get_Text(Update_Messages_Set.
                  Param(I).Entry_Value));

            -- Integer case
            --
            IF A_Param_Type=3 THEN
               IF(Current="")
                     THEN
                  A_Param.Integer_Value:=0;
               ELSE
                  To_Integer(Current,A_Param.Integer_Value,Ok);
                  IF NOT Ok
                        THEN
                     Show_Message_Box(To_Unbounded_String("Parameter " &
                           Integer'Image(Integer(I+1)) & " ") &
                        Lb_Must_Be(Current_Language) & To_Unbounded_String(
                           " integer"));
                     RETURN;
                  END IF;
               END IF;

               -- Double case
               --
            ELSE
               IF A_Param_Type=2 THEN
                  IF(Current="")
                        THEN
                     A_Param.Double_Value:=0.0;
                  ELSE
                     To_Double(Current,A_Param.Double_Value,Ok);
                     IF NOT Ok
                           THEN
                        Show_Message_Box(To_Unbounded_String("Parameter " &
                              Integer'Image(Integer(I+1)) & " ") &
                           Lb_Must_Be(Current_Language) &
                           To_Unbounded_String(" double"));
                        RETURN;
                     END IF;
                  END IF;

                  -- String case
                  --
               ELSE
                  IF A_Param_Type=1 THEN
                     A_Param.String_Value:=Current;

                     -- Boolean case
                     --
                  ELSE
                     IF(Current="")
                           THEN
                        A_Param.Boolean_Value:=True;
                     ELSE
                        To_Boolean(Current,A_Param.Boolean_Value,Ok);
                        IF NOT Ok
                              THEN
                           Show_Message_Box(To_Unbounded_String(
                                 "Parameter " &
                                 Integer'Image(Integer(I+1)) & " ") &
                              Lb_Must_Be(Current_Language) &
                              To_Unbounded_String(" boolean"));
                           RETURN;
                        END IF;
                     END IF;

                  END IF;
               END IF;
            END IF;


            Add(Message_Param, A_Param);
         END IF;

      END LOOP;


      --
      -- Update Message set
      --

      Add_Message (Sys.Messages, Message_Name,
         Natural(Message_Size),Natural(Message_Period),
         Natural(Message_Deadline),Natural(Message_Jitter),Message_Param,
         Natural(Response_Time),Natural(Communication_Time));

      Update_Messages_Set.Current_Message := Message_Name;

      Reset_Iterator(Sys.Messages, My_Iterator);
      Current_Element(Sys.Messages, A_Message, My_Iterator);
      Show_Messages(A_Message.Name);

      IF Add_New_Item /= NULL THEN
         Set_Name(Items_List(Last_Item-1),Message_Name);
         Refresh_Canvas(Graph_Window.Buffer);
      END IF;


   EXCEPTION
      WHEN Message_Already_Defined =>
         BEGIN
            Show_Message_Box(Lb_Message_Name(Current_Language) &
               Lb_Already_Defined(Current_Language) );
            RETURN;
         END;
      WHEN Message_Set.Generic_Message_Set.Full_Set =>
         BEGIN
            Show_Message_Box(Lb_Can_Not_Define_More_Messages(
                  Current_Language));
            RETURN;
         END;

   END On_Add_Pressed;


   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;
      A_Message   : Message_Ptr;

   BEGIN

      Reset_Iterator(Sys.Messages, My_Iterator);
      IF NOT Is_Empty(Sys.Messages) THEN
         LOOP

            Current_Element(Sys.Messages, A_Message, My_Iterator);
            IF A_Message.Name = Update_Messages_Set.Current_Message THEN

               Delete_Message(Sys, A_Message);
               Rebuild_Canvas(Graph_Window);

               -- Clear main entrys
               --
               Update_Messages_Set.Current_Message := To_Unbounded_String(
                  "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Jitter, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Deadline, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Period, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Size, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Response_Time, "");
               Gtk.Gentry.Set_Text(Update_Messages_Set.Communication_Time,
                  "");


               -- Clear User defined parameters
               --
               FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP
                  Gtk.Gentry.Set_Text(Update_Messages_Set.Param(I).
                     Entry_Name, "");
                  Gtk.Gentry.Set_Text(Update_Messages_Set.Param(I).
                     Entry_Value, "");
               END LOOP;

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);
            Next_Element(Sys.Messages, My_Iterator);

         END LOOP;
         Show_Messages(A_Message.Name);
      END IF;

   END On_Delete_Pressed;


   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Messages_Set.Undo, Sys);
      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Messages_Set);
   END On_Cancel_Pressed;

   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Messages_Set.Vbox_List) THEN
         Show_Advanced(Update_Messages_Set,False);
         NULL;
      ELSE
         Show_Advanced(Update_Messages_Set,True);
         NULL;
      END IF;

   END On_Advanced_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok           : Boolean;
      Message_Name : Unbounded_String;

      Message_Jitter     : Integer;
      Message_Deadline   : Integer;
      Message_Period     : Integer;
      Message_Size       : Integer;
      Response_Time      : Integer;
      Communication_Time : Integer;

      Message_Param : User_Defined_Parameters_Table;

      Message_Jitter_String     : Unbounded_String;
      Message_Deadline_String   : Unbounded_String;
      Message_Period_String     : Unbounded_String;
      Message_Size_String       : Unbounded_String;
      Response_Time_String      : Unbounded_String;
      Communication_Time_String : Unbounded_String;

      A_Message       : Message_Ptr;
      Created_Message : Message_Ptr;

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;

      Current : Unbounded_String;

      A_Param      : Parameter_Ptr;
      A_Param_Type : Gint;

   BEGIN

      -- First Message page : general information
      --

      Message_Name := To_Unbounded_String(Get_Text(Update_Messages_Set.
            Name));
      IF Update_Messages_Set.Current_Message = "" THEN
         Show_Message_Box(Lb_Select_Task_First(Current_Language));
         RETURN;
      END IF;

      Reset_Iterator(Sys.Messages, My_Iterator);
      IF NOT Is_Empty(Sys.Messages) THEN
         LOOP
            Current_Element(Sys.Messages, A_Message, My_Iterator);
            IF (Message_Name = A_Message.Name) THEN
               IF(A_Message.Name/=Update_Messages_Set.Current_Message) THEN
                  Show_Message_Box(Lb_Message_Name(Current_Language) &
                     Lb_Already_Defined(Current_Language));
                  RETURN;
               END IF;
            END IF;
            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);
            Next_Element(Sys.Messages, My_Iterator);
         END LOOP;
      ELSE
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;

      Message_Jitter_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Jitter));
      Message_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Messages_Set.Deadline));
      Message_Period_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Period));
      Message_Size_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Size));
      Response_Time_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Response_Time));
      Communication_Time_String := To_Unbounded_String(Get_Text(
            Update_Messages_Set.Communication_Time));


      To_Integer(Response_Time_String,Response_Time, Ok);
      IF NOT Ok
            THEN
         IF Response_Time_String /= "" THEN
            Show_Message_Box(Lb_Response_Time_Message(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            RETURN;
         END IF;
      END IF;


      To_Integer(Communication_Time_String,Communication_Time, Ok);
      IF NOT Ok
            THEN
         IF Communication_Time_String /= "" THEN
            Show_Message_Box(Lb_Communication_Time_Message(
                  Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Jitter_String,Message_Jitter, Ok);
      IF NOT Ok
            THEN
         IF Message_Jitter_String /= "" THEN
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Deadline_String,Message_Deadline, Ok);
      IF NOT Ok
            THEN
         IF Message_Deadline_String /= "" THEN
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Period_String,Message_Period, Ok);
      IF NOT Ok
            THEN
         IF Message_Period_String /= "" THEN
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            RETURN;
         END IF;
      END IF;

      To_Integer(Message_Size_String,Message_Size, Ok);
      IF NOT Ok
            THEN
         IF Message_Size_String /= "" THEN
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(
                  Current_Language));
            RETURN;
         END IF;
      END IF;

      IF (Message_Size<0)
            THEN
         Show_Message_Box(Lb_Size(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         RETURN;
      END IF;


      IF (Response_Time<0)
            THEN
         Show_Message_Box(Lb_Response_Time_Message(Current_Language) &
            Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;


      IF (Communication_Time<0)
            THEN
         Show_Message_Box(Lb_Communication_Time_Message(Current_Language) &
            Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Jitter<0)
            THEN
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Period<0)
            THEN
         Show_Message_Box(Lb_Period(Current_Language) & Lb_Must_Be(
               Current_Language) & "=>0");
         RETURN;
      END IF;

      IF (Message_Deadline<0)
            THEN
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & ">0");
         RETURN;
      END IF;

      IF (Message_Period/=0 AND Message_Jitter>Message_Deadline)
            THEN
         Show_Message_Box(Lb_Deadline(Current_Language) & Lb_Must_Be(
               Current_Language) & Lb_Superior(Current_Language) &
            Lb_Jitter(Current_Language));
         RETURN;
      END IF;

      IF (Message_Period/=0 AND Message_Jitter=0)
            THEN
         Show_Message_Box(Lb_Jitter(Current_Language) & Lb_Mandatory(
               Current_Language));
         RETURN;
      END IF;


      --
      --- Second page of the widget : user defined parameters
      --

      Initialize(Message_Param);

      FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP

         Current:=To_Unbounded_String(Get_Text(Update_Messages_Set.
               Param(I).Entry_Name));


         IF Current /= Empty_String THEN


            A_Param_Type:=  Gint(Selected_Button (Update_Messages_Set.
                  Param(I).Clist_Parameter_Menu));


            IF A_Param_Type = 3
                  THEN
               A_Param:= NEW Parameter(Integer_Parameter);
            ELSE
               IF A_Param_Type = 2
                     THEN
                  A_Param:= NEW Parameter(Double_Parameter);
               ELSE
                  IF A_Param_Type = 1
                        THEN
                     A_Param:= NEW Parameter(String_Parameter);
                  ELSE
                     A_Param:= NEW Parameter(Boolean_Parameter);
                  END IF;
               END IF;
            END IF;


            A_Param.Name:=Current;

            Current:=To_Unbounded_String(Get_Text(Update_Messages_Set.
                  Param(I).Entry_Value));

            -- Integer case
            --
            IF A_Param_Type=3 THEN
               IF(Current="")
                     THEN
                  A_Param.Integer_Value:=0;
               ELSE
                  To_Integer(Current,A_Param.Integer_Value,Ok);
                  IF NOT Ok
                        THEN
                     Show_Message_Box(To_Unbounded_String("Parameter " &
                           Integer'Image(Integer(I+1)) & " ") &
                        Lb_Must_Be(Current_Language) & To_Unbounded_String(
                           " integer"));
                     RETURN;
                  END IF;
               END IF;

               -- Double case
               --
            ELSE
               IF A_Param_Type=2 THEN
                  IF(Current="")
                        THEN
                     A_Param.Double_Value:=0.0;
                  ELSE
                     To_Double(Current,A_Param.Double_Value,Ok);
                     IF NOT Ok
                           THEN
                        Show_Message_Box(To_Unbounded_String("Parameter " &
                              Integer'Image(Integer(I+1)) & " ") &
                           Lb_Must_Be(Current_Language) &
                           To_Unbounded_String(" double"));
                        RETURN;
                     END IF;
                  END IF;

                  -- String case
                  --
               ELSE
                  IF A_Param_Type=1 THEN
                     A_Param.String_Value:=Current;

                     -- Boolean case
                     --
                  ELSE
                     IF(Current="")
                           THEN
                        A_Param.Boolean_Value:=True;
                     ELSE
                        To_Boolean(Current,A_Param.Boolean_Value,Ok);
                        IF NOT Ok
                              THEN
                           Show_Message_Box(To_Unbounded_String(
                                 "Parameter " &
                                 Integer'Image(Integer(I+1)) & " ") &
                              Lb_Must_Be(Current_Language) &
                              To_Unbounded_String(" boolean"));
                           RETURN;
                        END IF;
                     END IF;

                  END IF;
               END IF;
            END IF;


            Add(Message_Param, A_Param);
         END IF;

      END LOOP;


      --
      -- Update Message set
      --

      Reset_Iterator(Sys.Messages, My_Iterator);
      LOOP

         Current_Element(Sys.Messages, A_Message, My_Iterator);
         IF A_Message.Name = Update_Messages_Set.Current_Message THEN

            Delete(Sys.Messages, A_Message);

            Add_Message (Sys.Messages, Created_Message, Message_Name,
               Natural(Message_Size),
               Natural(Message_Period), Natural(Message_Deadline),
               Natural(Message_Jitter),Message_Param,Natural(
                  Response_Time), Natural(Communication_Time));

            Update_All_Task_Dependencies(Sys.Dependencies, Created_Message);


            Update_Messages_Set.Current_Message := Message_Name;
            EXIT;
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);
         Next_Element(Sys.Messages, My_Iterator);

      END LOOP;

      Show_Messages(A_Message.Name);

   END On_Modify_Pressed;

END Update_Messages_Set_Pkg.Callbacks;














