------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Glib;
USE Glib;
WITH Gtk;
USE Gtk;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gdk.Color;
USE Gdk.Color;

WITH Mmi_Tools;
USE Mmi_Tools;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE  Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Xpm;
USE Xpm;

WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Messages;
USE Messages;
WITH Message_Set;
USE Message_Set;
USE Message_Set.Generic_Message_Set;
WITH Update_Messages_Set_Pkg.Callbacks;
USE Update_Messages_Set_Pkg.Callbacks;
WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;


PACKAGE BODY Update_Messages_Set_Pkg IS

   PROCEDURE Gtk_New (
         Update_Messages_Set :    OUT Update_Messages_Set_Access;
         From                : IN     Unbounded_String) IS

   BEGIN
      Update_Messages_Set := NEW Update_Messages_Set_Record;
      Update_Messages_Set_Pkg.Initialize(Update_Messages_Set);
      Update_Messages_Set.Parent := From;

   END Gtk_New;

   PROCEDURE Show_Advanced (
         Update_Messages_Set : IN     Update_Messages_Set_Access;
         Advanced            : IN     Boolean) IS

   BEGIN

      IF NOT Advanced THEN
         Hide(Update_Messages_Set.Vbox_List);
         Hide(Update_Messages_Set.Delete_Button);
         Hide(Update_Messages_Set.Modify_Button);
         Show(Update_Messages_Set.Add_Button);
         Show(Update_Messages_Set.Close_Button);
         Show(Update_Messages_Set.Cancel_Button);
         Show(Update_Messages_Set.Advanced_Button);

         Show(Update_Messages_Set.Vbox_Main);
         Show(Update_Messages_Set.Hbox_Message);
         Show(Update_Messages_Set.Vbox_Update);
         Show(Update_Messages_Set.Hbutton_Box_Modify);
         Show_All(Update_Messages_Set.Frame_Message);

      ELSE
         Show_All(Update_Messages_Set.Vbox_Main);
      END IF;
      Show(Update_Messages_Set);


   END Show_Advanced;

   PROCEDURE Initialize (
         Update_Messages_Set : ACCESS Update_Messages_Set_Record'Class) IS


      A_Message       : Message_Ptr;
      My_Iterator     : Messages_Iterator;
      Parameter_Items : Strings_Table;

      Jitter             : Unbounded_String;
      Deadline           : Unbounded_String;
      Period             : Unbounded_String;
      Size               : Unbounded_String;
      Response_Time      : Unbounded_String;
      Communication_Time : Unbounded_String;
      A_Line             : Chars_Ptr_Array (1 .. 7);
      Line               : Gint;
      A_Periodic_Message : Periodic_Message_Ptr;

   BEGIN

      Initialize(Update_Messages_Set.Undo);
      Duplicate(Sys,Update_Messages_Set.Undo);

      Gtk.Window.Initialize(Update_Messages_Set, Window_Toplevel);
      Set_Policy (Update_Messages_Set, True, True, True);
      Set_Title (Update_Messages_Set, To_String(Lb_Update_Message(
               Current_Language)));
      Set_Position (Update_Messages_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Messages_Set, True);
      Grab_Focus(Update_Messages_Set);

      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Messages_Set.Vbox_Main, 10);
      Add(Update_Messages_Set, Update_Messages_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox(Update_Messages_Set.Hbox_Message, False, 10);
      Add(Update_Messages_Set.Vbox_Main, Update_Messages_Set.Hbox_Message);

      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_Update, False, 10);
      Add(Update_Messages_Set.Hbox_Message, Update_Messages_Set.
         Vbox_Update);

      Gtk.Box.Gtk_New_Vbox(Update_Messages_Set.Vbox_List, False, 10);
      Add(Update_Messages_Set.Hbox_Message, Update_Messages_Set.Vbox_List);

      Gtk.Frame.Gtk_New(Update_Messages_Set.Frame_Message,"");
      Add(Update_Messages_Set.Vbox_Update, Update_Messages_Set.
         Frame_Message);


      -- Button panel
      --
      Gtk_New(Update_Messages_Set.Hbutton_Box_Modify);
      Set_Spacing(Update_Messages_Set.Hbutton_Box_Modify, 10);
      Set_Layout(Update_Messages_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size(Update_Messages_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding(Update_Messages_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start(Update_Messages_Set.Vbox_Update,
         Update_Messages_Set.Hbutton_Box_Modify,
         False, True, 5);


      -- Create Notebook
      --
      Gtk_New (Update_Messages_Set.Notebook);
      Set_Tab_Pos (Update_Messages_Set.Notebook, Pos_Top);
      Add(Update_Messages_Set.Frame_Message, Update_Messages_Set.Notebook);
      Set_Border_Width (Update_Messages_Set.Notebook, 10);
      Realize (Update_Messages_Set.Notebook);


      Create_From_Xpm_D (Update_Messages_Set.Book_Opened,
         Get_Window (Update_Messages_Set.Notebook),
         Update_Messages_Set.Book_Opened_Mask, Null_Color,
         Book_Opened_Xpm);
      Create_From_Xpm_D (Update_Messages_Set.Book_Closed,
         Get_Window (Update_Messages_Set.Notebook),
         Update_Messages_Set.Book_Closed_Mask, Null_Color,
         Book_Closed_Xpm);

      -- Create Notebook pages
      --


      -- Main page
      --
      Gtk_New (Update_Messages_Set.Child_Page, To_String(Lb_Main(
               Current_Language)));
      Gtk.Table.Gtk_New (Update_Messages_Set.Message_Table,2, 5, False);
      Set_Border_Width (Update_Messages_Set.Message_Table, 5);

      Add(Update_Messages_Set.Child_Page, Update_Messages_Set.
         Message_Table);

      Gtk_New_Hbox (Update_Messages_Set.Label_Box, False, 0);
      Gtk_New (Update_Messages_Set.Pixmap, Update_Messages_Set.
         Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);
      Pack_Start (Update_Messages_Set.Label_Box, Update_Messages_Set.
         Pixmap,
         False, True, 0);
      Set_Padding (Update_Messages_Set.Pixmap, 3, 1);

      Gtk_New (Update_Messages_Set.Lab, To_String(Lb_Main(
               Current_Language)));
      Pack_Start (Update_Messages_Set.Label_Box, Update_Messages_Set.Lab,
         False,
         True, 0);
      Show_All (Update_Messages_Set.Label_Box);

      Gtk_New_Vbox (Update_Messages_Set.Menu_Box, False, 0);
      Gtk_New (Update_Messages_Set.Pixmap, Update_Messages_Set.
         Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);
      Pack_Start (Update_Messages_Set.Menu_Box, Update_Messages_Set.
         Pixmap,
         False, True, 0);
      Set_Padding (Update_Messages_Set.Pixmap, 3, 1);

      Gtk_New (Update_Messages_Set.Lab, To_String(Lb_Main(
               Current_Language)));
      Pack_Start (Update_Messages_Set.Menu_Box, Update_Messages_Set.Lab,
         False,
         True, 0);
      Show_All (Update_Messages_Set.Menu_Box);

      Append_Page_Menu (Update_Messages_Set.Notebook, Update_Messages_Set.
         Child_Page, Update_Messages_Set.Label_Box, Update_Messages_Set.
         Menu_Box);



      -- User defined parameter page
      --

      Gtk_New (Update_Messages_Set.Child_Page,  To_String(
            Lb_User_Defined_Parameter(Current_Language)));
      Gtk.Table.Gtk_New (Update_Messages_Set.Parameter_Table,2, 6, False);
      Set_Border_Width (Update_Messages_Set.Parameter_Table, 5);
      Add(Update_Messages_Set.Child_Page, Update_Messages_Set.
         Parameter_Table);
      Gtk_New_Hbox (Update_Messages_Set.Label_Box, False, 0);
      Gtk_New (Update_Messages_Set.Pixmap, Update_Messages_Set.
         Book_Closed,
         Update_Messages_Set.Book_Closed_Mask);
      Pack_Start (Update_Messages_Set.Label_Box, Update_Messages_Set.
         Pixmap,
         False, True, 0);
      Set_Padding (Update_Messages_Set.Pixmap, 3, 1);

      Gtk_New (Update_Messages_Set.Lab,  To_String(
            Lb_User_Defined_Parameter(
               Current_Language)));
      Pack_Start (Update_Messages_Set.Label_Box, Update_Messages_Set.Lab,
         False,
         True, 0);
      Show_All (Update_Messages_Set.Label_Box);

      Gtk_New_Vbox (Update_Messages_Set.Menu_Box, False, 0);
      Gtk_New (Update_Messages_Set.Pixmap, Update_Messages_Set.
         Book_Closed,
         Update_Messages_Set.Book_Closed_Mask);
      Pack_Start (Update_Messages_Set.Menu_Box, Update_Messages_Set.
         Pixmap,
         False, True, 0);
      Set_Padding (Update_Messages_Set.Pixmap, 3, 1);

      Gtk_New (Update_Messages_Set.Lab,  To_String(
            Lb_User_Defined_Parameter(
               Current_Language)));
      Pack_Start (Update_Messages_Set.Menu_Box, Update_Messages_Set.Lab,
         False,
         True, 0);
      Show_All (Update_Messages_Set.Menu_Box);

      Append_Page_Menu (Update_Messages_Set.Notebook, Update_Messages_Set.
         Child_Page, Update_Messages_Set.Label_Box, Update_Messages_Set.
         Menu_Box);


      -- Frame listings

      -- Begin Main page listing
      --

      Gtk.Label.Gtk_New(Update_Messages_Set.Name_Label, To_String(Lb_Name(
               Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Name_Label, 0, 1, 0, 1);

      Gtk.Label.Gtk_New(Update_Messages_Set.Jitter_Label, To_String(
            Lb_Jitter(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Jitter_Label, 0, 1, 1, 2);

      Gtk.Label.Gtk_New(Update_Messages_Set.Deadline_Label, To_String(
            Lb_Deadline(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Deadline_Label, 0, 1, 2, 3);

      Gtk.Label.Gtk_New(Update_Messages_Set.Period_Label,To_String(
            Lb_Period(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Period_Label, 0, 1, 3, 4);

      Gtk.Label.Gtk_New(Update_Messages_Set.Size_Label,To_String(Lb_Size(
               Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Size_Label, 0, 1, 4, 5);


      Gtk.Label.Gtk_New(Update_Messages_Set.Response_Time_Label,
         "Response Time" & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Response_Time_Label, 0, 1, 5, 6);


      Gtk.Label.Gtk_New(Update_Messages_Set.Communication_Time_Label,
         "Communication Time" & To_String(Lb_Colon));
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Communication_Time_Label, 0, 1, 6, 7);




      Gtk.Gentry.Gtk_New(Update_Messages_Set.Name, 50);
      Set_Editable (Update_Messages_Set.Name, True);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Name,1, 2, 0, 1);

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Jitter, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Jitter, 1, 2, 1, 2);

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Deadline, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Deadline, 1, 2, 2, 3);

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Period, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Period, 1, 2, 3, 4);


      Gtk.Gentry.Gtk_New(Update_Messages_Set.Size, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Size, 1, 2, 4, 5);

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Response_Time, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Response_Time, 1, 2, 5, 6);

      Gtk.Gentry.Gtk_New(Update_Messages_Set.Communication_Time, 50);
      Attach_Defaults(Update_Messages_Set.Message_Table,
         Update_Messages_Set.
         Communication_Time, 1, 2, 6, 7);

      -- User defined parameters
      --

      Show_All(Update_Messages_Set.Child_Page);


      Gtk_New (Update_Messages_Set.Lab, To_String(Lb_Name(
               Current_Language) & Lb_Colon));
      Set_Alignment (Update_Messages_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Messages_Set.Lab, 0, 0);
      Set_Justify (Update_Messages_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Messages_Set.Lab, False);
      Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
         Lab, 0, 1,
         0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Messages_Set.Lab, To_String(Lb_Role_Type(
               Current_Language) & Lb_Colon));
      Set_Alignment (Update_Messages_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Messages_Set.Lab, 0, 0);
      Set_Justify (Update_Messages_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Messages_Set.Lab, False);
      Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
         Lab, 1, 2,
         0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Messages_Set.Lab, To_String(Lb_Value(
               Current_Language) & Lb_Colon));
      Set_Alignment (Update_Messages_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Messages_Set.Lab, 0, 0);
      Set_Justify (Update_Messages_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Messages_Set.Lab, False);
      Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
         Lab, 2, 3,
         0,
         1,
         0, 0,
         0, 0);


      FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP


         Gtk_New (Update_Messages_Set.Param(I).Entry_Name);
         Set_Editable (Update_Messages_Set.Param(I).Entry_Name, True);
         Set_Max_Length (Update_Messages_Set.Param(I).Entry_Name, 0);
         Set_Text (Update_Messages_Set.Param(I).Entry_Name, "");
         Set_Visibility (Update_Messages_Set.Param(I).Entry_Name, True);
         Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
            Param(
               I).
            Entry_Name, 0,
            1,  1+Guint(I), 2+Guint(I),
            Expand OR Fill, 0,
            0, 0);


         Initialize(Parameter_Items);
         Add(Parameter_Items,To_Unbounded_String("Integer"));
         Add(Parameter_Items,To_Unbounded_String("Double"));
         Add(Parameter_Items,To_Unbounded_String("String"));
         Add(Parameter_Items,To_Unbounded_String("Boolean"));
         Update_Messages_Set.Param(I).Clist_Parameter_Menu:= Widget_Slist.
            Null_List;
         Build_Option_Menu (Update_Messages_Set.Param(I).Parameter_Menu,
            Update_Messages_Set.Param(I).Clist_Parameter_Menu,
            Parameter_Items, -1, NULL);
         Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
            Param(
               I).
            Parameter_Menu, 1, 2,
            1+Guint(I), 2+Guint(I),
            Expand OR Fill, 0,
            0, 0);


         Gtk_New (Update_Messages_Set.Param(I).Entry_Value);
         Set_Editable (Update_Messages_Set.Param(I).Entry_Value, True);
         Set_Max_Length (Update_Messages_Set.Param(I).Entry_Value, 0);
         Set_Text (Update_Messages_Set.Param(I).Entry_Value, "");
         Set_Visibility (Update_Messages_Set.Param(I).Entry_Value, True);
         Attach (Update_Messages_Set.Parameter_Table, Update_Messages_Set.
            Param(
               I).
            Entry_Value, 2,
            3,  1+Guint(I), 2+Guint(I),
            Expand OR Fill, 0,
            0, 0);

      END LOOP;


      -- End of Notebook


      Gtk.Button.Gtk_New(Update_Messages_Set.Close_Button,
         To_String(Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Cancel_Button,
         To_String(Lb_Cancel(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Advanced_Button,
         To_String(Lb_Advanced(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Delete_Button,
         To_String(Lb_Delete(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Modify_Button,
         To_String(Lb_Modify(Current_Language)));
      Gtk.Button.Gtk_New(Update_Messages_Set.Add_Button,
         To_String(Lb_Add(Current_Language)));

      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Close_Button);
      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Cancel_Button);
      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Advanced_Button);
      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Delete_Button);
      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Modify_Button);
      Add(Update_Messages_Set.Hbutton_Box_Modify,
         Update_Messages_Set.Add_Button);

      -- Spreadsheet for Message information
      --
      Gtk.Scrolled_Window.Gtk_New(Update_Messages_Set.Scroll_List, NULL, NULL);
      Add(Update_Messages_Set.Vbox_List,Update_Messages_Set.Scroll_List);

      Update_Messages_Set.List_Titles:= To_String(Lb_Name(
            Current_Language))
         +To_String(Lb_Jitter(Current_Language))
         +To_String(Lb_Deadline(Current_Language))
         +To_String(Lb_Period(Current_Language))
         +To_String(Lb_Size(Current_Language))
         +To_String(Lb_Response_Time_Message(Current_Language))
         +To_String(Lb_Communication_Time_Message(Current_Language));

      Gtk.Clist.Gtk_New(Update_Messages_Set.Messages_List,7,
         Update_Messages_Set.
         List_Titles);
      Set_Usize(Update_Messages_Set.Messages_List,475,200);
      Set_Sort_Column(Update_Messages_Set.Messages_List,0);
      Set_Sort_Type(Update_Messages_Set.Messages_List,Ascending);
      Set_Auto_Sort(Update_Messages_Set.Messages_List, True);
      Add(Update_Messages_Set.Scroll_List,Update_Messages_Set.
         Messages_List);


      Reset_Iterator(Sys.Messages, My_Iterator);

      IF NOT Is_Empty(Sys.Messages) THEN
         LOOP

            Current_Element(Sys.Messages, A_Message, My_Iterator);

            Deadline := To_Unbounded_String(Natural'Image(A_Message.
                  Deadline));
            Size    := To_Unbounded_String(Natural'Image(A_Message.Size));
            Response_Time := To_Unbounded_String(Natural'Image(A_Message.
                  Response_Time));
            Communication_Time := To_Unbounded_String(Natural'Image(
                  A_Message.
                  Communication_Time));


            CASE A_Message.Message_Type IS
               WHEN Periodic_Type =>
                  A_Periodic_Message := Periodic_Message_Ptr(A_Message);
                  Period := To_Unbounded_String(Natural'Image(
                        A_Periodic_Message.Period));
                  Jitter := To_Unbounded_String(Natural'Image(
                        A_Periodic_Message.Jitter));
               WHEN Aperiodic_Type =>
                  Period := To_Unbounded_String("");
                  Jitter := To_Unbounded_String("");
               WHEN Generic_Type =>
                  Period := To_Unbounded_String("");
                  Jitter := To_Unbounded_String("");
            END CASE;


            -- Create the line
            --
            A_Line := To_String(A_Message.Name)+To_String(Jitter)+
               To_String(Deadline)+To_String(Period)+To_String(Size)+
               To_String(Response_Time)+To_String(Communication_Time);

            -- Display the Line
            --
            Line := Append(Update_Messages_Set.Messages_List, A_Line);

            EXIT WHEN Is_Last_Element(Sys.Messages, My_Iterator);

            Next_Element(Sys.Messages, My_Iterator);

         END LOOP;
      END IF;

      -- Notebook page switching activation
      --
      Two_Notebook_Callback.Connect
         (Update_Messages_Set.Notebook, "switch_page",
         Two_Notebook_Callback.To_Marshaller (Page_Switch'ACCESS),
         Update_Messages_Set.Notebook);

      -- Select Message from spreadsheet activation
      --
      List_Callback.Connect (Update_Messages_Set.Messages_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'ACCESS));

      Button_Callback.Connect (Update_Messages_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'ACCESS));
      Button_Callback.Connect (Update_Messages_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));
      Button_Callback.Connect (Update_Messages_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'ACCESS));
      Button_Callback.Connect (Update_Messages_Set.Close_Button, "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));
      Button_Callback.Connect (Update_Messages_Set.Add_Button, "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'ACCESS));
      Button_Callback.Connect (Update_Messages_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'ACCESS));


   END Initialize;


END Update_Messages_Set_Pkg;



