------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------
WITH Gdk;
WITH Gdk.Bitmap;
USE Gdk.Bitmap;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Notebook;
USE Gtk.Notebook;
WITH Gtk.Pixmap;
USE Gtk.Pixmap;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk;
USE Gtk;

WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Hbutton_Box;
USE Gtk.Hbutton_Box;
WITH Gtk.Radio_Button;
USE Gtk.Radio_Button;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Gtk.Option_Menu;
USE Gtk.Option_Menu;
WITH Gtk.Widget;
USE Gtk.Widget;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Tasks_Widget_Pkg;
USE Tasks_Widget_Pkg;

WITH Systems;
USE Systems;

WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Messages;
USE Messages;

WITH Systems;
USE Systems;


PACKAGE Update_Messages_Set_Pkg IS

   TYPE Update_Messages_Set_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Update_Messages_Set_Access IS ACCESS ALL Update_Messages_Set_Record'Class;

   PROCEDURE Gtk_New (
         Update_Messages_Set :    OUT Update_Messages_Set_Access;
         From                : IN     Unbounded_String);

   PROCEDURE Initialize (
         Update_Messages_Set : ACCESS Update_Messages_Set_Record'Class);

   PROCEDURE Show_Advanced (
         Update_Messages_Set : IN     Update_Messages_Set_Access;
         Advanced            : IN     Boolean);



   Update_Messages_Set : Update_Messages_Set_Access;

PRIVATE
   TYPE Update_Messages_Set_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Parent : Unbounded_String;

         Scroll : Gtk_Scrolled_Window;

         Messages_List           : Gtk.Clist.Gtk_Clist;
         Message_Type_Menu       : Gtk_Option_Menu;
         Clist_Message_Type_Menu : Widget_Slist.Gslist;
         Clist_Processor_Menu    : Widget_Slist.Gslist;
         Clist_Policy_Menu       : Widget_Slist.Gslist;
         Message_Items           : String_List.Glist;
         Current_Message         : Unbounded_String;
         Current_Message_Ptr     : Message_Ptr;

         Name               : Gtk_Entry;
         Size               : Gtk_Entry;
         Period             : Gtk_Entry;
         Deadline           : Gtk_Entry;
         Jitter             : Gtk_Entry;
         Response_Time      : Gtk_Entry;
         Communication_Time : Gtk_Entry;

         Name_Label               : Gtk_Label;
         Size_Label               : Gtk_Label;
         Period_Label             : Gtk_Label;
         Deadline_Label           : Gtk_Label;
         Jitter_Label             : Gtk_Label;
         Response_Time_Label      : Gtk_Label;
         Communication_Time_Label : Gtk_Label;

         Frame_Message   : Gtk.Frame.Gtk_Frame;
         Message_Table   : Gtk.Table.Gtk_Table;
         Offset_Table    : Gtk.Table.Gtk_Table;
         Parameter_Table : Gtk.Table.Gtk_Table;

         Vbox_Main    : Gtk.Box.Gtk_Box;
         Hbox_Message : Gtk.Box.Gtk_Box;
         Hbox_Confirm : Gtk.Box.Gtk_Box;
         Vbox_List    : Gtk.Box.Gtk_Box;
         Hbox_Modify  : Gtk.Box.Gtk_Box;
         Vbox_Update  : Gtk.Box.Gtk_Box;
         Hbox_File    : Gtk.Box.Gtk_Box;
         Hbox_Seed    : Gtk.Box.Gtk_Box;

         Seed_Group         : Widget_Slist.Gslist;
         Seed_Unpredictable : Gtk_Radio_Button;
         Seed_Predictable   : Gtk_Radio_Button;

         Hbutton_Box_Modify : Gtk_Hbutton_Box;

         Modify_Button   : Gtk_Button;
         Advanced_Button : Gtk_Button;
         Close_Button    : Gtk_Button;
         Cancel_Button   : Gtk_Button;
         Add_Button      : Gtk_Button;
         Delete_Button   : Gtk_Button;
         Seed_Button     : Gtk_Button;

         Hseparator1 : Gtk_Hseparator;
         Vseparator  : Gtk_Vseparator;
         Vseparator2 : Gtk_Vseparator;

         Child_Page : Gtk_Frame;
         Label_Box  : Gtk_Box;
         Menu_Box   : Gtk_Box;
         Pixmap     : Gtk_Pixmap;
         Vbox       : Gtk_Box;
         Button     : Gtk_Button;
         Lab        : Gtk_Label;

         Offset : Widget_Offset_Table;
         Param  : Widget_Parameter_Table;

         Book_Opened      : Gdk_Pixmap;
         Book_Opened_Mask : Gdk_Bitmap;
         Book_Closed      : Gdk_Pixmap;
         Book_Closed_Mask : Gdk_Bitmap;
         Notebook         : Gtk_Notebook;

         Undo : System;

         Scroll_List : Gtk_Scrolled_Window;
         List_Titles : Chars_Ptr_Array (1 .. 7);

         Sys      : System;
         Advanced : Boolean := False;
      END RECORD;

END Update_Messages_Set_Pkg;








