------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Project;
USE Project;
WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Update_Networks_Set_Pkg;
USE Update_Networks_Set_Pkg;





PACKAGE BODY Update_Networks_Set_Pkg.Callbacks IS

   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Networks_Set.Undo, Sys);
      Destroy(Update_Networks_Set);
   END  On_Cancel_Pressed;

   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Network_Modify;
      Destroy(Update_Networks_Set);
   END On_Modify_Pressed;

   PROCEDURE Network_Modify IS
      A_Network      : Network_Ptr;
      A_Network_Type : Type_Of_Network;
      My_Iterator    : Network_Iterator;
   BEGIN

      IF Get_Active(Update_Networks_Set.Button_Bounded_Delay)
            THEN
         A_Network_Type:= Bounded_Delay;
      ELSE
         A_Network_Type := Parametric_Delay;
      END IF;


      Reset_Iterator(Sys.Networks, My_Iterator);

      LOOP

         Current_Element(Sys.Networks, A_Network, My_Iterator);

         IF A_Network.Name = Update_Networks_Set.Last_Network_Selected
               THEN
            EXIT;
         END IF;
         IF Is_Last_Element(Sys.Networks, My_Iterator)
               THEN
            RAISE Network_Not_Found;
         END IF;
         Next_Element(Sys.Networks, My_Iterator);
      END LOOP;

      Update_Network (Sys.Networks,A_Network.Name,To_Unbounded_String(
            A_Network_Type'Img));

   END Network_Modify;

   PROCEDURE On_Change_Network (
         Widget : ACCESS Gtk_Editable_Record'Class) IS

      Network_Name : Unbounded_String;
      A_Network    : Network_Ptr;
      My_Iterator  : Network_Iterator;

   BEGIN

      Network_Modify;
      IF To_Unbounded_String(Get_Text(Get_Entry(Update_Networks_Set.Name))) =
            To_Unbounded_String("")
            THEN
         Update_Networks_Set.Last_Network_Selected:= To_Unbounded_String(
            "");
         RETURN;
      END IF;

      Reset_Iterator(Sys.Networks, My_Iterator);
      LOOP
         Current_Element(Sys.Networks, A_Network, My_Iterator);
         IF A_Network.Name = To_Unbounded_String(Get_Text(Get_Entry(
                     Update_Networks_Set.Name)))
               THEN
            EXIT;
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Networks, My_Iterator);
         Next_Element(Sys.Networks, My_Iterator);
      END LOOP;

      IF A_Network.Network_Type = Bounded_Delay
            THEN
         Set_Active(Update_Networks_Set.Button_Bounded_Delay, True);
      ELSIF A_Network.Network_Type = Parametric_Delay
            THEN
         Set_Active(Update_Networks_Set.Button_Parametric_Delay, True);
      END IF;

      Network_Name := To_Unbounded_String(To_String(A_Network.Name));
      Update_Networks_Set.Last_Network_Selected := Network_Name;

   END On_Change_Network;


END Update_Networks_Set_Pkg.Callbacks;



