------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Gtk.Window;
USE Gtk.Window;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Gtk.Combo;
USE Gtk.Combo;
WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH  Unbounded_Strings;
USE  Unbounded_Strings;
USE  Unbounded_Strings.Strings_Table_Package;

WITH Gtk.Table;
USE Gtk.Table;
WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Gtk;
USE Gtk;
WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Button;

WITH Gtk.Box;
USE Gtk.Box;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Project;
USE Project;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH Update_Networks_Set_Pkg.Callbacks;
USE Update_Networks_Set_Pkg.Callbacks;

PACKAGE BODY Update_Networks_Set_Pkg IS

   PROCEDURE Gtk_New (
         Update_Networks_Set :    OUT Update_Networks_Set_Access) IS

   BEGIN
      Update_Networks_Set := NEW Update_Networks_Set_Record;
      Update_Networks_Set_Pkg.Initialize(Update_Networks_Set);
   END Gtk_New;

   PROCEDURE Initialize (
         Update_Networks_Set : ACCESS Update_Networks_Set_Record'Class) IS

      My_Iterator : Iterator;
      A_Network   : Network_Ptr;
      Name_Entry  : Gtk.Gentry.Gtk_Entry;

   BEGIN

      Initialize(Update_Networks_Set.Undo);
      Duplicate(Sys,Update_Networks_Set.Undo);

      --boxes parameters
      Gtk.Window.Initialize(Update_Networks_Set, Window_Toplevel);
      Set_Policy (Update_Networks_Set, True, True, True);
      Set_Title (Update_Networks_Set, To_String(Lb_Add_Network(
               Current_Language)));
      Set_Position (Update_Networks_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Networks_Set, True);
      Grab_Focus(Update_Networks_Set);

      Gtk.Box.Gtk_New_Vbox (Update_Networks_Set.Vbox_Main,False,10);
      Gtk.Box.Gtk_New_Hbox (Update_Networks_Set.Hbox_Name_Type,False,10);
      Gtk.Box.Gtk_New_Hbox (Update_Networks_Set.Hbox_Add_Cancel,False,10);

      Add(Update_Networks_Set,Update_Networks_Set.Vbox_Main);
      Add(Update_Networks_Set.Vbox_Main,Update_Networks_Set.
         Hbox_Name_Type);
      Add(Update_Networks_Set.Vbox_Main,Update_Networks_Set.
         Hbox_Add_Cancel);

      ------------------------------------------------------------------------
      --Name and Type Fields
      ------------------------------------------------------------------------
      -- Using a Table of 2 columns and 2 lines

      Gtk.Table.Gtk_New (Update_Networks_Set.Table_Name_Type,2 , 3, False);
      --(lines,columns)
      Add(Update_Networks_Set.Hbox_Name_Type,Update_Networks_Set.
         Table_Name_Type);

      -- Widgets in Table Name_Type

      Gtk.Combo.Gtk_New(Update_Networks_Set.Name);
      Set_Editable(Get_Entry(Update_Networks_Set.Name),False);

      Gtk.Label.Gtk_New(Update_Networks_Set.Name_Label,To_String(
            Lb_Networks_Name(Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New(Update_Networks_Set.Type_Label,To_String(
            Lb_Networks_Type(Current_Language) & Lb_Colon));

      Reset_Iterator(Sys.Networks, My_Iterator);

      LOOP
         Current_Element(Sys.Networks, A_Network, My_Iterator);
         String_List.Append(Update_Networks_Set.Network_Menu, To_String(
               A_Network.Name));
         EXIT WHEN Is_Last_Element(Sys.Networks, My_Iterator);
         Next_Element(Sys.Networks, My_Iterator);
      END LOOP;
      Set_Popdown_Strings (Update_Networks_Set.Name, Update_Networks_Set.
         Network_Menu);

      -- Radio Button Type
      --
      Gtk_New(Update_Networks_Set.Button_Bounded_Delay, Label =>
         "Bounded Delay");
      Gtk_New(Update_Networks_Set.Button_Parametric_Delay, Group =>
         Update_Networks_Set.Button_Bounded_Delay, Label =>
         "Parametric Delay");

      -- Attach Table
      --
      Attach_Defaults(Update_Networks_Set.Table_Name_Type,
         Update_Networks_Set.Name_Label, 0, 1, 0, 1);
      Attach_Defaults(Update_Networks_Set.Table_Name_Type,
         Update_Networks_Set.Name, 1, 2, 0, 1);

      Attach_Defaults(Update_Networks_Set.Table_Name_Type,
         Update_Networks_Set.Type_Label, 0, 1, 1, 2);
      Attach_Defaults(Update_Networks_Set.Table_Name_Type,
         Update_Networks_Set.Button_Bounded_Delay, 1, 2, 1, 2);
      Attach_Defaults(Update_Networks_Set.Table_Name_Type,
         Update_Networks_Set.Button_Parametric_Delay, 2, 3, 1, 2);

      ---------------------------------------------------------------------------
      --  Modify and Cancel Button
      ---------------------------------------------------------------------------

      Gtk.Button.Gtk_New(Update_Networks_Set.Modify_Button,To_String(
            Lb_Modify(Current_Language)));
      Gtk.Button.Gtk_New(Update_Networks_Set.Cancel_Button,To_String(
            Lb_Cancel(Current_Language)));

      Update_Networks_Set.Last_Network_Selected := To_Unbounded_String(
         Get_Text(Get_Entry(Update_Networks_Set.Name)));

      Name_Entry:=Get_Entry(Update_Networks_Set.Name);


      Button_Callback.Connect (Update_Networks_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));
      Button_Callback.Connect (Update_Networks_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'ACCESS));
      Combo_Callback.Connect (Name_Entry,"changed",
         Combo_Callback.To_Marshaller (On_Change_Network'ACCESS));

      Add(Update_Networks_Set.Hbox_Add_Cancel,Update_Networks_Set.
         Modify_Button);
      Add(Update_Networks_Set.Hbox_Add_Cancel,Update_Networks_Set.
         Cancel_Button);

   END Initialize;

END Update_Networks_Set_Pkg;




