------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;

WITH Project;
USE Project;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Scheduler;
USE Scheduler;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;



PACKAGE BODY Update_Processors_Set_Pkg.Callbacks.Integrity_Checks IS



   PROCEDURE Check_Integrity_Constraints (
         Update_Processors_Set : IN OUT Update_Processors_Set_Access;
         Error_Message         : IN OUT Unbounded_String;
         Ok                    : IN OUT Boolean) IS



      Sched          : Gint;
      Net            : Gint;
      A_Network      : Network_Ptr;
      Quantum_String : Unbounded_String;
      Integer_Ok     : Boolean;


   BEGIN

      Ok:=True;


      Processor_Name:=To_Unbounded_String(Get_Text(
            Update_Processors_Set.Entry_Processor_Name));


      Sched:= Gint'Val (15 -
         Gint(Selected_Button (Update_Processors_Set.Clist_Scheduler_Menu)));

      Net:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Networks)) - Gint(
            Selected_Button (Update_Processors_Set.Clist_Network_Menu)));


      Processor_File_Name :=To_Unbounded_String(Get_Text(
            Update_Processors_Set.
            Entry_File_Name));


      IF (Sched=0)
            THEN
         Processor_Scheduler := Earliest_Deadline_First_Protocol;
      END IF;
      IF (Sched=1)
            THEN
         Processor_Scheduler := Least_Laxity_First_Protocol;
      END IF;
      IF (Sched=2)
            THEN
         Processor_Scheduler := Rate_Monotonic_Protocol;
      END IF;
      IF (Sched=3)
            THEN
         Processor_Scheduler := Deadline_Monotonic_Protocol;
      END IF;
      IF (Sched=4)
            THEN
         Processor_Scheduler := Posix_1003_Highest_Priority_First_Protocol;
      END IF;
      IF (Sched=5)
            THEN
         Processor_Scheduler := Time_Sharing_Based_On_Wait_Time_Protocol;
      END IF;
      IF (Sched=6)
            THEN
         Processor_Scheduler := Round_Robin_Protocol;
      END IF;
      IF (Sched=7)
            THEN
         Processor_Scheduler :=
            Maximum_Urgency_First_Based_On_Laxity_Protocol;
      END IF;
      IF (Sched=8)
            THEN
         Processor_Scheduler :=
            Maximum_Urgency_First_Based_On_Deadline_Protocol;
      END IF;
      IF (Sched=9)
            THEN
         Processor_Scheduler := D_Over_Protocol;
      END IF;
      IF (Sched=10)
            THEN
         Processor_Scheduler := Time_Sharing_Based_On_Cpu_Usage_Protocol;
      END IF;
      IF (Sched=11)
            THEN
         Processor_Scheduler := Parametric_Protocol;
      END IF;
      IF (Sched=12)
            THEN
         Processor_Scheduler := Hierarchical_Cyclic_Protocol;
      END IF;
      IF (Sched=13)
            THEN
         Processor_Scheduler := Hierarchical_Round_Robin_Protocol;
      END IF;
      IF (Sched=14)
            THEN
         Processor_Scheduler := Hierarchical_Fixed_Priority_Protocol;
      END IF;
      IF (Sched=15)
            THEN
         Processor_Scheduler := Hierarchical_Parametric_Protocol;
      END IF;




      IF (Net=0)
            THEN
         Processor_Network :=  To_Unbounded_String("No_Network");
      ELSE
         Get_Element_Number(Sys.Networks,A_Network,
            Network_Range(
               Net-1));
         Processor_Network:=A_Network.Name;
      END IF;



      Quantum_String:=To_Unbounded_String(
         Get_Text(
            Update_Processors_Set.
            Entry_Quantum));
      IF(Quantum_String="")
            THEN
         Processor_Quantum:=0;
      ELSE
         To_Integer(Quantum_String, Processor_Quantum, Integer_Ok);
         IF NOT Integer_Ok
               THEN
            Error_Message:="Quantum" &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      IF Get_Active(Update_Processors_Set.
            Entry_Option)
            THEN
         Processor_Preemptive:=Preemptive;
      ELSE
         Processor_Preemptive:=Not_Preemptive;
      END IF;


      Check_Processor(Sys.Processors,
         Integrity_Checks.Processor_Name,
         Integrity_Checks.Processor_Preemptive,
         Integrity_Checks.Processor_Quantum,
         Integrity_Checks.Processor_File_Name,
         Integrity_Checks.Processor_Scheduler,
         Integrity_Checks.Processor_Network);




   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);



   END Check_Integrity_Constraints;




END Update_Processors_Set_Pkg.Callbacks.Integrity_Checks;


