------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Project;
USE Project;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;

WITH Scheduler;
USE Scheduler;
WITH Scheduler.Parametrics;
USE Scheduler.Parametrics;


WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Framework_Config;
USE Framework_Config;


WITH Translate;
USE Translate;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH Glib;
USE Glib;
WITH Gtk.Check_Button;
USE Gtk.Check_Button;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;



WITH Generic_Select_File_Pkg;
WITH Update_Processors_Set_Pkg.Callbacks.Integrity_Checks;



PACKAGE BODY Update_Processors_Set_Pkg.Callbacks IS



   PROCEDURE Read_Sc_File (
         File_Name : IN     Unbounded_String) IS


   BEGIN
      Set_Text(Update_Processors_Set.
         Entry_File_Name,
         To_String(File_Name) );
   END Read_Sc_File;



   PROCEDURE Dummy IS
   BEGIN
      NULL;
   END Dummy;

   PACKAGE Read_Sc_File_Package IS NEW Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   USE Read_Sc_File_Package;


   PROCEDURE Show_Processors (
         Update_Processors_Set : ACCESS Update_Processors_Set_Record'Class) IS



      A_Processor : Processor_Ptr;
      My_Iterator : Processors_Iterator;
      Line        : Gint;

      My_Scheduler : Scheduler_Ptr;
      A_Line       : Chars_Ptr_Array (1 .. 5);


   BEGIN
      Clear(Update_Processors_Set.Processors_List);

      Reset_Iterator(Sys.Processors, My_Iterator);
      IF NOT Is_Empty(Sys.Processors) THEN
         LOOP

            Current_Element(Sys.Processors, A_Processor,
               My_Iterator);


            My_Scheduler:=A_Processor.All.Scheduler;

            -- Create the line
            --
            A_Line := To_String(A_Processor.Name)+To_String(
               Get_Name(My_Scheduler.All))+
               To_String(A_Processor.Network)+Get_Quantum(
               My_Scheduler.All)+Get_Preemptive(My_Scheduler.All);

            -- Display the Line
            --
            Line := Append(Update_Processors_Set.Processors_List,
               A_Line);


            EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);

            Next_Element(Sys.Processors, My_Iterator);

         END LOOP;
      END IF;
   END Show_Processors;



   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS


      Name                : Unbounded_String;
      Quantum             : Unbounded_String;
      Row                 : Gint;
      A_Processor         : Processor_Ptr;
      Index               : Strings_Range    := Strings_Range'First;
      A_Network           : Network_Ptr;
      My_Network_Iterator : Network_Iterator;

      My_Scheduler : Scheduler_Ptr;
      Current      : Gint;

      Network_Items,
      Items         : Strings_Table;
      Sched_Id      : Gint;


   BEGIN

      Initialize(Items);
      Build_Global_Scheduler_Table_Name(Items);


      Row:=Get_Focus_Row(Update_Processors_Set.Processors_List);
      Name := To_Unbounded_String(Get_Text(Update_Processors_Set.
            Processors_List,
            Row,0));

      Update_Processors_Set.Current_Processor := Name;

      Quantum      := To_Unbounded_String(Get_Text(
            Update_Processors_Set.
            Processors_List,Row,3));




      A_Processor:=Search_Processor(Sys.Processors,
         Name);


      -- Preemptive push button
      --
      IF Get_Preemptive(A_Processor.Scheduler.All) = Not_Preemptive
            THEN
         Set_Active(Update_Processors_Set.Entry_Option, False);
      ELSE
         Set_Active(Update_Processors_Set.Entry_Option, True);
      END IF;



      -- Quantum
      --
      Gtk.Gentry.Set_Text(Update_Processors_Set.Entry_Quantum,
         To_String(Quantum));


      -- Processor name
      --
      Gtk.Gentry.Set_Text(Update_Processors_Set.Entry_Processor_Name,
         To_String(Name));



      -- Rebuild The Scheduler Menu
      --

      My_Scheduler:=A_Processor.All.Scheduler;


      IF (Get_Name(My_Scheduler.All) = Earliest_Deadline_First_Protocol)
            THEN
         Sched_Id:=0;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Least_Laxity_First_Protocol)
            THEN
         Sched_Id:=1;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Rate_Monotonic_Protocol)
            THEN
         Sched_Id:=2;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Deadline_Monotonic_Protocol)
            THEN
         Sched_Id:=3;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Posix_1003_Highest_Priority_First_Protocol)
            THEN
         Sched_Id:=4;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Time_Sharing_Based_On_Wait_Time_Protocol)
            THEN
         Sched_Id:=5;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Round_Robin_Protocol)
            THEN
         Sched_Id:=6;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Maximum_Urgency_First_Based_On_Laxity_Protocol)
            THEN
         Sched_Id:=7;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Maximum_Urgency_First_Based_On_Deadline_Protocol)
            THEN
         Sched_Id:=8;
      END IF;
      IF (Get_Name(My_Scheduler.All) = D_Over_Protocol)
            THEN
         Sched_Id:=9;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Time_Sharing_Based_On_Cpu_Usage_Protocol)
            THEN
         Sched_Id:=10;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Parametric_Protocol)
            THEN
         Sched_Id:=11;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Hierarchical_Cyclic_Protocol)
            THEN
         Sched_Id:=12;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Hierarchical_Round_Robin_Protocol)
            THEN
         Sched_Id:=13;
      END IF;
      IF (Get_Name(My_Scheduler.All) =
            Hierarchical_Fixed_Priority_Protocol)
            THEN
         Sched_Id:=14;
      END IF;
      IF (Get_Name(My_Scheduler.All) = Hierarchical_Parametric_Protocol)
            THEN
         Sched_Id:=15;
      END IF;



      Update_Processors_Set.Clist_Scheduler_Menu:= Widget_Slist.
         Null_List;
      Gtk.Option_Menu.Gtk_New(Update_Processors_Set.Scheduler_Menu);
      Build_Option_Menu (Update_Processors_Set.Scheduler_Menu,
         Update_Processors_Set.Clist_Scheduler_Menu, Items,
         Sched_Id, NULL);
      Show_All(Update_Processors_Set.Scheduler_Menu);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Scheduler_Menu, 1, 2, 2, 3);


      -- Rebuild network menu
      --

      Initialize(Network_Items);
      Index:=1;
      Reset_Iterator(Sys.Networks, My_Network_Iterator);
      Update_Processors_Set.Clist_Network_Menu:= Widget_Slist.Null_List;
      Add(Network_Items,To_Unbounded_String ("No_Network"));

      IF NOT Is_Empty(Sys.Networks) THEN
         LOOP
            Current_Element(Sys.Networks, A_Network,
               My_Network_Iterator);
            Add(Network_Items,A_Network.Name);

            IF A_Network.Name = A_Processor.Network THEN
               Current:=Gint(Index);
            END IF;
            Index:=Index+1;

            EXIT WHEN Is_Last_Element(Sys.Networks, My_Network_Iterator);

            Next_Element(Sys.Networks, My_Network_Iterator);
         END LOOP;
      END IF;

      Gtk.Option_Menu.Gtk_New(Update_Processors_Set.Network_Menu);
      Build_Option_Menu (Update_Processors_Set.Network_Menu,
         Update_Processors_Set.Clist_Network_Menu,
         Network_Items, Current, NULL);
      Show_All(Update_Processors_Set.Network_Menu);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Network_Menu,1,2,3,4);


      -- Source code file name
      --
      IF (Sched_Id=11) OR (Sched_Id=15)
            THEN
         Gtk.Gentry.Set_Text(Update_Processors_Set.Entry_File_Name,
            To_String(Get_Behavior_File_Name(Parametric_Scheduler(
                     A_Processor.Scheduler.All))));
      ELSE
         Gtk.Gentry.Set_Text(Update_Processors_Set.Entry_File_Name,
            "");
      END IF;




      Show_All(Update_Processors_Set);

   END On_Click_List;



   PROCEDURE On_Read_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Sched : Gint;

   BEGIN

      Sched:= Gint'Val (15 - Gint(Selected_Button(
               Update_Processors_Set.
               Clist_Scheduler_Menu)));

      IF (Sched/=11) AND (Sched/=15)
            THEN
         Show_Message_Box(Lb_Only_On_Parametric(Current_Language));
         RETURN;
      END IF;

      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Sc_File_Package.Generic_Select_File);

   END On_Read_Pressed;



   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Processors_Set);
   END On_Close_Pressed;



   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Processor_Set.Generic_Processor_Set.Iterator;
      A_Processor : Processor_Ptr;

   BEGIN


      IF Update_Processors_Set.Current_Processor = "" THEN
         Show_Message_Box(Lb_Select_Processor_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Processors) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Processors, My_Iterator);
      IF NOT Is_Empty(Sys.Processors) THEN
         LOOP

            Current_Element(Sys.Processors, A_Processor,
               My_Iterator);
            IF A_Processor.Name = Update_Processors_Set.
                  Current_Processor THEN

               Delete_Processor(Sys, A_Processor);
               Rebuild_Canvas(Graph_Window);

               -- Clear  entries
               --
               Update_Processors_Set.Current_Processor :=
                  To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Processors_Set.
                  Entry_Quantum, "");
               Set_Text(Update_Processors_Set.Entry_Processor_Name, "");
               Set_Text(Update_Processors_Set.
                  Entry_File_Name, "");
               Set_Active(Update_Processors_Set.Entry_Option, False);


               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);
            Next_Element(Sys.Processors, My_Iterator);

         END LOOP;

      END IF;


      Show_Processors(Update_Processors_Set);


   END On_Delete_Pressed;




   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Processors_Set.Undo, Sys);
      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Processors_Set);
   END On_Cancel_Pressed;



   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Processors_Set.Vbox_List) THEN
         Show_Advanced(Update_Processors_Set,False);
      ELSE
         Show_Advanced(Update_Processors_Set,True);
      END IF;
   END On_Advanced_Pressed;



   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS



      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Processors_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;

      Add_Processor(Sys.Processors,
         Integrity_Checks.Processor_Name,
         Integrity_Checks.Processor_Preemptive,
         Integrity_Checks.Processor_Quantum,
         Integrity_Checks.Processor_File_Name,
         Integrity_Checks.Processor_Scheduler,
         Integrity_Checks.Processor_Network);


      Show_Processors(Update_Processors_Set);


   EXCEPTION
      WHEN Processor_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Add_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS



      Ok            : Boolean             := True;
      Error_Message : Unbounded_String;
      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;

   BEGIN



      IF Update_Processors_Set.Current_Processor = "" THEN
         Show_Message_Box(Lb_Select_Processor_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Processors) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Processors_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      --
      -- Update set
      --

      Reset_Iterator(Sys.Processors, My_Iterator);
      LOOP

         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         IF A_Processor.Name = Update_Processors_Set.
               Current_Processor THEN

            Delete(Sys.Processors, A_Processor);

            Add_Processor(Sys.Processors,
               Integrity_Checks.Processor_Name,
               Integrity_Checks.Processor_Preemptive,
               Integrity_Checks.Processor_Quantum,
               Integrity_Checks.Processor_File_Name,
               Integrity_Checks.Processor_Scheduler,
               Integrity_Checks.Processor_Network);

            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Processors, My_Iterator);
         Next_Element(Sys.Processors, My_Iterator);

      END LOOP;




      Show_Processors(Update_Processors_Set);



   EXCEPTION
      WHEN Processor_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Modify_Pressed;




END Update_Processors_Set_Pkg.Callbacks;
