------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Glib;
USE Glib;
WITH Gtk.Enums;
USE Gtk.Enums;



WITH Project;
USE Project;
WITH Mmi_Tools;
USE Mmi_Tools;


WITH Framework_Config;
USE Framework_Config;

WITH Translate;
USE Translate;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Networks;
USE Networks;
WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;


WITH Gtk;
USE Gtk;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Callbacks_Cheddar;
USE Callbacks_Cheddar;
WITH Update_Processors_Set_Pkg.Callbacks;
USE Update_Processors_Set_Pkg.Callbacks;

WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;


PACKAGE BODY Update_Processors_Set_Pkg IS


   PROCEDURE Gtk_New (
         Update_Processors_Set :    OUT Update_Processors_Set_Access;
         Title                 : IN     Unbounded_String) IS
   BEGIN
      Update_Processors_Set := NEW Update_Processors_Set_Record;
      Update_Processors_Set_Pkg.Initialize (Update_Processors_Set,
         Title);
   END Gtk_New;


   PROCEDURE Show_Advanced (
         Update_Processors_Set : IN     Update_Processors_Set_Access;
         Advanced              : IN     Boolean) IS

   BEGIN

      IF NOT Advanced THEN
         Hide(Update_Processors_Set.Vbox_List);
         Hide(Update_Processors_Set.Delete_Button);
         Hide(Update_Processors_Set.Modify_Button);
         Show(Update_Processors_Set.Add_Button);
         Show(Update_Processors_Set.Close_Button);
         Show(Update_Processors_Set.Cancel_Button);
         Show(Update_Processors_Set.Advanced_Button);

         Show(Update_Processors_Set.Vbox_Main);
         Show(Update_Processors_Set.Hbox_Processors);
         Show_All(Update_Processors_Set.Vbox_Update);
         Show(Update_Processors_Set.Hbutton_Box_Modify);
         Show_All(Update_Processors_Set.Frame_Processors);

      ELSE
         Show_All(Update_Processors_Set.Vbox_Main);
      END IF;
      Show(Update_Processors_Set);

   END Show_Advanced;


   PROCEDURE Initialize (
         Update_Processors_Set : ACCESS Update_Processors_Set_Record'Class;
         Title                 : IN     Unbounded_String) IS


      Scheduler_Items     : Strings_Table;
      My_Network_Iterator : Network_Iterator;
      Network_Items       : Strings_Table;
      A_Network           : Network_Ptr;


   BEGIN

      Gtk.Window.Initialize(Update_Processors_Set, Window_Toplevel);
      Set_Policy (Update_Processors_Set, True, True, True);
      Set_Title (Update_Processors_Set, To_String(Title));
      Set_Position (Update_Processors_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Processors_Set, True);
      Grab_Focus(Update_Processors_Set);



      Gtk.Box.Gtk_New_Vbox(Update_Processors_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Processors_Set.Vbox_Main, 10);
      Add(Update_Processors_Set, Update_Processors_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox(Update_Processors_Set.Hbox_Processors,
         False, 10);
      Add(Update_Processors_Set.Vbox_Main, Update_Processors_Set.
         Hbox_Processors);

      Gtk.Box.Gtk_New_Vbox(Update_Processors_Set.Vbox_Update, False,
         10);
      Add(Update_Processors_Set.Hbox_Processors,
         Update_Processors_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox(Update_Processors_Set.Vbox_List, False, 10);
      Add(Update_Processors_Set.Hbox_Processors,
         Update_Processors_Set.Vbox_List);

      Initialize(Update_Processors_Set.Undo);
      Duplicate(Sys,Update_Processors_Set.Undo);

      Gtk.Frame.Gtk_New(Update_Processors_Set.Frame_Processors,"");
      Add(Update_Processors_Set.Vbox_Update, Update_Processors_Set.
         Frame_Processors);

      Gtk_New(Update_Processors_Set.Hbutton_Box_Modify);
      Set_Spacing(Update_Processors_Set.Hbutton_Box_Modify, 10);
      Set_Layout(Update_Processors_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size(Update_Processors_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding(Update_Processors_Set.Hbutton_Box_Modify, 7,
         0);
      Pack_Start(Update_Processors_Set.Vbox_Update,
         Update_Processors_Set.Hbutton_Box_Modify,
         False, True, 5);

      Gtk.Table.Gtk_New (Update_Processors_Set.Processors_Table,2,
         15, False);
      Set_Border_Width (Update_Processors_Set.Processors_Table, 5);
      Add(Update_Processors_Set.Frame_Processors,
         Update_Processors_Set.Processors_Table);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab, To_String(
            Lb_Processor_Name(
               Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 0, 1);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab,To_String(
            Lb_Scheduler(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 2, 3);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab, "Quantum :");
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 1, 2);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab, To_String(
            Lb_Network(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 3, 4);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab,To_String(
            Lb_Option(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 4, 5);

      Gtk.Label.Gtk_New(Update_Processors_Set.Lab,To_String(
            Lb_File_Name(Current_Language)) & To_String(Lb_Colon));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Lab, 0, 1, 5, 6);

      Gtk.Button.Gtk_New(Update_Processors_Set.Read_Button,
         To_String(Lb_Read(Current_Language)));
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Read_Button, 1, 2, 5, 6);


      -- List of Network
      --

      Initialize(Network_Items);
      Reset_Iterator(Sys.Networks, My_Network_Iterator);

      Add(Network_Items,To_Unbounded_String ("No_Network"));

      IF NOT Is_Empty(Sys.Networks) THEN
         LOOP
            Current_Element(Sys.Networks, A_Network,My_Network_Iterator);
            Add(Network_Items,A_Network.Name);

            EXIT WHEN Is_Last_Element(Sys.Networks, My_Network_Iterator);

            Next_Element(Sys.Networks, My_Network_Iterator);
         END LOOP;
      END IF;

      -- List of scheduler
      --
      Initialize(Scheduler_Items);
      Build_Global_Scheduler_Table_Name(Scheduler_Items);


      Gtk.Gentry.Gtk_New(Update_Processors_Set.Entry_Processor_Name,
         50);
      Set_Editable (Update_Processors_Set.Entry_Processor_Name, True);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Entry_Processor_Name, 1, 2, 0, 1);

      Gtk.Gentry.Gtk_New(Update_Processors_Set.Entry_Quantum, 50);
      Set_Editable (Update_Processors_Set.Entry_Quantum, True);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Entry_Quantum, 1, 2, 1, 2);
      Gtk.Gentry.Set_Text(Update_Processors_Set.Entry_Quantum, "");

      Gtk.Option_Menu.Gtk_New(Update_Processors_Set.Scheduler_Menu);
      Build_Option_Menu (Update_Processors_Set.Scheduler_Menu,
         Update_Processors_Set.Clist_Scheduler_Menu, Scheduler_Items,
         -1, NULL);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Scheduler_Menu, 1, 2, 2, 3);

      Gtk.Gentry.Gtk_New(Update_Processors_Set.Entry_Network, 50);
      Set_Editable (Update_Processors_Set.Entry_Network, True);
      Gtk.Option_Menu.Gtk_New(Update_Processors_Set.Network_Menu);
      Build_Option_Menu (Update_Processors_Set.Network_Menu,
         Update_Processors_Set.Clist_Network_Menu, Network_Items,
         -1, NULL);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Network_Menu, 1, 2, 3, 4);


      Gtk.Check_Button.Gtk_New(Update_Processors_Set.Entry_Option,
         To_String(
            Lb_Preemptive(Current_Language)));
      Set_Active(Update_Processors_Set.Entry_Option, True);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Entry_Option, 1, 2, 4, 5);




      Gtk.Gentry.Gtk_New(Update_Processors_Set.Entry_File_Name, 50);
      Attach_Defaults(Update_Processors_Set.Processors_Table,
         Update_Processors_Set.
         Entry_File_Name, 1, 2, 6, 7);



      Gtk.Button.Gtk_New(Update_Processors_Set.Close_Button,
         To_String(Lb_Close(Current_Language)));
      Gtk.Button.Gtk_New(Update_Processors_Set.Cancel_Button,
         To_String(Lb_Cancel(Current_Language)));
      Gtk.Button.Gtk_New(Update_Processors_Set.Advanced_Button,
         To_String(Lb_Advanced(Current_Language)));
      Gtk.Button.Gtk_New(Update_Processors_Set.Delete_Button,
         To_String(Lb_Delete(Current_Language)));
      Gtk.Button.Gtk_New(Update_Processors_Set.Modify_Button,
         To_String(Lb_Modify(Current_Language)));
      Gtk.Button.Gtk_New(Update_Processors_Set.Add_Button,
         To_String(Lb_Add(Current_Language)));


      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Close_Button);
      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Cancel_Button);
      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Advanced_Button);
      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Delete_Button);
      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Modify_Button);
      Add(Update_Processors_Set.Hbutton_Box_Modify,
         Update_Processors_Set.Add_Button);


      -- Spreadsheet for processor information
      --
      Gtk.Scrolled_Window.Gtk_New(Update_Processors_Set.Scroll_List, NULL, NULL);
      Add(Update_Processors_Set.Vbox_List,Update_Processors_Set.
         Scroll_List);

      Update_Processors_Set.List_Titles:= To_String(Lb_Processor_Name(
            Current_Language))
         +To_String(Lb_Scheduler(Current_Language))
         +To_String(Lb_Network(Current_Language))
         + "Quantum"
         +To_String(Lb_Option(Current_Language));

      Gtk.Clist.Gtk_New(Update_Processors_Set.Processors_List, 5,
         Update_Processors_Set.
         List_Titles);
      Set_Usize(Update_Processors_Set.Processors_List,475,200);
      Set_Sort_Column(Update_Processors_Set.Processors_List,0);
      Set_Sort_Type(Update_Processors_Set.Processors_List,
         Ascending);
      Set_Auto_Sort(Update_Processors_Set.Processors_List, True);
      Add(Update_Processors_Set.Scroll_List,Update_Processors_Set.
         Processors_List);




      Show_Processors(Update_Processors_Set);



      List_Callback.Connect (Update_Processors_Set.
         Processors_List,"select_row",
         List_Callback.To_Marshaller (On_Click_List'ACCESS));

      Button_Callback.Connect (Update_Processors_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'ACCESS));
      Button_Callback.Connect (Update_Processors_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'ACCESS));
      Button_Callback.Connect (Update_Processors_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'ACCESS));
      Button_Callback.Connect (Update_Processors_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'ACCESS));
      Button_Callback.Connect (Update_Processors_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'ACCESS));
      Button_Callback.Connect (Update_Processors_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'ACCESS));
      Button_Callback.Connect (
         Update_Processors_Set.Read_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Read_Pressed'ACCESS));



   END Initialize;


END Update_Processors_Set_Pkg;
