------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------





WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;


WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Resources;
USE Resources;
USE Resources.Task_Lists;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Gtk.Gentry;
USE Gtk.Gentry;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Resources;
USE Resources;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;


WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;


WITH Update_Resources_Set_Pkg.Callbacks.Integrity_Checks;



PACKAGE BODY Update_Resources_Set_Pkg.Callbacks IS


   PROCEDURE Show_Resources (
         Update_Resources_Set : ACCESS Update_Resources_Set_Record'Class) IS

      A_Resource      : Resource_Ptr;
      My_Iterator     : Resources_Iterator;
      Line            : Gint;
      Resource_Share  : Chars_Ptr_Array (1 .. 3);
      Protocol_String : Unbounded_String;


   BEGIN
      Clear(Update_Resources_Set.Resources_List);

      Reset_Iterator(Sys.Resources, My_Iterator);
      IF NOT Is_Empty(Sys.Resources) THEN
         LOOP

            Current_Element(Sys.Resources, A_Resource,
               My_Iterator);

            Protocol_String := To_Unbounded_String(
               Resources_Type'Image(A_Resource.Protocol));

            -- Create the line
            --
            Resource_Share := To_String(A_Resource.Name)+
               To_String(A_Resource.Cpu_Name)+To_String(Protocol_String);

            -- Display the Line
            --
            Line := Append(Update_Resources_Set.Resources_List,
               Resource_Share);

            EXIT WHEN Is_Last_Element(Sys.Resources, My_Iterator);

            Next_Element(Sys.Resources, My_Iterator);

         END LOOP;
      END IF;
   END Show_Resources;




   PROCEDURE Show_Task_Affect_List (
         Update_Resources_Set : ACCESS Update_Resources_Set_Record'Class) IS


      Affect_Task_Share : Affected_Task_List;
      Task_Name         : Unbounded_String;
      Line              : Gint;
      Task_Share        : Chars_Ptr_Array (1 .. 3);


   BEGIN
      Clear(Update_Resources_Set.Affected_Task_List);


      FOR I IN 0 .. Update_Resources_Set.Task_List_Share.Nb_Entries-1
            LOOP

         Task_Name:=Update_Resources_Set.Task_List_Share.Entries(I).Item;
         Affect_Task_Share.Task_Begin :=
            Update_Resources_Set.Task_List_Share.Entries(I).
            Data.Task_Begin;
         Affect_Task_Share.Task_End :=
            Update_Resources_Set.Task_List_Share.Entries(I).
            Data.Task_End;

         Task_Share:= To_String(Task_Name)
            +Affect_Task_Share.Task_Begin'Img
            +Affect_Task_Share.Task_End'Img;

         -- Show Task on Clist
         --
         Line := Append(Update_Resources_Set.Affected_Task_List,
            Task_Share);

      END LOOP;

   END Show_Task_Affect_List;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      A_Address_Space        : Address_Space_Ptr;
      Address_Space_Iterator : Address_Space_Set.Generic_Address_Space_Set.Iterator;
      Empty_Menu             : String_List.Glist;

      Name     : Unbounded_String;
      Protocol : Unbounded_String;
      Row      : Gint;

      A_Resource : Resource_Ptr;

      Protocol_Items : String_List.Glist;



   BEGIN

      Row:=Get_Focus_Row(Update_Resources_Set.Resources_List);

      Name := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Resources_List,
            Row,0));
      Update_Resources_Set.Current_Resource := Name;

      Protocol     := To_Unbounded_String(Get_Text(
            Update_Resources_Set.
            Resources_List,Row,2));


      A_Resource:=Search_Resource(Sys.Resources, Name);


      -- Show name
      --
      Gtk.Gentry.Set_Text(Update_Resources_Set.Name, To_String(Name));



      -- Show state
      --
      Gtk.Gentry.Set_Text(Update_Resources_Set.State, A_Resource.State'
         Img);


      -- Show address space and processor names
      --
      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces,
         A_Resource.
         Address_Space_Name);
      Set_Text(Update_Resources_Set.Cpu_Name, To_String(
            A_Address_Space.
            Cpu_Name));

      Update_Resources_Set.Clist_Address_Space_Menu:=Empty_Menu;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name =  A_Resource.Address_Space_Name THEN
            String_List.Append(Update_Resources_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name /=  A_Resource.Address_Space_Name THEN
            String_List.Append(Update_Resources_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;

      Set_Popdown_Strings (Update_Resources_Set.Address_Space_Menu,
         Update_Resources_Set.Clist_Address_Space_Menu);



      -- Show protocol
      --
      IF Protocol = "NO_PROTOCOL"
            THEN
         String_List.Append (Protocol_Items,
            "No Protocol");
         String_List.Append (Protocol_Items, "PCP");
         String_List.Append (Protocol_Items, "PIP");
         String_List.Append (Protocol_Items, "IPCP");
      ELSE
         IF Protocol = "PRIORITY_CEILING_PROTOCOL"
               THEN
            String_List.Append (Protocol_Items,
               "PCP");
            String_List.Append (Protocol_Items,
               "No Protocol");
            String_List.Append (Protocol_Items,
               "PIP");
            String_List.Append (Protocol_Items, "IPCP");
         ELSE
            IF Protocol = "IMMEDIATE_PRIORITY_CEILING_PROTOCOL"
                  THEN
               String_List.Append (Protocol_Items,
                  "IPCP");
               String_List.Append (Protocol_Items,
                  "No Protocol");
               String_List.Append (Protocol_Items,
                  "PIP");
               String_List.Append (Protocol_Items, "PCP");
            ELSE
               IF Protocol = "PRIORITY_INHERITANCE_PROTOCOL"
                     THEN
                  String_List.Append (Protocol_Items,
                     "PIP");
                  String_List.Append (Protocol_Items,
                     "No Protocol");
                  String_List.Append (Protocol_Items,
                     "PCP");
                  String_List.Append (Protocol_Items, "IPCP");
               END IF;
            END IF;
         END IF;
      END IF;

      Set_Popdown_Strings (Update_Resources_Set.Protocol,
         Protocol_Items);


      --  Show task using the resource
      --
      Initialize(Update_Resources_Set.Task_List_Share);
      FOR I IN 0 .. A_Resource.Task_Tab.Nb_Entries-1
            LOOP
         Update_Resources_Set.Task_List_Share.Entries(I).Item:=
            A_Resource.Task_Tab.Entries(I).Item;
         Update_Resources_Set.Task_List_Share.Entries(I).Data :=
            A_Resource.Task_Tab.Entries(I).Data;
         Update_Resources_Set.Task_List_Share.Nb_Entries:=
            Update_Resources_Set.Task_List_Share.Nb_Entries+1;
      END LOOP;

      Show_Task_Affect_List (Update_Resources_Set);
      Show_All(Update_Resources_Set);

   END On_Click_List;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Resources_Set);
   END On_Close_Pressed;



   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Resources_Set.Undo, Sys);
      Destroy(Update_Resources_Set);
   END On_Cancel_Pressed;


   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Resources_Set.Vbox_List) THEN
         Show_Advanced(Update_Resources_Set,False);
      ELSE
         Show_Advanced(Update_Resources_Set,True);
      END IF;
   END On_Advanced_Pressed;


   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Resource_Set.Generic_Resource_Set.Iterator;
      A_Resource  : Resource_Ptr;

   BEGIN

      IF Update_Resources_Set.Current_Resource = "" THEN
         Show_Message_Box(Lb_Select_Resource_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Resources) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;


      Reset_Iterator(Sys.Resources, My_Iterator);
      IF NOT Is_Empty(Sys.Resources) THEN
         LOOP

            Current_Element(Sys.Resources, A_Resource,
               My_Iterator);
            IF A_Resource.Name = Update_Resources_Set.
                  Current_Resource THEN


               Delete(Sys.Resources, A_Resource);


               -- Clear  entries
               --
               Update_Resources_Set.Current_Resource :=
                  To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Resources_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Resources_Set.
                  State, "1");
               Clear(Update_Resources_Set.Affected_Task_List);

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Resources, My_Iterator);
            Next_Element(Sys.Resources, My_Iterator);

         END LOOP;

      END IF;


      Show_Resources(Update_Resources_Set);


   END On_Delete_Pressed;




   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Resources_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      -- Add the new item in the set
      --
      Add_Resource (Sys.Resources,
         Integrity_Checks.Resource_Name,
         Integrity_Checks.Resource_State,
         Integrity_Checks.Resource_Processor,
         Integrity_Checks.Resource_Address_Space,
         Integrity_Checks.Resource_Protocol,
         Update_Resources_Set.Task_List_Share);

      Update_Resources_Set.Current_Resource :=
         Integrity_Checks.Resource_Name;

      Show_Resources(Update_Resources_Set);

   EXCEPTION
      WHEN Resource_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;



   END On_Add_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      A_Resource    : Resource_Ptr;
      My_Iterator   : Resource_Set.Generic_Resource_Set.Iterator;
      Ok            : Boolean                                    := True;
      Error_Message : Unbounded_String;

   BEGIN


      IF Update_Resources_Set.Current_Resource = "" THEN
         Show_Message_Box(Lb_Select_Resource_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Resources) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;



      Integrity_Checks.Check_Integrity_Constraints (
         Update_Resources_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      --
      -- Update resource set
      --

      Reset_Iterator(Sys.Resources, My_Iterator);
      LOOP

         Current_Element(Sys.Resources, A_Resource, My_Iterator);
         IF A_Resource.Name = Update_Resources_Set.Current_Resource THEN

            Delete(Sys.Resources, A_Resource);


            Update_Resources_Set.Current_Resource :=
               Integrity_Checks.Resource_Name;
            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Resources, My_Iterator);
         Next_Element(Sys.Resources, My_Iterator);

      END LOOP;

      Add_Resource (Sys.Resources,
         Integrity_Checks.Resource_Name,
         Integrity_Checks.Resource_State,
         Integrity_Checks.Resource_Processor,
         Integrity_Checks.Resource_Address_Space,
         Integrity_Checks.Resource_Protocol,
         Update_Resources_Set.Task_List_Share);

      Show_Resources(Update_Resources_Set);


   EXCEPTION
      WHEN Resource_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Modify_Pressed;


   PROCEDURE On_Change_Cpu (
         Widget : ACCESS Gtk_Editable_Record'Class) IS

      Resource_Address_Space : Unbounded_String;
      A_Address_Space        : Address_Space_Ptr;

   BEGIN
      Resource_Address_Space:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Resources_Set.Address_Space_Menu)));
      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces,
         Resource_Address_Space);
      Set_Text(Update_Resources_Set.Cpu_Name, To_String(
            A_Address_Space.
            Cpu_Name));
   END On_Change_Cpu;



   PROCEDURE On_Affected_Task_List_Click (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name       : Unbounded_String;
      Task_Begin : Unbounded_String;
      Task_End   : Unbounded_String;
      Row        : Gint;

      A_Task     : Task_Ptr;
      Ite        : Tasks_Iterator;
      Task_Items : String_List.Glist;

   BEGIN

      Row:=Get_Focus_Row(Update_Resources_Set.Affected_Task_List);
      Name       := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,0));
      Task_Begin := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,1));
      Task_End   := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,2));


      String_List.Append (Task_Items, To_String(Name));
      Reset_Iterator(Sys.Tasks, Ite);
      LOOP
         Current_Element(Sys.Tasks, A_Task, Ite);

         IF A_Task.Name/=Name
               THEN
            String_List.Append (Task_Items,
               To_String(A_Task.Name));
         END IF;

         EXIT WHEN Is_Last_Element(Sys.Tasks, Ite);

         Next_Element(Sys.Tasks, Ite);
      END LOOP;


      Set_Popdown_Strings (Update_Resources_Set.Resource_Affect_Task,
         Task_Items);


      Gtk.Gentry.Set_Text(Update_Resources_Set.Resource_Affect_Begin,
         To_String(
            Task_Begin));
      Gtk.Gentry.Set_Text(Update_Resources_Set.Resource_Affect_End,
         To_String(
            Task_End));

   END On_Affected_Task_List_Click;



   PROCEDURE On_Add_Affect_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Affect_Task_Share : Affected_Task_List;
      Ok                : Boolean            := True;
      Error_Message     : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints (
         Update_Resources_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      Affect_Task_Share.Task_Begin := Natural(
         Integrity_Checks.Resource_Share_Begin);
      Affect_Task_Share.Task_End := Natural(
         Integrity_Checks.Resource_Share_End);

      -- Add Task on list
      --
      Update_Resources_Set.Task_List_Share.Entries(
         Update_Resources_Set.
         Task_List_Share.Nb_Entries).Item:=
         Integrity_Checks.Resource_Task_Name;
      Update_Resources_Set.Task_List_Share.Entries(
         Update_Resources_Set.
         Task_List_Share.Nb_Entries).Data := Affect_Task_Share;
      Update_Resources_Set.Task_List_Share.Nb_Entries:=
         Update_Resources_Set.Task_List_Share.Nb_Entries+1;



      -- Show Task on Clist
      --
      Show_Task_Affect_List (Update_Resources_Set);


   END On_Add_Affect_Pressed;



   PROCEDURE On_Modify_Affect_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

      Name              : Unbounded_String;
      Task_Begin_String : Unbounded_String;
      Task_End_String   : Unbounded_String;
      Task_Begin        : Natural;
      Task_End          : Natural;
      Natural_Ok        : Boolean;
      Row               : Gint;



   BEGIN

      Row:=Get_Focus_Row(Update_Resources_Set.Affected_Task_List);
      Name       := To_Unbounded_String(Get_Text(Update_Resources_Set.
            Affected_Task_List,Row,0));
      Task_Begin_String := To_Unbounded_String(Get_Text(
            Update_Resources_Set.
            Affected_Task_List,Row,1));
      Task_End_String   := To_Unbounded_String(Get_Text(
            Update_Resources_Set.
            Affected_Task_List,Row,2));
      To_Natural(Task_Begin_String, Task_Begin, Natural_Ok);
      To_Natural(Task_End_String, Task_End, Natural_Ok);


      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints (
         Update_Resources_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      -- Find old values and replace them
      --
      FOR I IN 0..Update_Resources_Set.Task_List_Share.Nb_Entries-1 LOOP
         IF (Update_Resources_Set.Task_List_Share.Entries(I).Item=Name)
               AND (Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_Begin=Task_Begin)
               AND (Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_End=Task_End)
               THEN
            Update_Resources_Set.Task_List_Share.Entries(I).Item:=
               Integrity_Checks.Resource_Task_Name;
            Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_End:=Integrity_Checks.Resource_Share_End;
            Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_Begin :=Integrity_Checks.Resource_Share_Begin;
            EXIT;
         END IF;
      END LOOP;


      --  Show task using the resource
      --
      Show_Task_Affect_List (Update_Resources_Set);


   END On_Modify_Affect_Pressed;






   PROCEDURE On_Delete_Affect_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;


   BEGIN


      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints (
         Update_Resources_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;


      -- Find item and delete it
      --
      FOR I IN 0..Update_Resources_Set.Task_List_Share.Nb_Entries-1 LOOP
         IF (Update_Resources_Set.Task_List_Share.Entries(I).Item=
               Integrity_Checks.Resource_Task_Name)
               AND (Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_Begin=Integrity_Checks.Resource_Share_Begin)
               AND (Update_Resources_Set.Task_List_Share.Entries(I).
               Data.Task_End=Integrity_Checks.Resource_Share_End)
               THEN
            Update_Resources_Set.Task_List_Share.Entries(I):=
               Update_Resources_Set.Task_List_Share.Entries(
               Update_Resources_Set.Task_List_Share.Nb_Entries-1);
            Update_Resources_Set.Task_List_Share.Nb_Entries:=
               Update_Resources_Set.Task_List_Share.Nb_Entries-1;
            EXIT;
         END IF;
      END LOOP;


      --  Show task using the resource
      --
      Show_Task_Affect_List (Update_Resources_Set);


   END On_Delete_Affect_Pressed;





END Update_Resources_Set_Pkg.Callbacks;


