------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Pixmap;
USE Gtk.Pixmap;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk;
USE Gtk;

WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Hbutton_Box;
USE Gtk.Hbutton_Box;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Gtk.Option_Menu;
USE Gtk.Option_Menu;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Resources;
USE Resources;
WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtk.Combo;
USE Gtk.Combo;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Option_Menu;
USE Gtk.Option_Menu;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;

WITH Resources;
USE Resources;
WITH Systems;
USE Systems;


PACKAGE Update_Resources_Set_Pkg IS

   TYPE Update_Resources_Set_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Update_Resources_Set_Access IS ACCESS ALL Update_Resources_Set_Record'Class;

   PROCEDURE Gtk_New (
         Update_Resources_Set :    OUT Update_Resources_Set_Access;
         Title                : IN     Unbounded_String);
   PROCEDURE Initialize (
         Update_Resources_Set : ACCESS Update_Resources_Set_Record'Class;
         Title                : IN     Unbounded_String);
   PROCEDURE Show_Advanced (
         Update_Resources_Set : IN     Update_Resources_Set_Access;
         Advanced             : IN     Boolean);

   Update_Resources_Set : Update_Resources_Set_Access;
   Parent               : Unbounded_String;


PRIVATE

   TYPE Update_Resources_Set_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Scroll : Gtk_Scrolled_Window;

         Resources_List           : Gtk.Clist.Gtk_Clist;
         Resource_Type_Menu       : Gtk_Option_Menu;
         Clist_Resource_Type_Menu : Widget_Slist.Gslist;
         Processor_Menu           : Gtk_Option_Menu;
         Clist_Processor_Menu     : Widget_Slist.Gslist;
         Resource_Items           : String_List.Glist;

         Current_Resource : Unbounded_String;

         Lab : Gtk_Label;

         Frame_Resource : Gtk.Frame.Gtk_Frame;
         Resource_Table : Gtk.Table.Gtk_Table;

         Vbox_Main     : Gtk.Box.Gtk_Box;
         Hbox_Resource : Gtk.Box.Gtk_Box;
         Hbox_Confirm  : Gtk.Box.Gtk_Box;
         Vbox_List     : Gtk.Box.Gtk_Box;
         Hbox_Modify   : Gtk.Box.Gtk_Box;
         Vbox_Update   : Gtk.Box.Gtk_Box;

         Hbutton_Box_Modify : Gtk_Hbutton_Box;

         Modify_Button        : Gtk_Button;
         Advanced_Button      : Gtk_Button;
         Close_Button         : Gtk_Button;
         Cancel_Button        : Gtk_Button;
         Add_Button           : Gtk_Button;
         Delete_Button        : Gtk_Button;
         Button_Add_Affect    : Gtk.Button.Gtk_Button;
         Button_Modify_Affect : Gtk.Button.Gtk_Button;
         Button_Delete_Affect : Gtk.Button.Gtk_Button;

         Hseparator1 : Gtk_Hseparator;
         Vseparator  : Gtk_Vseparator;
         Vseparator2 : Gtk_Vseparator;

         Child_Page : Gtk_Frame;
         Label_Box  : Gtk_Box;
         Menu_Box   : Gtk_Box;
         Pixmap     : Gtk_Pixmap;
         Vbox       : Gtk_Box;
         Button     : Gtk_Button;

         Undo : System;

         Scroll_List : Gtk_Scrolled_Window;
         List_Titles : Chars_Ptr_Array (1 .. 3);

         Advanced : Boolean := False;

         Name                     : Gtk.Gentry.Gtk_Entry;
         State                    : Gtk.Gentry.Gtk_Entry;
         Protocol                 : Gtk.Combo.Gtk_Combo;
         Cpu_Name                 : Gtk_Entry;
         Address_Space_Menu       : Gtk.Combo.Gtk_Combo;
         Clist_Address_Space_Menu : String_List.Glist;

         Affected_Task_List        : Gtk.Clist.Gtk_Clist;
         Scroll_Affected_Task_List : Gtk_Scrolled_Window;

         Hbox_Add_Cancel       : Gtk.Box.Gtk_Box;
         Hbox_Add_Affect       : Gtk.Box.Gtk_Box;
         Hbox_Table_Name_State : Gtk.Box.Gtk_Box;

         Table_Name_State : Gtk.Table.Gtk_Table;
         Table_Add_Affect : Gtk.Table.Gtk_Table;

         Resource_Affect_Task  : Gtk_Combo;
         Items                 : String_List.Glist;
         Resource_Affect_Begin : Gtk.Gentry.Gtk_Entry;
         Resource_Affect_End   : Gtk.Gentry.Gtk_Entry;

         Resource_Name_Label          : Gtk.Label.Gtk_Label;
         Resource_State_Label         : Gtk.Label.Gtk_Label;
         Resource_Task_Label          : Gtk.Label.Gtk_Label;
         Resource_Begin_Label         : Gtk.Label.Gtk_Label;
         Resource_End_Label           : Gtk.Label.Gtk_Label;
         Resource_Combo_Label         : Gtk.Label.Gtk_Label;
         Resource_Cpu_Label           : Gtk.Label.Gtk_Label;
         Resource_Address_Space_Label : Gtk.Label.Gtk_Label;
         Frame_Name_State             : Gtk.Frame.Gtk_Frame;
         Frame_Affect                 : Gtk.Frame.Gtk_Frame;
         Affect_List_Titles           : Chars_Ptr_Array (1 .. 3);

         Task_List_Share : Resources.Task_List;
      END RECORD;



END Update_Resources_Set_Pkg;
