------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Glib;
USE Glib;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;
WITH Task_Offsets;
USE Task_Offsets;
USE Task_Offsets.Offsets_Table_Package;

WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Radio_Menu_Item;
USE Gtk.Radio_Menu_Item;

WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;


WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;




PACKAGE BODY Update_Tasks_Set_Pkg.Callbacks.Integrity_Checks IS



   PROCEDURE Check_Integrity_Constraints (
         Update_Tasks_Set : IN OUT Update_Tasks_Set_Access;
         Error_Message    : IN OUT Unbounded_String;
         Ok               : IN OUT Boolean) IS


      Task_Type_Index : Gint := 0;
      Policy_Index    : Gint := 0;

      Integer_Ok,
      Double_Ok,
      Boolean_Ok : Boolean;

      Task_Priority_String          : Unbounded_String;
      Task_Capacity_String          : Unbounded_String;
      Task_Jitter_String            : Unbounded_String;
      Task_Deadline_String          : Unbounded_String;
      Task_Period_String            : Unbounded_String;
      Task_Start_Time_String        : Unbounded_String;
      Task_Blocking_Time_String     : Unbounded_String;
      Task_Seed_Value_String        : Unbounded_String;
      Task_Stack_Memory_Size_String : Unbounded_String;
      Task_Text_Memory_Size_String  : Unbounded_String;
      Task_Criticality_String       : Unbounded_String;

      Current      : Unbounded_String;
      A_Offset     : Offset_Type;
      A_Param      : Parameter_Ptr;
      A_Param_Type : Gint;

   BEGIN

      Ok:=True;


      --
      -- First task page : general information
      --

      Task_Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Name));
      Task_Priority_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Priority));
      Task_Capacity_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Capacity));
      Task_Jitter_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.
            Jitter));
      Task_Deadline_String :=  To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Deadline));
      Task_Period_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.
            Period));
      Task_Criticality_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.
            Criticality));
      Task_Start_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Start_Time));
      Task_Blocking_Time_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Blocking_Time));
      Task_Parametric_Rule := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Activation_Rule));
      Task_Text_Memory_Size_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Text_Memory_Size));
      Task_Stack_Memory_Size_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Stack_Memory_Size));
      Task_Seed_Value_String := To_Unbounded_String(Get_Text(
            Update_Tasks_Set.Seed_Value));
      Task_Processor :=To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Cpu_Name));
      Task_Address_Space:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Tasks_Set.Address_Space_Menu)));


      -- Convert value from menu to Tasks_Type
      --
      Task_Type_Index:= Gint(4 - Selected_Button (Update_Tasks_Set.
            Clist_Task_Type_Menu));
      Task_Type := Tasks_Type'Val(Task_Type_Index);

      -- Convert value from menu to Policies
      --
      Policy_Index:= Gint(2 - Selected_Button (Update_Tasks_Set.
            Clist_Policy_Menu));
      Task_Policy := Policies'Val(Policy_Index);



      To_Integer(Task_Priority_String,Task_Priority, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Priority_String /= "" THEN
            Error_Message:=Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Capacity_String,Task_Capacity, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Capacity_String /= "" THEN
            Error_Message:=Lb_Capacity(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Jitter_String,Task_Jitter, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Jitter_String /= "" THEN
            Error_Message:=Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Deadline_String,Task_Deadline, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Deadline_String /= "" THEN
            Error_Message:=Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Criticality_String,Task_Criticality, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Criticality_String /= "" THEN
            Error_Message:=Lb_Criticality(Current_Language) &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Period_String,Task_Period, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Period_String /= "" THEN
            Error_Message:=Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Seed_Value_String,Task_Seed_Value, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Seed_Value_String /= "" THEN
            Error_Message:=Lb_Seed(Current_Language) &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;


      IF Get_Active(Update_Tasks_Set.Seed_Predictable)
            THEN
         Task_Predictable :=True;
      ELSE
         Task_Predictable :=False;
      END IF;


      To_Integer(Task_Start_Time_String,Task_Start_Time, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Start_Time_String /= "" THEN
            Error_Message:=Lb_Start_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Blocking_Time_String,Task_Blocking_Time, Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Blocking_Time_String /= "" THEN
            Error_Message:=Lb_Blocking_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Text_Memory_Size_String,Task_Text_Memory_Size,
         Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Text_Memory_Size_String /= "" THEN
            Error_Message:=Lb_Text_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;

      To_Integer(Task_Stack_Memory_Size_String,Task_Stack_Memory_Size,
         Integer_Ok);
      IF NOT Integer_Ok
            THEN
         IF Task_Stack_Memory_Size_String /= "" THEN
            Error_Message:=Lb_Stack_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            RETURN;
         END IF;
      END IF;


      --
      --- Second page of the widget : user defined parameters
      --
      Initialize(Task_Param);

      FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
               Param(I).Entry_Name));


         IF Current /= Empty_String THEN


            A_Param_Type:=  Gint(Selected_Button (Update_Tasks_Set.
                  Param(I).Clist_Parameter_Menu));


            IF A_Param_Type = 3
                  THEN
               A_Param:= NEW Parameter(Integer_Parameter);
            ELSE
               IF A_Param_Type = 2
                     THEN
                  A_Param:= NEW Parameter(Double_Parameter);
               ELSE
                  IF A_Param_Type = 1
                        THEN
                     A_Param:= NEW Parameter(String_Parameter);
                  ELSE
                     A_Param:= NEW Parameter(Boolean_Parameter);
                  END IF;
               END IF;
            END IF;


            A_Param.Name:=Current;

            Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
                  Param(I).Entry_Value));

            -- Integer case
            --
            IF A_Param_Type=3 THEN
               IF(Current="")
                     THEN
                  A_Param.Integer_Value:=0;
               ELSE
                  To_Integer(Current,A_Param.Integer_Value,Integer_Ok);
                  IF NOT Integer_Ok
                        THEN
                     Error_Message:=To_Unbounded_String("Parameter " &
                        Integer'Image(Integer(I+1)) & " ") &
                        Lb_Must_Be(Current_Language) & To_Unbounded_String(
                        " integer");
                     Ok:=False;
                     RETURN;
                  END IF;
               END IF;

               -- Double case
               --
            ELSE
               IF A_Param_Type=2 THEN
                  IF(Current="")
                        THEN
                     A_Param.Double_Value:=0.0;
                  ELSE
                     To_Double(Current,A_Param.Double_Value,Double_Ok);
                     IF NOT Double_Ok
                           THEN
                        Error_Message:=To_Unbounded_String("Parameter " &
                           Integer'Image(Integer(I+1)) & " ") &
                           Lb_Must_Be(Current_Language) &
                           To_Unbounded_String(" double");
                        Ok:=False;
                        RETURN;
                     END IF;
                  END IF;

                  -- String case
                  --
               ELSE
                  IF A_Param_Type=1 THEN
                     A_Param.String_Value:=Current;

                     -- Boolean case
                     --
                  ELSE
                     IF(Current="")
                           THEN
                        A_Param.Boolean_Value:=True;
                     ELSE
                        To_Boolean(Current,A_Param.Boolean_Value,
                           Boolean_Ok);
                        IF NOT Boolean_Ok
                              THEN
                           Error_Message:=To_Unbounded_String(
                              "Parameter " &
                              Integer'Image(Integer(I+1)) & " ") &
                              Lb_Must_Be(Current_Language) &
                              To_Unbounded_String(" boolean");
                           Ok:=False;
                           RETURN;
                        END IF;
                     END IF;

                  END IF;
               END IF;
            END IF;


            Add(Task_Param, A_Param);
         END IF;

      END LOOP;


      --
      -- Offset information
      --
      Initialize(Task_Offset);

      FOR I IN 0..Offsets_Range'Last-1 LOOP


         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Val));
         IF(Current="")
               THEN
            A_Offset.Value:=0;
         ELSE
            To_Integer(Current, A_Offset.Value,Integer_Ok);
            IF NOT Integer_Ok
                  THEN
               Error_Message:=To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ")") & Lb_Value(
                  Current_Language);
               Ok:=False;
               RETURN;
            END IF;
         END IF;

         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Offset(I).
               Entry_Act));
         IF(Current="")
               THEN
            A_Offset.Activation:=0;
         ELSE
            To_Integer(Current, A_Offset.Activation,Integer_Ok);
            IF NOT Integer_Ok
                  THEN
               Error_Message:=To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String("Offset(" & I'Img & ") ") &
                  Lb_Activation(
                  Current_Language);
               Ok:=False;
               RETURN;
            END IF;
         END IF;

         IF A_Offset.Activation/=0
               THEN
            Add(Task_Offset,A_Offset);
         END IF;

      END LOOP;




      Check_Task (Sys.Tasks,
         Integrity_Checks.Task_Name,
         Integrity_Checks.Task_Processor,
         Integrity_Checks.Task_Address_Space,
         Integrity_Checks.Task_Type,
         Integrity_Checks.Task_Start_Time,
         Integrity_Checks.Task_Capacity,
         Integrity_Checks.Task_Period,
         Integrity_Checks.Task_Deadline,
         Integrity_Checks.Task_Jitter,
         Integrity_Checks.Task_Blocking_Time,
         Integrity_Checks.Task_Priority,
         Integrity_Checks.Task_Criticality,
         Integrity_Checks.Task_Policy,
         Integrity_Checks.Task_Offset,
         Integrity_Checks.Task_Stack_Memory_Size,
         Integrity_Checks.Task_Text_Memory_Size,
         Integrity_Checks.Task_Param,
         Integrity_Checks.Task_Parametric_Rule,
         Integrity_Checks.Task_Seed_Value, Integrity_Checks.
         Task_Predictable);




   EXCEPTION
      WHEN OTHERS =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   END Check_Integrity_Constraints;




END Update_Tasks_Set_Pkg.Callbacks.Integrity_Checks;

