------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Glib;
USE Glib;

WITH Mmi_Tools;
USE Mmi_Tools;

WITH Text_Io;
USE Text_Io;

WITH Translate;
USE Translate;
WITH Framework_Config;
USE Framework_Config;

WITH Task_Offsets;
USE Task_Offsets;
USE Task_Offsets.Offsets_Table_Package;

WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;

WITH Unbounded_Strings;
USE Unbounded_Strings;
USE Unbounded_Strings.Strings_Table_Package;

WITH Project;
USE Project;

WITH Tasks;
USE Tasks;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Address_Spaces;
USE Address_Spaces;
WITH Address_Space_Set;
USE Address_Space_Set;
USE Address_Space_Set.Generic_Address_Space_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Graph_Window_Pkg;
USE Graph_Window_Pkg;
WITH Graph_Item_Pkg;
USE Graph_Item_Pkg;
WITH Graph_Add_Pkg;
USE Graph_Add_Pkg;
WITH My_Canvas;
USE My_Canvas;



WITH Buffer_Set;
USE Buffer_Set;
USE Buffer_Set.Generic_Buffer_Set;

WITH Double_Util;
USE Double_Util;
WITH Ada.Numerics.Float_Random;
USE Ada.Numerics.Float_Random;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;


WITH Resource_Set;
USE Resource_Set;
USE Resource_Set.Generic_Resource_Set;

WITH Network_Set;
USE Network_Set;
USE Network_Set.Generic_Network_Set;

WITH Task_Dependencies;
USE Task_Dependencies;


WITH  Gnat.Current_Exception;
USE  Gnat.Current_Exception;

WITH Systems;
USE Systems;


WITH Update_Tasks_Set_Pkg.Callbacks.Integrity_Checks;




PACKAGE BODY Update_Tasks_Set_Pkg.Callbacks IS


   PROCEDURE Page_Switch (
         Notebook : ACCESS Gtk_Notebook_Record'Class;
         Page     : IN     Gtk.Gtk_Notebook_Page;
         Page_Num : IN     Gtk_Notebook) IS
      PRAGMA Warnings (Off, Page_Num);
      Old_Page : CONSTANT Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   :          Gtk_Pixmap;
   BEGIN
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);
      Pixmap := Gtk_Pixmap
         (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);

      IF Old_Page /= NULL THEN
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Closed,
            Update_Tasks_Set.Book_Closed_Mask);
         Pixmap := Gtk_Pixmap
            (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set (Pixmap, Update_Tasks_Set.Book_Closed,
            Update_Tasks_Set.Book_Closed_Mask);
      END IF;
   END Page_Switch;


   PROCEDURE Show_Tasks (
         Update_Tasks_Set : ACCESS Update_Tasks_Set_Record'Class) IS

      A_Task          : Task_Ptr;
      My_Iterator     : Tasks_Iterator;
      Line            : Gint;
      Task_Share      : Chars_Ptr_Array (1 .. 12);
      Capacity        : Unbounded_String;
      Start_Time      : Unbounded_String;
      Jitter          : Unbounded_String;
      Deadline        : Unbounded_String;
      Period          : Unbounded_String;
      Priority        : Unbounded_String;
      Blocking_Time   : Unbounded_String;
      Parametric_Rule : Unbounded_String;

      Policy    : Unbounded_String;
      Task_Type : Unbounded_String;

   BEGIN

      Clear(Update_Tasks_Set.Tasks_List);

      Reset_Iterator(Sys.Tasks, My_Iterator);
      IF NOT Is_Empty(Sys.Tasks) THEN
         LOOP

            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            Capacity      := To_Unbounded_String(
               Natural'Image(A_Task.Capacity));
            Deadline      := To_Unbounded_String(
               Natural'Image(A_Task.Deadline));
            Start_Time    := To_Unbounded_String(
               Natural'Image(A_Task.Start_Time));
            Priority      := To_Unbounded_String(
               Natural'Image(Natural(A_Task.Priority)));
            Blocking_Time := To_Unbounded_String(
               Natural'Image(A_Task.Blocking_Time));

            CASE A_Task.Task_Type IS
               WHEN Aperiodic_Type =>
                  Jitter    := To_Unbounded_String(" 0");
                  Period    := To_Unbounded_String(" 0");
                  Parametric_Rule   := To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Aperiodic");
               WHEN Periodic_Type =>
                  Jitter    := To_Unbounded_String(
                     Natural'Image(Periodic_Task_Ptr(A_Task).Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(Periodic_Task_Ptr(A_Task).Period));
                  Parametric_Rule   := To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Periodic");
               WHEN Poisson_Type =>
                  Jitter    := To_Unbounded_String(
                     Natural'Image(Poisson_Task_Ptr(A_Task).Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(Poisson_Task_Ptr(A_Task).Period));
                  Parametric_Rule   := To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Poisson");
               WHEN Sporadic_Type =>
                  Jitter    := To_Unbounded_String(
                     Natural'Image(Sporadic_Task_Ptr(A_Task).Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(Sporadic_Task_Ptr(A_Task).Period));
                  Parametric_Rule   := To_Unbounded_String("");
                  Task_Type := To_Unbounded_String("Sporadic");
               WHEN Parametric_Type =>
                  Jitter    := To_Unbounded_String(
                     Natural'Image(Poisson_Task_Ptr(A_Task).Jitter));
                  Period    := To_Unbounded_String(
                     Natural'Image(Poisson_Task_Ptr(A_Task).Period));
                  Parametric_Rule   := Parametric_Task_Ptr(A_Task).
                     Activation_Rule;
                  Task_Type := To_Unbounded_String("Parametric");
            END CASE;


            CASE A_Task.Policy IS
               WHEN Sched_Fifo  =>
                  Policy := To_Unbounded_String("Fifo");
               WHEN Sched_Rr =>
                  Policy := To_Unbounded_String("RR");
               WHEN Sched_Others =>
                  Policy := To_Unbounded_String("Other");
            END CASE;


            -- Create the line
            --
            Task_Share := To_String(A_Task.Name)+To_String(Task_Type)+
               To_String(A_Task.Cpu_Name)+To_String(Policy)+
               To_String(Priority)+To_String(Capacity)+To_String(Jitter)+
               To_String(Deadline)+To_String(Period)+To_String(Start_Time)+
               To_String(Blocking_Time)+To_String(Parametric_Rule);

            -- Display the Line
            --
            Line := Append(Update_Tasks_Set.Tasks_List, Task_Share);

            EXIT WHEN Is_Last_Element(Sys.Tasks, My_Iterator);

            Next_Element(Sys.Tasks, My_Iterator);

         END LOOP;
      END IF;
   END Show_Tasks;


   PROCEDURE On_Click_List (
         List : ACCESS Gtk_Clist_Record'Class) IS

      Name            : Unbounded_String;
      Capacity        : Unbounded_String;
      Jitter          : Unbounded_String;
      Deadline        : Unbounded_String;
      Period          : Unbounded_String;
      Start_Time      : Unbounded_String;
      Priority        : Unbounded_String;
      Blocking_Time   : Unbounded_String;
      Parametric_Rule : Unbounded_String;
      Row             : Gint;

      A_Task  : Task_Ptr;
      Current : Gint;

      Location_Items  : Strings_Table;
      Parameter_Items : ARRAY (User_Defined_Parameters_Range) OF Strings_Table;

      A_Address_Space        : Address_Space_Ptr;
      Address_Space_Iterator : Address_Space_Set.Generic_Address_Space_Set.Iterator;

      Empty_Menu        : String_List.Glist;
      Current_Task_Type : Gint;
      Current_Policy    : Gint;

   BEGIN

      Row:=Get_Focus_Row(Update_Tasks_Set.Tasks_List);
      Name := To_Unbounded_String(Get_Text(Update_Tasks_Set.Tasks_List,
            Row,0));

      Update_Tasks_Set.Current_Task := Name;

      Priority      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,4));
      Capacity      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,5));
      Jitter        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,6));
      Deadline      := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,7));
      Period        := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,8));
      Start_Time    := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,9));
      Blocking_Time := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,10));
      Parametric_Rule := To_Unbounded_String(Get_Text(Update_Tasks_Set.
            Tasks_List,Row,11));

      Gtk.Gentry.Set_Text(Update_Tasks_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Priority, To_String(Priority));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Capacity, To_String(Capacity));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Jitter, To_String(Jitter));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Deadline, To_String(Deadline));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Period, To_String(Period));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Start_Time, To_String(
            Start_Time));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Blocking_Time, To_String(
            Blocking_Time));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Activation_Rule, To_String(
            Parametric_Rule));


      A_Task:=Search_Task(Sys.Tasks, Update_Tasks_Set.Current_Task);


      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces, A_Task.
         Address_Space_Name);
      Set_Text(Update_Tasks_Set.Cpu_Name, To_String(A_Address_Space.
            Cpu_Name));

      Update_Tasks_Set.Clist_Address_Space_Menu:=Empty_Menu;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name =  A_Task.Address_Space_Name THEN
            String_List.Append(Update_Tasks_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;
      Reset_Iterator(Sys.Address_Spaces, Address_Space_Iterator);
      LOOP
         Current_Element(Sys.Address_Spaces, A_Address_Space,
            Address_Space_Iterator);
         IF A_Address_Space.Name /=  A_Task.Address_Space_Name THEN
            String_List.Append(Update_Tasks_Set.
               Clist_Address_Space_Menu, To_String(
                  A_Address_Space.Name));
         END IF;
         EXIT WHEN Is_Last_Element(Sys.Address_Spaces,
            Address_Space_Iterator);

         Next_Element(Sys.Address_Spaces, Address_Space_Iterator);
      END LOOP;

      Set_Popdown_Strings (Update_Tasks_Set.Address_Space_Menu,
         Update_Tasks_Set.Clist_Address_Space_Menu);



      Show_All(Update_Tasks_Set);


      -- Rebuild type menu
      --
      Initialize(Location_Items);
      Add(Location_Items,To_Unbounded_String("Periodic"));
      Add(Location_Items,To_Unbounded_String("Aperiodic"));
      Add(Location_Items,To_Unbounded_String("Sporadic"));
      Add(Location_Items,To_Unbounded_String("Poisson"));
      Add(Location_Items,To_Unbounded_String("Parametric"));

      Current_Task_Type := Gint'(Tasks_Type'Pos(A_Task.Task_Type));

      Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Task_Type_Menu);
      Update_Tasks_Set.Clist_Task_Type_Menu:= Widget_Slist.Null_List;

      Build_Option_Menu (Update_Tasks_Set.Task_Type_Menu,
         Update_Tasks_Set.
         Clist_Task_Type_Menu, Location_Items, Current_Task_Type, NULL);
      Show_All(Update_Tasks_Set.Task_Type_Menu);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Task_Type_Menu,1,2,1,2);

      Show_All(Update_Tasks_Set);


      -- Rebuild policy menu
      --
      Initialize(Location_Items);
      Add(Location_Items,To_Unbounded_String("SCHED_FIFO"));
      Add(Location_Items,To_Unbounded_String("SCHED_RR"));
      Add(Location_Items,To_Unbounded_String("SCHED_OTHERS"));

      CASE A_Task.Policy IS
         WHEN Sched_Fifo   =>
            Current_Policy:=0;
         WHEN Sched_Rr     =>
            Current_Policy:=1;
         WHEN Sched_Others =>
            Current_Policy:=2;
      END CASE;

      Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Policy_Menu);
      Update_Tasks_Set.Clist_Policy_Menu:= Widget_Slist.Null_List;

      Build_Option_Menu (Update_Tasks_Set.Policy_Menu,
         Update_Tasks_Set.
         Clist_Policy_Menu, Location_Items, Current_Policy, NULL);
      Show_All(Update_Tasks_Set.Policy_Menu);
      Attach_Defaults(Update_Tasks_Set.Task_Table, Update_Tasks_Set.
         Policy_Menu,1,2,4,5);
      Show_All(Update_Tasks_Set);


      -- Seed value and predictable attribute
      --
      IF (A_Task.Task_Type = Poisson_Type) OR
            (A_Task.Task_Type = Parametric_Type)
            OR
            (A_Task.Task_Type = Sporadic_Type)
            THEN

         IF (Poisson_Task_Ptr(A_Task).Predictable)
               THEN
            Set_Active(Update_Tasks_Set.Seed_Predictable,True);
            Set_Active(Update_Tasks_Set.Seed_Unpredictable,False);
         ELSE
            Set_Active(Update_Tasks_Set.Seed_Predictable,False);
            Set_Active(Update_Tasks_Set.Seed_Unpredictable,True);
         END IF;

         Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,
            Poisson_Task_Ptr(A_Task).Seed'Img);

      ELSE
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,"");
      END IF;

      -- Criticality level
      --
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Criticality,
         A_Task.Criticality'Img);


      -- Task memory information
      --
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Text_Memory_Size,
         A_Task.Text_Memory_Size'Img);
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Stack_Memory_Size,
         A_Task.Stack_Memory_Size'Img);


      -- Offset information
      --
      FOR I IN 0..Offsets_Range'Last-1 LOOP
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Act, "");
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Val, "");
      END LOOP;
      FOR I IN 0 .. A_Task.Offset.Nb_Entries-1 LOOP
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Act,
            A_Task.Offset.Entries(I).Activation'Img);
         Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).Entry_Val,
            A_Task.Offset.Entries(I).Value'Img);
      END LOOP;


      -- User defined parameters
      --
      FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP
         Set_Text(Update_Tasks_Set.Param(I).Entry_Name, "");
         Set_Text(Update_Tasks_Set.Param(I).Entry_Value, "");
      END LOOP;

      FOR I IN A_Task.Param.Nb_Entries .. User_Defined_Parameters_Range'
            Last-1 LOOP
         Initialize(Parameter_Items(I));
         Add(Parameter_Items(I),To_Unbounded_String("Integer"));
         Add(Parameter_Items(I),To_Unbounded_String("Double"));
         Add(Parameter_Items(I),To_Unbounded_String("String"));
         Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

         Destroy(Update_Tasks_Set.Param(I).Parameter_Menu);
         Update_Tasks_Set.Param(I).Clist_Parameter_Menu :=
            Widget_Slist.Null_List;
         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Param(I).
            Parameter_Menu);

         Build_Option_Menu (Update_Tasks_Set.Param(I).Parameter_Menu,
            Update_Tasks_Set.Param(I).Clist_Parameter_Menu,
            Parameter_Items(I),
            -1, NULL);
         Show_All(Update_Tasks_Set.Param(I).Parameter_Menu);
         Attach (Update_Tasks_Set.Parameter_Table, Update_Tasks_Set.
            Param(I).Parameter_Menu, 1, 2,
            1+Guint(I), 2+Guint(I),
            Expand OR Fill, 0,
            0, 0);
      END LOOP;
      Show_All(Update_Tasks_Set);


      FOR I IN 0..A_Task.Param.Nb_Entries-1 LOOP
         Set_Text(Update_Tasks_Set.Param(I).Entry_Name, To_String(
               A_Task.Param.Entries(I).Name));


         IF A_Task.Param.Entries(I).Ptype=Boolean_Parameter THEN
            Set_Text(Update_Tasks_Set.Param(I).Entry_Value, A_Task.
               Param.Entries(I).Boolean_Value'Img);
            Current:=3;
         ELSE
            IF A_Task.Param.Entries(I).Ptype=Integer_Parameter THEN
               Set_Text(Update_Tasks_Set.Param(I).Entry_Value, A_Task.
                  Param.Entries(I).Integer_Value'Img);
               Current:=0;
            ELSE
               IF A_Task.Param.Entries(I).Ptype=Double_Parameter THEN
                  Set_Text(Update_Tasks_Set.Param(I).Entry_Value,
                     To_String(Format(A_Task.Param.Entries(I).
                           Double_Value)));
                  Current:=1;
               ELSE
                  Set_Text(Update_Tasks_Set.Param(I).Entry_Value,
                     To_String(A_Task.Param.Entries(I).String_Value));
                  Current:=2;
               END IF;
            END IF;
         END IF;

         Initialize(Parameter_Items(I));
         Add(Parameter_Items(I),To_Unbounded_String("Integer"));
         Add(Parameter_Items(I),To_Unbounded_String("Double"));
         Add(Parameter_Items(I),To_Unbounded_String("String"));
         Add(Parameter_Items(I),To_Unbounded_String("Boolean"));

         Destroy(Update_Tasks_Set.Param(I).Parameter_Menu);
         Update_Tasks_Set.Param(I).Clist_Parameter_Menu :=
            Widget_Slist.
            Null_List;
         Gtk.Option_Menu.Gtk_New(Update_Tasks_Set.Param(I).
            Parameter_Menu);

         Build_Option_Menu (Update_Tasks_Set.Param(I).Parameter_Menu,
            Update_Tasks_Set.Param(I).Clist_Parameter_Menu,
            Parameter_Items(I),
            Current, NULL);
         Show_All(Update_Tasks_Set.Param(I).Parameter_Menu);
         Attach (Update_Tasks_Set.Parameter_Table, Update_Tasks_Set.
            Param(I).Parameter_Menu, 1, 2,
            1+Guint(I), 2+Guint(I),
            Expand OR Fill, 0,
            0, 0);


      END LOOP;
      Show_All(Update_Tasks_Set);




   END On_Click_List;


   PROCEDURE On_Close_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Tasks_Set);
   END On_Close_Pressed;


   PROCEDURE On_Add_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      Ok            : Boolean          := True;
      Error_Message : Unbounded_String;

   BEGIN


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Tasks_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;

      --
      -- Update task set
      --

      Add_Task (Sys.Tasks, Integrity_Checks.Task_Name,
         Integrity_Checks.Task_Processor,
         Integrity_Checks.Task_Address_Space,
         Integrity_Checks.Task_Type,
         Integrity_Checks.Task_Start_Time,
         Integrity_Checks.Task_Capacity,
         Integrity_Checks.Task_Period,
         Integrity_Checks.Task_Deadline,
         Integrity_Checks.Task_Jitter,
         Integrity_Checks.Task_Blocking_Time,
         Integrity_Checks.Task_Priority,
         Integrity_Checks.Task_Criticality,
         Integrity_Checks.Task_Policy,
         Integrity_Checks.Task_Offset,
         Integrity_Checks.Task_Stack_Memory_Size,
         Integrity_Checks.Task_Text_Memory_Size,
         Integrity_Checks.Task_Param,
         Integrity_Checks.Task_Parametric_Rule,
         Integrity_Checks.Task_Seed_Value, Integrity_Checks.
         Task_Predictable);

      Update_Tasks_Set.Current_Task := Integrity_Checks.Task_Name;

      Show_Tasks(Update_Tasks_Set);

      IF Add_New_Item /= NULL THEN
         Set_Name(Items_List(Last_Item-1), Integrity_Checks.Task_Name);
         Refresh_Canvas(Graph_Window.Buffer);
      END IF;



   EXCEPTION
      WHEN Task_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Add_Pressed;


   PROCEDURE On_Seed_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
      Seed : Generator;
      Nmb  : Integer;
   BEGIN

      -- Give random value for the tasks seed
      --
      Reset(Seed);

      -- Range from integer'first .. integer'last
      --
      Nmb := Integer(Double(Random(Seed))*Double(Integer'Last-Integer'
            First)+
         Double(Integer'First));
      Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value,Nmb'Img);

   END On_Seed_Pressed;


   PROCEDURE On_Delete_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS

      My_Iterator : Task_Set.Generic_Task_Set.Iterator;
      A_Task      : Task_Ptr;

   BEGIN


      IF Update_Tasks_Set.Current_Task = "" THEN
         Show_Message_Box(Lb_Select_Task_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Tasks) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;

      Reset_Iterator(Sys.Tasks, My_Iterator);
      IF NOT Is_Empty(Sys.Tasks) THEN
         LOOP

            Current_Element(Sys.Tasks, A_Task, My_Iterator);
            IF A_Task.Name = Update_Tasks_Set.Current_Task THEN

               Delete_Task(Sys, A_Task);
               Rebuild_Canvas(Graph_Window);


               -- Clear main entrys
               --
               Update_Tasks_Set.Current_Task := To_Unbounded_String("");

               Gtk.Gentry.Set_Text(Update_Tasks_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Priority, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Capacity, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Jitter, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Deadline, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Criticality, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Period, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Start_Time, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Blocking_Time, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Activation_Rule, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Seed_Value, "");
               Gtk.Gentry.Set_Text(Update_Tasks_Set.Cpu_Name, "");

               -- Clear User defined parameters
               --
               FOR I IN 0..User_Defined_Parameters_Range'Last-1 LOOP
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Param(I).
                     Entry_Name, "");
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Param(I).
                     Entry_Value, "");
               END LOOP;

               -- Clear Offset parameters
               --
               FOR I IN 0..Offsets_Range'Last-1 LOOP
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).
                     Entry_Val, "");
                  Gtk.Gentry.Set_Text(Update_Tasks_Set.Offset(I).
                     Entry_Act, "");
               END LOOP;

               EXIT;
            END IF;

            EXIT WHEN Is_Last_Element(Sys.Tasks, My_Iterator);
            Next_Element(Sys.Tasks, My_Iterator);

         END LOOP;
         Show_Tasks(Update_Tasks_Set);
      END IF;

   END On_Delete_Pressed;


   PROCEDURE On_Cancel_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      Duplicate(Update_Tasks_Set.Undo, Sys);
      Rebuild_Canvas(Graph_Window);
      Destroy(Update_Tasks_Set);
   END On_Cancel_Pressed;


   PROCEDURE On_Advanced_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS
   BEGIN
      IF Visible_Is_Set(Update_Tasks_Set.Vbox_List) THEN
         Show_Advanced(Update_Tasks_Set,False);
      ELSE
         Show_Advanced(Update_Tasks_Set,True);
      END IF;
   END On_Advanced_Pressed;



   PROCEDURE On_Modify_Pressed (
         Object : ACCESS Gtk_Button_Record'Class) IS


      A_Task        : Task_Ptr;
      Created_Task  : Task_Ptr;
      My_Iterator   : Task_Set.Generic_Task_Set.Iterator;
      Ok            : Boolean                            := True;
      Error_Message : Unbounded_String;

   BEGIN



      IF Update_Tasks_Set.Current_Task = "" THEN
         Show_Message_Box(Lb_Select_Task_First(Current_Language));
         RETURN;
      END IF;

      IF Is_Empty(Sys.Tasks) THEN
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         RETURN;
      END IF;



      Integrity_Checks.Check_Integrity_Constraints (
         Update_Tasks_Set, Error_Message, Ok);
      IF NOT Ok THEN
         Show_Message_Box(Error_Message);
         RETURN;
      END IF;



      --
      -- Update task set
      --

      Reset_Iterator(Sys.Tasks, My_Iterator);
      LOOP

         Current_Element(Sys.Tasks, A_Task, My_Iterator);
         IF A_Task.Name = Update_Tasks_Set.Current_Task THEN

            Delete(Sys.Tasks, A_Task);


            Add_Task (Sys.Tasks, Created_Task,
               Integrity_Checks.Task_Name,
               Integrity_Checks.Task_Processor,
               Integrity_Checks.Task_Address_Space,
               Integrity_Checks.Task_Type,
               Integrity_Checks.Task_Start_Time,
               Integrity_Checks.Task_Capacity,
               Integrity_Checks.Task_Period,
               Integrity_Checks.Task_Deadline,
               Integrity_Checks.Task_Jitter,
               Integrity_Checks.Task_Blocking_Time,
               Integrity_Checks.Task_Priority,
               Integrity_Checks.Task_Criticality,
               Integrity_Checks.Task_Policy,
               Integrity_Checks.Task_Offset,
               Integrity_Checks.Task_Stack_Memory_Size,
               Integrity_Checks.Task_Text_Memory_Size,
               Integrity_Checks.Task_Param,
               Integrity_Checks.Task_Parametric_Rule,
               Integrity_Checks.Task_Seed_Value, Integrity_Checks.
               Task_Predictable);



            Update_All_Task_Dependencies(Sys.Dependencies, Created_Task);


            Update_Tasks_Set.Current_Task := Integrity_Checks.Task_Name;
            EXIT;
         END IF;


         EXIT WHEN Is_Last_Element(Sys.Tasks, My_Iterator);
         Next_Element(Sys.Tasks, My_Iterator);

      END LOOP;

      Show_Tasks(Update_Tasks_Set);



   EXCEPTION
      WHEN Task_Set.Invalid_Parameter =>
         BEGIN
            Show_Message_Box(Exception_Message);
            RETURN;
         END;
      WHEN OTHERS =>
         BEGIN
            Show_Message_Box(Exception_Name & Exception_Message);
            RETURN;
         END;


   END On_Modify_Pressed;



   PROCEDURE On_Change_Cpu (
         Widget : ACCESS Gtk_Editable_Record'Class) IS

      Task_Address_Space : Unbounded_String;
      A_Address_Space    : Address_Space_Ptr;

   BEGIN
      Task_Address_Space:=To_Unbounded_String(Get_Text(Get_Entry(
               Update_Tasks_Set.Address_Space_Menu)));
      A_Address_Space:=Search_Address_Space(Sys.Address_Spaces,
         Task_Address_Space);
      Set_Text(Update_Tasks_Set.Cpu_Name, To_String(A_Address_Space.
            Cpu_Name));
   END On_Change_Cpu;


END Update_Tasks_Set_Pkg.Callbacks;
