------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Gtk.Combo;
USE Gtk.Combo;

WITH Gdk;
WITH Gdk.Bitmap;
USE Gdk.Bitmap;
WITH Gdk.Pixmap;
USE Gdk.Pixmap;
WITH Gtk.Enums;
USE Gtk.Enums;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Notebook;
USE Gtk.Notebook;
WITH Gtk.Pixmap;
USE Gtk.Pixmap;
WITH Gtk;
USE Gtk;

WITH Gtk.Window;
USE Gtk.Window;
WITH Gtk.Table;
USE Gtk.Table;
WITH Gtk.Label;
USE Gtk.Label;
WITH Gtk.Gentry;
USE Gtk.Gentry;
WITH Gtk.Separator;
USE Gtk.Separator;
WITH Gtk.Button;
USE Gtk.Button;
WITH Gtk.Box;
USE Gtk.Box;
WITH Gtk.Hbutton_Box;
USE Gtk.Hbutton_Box;
WITH Gtk.Radio_Button;
USE Gtk.Radio_Button;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Gtk.Option_Menu;
USE Gtk.Option_Menu;
WITH Gtk.Widget;
USE Gtk.Widget;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;

WITH Gtk.Clist;
USE Gtk.Clist;
WITH Gtkada.Types;
USE Gtkada.Types;
WITH Gtk.Frame;
USE Gtk.Frame;
WITH Gtk.Scrolled_Window;
USE Gtk.Scrolled_Window;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Gtk.Enums;
USE Gtk.Enums;

WITH Tasks_Widget_Pkg;
USE Tasks_Widget_Pkg;

WITH Systems;
USE Systems;


PACKAGE Update_Tasks_Set_Pkg IS

   TYPE Update_Tasks_Set_Record IS NEW Gtk_Window_Record WITH PRIVATE;
   TYPE Update_Tasks_Set_Access IS ACCESS ALL Update_Tasks_Set_Record'Class;

   PROCEDURE Gtk_New (
         Update_Tasks_Set :    OUT Update_Tasks_Set_Access;
         Title            : IN     Unbounded_String);
   PROCEDURE Initialize (
         Update_Tasks_Set : ACCESS Update_Tasks_Set_Record'Class;
         Title            : IN     Unbounded_String);
   PROCEDURE Show_Advanced (
         Update_Tasks_Set : IN     Update_Tasks_Set_Access;
         Advanced         : IN     Boolean);

   Update_Tasks_Set : Update_Tasks_Set_Access;
   Parent           : Unbounded_String;


PRIVATE

   TYPE Update_Tasks_Set_Record IS NEW Gtk_Window_Record WITH
      RECORD
         Scroll : Gtk_Scrolled_Window;

         Tasks_List               : Gtk.Clist.Gtk_Clist;
         Task_Type_Menu           : Gtk_Option_Menu;
         Clist_Task_Type_Menu     : Widget_Slist.Gslist;
         Address_Space_Menu       : Gtk.Combo.Gtk_Combo;
         Clist_Address_Space_Menu : String_List.Glist;
         Cpu_Name                 : Gtk_Entry;
         Policy_Menu              : Gtk_Option_Menu;
         Clist_Policy_Menu        : Widget_Slist.Gslist;
         Task_Items               : String_List.Glist;

         Current_Task : Unbounded_String;

         Name              : Gtk_Entry;
         Start_Time        : Gtk_Entry;
         Capacity          : Gtk_Entry;
         Period            : Gtk_Entry;
         Deadline          : Gtk_Entry;
         Jitter            : Gtk_Entry;
         Priority          : Gtk_Entry;
         Criticality       : Gtk_Entry;
         Blocking_Time     : Gtk_Entry;
         Seed_Value        : Gtk_Entry;
         Activation_Rule   : Gtk_Entry;
         Stack_Memory_Size : Gtk_Entry;
         Text_Memory_Size  : Gtk_Entry;

         Frame_Task      : Gtk.Frame.Gtk_Frame;
         Task_Table      : Gtk.Table.Gtk_Table;
         Offset_Table    : Gtk.Table.Gtk_Table;
         Parameter_Table : Gtk.Table.Gtk_Table;

         Vbox_Main    : Gtk.Box.Gtk_Box;
         Hbox_Task    : Gtk.Box.Gtk_Box;
         Hbox_Confirm : Gtk.Box.Gtk_Box;
         Vbox_List    : Gtk.Box.Gtk_Box;
         Hbox_Modify  : Gtk.Box.Gtk_Box;
         Vbox_Update  : Gtk.Box.Gtk_Box;
         Hbox_File    : Gtk.Box.Gtk_Box;
         Hbox_Seed    : Gtk.Box.Gtk_Box;

         Seed_Group         : Widget_Slist.Gslist;
         Seed_Unpredictable : Gtk_Radio_Button;
         Seed_Predictable   : Gtk_Radio_Button;

         Hbutton_Box_Modify : Gtk_Hbutton_Box;

         Modify_Button   : Gtk_Button;
         Advanced_Button : Gtk_Button;
         Close_Button    : Gtk_Button;
         Cancel_Button   : Gtk_Button;
         Add_Button      : Gtk_Button;
         Delete_Button   : Gtk_Button;
         Seed_Button     : Gtk_Button;

         Hseparator1 : Gtk_Hseparator;
         Vseparator  : Gtk_Vseparator;
         Vseparator2 : Gtk_Vseparator;

         Child_Page : Gtk_Frame;
         Label_Box  : Gtk_Box;
         Menu_Box   : Gtk_Box;
         Pixmap     : Gtk_Pixmap;
         Vbox       : Gtk_Box;
         Button     : Gtk_Button;

         Lab : Gtk_Label;

         Offset : Widget_Offset_Table;
         Param  : Widget_Parameter_Table;

         Book_Opened      : Gdk_Pixmap;
         Book_Opened_Mask : Gdk_Bitmap;
         Book_Closed      : Gdk_Pixmap;
         Book_Closed_Mask : Gdk_Bitmap;
         Notebook         : Gtk_Notebook;

         Undo : System;

         Scroll_List : Gtk_Scrolled_Window;
         List_Titles : Chars_Ptr_Array (1 .. 12);

         Advanced : Boolean := False;
      END RECORD;



END Update_Tasks_Set_Pkg;
