------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


GENERIC
   TYPE Element IS PRIVATE;
   WITH PROCEDURE Put (
         E : IN     Element);

PACKAGE Binary_Trees IS


   No_Cell : EXCEPTION;


   TYPE Cell IS PRIVATE;
   TYPE Binary_Tree IS ACCESS Cell;



   PROCEDURE Add_Left (
         L : IN OUT Binary_Tree;
         E : IN     Element);
   PROCEDURE Add_Right (
         L : IN OUT Binary_Tree;
         E : IN     Element);


   FUNCTION Get_Left (
         L : IN     Binary_Tree)
     RETURN Binary_Tree;
   FUNCTION Get_Right (
         L : IN     Binary_Tree)
     RETURN Binary_Tree;


   PROCEDURE Put (
         L : IN     Binary_Tree);

   PROCEDURE Delete (
         L : IN OUT Binary_Tree;
         E : IN     Element);

   PROCEDURE Duplicate (
         Src  : IN     Binary_Tree;
         Dest : IN OUT Binary_Tree);

PRIVATE

   TYPE Cell IS
      RECORD
         Left,
         Right : Binary_Tree;
         Info  : Element;
      END RECORD;


END Binary_Trees;
