------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Text_Io;
USE Text_Io;

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;



PACKAGE BODY Discrete_Util IS


   PACKAGE Integer_Io IS NEW Text_Io.Integer_Io(Discrete_Element);


   FUNCTION Format (
         F : IN     Discrete_Element)
     RETURN Unbounded_String IS
      S      : String (1 .. 10000);
      Result : Unbounded_String;
   BEGIN
      Integer_Io.Put(S,F,10);

      -- Suppress blank
      --
      FOR I IN 1..S'Length LOOP
         IF S(I) /= ' '
               THEN
            Append(Result,S(I));
         END IF;
      END LOOP;

      RETURN(Result);
   END Format;



   FUNCTION Gcd (
         Left  : Discrete_Element;
         Right : Discrete_Element)
     RETURN Discrete_Element IS
      Min       : Discrete_Element := Discrete_Element'Min (Left, Right);
      Max       : Discrete_Element := Discrete_Element'Max (Left, Right);
      Remainder : Discrete_Element;
   BEGIN
      Reduce : LOOP
         IF Min <= 0 THEN
            RETURN Max;
         END IF;

         Remainder := Max rem Min;
         Max := Min;
         Min := Remainder;
      END LOOP Reduce;
   END Gcd;



   FUNCTION Lcm (
         A : IN     Discrete_Element;
         B : IN     Discrete_Element)
     RETURN Discrete_Element IS

      Tmp    : Double           := Double (A) * Double (B) / Double (Gcd
        (A, B));
      Result : Discrete_Element;
   BEGIN
      Tmp:=Double'Floor(Tmp);
      Result:=Discrete_Element(Tmp);

      RETURN Result;
   END Lcm;



   FUNCTION Lcm (
         A : IN     Discrete_Element;
         B : IN     Discrete_Element)
     RETURN Double IS

      Tmp : CONSTANT Double := Double (A) * Double (B) / Double (Gcd (A,
        B));

   BEGIN
      RETURN Double'Floor(Tmp);
   END Lcm;




END Discrete_Util;


