------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Text_Io;
USE Text_Io;

WITH Real_Util;

PACKAGE BODY Double_Util IS

   PACKAGE Double_Io IS NEW Text_Io.Float_Io(Double);
   PACKAGE Double_Real  IS NEW Real_Util(Double);


   FUNCTION Format (
         F     : IN     Double;
         After : IN     Integer := 5)
     RETURN Unbounded_String IS
   BEGIN
      RETURN Double_Real.Format(F,After);
   END Format;



   PROCEDURE Initialize (
         F :    OUT Double) IS
   BEGIN
      F:=0.0;
   END Initialize;

   PROCEDURE Put (
         F : IN     Double) IS
   BEGIN
      Double_Io.Put(F, Exp=>5);
   END Put;

   PROCEDURE Put (
         F : IN     Double_Ptr) IS
   BEGIN
      Double_Io.Put(F.All, Exp=>5);
   END Put;

   PROCEDURE Put_Line (
         F : IN     Double) IS
   BEGIN
      Double_Io.Put(F, Exp=>5);
      New_Line;
   END Put_Line;

   FUNCTION Equal (
         Num1,
         Num2     : IN     Double;
         Accuracy : IN     Integer := 10000)
     RETURN Boolean IS

   BEGIN
      RETURN Double_Real.Equal(Num1,Num2,Accuracy);
   END Equal;

   FUNCTION My_Mod (
         A : Double;
         B : Double)
     RETURN Double IS
      Result : Double;
   BEGIN
      Result := A - Double'Floor(A/B)*B;
      RETURN(Result);
   END My_Mod;

END Double_Util;
