------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;

WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;


WITH Unchecked_Deallocation;



PACKAGE Double_Util IS


   -- Pointer to a double
   --
   TYPE Double_Ptr IS ACCESS Double;


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Double, Double_Ptr);


   -- Initialize a double
   --
   PROCEDURE Initialize (
         F :    OUT Double);


   -- Convert a double into a string
   --
   FUNCTION Format (
         F     : IN     Double;
         After : IN     Integer := 5)
     RETURN Unbounded_String;


   -- Display a double
   --
   PROCEDURE Put (
         F : IN     Double);
   PROCEDURE Put (
         F : IN     Double_Ptr);
   PROCEDURE Put_Line (
         F : IN     Double);


   -- New equal function for numeric stability
   --
   FUNCTION Equal (
         Num1,
         Num2     : IN     Double;
         Accuracy : IN     Integer := 10000)
     RETURN Boolean;

   -- return A Modulo B
   --
   FUNCTION My_Mod (
         A : Double;
         B : Double)
     RETURN Double;


END Double_Util;
