------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;




PACKAGE BODY Fifos IS





   PROCEDURE Reset (
         My_Fifo : IN OUT Fifo) IS
   BEGIN
      My_Fifo.Queue:=0;
      My_Fifo.Head:=0;
      My_Fifo.Size:=0;
   END Reset;


   PROCEDURE Put (
         My_Fifo : IN     Fifo) IS

      Index : Element_Range := My_Fifo.Queue;

   BEGIN
      Put_Line("Fifo : queue = " & My_Fifo.Queue'Img
         & "; head = " & My_Fifo.Head'Img
         & "; size = " & My_Fifo.Size'Img);

      WHILE (Index /= My_Fifo.Head) LOOP
         Put(My_Fifo.Data(Index));
         New_Line;

         IF (Index = Element_Range'Last)
               THEN
            Index:=Element_Range'First;
         ELSE
            Index:=Index+1;
         END IF;
      END LOOP;

      New_Line;
      New_Line;

   END Put;


   PROCEDURE Insert (
         My_Fifo : IN OUT Fifo;
         E       : IN     Element) IS

      Tmp : Element_Range;
   BEGIN
      IF (My_Fifo.Head = Element_Range'Last)
            THEN
         Tmp:=Element_Range'First;
      ELSE
         Tmp:=My_Fifo.Head+1;
      END IF;
      IF (My_Fifo.Queue = Tmp)
            THEN
         RAISE Full_Fifo;
      END IF;

      My_Fifo.Data(My_Fifo.Head):=E;
      IF (My_Fifo.Head = Element_Range'Last)
            THEN
         My_Fifo.Head:=Element_Range'First;
      ELSE
         My_Fifo.Head:=My_Fifo.Head+1;
      END IF;

      My_Fifo.Size:=My_Fifo.Size+1;
   END Insert;


   PROCEDURE Extract (
         My_Fifo : IN OUT Fifo;
         E       : IN OUT Element) IS
   BEGIN
      IF Is_Empty(My_Fifo)
            THEN
         RAISE Empty_Fifo;
      END IF;

      E:=My_Fifo.Data(My_Fifo.Queue);

      IF (My_Fifo.Queue = Element_Range'Last)
            THEN
         My_Fifo.Queue:=Element_Range'First;
      ELSE
         My_Fifo.Queue:=My_Fifo.Queue+1;
      END IF;

      My_Fifo.Size:=My_Fifo.Size-1;
   END Extract;



   PROCEDURE Extract_Any_Where (
         My_Fifo : IN OUT Fifo;
         E       : IN     Element) IS

      Index : Element_Range := My_Fifo.Queue;

   BEGIN
      IF Is_Empty(My_Fifo)
            THEN
         RAISE Empty_Fifo;
      END IF;

      -- Looking for the element
      --
      WHILE (Index /= My_Fifo.Head) LOOP

         IF E=My_Fifo.Data(Index)
               THEN
            WHILE Index/=My_Fifo.Head LOOP
               IF Index=Element_Range'Last THEN
                  My_Fifo.Data(Index):=
                     My_Fifo.Data(Element_Range'First);
               ELSE
                  My_Fifo.Data(Index):=
                     My_Fifo.Data(Index+1);
               END IF;

               IF (Index = Element_Range'Last)
                     THEN
                  Index:=Element_Range'First;
               ELSE
                  Index:=Index+1;
               END IF;
            END LOOP;

            IF (My_Fifo.Head = Element_Range'First)
                  THEN
               My_Fifo.Head:=Element_Range'Last;
            ELSE
               My_Fifo.Head:=My_Fifo.Head-1;
            END IF;
            My_Fifo.Size:=My_Fifo.Size-1;
            RETURN;
         END IF;

         IF (Index = Element_Range'Last)
               THEN
            Index:=Element_Range'First;
         ELSE
            Index:=Index+1;
         END IF;
      END LOOP;

      RAISE Element_Not_Found;

   END Extract_Any_Where;



   FUNCTION Consult (
         My_Fifo : IN     Fifo)
     RETURN Element IS
   BEGIN
      IF Is_Empty(My_Fifo)
            THEN
         RAISE Empty_Fifo;
      END IF;

      RETURN (My_Fifo.Data(My_Fifo.Queue));
   END Consult;


   FUNCTION Is_Empty (
         My_Fifo : IN     Fifo)
     RETURN Boolean IS
   BEGIN
      RETURN (My_Fifo.Queue=My_Fifo.Head);
   END Is_Empty;


   FUNCTION Get_Size (
         My_Fifo : IN     Fifo)
     RETURN Natural IS
   BEGIN
      RETURN (My_Fifo.Size);
   END Get_Size;





END Fifos;




