------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



GENERIC

   Max_Element : Natural := 100;
   TYPE Element IS PRIVATE;
   WITH PROCEDURE Put (
         A_Element : IN     Element);


PACKAGE Fifos IS



   TYPE Fifo IS TAGGED PRIVATE;
   TYPE Element_Range IS NEW Natural RANGE 0 .. Max_Element;
   TYPE Fifo_Ptr IS ACCESS ALL Fifo'Class;


   Full_Fifo : EXCEPTION;
   Empty_Fifo : EXCEPTION;
   Element_Not_Found : EXCEPTION;




   -- Clear the fifo
   --
   PROCEDURE Reset (
         My_Fifo : IN OUT Fifo);


   -- Put the content of the fifo in the
   -- screen
   --
   PROCEDURE Put (
         My_Fifo : IN     Fifo);


   -- Insert a element into the fifo
   -- We insert with the head pointer
   --
   PROCEDURE Insert (
         My_Fifo : IN OUT Fifo;
         E       : IN     Element);


   -- Extract a element from the fifo
   -- We extract with the queue pointer
   --
   PROCEDURE Extract (
         My_Fifo : IN OUT Fifo;
         E       : IN OUT Element);


   -- Extract "Element" from any where
   --
   PROCEDURE Extract_Any_Where (
         My_Fifo : IN OUT Fifo;
         E       : IN     Element);


   -- Read (but not extract) a element from the fifo
   -- We read with the queue pointer
   --
   FUNCTION Consult (
         My_Fifo : IN     Fifo)
     RETURN Element;


   -- Get the number of elements currently present
   -- in the fifo
   --
   FUNCTION Get_Size (
         My_Fifo : IN     Fifo)
     RETURN Natural;


   -- Is my_fifo empty ?
   --
   FUNCTION Is_Empty (
         My_Fifo : IN     Fifo)
     RETURN Boolean;


PRIVATE


   TYPE Element_Table IS ARRAY (Element_Range) OF Element;

   TYPE Fifo IS TAGGED
      RECORD
         Size  : Natural       := 0;
         Data  : Element_Table;
         Queue : Element_Range;
         Head  : Element_Range;
      END RECORD;




END Fifos;




