------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;



PACKAGE BODY Hashs IS



   PROCEDURE Put (
         H : IN     Hash) IS

      Current : Link;

   BEGIN
      FOR I IN Hash'RANGE LOOP
         Current:=H(I);
         WHILE Current/=NULL LOOP
            Put("Index = " & To_String(Current.Index) );
            Put("   Data = ");
            Put(Current.A_Data);
            New_Line;
            New_Line;

            Current:=Current.Next;
         END LOOP;
      END LOOP;
   END Put;


   PROCEDURE Initialize (
         H : IN OUT Hash) IS
   BEGIN
      FOR I IN Hash'RANGE LOOP
         H(I):=NULL;
      END LOOP;
   END Initialize;


   PROCEDURE Add (
         H     : IN OUT Hash;
         Index : IN     Unbounded_String;
         D     : IN     Data_Type) IS

      Item : Natural := Compute_Hash (Index);
      C    : Link;

   BEGIN
      IF H(Item)/=NULL THEN
         C := NEW Cell;
         C.A_Data:=D;
         C.Index:=Index;
         C.Next:=H(Item).Next;
         H(Item).Next:=C;
      ELSE
         H(Item):= NEW Cell;
         H(Item).A_Data:=D;
         H(Item).Index:=Index;
      END IF;
   END Add;


   FUNCTION Consult (
         H     : IN     Hash;
         Index : IN     Unbounded_String)
     RETURN Data_Type IS

      Item    : Natural := Compute_Hash (Index);
      Current : Link    := H (Item);

   BEGIN

      WHILE Current/=NULL LOOP
         IF Current.Index = Index
               THEN
            RETURN Current.A_Data;
         END IF;
         Current:=Current.Next;
      END LOOP;

      RAISE Data_Not_Found;

   END Consult;


END Hashs;


