------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;



GENERIC
   Size : Positive := 100;
   TYPE Data_Type IS PRIVATE;

   WITH FUNCTION Compute_Hash (
         Index : IN     Unbounded_String)
     RETURN Natural;
   WITH PROCEDURE Put (
         D : Data_Type);


PACKAGE Hashs IS


   TYPE Hash IS PRIVATE;


   PROCEDURE Initialize (
         H : IN OUT Hash);

   PROCEDURE Add (
         H     : IN OUT Hash;
         Index : IN     Unbounded_String;
         D     : IN     Data_Type);

   FUNCTION Consult (
         H     : IN     Hash;
         Index : IN     Unbounded_String)
     RETURN Data_Type;

   PROCEDURE Put (
         H : IN     Hash);


   Data_Not_Found : EXCEPTION;


PRIVATE


   TYPE Cell;
   TYPE Link IS ACCESS Cell;


   TYPE Cell IS
      RECORD
         Next   : Link;
         Index  : Unbounded_String;
         A_Data : Data_Type;
      END RECORD;


   TYPE Hash IS ARRAY (1 .. Size) OF Link;



END Hashs;


