------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Text_Io;
USE Text_Io;




PACKAGE BODY Indexed_Tables IS



   PROCEDURE Initialize (
         A_Table :    OUT Indexed_Table) IS
   BEGIN
      A_Table.Nb_Entries:=0;
      FOR I IN Indexed_Table_Range LOOP
         Initialize(A_Table.Entries(I).Data);
      END LOOP;
   END Initialize;


   PROCEDURE Put (
         A_Item : IN     Item) IS
   BEGIN
      Put (A_Item.Item);
      Put ("/");
      Put (A_Item.Data);
      New_Line;
   END Put;


   PROCEDURE Put (
         A_Table : IN     Indexed_Table) IS
   BEGIN
      Put_Line(" Nb entries : " & A_Table.Nb_Entries'Img);
      FOR I IN First_Range..First_Range+A_Table.Nb_Entries-1 LOOP
         Put(A_Table.Entries(I));
      END LOOP;
   END Put;



   PROCEDURE Delete (
         A_Table : IN OUT Indexed_Table;
         A_Item  :        Indexed_Table_Range) IS
   BEGIN

      A_Table.Entries(A_Item):=
         A_Table.Entries(A_Table.Nb_Entries-1);
      A_Table.Nb_Entries:=
         A_Table.Nb_Entries-1;

   END Delete;



   PROCEDURE Add (
         A_Table   : IN OUT Indexed_Table;
         A_Index   :        Index;
         A_Element :        Element) IS
   BEGIN

      IF A_Table.Nb_Entries >   Indexed_Table_Range'Last-1 THEN
         RAISE Indexed_Table_Full;
      END IF;

      A_Table.Entries(A_Table.Nb_Entries).Item:=A_Index;
      A_Table.Entries(A_Table.Nb_Entries).Data:=A_Element;
      A_Table.Nb_Entries:=
         A_Table.Nb_Entries+1;
   END Add;




END Indexed_Tables;



