------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Unchecked_Deallocation;



GENERIC
   TYPE Element IS PRIVATE;
   TYPE Index IS PRIVATE;

   Size  : Natural := 100;
   First : Natural := 0;
   WITH PROCEDURE Put (
         E : IN     Element);
   WITH PROCEDURE Initialize (
         E :    OUT Element);
   WITH PROCEDURE Put (
         E : IN     Index);

PACKAGE Indexed_Tables IS



   -- This exception is raised when adding a new item
   -- in the table will conduct to have Ada check bound exception
   --
   Indexed_Table_Full : EXCEPTION;


   -- Do not change range 0..first+size
   -- intro first..first+size : it's a tips to
   -- avoid constraint error with nb_entries
   -- define by a table_range (and not a natural)
   --
   TYPE Indexed_Table_Range IS NEW Natural RANGE 0 .. First + Size;
   First_Range : Indexed_Table_Range :=Indexed_Table_Range(First);


   TYPE Item IS
      RECORD
         Data : Element;
         Item : Index;
      END RECORD;

   TYPE Data_Table IS ARRAY (Indexed_Table_Range) OF Item;

   TYPE Indexed_Table IS
      RECORD
         Nb_Entries : Indexed_Table_Range := 0;
         Entries    : Data_Table;
      END RECORD;

   TYPE Indexed_Table_Ptr IS ACCESS Indexed_Table;


   PROCEDURE Put (
         A_Item : IN     Item);

   PROCEDURE Put (
         A_Table : IN     Indexed_Table);

   PROCEDURE Initialize (
         A_Table :    OUT Indexed_Table);


   PROCEDURE Delete (
         A_Table : IN OUT Indexed_Table;
         A_Item  :        Indexed_Table_Range);

   PROCEDURE Add (
         A_Table   : IN OUT Indexed_Table;
         A_Index   :        Index;
         A_Element :        Element);


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Indexed_Table, Indexed_Table_Ptr);




   PRAGMA Inline(Add);
   PRAGMA Inline(Delete);

END Indexed_Tables;



