------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;


PACKAGE Io_Tools IS

   TYPE File_Mode IS
         (Save_In_New_File,
          Append_To_Old_File);


   Can_Not_Open_File_For_Reading : EXCEPTION;
   Can_Not_Open_File_For_Writing : EXCEPTION;


   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   FUNCTION Read_Sequential_File (
         File_Name : Unbounded_String)
     RETURN Unbounded_String;
   FUNCTION Read_Sequential_File (
         File_Name : String)
     RETURN Unbounded_String;



   -- Read a ASCII file and store it in a
   -- ADA string
   -- I/O Exceptions can be raised in case of I/O error
   --
   PROCEDURE Write_Sequential_File (
         File_Name : IN     Unbounded_String;
         Data      : IN     Unbounded_String);
   PROCEDURE Write_Sequential_File (
         File_Name : IN     String;
         Data      : IN     Unbounded_String);


   -- Check is a file can be read
   -- Return True if the file can be read
   --
   FUNCTION Can_Be_Read (
         File_Name : Unbounded_String)
     RETURN Boolean;


   -- Return the directory separator for Ubnix or windows
   -- depending on the file_name and the already directory
   -- separators it contains
   --
   FUNCTION Directory_Separator (
         File_Name : Unbounded_String)
     RETURN String;



END Io_Tools;


