------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




GENERIC
   TYPE Element IS PRIVATE;
   WITH PROCEDURE Put (
         E : IN     Element);
   WITH PROCEDURE Free (
         E : IN OUT Element);


PACKAGE Lists IS



   TYPE Cell IS PRIVATE;
   TYPE List IS PRIVATE;


   TYPE Iterator IS PRIVATE;


   Element_Not_Found : EXCEPTION;



   PROCEDURE Initialize (
         L : IN OUT List);


   PROCEDURE Add (
         L : IN OUT List;
         E : IN     Element);


   PROCEDURE Put (
         L : IN     List);


   -- Remove from "L" the element "E"
   --
   PROCEDURE Delete (
         L : IN OUT List;
         E : IN     Element);


   -- Delete from "L" all elements stored into "E"
   --
   PROCEDURE Delete (
         L : IN OUT List;
         E : IN     List);


   -- Delete  all elements of the list
   --
   PROCEDURE Delete (
         L : IN OUT List);



   PROCEDURE Duplicate (
         Src  : IN     List;
         Dest : IN OUT List);


   FUNCTION Is_Empty (
         L : IN     List)
     RETURN Boolean;




   FUNCTION Get_Number_Of_Elements (
         L : IN     List)
     RETURN Natural;



   FUNCTION Get_Head (
         L : IN     List)
     RETURN Element;


   FUNCTION Get_Tail (
         L : IN     List)
     RETURN Element;


   -- Move the current iterator to the next or previous element
   --
   PROCEDURE Next_Element (
         L           : IN     List;
         My_Iterator : IN OUT Iterator);
   PROCEDURE Previous_Element (
         L           : IN     List;
         My_Iterator : IN OUT Iterator);


   -- Get the current element pointed by the iterator
   --
   PROCEDURE Current_Element (
         L              : IN     List;
         Return_Element :    OUT Element;
         My_Iterator    : IN     Iterator);


   -- Initiliaze an iterator the the head or the tail of a list
   --
   PROCEDURE Reset_Tail_Iterator (
         L           : IN     List;
         My_Iterator : IN OUT Iterator);
   PROCEDURE Reset_Head_Iterator (
         L           : IN     List;
         My_Iterator : IN OUT Iterator);


   -- the element is the tail of the head of a list ?
   --
   FUNCTION Is_Tail_Element (
         L           : IN     List;
         My_Iterator : IN     Iterator)
     RETURN Boolean;
   FUNCTION Is_Head_Element (
         L           : IN     List;
         My_Iterator : IN     Iterator)
     RETURN Boolean;



PRIVATE

   TYPE Cell_Ptr IS ACCESS Cell;

   TYPE Cell IS
      RECORD
         Next     : Cell_Ptr;
         Previous : Cell_Ptr;
         Info     : Element;
      END RECORD;


   TYPE List IS
      RECORD
         Head : Cell_Ptr;
         Tail : Cell_Ptr;
      END RECORD;


   TYPE Iterator IS
      RECORD
         Current : Cell_Ptr;
      END RECORD;



END Lists;
