------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Text_Io;
USE Text_Io;


WITH Discrete_Util;



PACKAGE BODY Natural_Util IS

   PACKAGE Natural_Io IS NEW Text_Io.Integer_Io(Natural);
   PACKAGE Natural_Discrete  IS NEW Discrete_Util(Natural);


   FUNCTION Format (
         F : IN     Natural)
     RETURN Unbounded_String IS
   BEGIN
      RETURN(Natural_Discrete.Format(F));
   END Format;


   PROCEDURE Put (
         E : IN     Natural_Ptr) IS
   BEGIN
      Natural_Io.Put(E.All);
   END Put;

   PROCEDURE Put (
         E : IN     Natural) IS
   BEGIN
      Natural_Io.Put(E);
   END Put;

   PROCEDURE Put_Line (
         E : IN     Natural) IS
   BEGIN
      Natural_Io.Put(E);
      New_Line;
   END Put_Line;


   PROCEDURE Initialize (
         A :    OUT Natural) IS
   BEGIN
      A:=0;
   END Initialize;


   FUNCTION Gcd (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Natural IS

   BEGIN
      RETURN(Natural_Discrete.Gcd(A,B));
   END Gcd;

   FUNCTION Lcm (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Natural IS

   BEGIN
      RETURN(Natural_Discrete.Lcm(A,B));
   END Lcm;

   FUNCTION Lcm (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Double IS

   BEGIN
      RETURN(Natural_Discrete.Lcm(A,B));
   END Lcm;



END Natural_Util;


