------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;


WITH Lists;
WITH Unchecked_Deallocation;




PACKAGE Natural_Util IS


   TYPE Natural_Ptr IS ACCESS Natural;



   -- Convert and format a natural to
   -- an unbounded_string
   --
   FUNCTION Format (
         F : IN     Natural)
     RETURN Unbounded_String;


   -- Display a natural
   --
   PROCEDURE Put (
         E : IN     Natural);
   PROCEDURE Put (
         E : IN     Natural_Ptr);
   PROCEDURE Put_Line (
         E : IN     Natural);


   -- Initialize a natural
   --
   PROCEDURE Initialize (
         A :    OUT Natural);


   -- Compute Great common Divisor
   -- of two natural
   --
   FUNCTION Gcd (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Natural;


   -- Compute Last common multiplier
   -- of two natural
   --
   FUNCTION Lcm (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Natural;
   FUNCTION Lcm (
         A : IN     Natural;
         B : IN     Natural)
     RETURN Double;




   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Natural, Natural_Ptr);


   PACKAGE Natural_List IS NEW Lists(Natural_Ptr, Put,
      Free);
   USE Natural_List;





END Natural_Util;


