------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Text_Io;
USE Text_Io;

PACKAGE BODY Real_Util IS


   PACKAGE Real_Element_Io IS NEW Text_Io.Float_Io(Real_Element);



   FUNCTION Equal (
         Num1,
         Num2     : Real_Element;
         Accuracy : Integer      := 10000)
     RETURN Boolean IS

      My_Epsi : CONSTANT Real_Element := 1.0 / Real_Element (Accuracy);
      Diff    : CONSTANT Real_Element :=
      ABS(Num1 - Num2);
   BEGIN
      RETURN (Diff < My_Epsi);
   END Equal;


   FUNCTION Format (
         F     : IN     Real_Element;
         After : IN     Natural      := 5)
     RETURN Unbounded_String IS
      S      : String (1 .. 10000);
      Result : Unbounded_String;
   BEGIN
      Real_Element_Io.Put(S, F, After, Exp=>0);

      -- suppress blank
      FOR I IN 1..S'Length LOOP
         IF S(I) /= ' '
               THEN
            Append(Result,S(I));
         END IF;
      END LOOP;

      RETURN(Result);
   END Format;


END Real_Util;
