------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Text_Io;
USE Text_Io;



PACKAGE BODY Stacks IS



   PROCEDURE Reset (
         A_Stack : IN OUT Stack) IS

   BEGIN
      A_Stack.First_Free:=0;
   END Reset;


   FUNCTION Get_Size (
         My_Stack : IN     Stack)
     RETURN Element_Range IS

   BEGIN
      RETURN My_Stack.First_Free;
   END Get_Size;


   -- Is my_stack empty ?
   --
   FUNCTION Is_Empty (
         My_Stack : IN     Stack)
     RETURN Boolean IS

   BEGIN
      RETURN My_Stack.First_Free=0;
   END Is_Empty;


   -- Put the content of the stack in the
   -- screen
   --
   PROCEDURE Put (
         My_Stack : IN     Stack) IS

   BEGIN
      Put_Line("Number of elements : " & My_Stack.First_Free'Img);
      FOR I IN 0..My_Stack.First_Free-1 LOOP
         Put(My_Stack.Data(I));
      END LOOP;
      New_Line;
   END Put;


   -- Get the last inserted element
   --
   FUNCTION Get_Top (
         My_Stack : IN     Stack)
     RETURN Element IS

   BEGIN
      IF My_Stack.First_Free=0 THEN
         RAISE Empty_Stack;
      END IF;
      RETURN My_Stack.Data(My_Stack.First_Free-1);
   END Get_Top;


   -- Push a new element
   --
   PROCEDURE Push (
         My_Stack  : IN OUT Stack;
         A_Element : IN     Element) IS
   BEGIN
      IF My_Stack.First_Free=Element_Range'Last THEN
         RAISE Full_Stack;
      END IF;
      My_Stack.Data(My_Stack.First_Free):=A_Element;
      My_Stack.First_Free:=My_Stack.First_Free+1;

   END Push;


   -- Pop a element
   --
   PROCEDURE Pop (
         My_Stack  : IN OUT Stack;
         A_Element : IN OUT Element) IS

   BEGIN
      IF My_Stack.First_Free=0 THEN
         RAISE Empty_Stack;
      END IF;
      A_Element:=My_Stack.Data(My_Stack.First_Free-1);
      My_Stack.First_Free:=My_Stack.First_Free-1;
   END Pop;



END Stacks;




