------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


GENERIC
   Max_Element : Natural := 100;
   TYPE Element IS PRIVATE;
   WITH PROCEDURE Put (
         A_Element : IN     Element);

PACKAGE Stacks IS




   TYPE Stack IS TAGGED PRIVATE;
   TYPE Element_Range IS NEW Natural RANGE 0 .. Max_Element;
   TYPE Stack_Ptr IS ACCESS ALL Stack'Class;


   Empty_Stack, Full_Stack : EXCEPTION;


   -- Clear the stack
   --	
   PROCEDURE Reset (
         A_Stack : IN OUT Stack);


   -- Get the number of elements currently present
   -- in the stack
   --
   FUNCTION Get_Size (
         My_Stack : IN     Stack)
     RETURN Element_Range;


   -- Is my_stack empty ?
   --
   FUNCTION Is_Empty (
         My_Stack : IN     Stack)
     RETURN Boolean;


   -- Put the content of the stack in the
   -- screen
   --
   PROCEDURE Put (
         My_Stack : IN     Stack);


   -- Get the last inserted element
   --
   FUNCTION Get_Top (
         My_Stack : IN     Stack)
     RETURN Element;


   -- Push a new element
   --
   PROCEDURE Push (
         My_Stack  : IN OUT Stack;
         A_Element : IN     Element);


   -- Pop a element
   --
   PROCEDURE Pop (
         My_Stack  : IN OUT Stack;
         A_Element : IN OUT Element);



PRIVATE


   TYPE Element_Table IS ARRAY (Element_Range) OF Element;

   TYPE Stack IS TAGGED
      RECORD
         First_Free : Element_Range := 0;
         Data       : Element_Table;
      END RECORD;


END Stacks;




